  .syntax unified
  .cpu cortex-m0plus
  .fpu softvfp
  .thumb

.global  g_pfnVectors
.global  Default_Handler

/* start address for the initialization values of the .data section.
defined in linker script */
.word  _sidata
/* start address for the .data section. defined in linker script */
.word  _sdata
/* end address for the .data section. defined in linker script */
.word  _edata
/* start address for the .bss section. defined in linker script */
.word  _sbss
/* end address for the .bss section. defined in linker script */
.word  _ebss

    .section  .text.Reset_Handler
  .weak  Reset_Handler
  .type  Reset_Handler, %function
Reset_Handler:  
   ldr   r0, =_estack
   mov   sp, r0          /* set stack pointer */

ApplicationStart:
/* Copy the data segment initializers from flash to SRAM */
  ldr r0, =_sdata
  ldr r1, =_edata
  ldr r2, =_sidata
  movs r3, #0
  b LoopCopyDataInit

CopyDataInit:
  ldr r4, [r2, r3]
  str r4, [r0, r3]
  adds r3, r3, #4

LoopCopyDataInit:
  adds r4, r0, r3
  cmp r4, r1
  bcc CopyDataInit
  
/* Zero fill the bss segment. */
  ldr r2, =_sbss
  ldr r4, =_ebss
  movs r3, #0
  b LoopFillZerobss

FillZerobss:
  str  r3, [r2]
  adds r2, r2, #4

LoopFillZerobss:
  cmp r2, r4
  bcc FillZerobss

/* Call the clock system intitialization function.*/
  bl  SystemInit
/* Call static constructors */
    bl __libc_init_array
/* Call the application's entry point.*/
  bl  main

LoopForever:
    b LoopForever


.size  Reset_Handler, .-Reset_Handler

/**
 * @brief  This is the code that gets called when the processor receives an
 *         unexpected interrupt.  This simply enters an infinite loop, preserving
 *         the system state for examination by a debugger.
*/
    .section  .text.Default_Handler,"ax",%progbits
Default_Handler:
Infinite_Loop:
  b  Infinite_Loop
  .size  Default_Handler, .-Default_Handler

/******************************************************************************
*
* The minimal vector table for a Cortex M0.  Note that the proper constructs
* must be placed on this to ensure that it ends up at physical address
* 0x0000.0000.
*
******************************************************************************/
  .section  .isr_vector,"a",%progbits
  .type  g_pfnVectors, %object
  .size  g_pfnVectors, .-g_pfnVectors
g_pfnVectors:
  .word  _estack
  .word  Reset_Handler
  .word  NMI_Handler
  .word  HardFault_Handler
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  SVC_Handler
  .word  0
  .word  0
  .word  PendSV_Handler
  .word  SysTick_Handler
  /* External Interrupts */
  .word  WWDT_IRQHandler           /* 0:  WWDT  */       
  .word  SVD_IRQHandler            /* 1:  SVD  */	       
  .word  RTC_IRQHandler            /* 2:  RTC	  */      
  .word  FLASH_IRQHandler          /* 3:  FLASH	*/     
  .word  CMU_IRQHandler            /* 4:  CMU	  */      
  .word  ADC_IRQHandler            /* 5:  ADC	  */      
  .word  SPI0_IRQHandler           /* 6:  SPI0   */      
  .word  SPI1_IRQHandler           /* 7:  SPI1 	 */     
  .word  SPI2_IRQHandler           /* 8:  SPI2   */     
  .word  UART0_IRQHandler          /* 9:  UART0	  */    
  .word  UART1_IRQHandler          /* 10:  UART1	*/     
  .word  UART2_IRQHandler          /* 11:  UART2	*/	   
  .word  UART3_IRQHandler          /* 12:  UART3	*/     
  .word  UART4_IRQHandler          /* 13:  UART4	*/     
  .word  UART5_IRQHandler          /* 14:  UART5	*/	   
  .word  U7816_IRQHandler		       /* 15:  U7816	*/   
  .word  LPUART0_IRQHandler			   /* 16:  LPUART0*/   
  .word  I2Cx_IRQHandler           /* 17:  I2Cx   */   
  .word  0                         /* 18:  NULL	  */  
  .word  CRYPTO_IRQHandler         /* 19:  CRYPTO	*/  
  .word  LPTIM_IRQHandler          /* 20:  LPTIM	*/    
  .word  DMA_IRQHandler            /* 21:  DMA	  */    
  .word  WKUPx_IRQHandler          /* 22:  WKUPx	*/     
  .word  COMP_IRQHandler           /* 23:  COMP	  */   
  .word  BTx_IRQHandler            /* 24:  BTx    */    
  .word  QSPI_IRQHandler           /* 25:  QSPI   */    
  .word  ETx_IRQHandler            /* 26:  ETx	  */     
  .word  BSTIM_IRQHandler          /* 27:  BSTIM	*/     
  .word  SPI3_IRQHandler           /* 28:  SPI3	  */    
  .word  SPI4_IRQHandler           /* 29:  SPI4   */    
  .word  GPIO_IRQHandler           /* 30:  GPIO   */    
  .word  LPUART1_IRQHandler 			 /* 31:  LPUART0*/  

/*******************************************************************************
*
* Provide weak aliases for each Exception handler to the Default_Handler.
* As they are weak aliases, any function with the same name will override
* this definition.
*
*******************************************************************************/
   .weak      NMI_Handler
   .thumb_set NMI_Handler,Default_Handler

   .weak      HardFault_Handler
   .thumb_set HardFault_Handler,Default_Handler

   .weak      SVC_Handler
   .thumb_set SVC_Handler,Default_Handler

   .weak      PendSV_Handler
   .thumb_set PendSV_Handler,Default_Handler

   .weak      SysTick_Handler
   .thumb_set SysTick_Handler,Default_Handler

   .weak      WWDT_IRQHandler
   .thumb_set WWDT_IRQHandler,Default_Handler

   .weak      SVD_IRQHandler
   .thumb_set SVD_IRQHandler,Default_Handler

   .weak      RTC_IRQHandler
   .thumb_set RTC_IRQHandler,Default_Handler

   .weak      FLASH_IRQHandler
   .thumb_set FLASH_IRQHandler,Default_Handler

   .weak      CMU_IRQHandler
   .thumb_set CMU_IRQHandler,Default_Handler

   .weak      ADC_IRQHandler
   .thumb_set ADC_IRQHandler,Default_Handler

   .weak      SPI0_IRQHandler
   .thumb_set SPI0_IRQHandler,Default_Handler

   .weak      SPI1_IRQHandler
   .thumb_set SPI1_IRQHandler,Default_Handler

   .weak      SPI2_IRQHandler
   .thumb_set SPI2_IRQHandler,Default_Handler

   .weak      UART0_IRQHandler
   .thumb_set UART0_IRQHandler,Default_Handler

   .weak      UART1_IRQHandler
   .thumb_set UART1_IRQHandler,Default_Handler

   .weak      UART2_IRQHandler
   .thumb_set UART2_IRQHandler,Default_Handler

   .weak      UART3_IRQHandler
   .thumb_set UART3_IRQHandler,Default_Handler
   
   .weak      UART4_IRQHandler                   
   .thumb_set UART4_IRQHandler,Default_Handler   
 
   .weak      UART5_IRQHandler
   .thumb_set UART5_IRQHandler,Default_Handler

   .weak      U7816_IRQHandler
   .thumb_set U7816_IRQHandler,Default_Handler

   .weak      LPUART0_IRQHandler
   .thumb_set LPUART0_IRQHandler,Default_Handler

   .weak      I2Cx_IRQHandler
   .thumb_set I2Cx_IRQHandler,Default_Handler

   .weak      CRYPTO_IRQHandler
   .thumb_set CRYPTO_IRQHandler,Default_Handler

   .weak      LPTIM_IRQHandler
   .thumb_set LPTIM_IRQHandler,Default_Handler

   .weak      DMA_IRQHandler
   .thumb_set DMA_IRQHandler,Default_Handler

   .weak      WKUPx_IRQHandler
   .thumb_set WKUPx_IRQHandler,Default_Handler

   .weak      COMP_IRQHandler
   .thumb_set COMP_IRQHandler,Default_Handler

   .weak      BTx_IRQHandler
   .thumb_set BTx_IRQHandler,Default_Handler

   .weak      QSPI_IRQHandler
   .thumb_set QSPI_IRQHandler,Default_Handler

   .weak      ETx_IRQHandler
   .thumb_set ETx_IRQHandler,Default_Handler

   .weak      BSTIM_IRQHandler
   .thumb_set BSTIM_IRQHandler,Default_Handler

   .weak      SPI3_IRQHandler
   .thumb_set SPI3_IRQHandler,Default_Handler

   .weak      SPI4_IRQHandler
   .thumb_set SPI4_IRQHandler,Default_Handler

   .weak      GPIO_IRQHandler
   .thumb_set GPIO_IRQHandler,Default_Handler
   
   .weak      LPUART1_IRQHandler
   .thumb_set LPUART1_IRQHandler,Default_Handler

