/**
  ******************************************************************************
  * @file    fm33a0xxev_cmu.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the CMU firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_CMU_H
#define __FM33A0XXEV_CMU_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
  

/* Defines------------------------------------------------------------------*/
#define HXVAR(object, addr)   (*((object *) (addr)))
#define const_rchf_Trim8 	HXVAR( uint32_t, 0x1FFFFB40 )	//RCHF 8MHzУֵ
#define const_rchf_Trim16 	HXVAR( uint32_t, 0x1FFFFB3C )	//RCHF 16MHzУֵ
#define const_rchf_Trim24 	HXVAR( uint32_t, 0x1FFFFB38 )	//RCHF 24MHzУֵ
#define const_rchf_Trim32 	HXVAR( uint32_t, 0x1FFFFB34)   //RCHF 32MHzУֵ
#define const_rchf_Trim40 	HXVAR( uint32_t, 0x1FFFFB48)   //RCHF 40MHzУֵ
#define const_rchf_Trim48 	HXVAR( uint32_t, 0x1FFFFB4C)   //RCHF 48MHzУֵ
	 
#define __XTHF_CLOCK                (8000000) 
/* Exported constants --------------------------------------------------------*/

/* Exported types ------------------------------------------------------------*/
typedef struct
{
	uint32_t			FSEL;		/*!<RCHFƵ  */
	FunState			RCHFEN;			/*!<RCHFʹܿ  */
}CMU_RCHF_InitTypeDef;

typedef struct
{            
  uint32_t		PLLLDB;		/*!<PLLƵ1023,pllƵ = PLLDB + 1  */
	FunState		PLLL_EN;		/*!<PLLʹܿ  */
}CMU_PLL_L_InitTypeDef;


typedef struct
{            
  uint32_t		PLLHDB;		/*!<PLLƵ1023,pllƵ = PLLDB + 1  */
	uint32_t 		REFPRSC;	/*!<PLLHοʱԤƵ  */
	uint32_t		PLLH_OSEL;	/*!<PLLѡ񣬵Ҫ1024ʱģʽ  */
	uint32_t		PLLH_INSEL;	/*!<PLLԴѡ  */
	FunState		PLLH_EN;		/*!<PLLʹܿ  */
}CMU_PLL_H_InitTypeDef;


typedef struct
{
  uint32_t			SYSCLKSEL;		/*!<ϵͳʱѡ񣬴״̬ѺԶָΪRCHF8M  */	
	uint32_t			AHBPRES;		/*!<AHBʱӷƵѡ  */
	uint32_t			APBPRES;		/*!<APBʱӷƵѡ  */
	FunState			SLP_ENEXTI;		/*!<Sleep/DeepSleepģʽEXTI  */
	
}CMU_SYSCLK_InitTypeDef;

typedef struct
{
  uint32_t SYSCLK_Frequency;		/*!<  SYSCLK clock frequency expressed in Hz */
  uint32_t AHBCLK_Frequency;		/*!<  AHB clock frequency expressed in Hz  */
  uint32_t APBCLK_Frequency;		/*!<  APB clock frequency expressed in Hz  */
  uint32_t RCHF_Frequency;			/*!<  RCHF clock frequency expressed in Hz   */
  uint32_t PLL_H_Frequency;			/*!<  PLL H clock frequency expressed in Hz   */
  uint32_t XTHF_Frequency;			/*!<  XTHF clock frequency expressed in Hz   */
  uint32_t LSCLK_Frequency;			/*!<  LSCLK clock frequency expressed in Hz   */
}CMU_ClocksType;



#define	CMU_SYSCLKCR_SLP_ENEXTI_Pos	25
#define	CMU_SYSCLKCR_SLP_ENEXTI_Msk	(0x1U << CMU_SYSCLKCR_SLP_ENEXTI_Pos)

#define	CMU_SYSCLKCR_APBPRES_Pos	16	/* APBʱӷƵѡ (APB1bus clock Prescaler)
000/001/010/011Ƶ
1002Ƶ
1014Ƶ
1108Ƶ
11116Ƶ */
#define	CMU_SYSCLKCR_APBPRES_Msk	(0x7U << CMU_SYSCLKCR_APBPRES_Pos)
#define	CMU_SYSCLKCR_APBPRES_DIV1	(0x0U << CMU_SYSCLKCR_APBPRES_Pos)	/* Ƶ */
#define	CMU_SYSCLKCR_APBPRES_DIV2	(0x4U << CMU_SYSCLKCR_APBPRES_Pos)	/* 2Ƶ */
#define	CMU_SYSCLKCR_APBPRES_DIV4	(0x5U << CMU_SYSCLKCR_APBPRES_Pos)	/* 4Ƶ */
#define	CMU_SYSCLKCR_APBPRES_DIV8	(0x6U << CMU_SYSCLKCR_APBPRES_Pos)	/* 8Ƶ */
#define	CMU_SYSCLKCR_APBPRES_DIV16	(0x7U << CMU_SYSCLKCR_APBPRES_Pos)	/* 16Ƶ */

#define	CMU_SYSCLKCR_AHBPRES_Pos	8	/* AHBʱӷƵѡ (AHB bus clock Prescaler)
000/001/010/011Ƶ
1002Ƶ
1014Ƶ
1108Ƶ
11116Ƶ */
#define	CMU_SYSCLKCR_AHBPRES_Msk	(0x7U << CMU_SYSCLKCR_AHBPRES_Pos)
#define	CMU_SYSCLKCR_AHBPRES_DIV1	(0x0U << CMU_SYSCLKCR_AHBPRES_Pos)	/* Ƶ */
#define	CMU_SYSCLKCR_AHBPRES_DIV2	(0x4U << CMU_SYSCLKCR_AHBPRES_Pos)	/* 2Ƶ */
#define	CMU_SYSCLKCR_AHBPRES_DIV4	(0x5U << CMU_SYSCLKCR_AHBPRES_Pos)	/* 4Ƶ */
#define	CMU_SYSCLKCR_AHBPRES_DIV8	(0x6U << CMU_SYSCLKCR_AHBPRES_Pos)	/* 8Ƶ */
#define	CMU_SYSCLKCR_AHBPRES_DIV16	(0x7U << CMU_SYSCLKCR_AHBPRES_Pos)	/* 16Ƶ */

#define	CMU_SYSCLKCR_STCLKSEL_Pos	6	/* CPUںsystickʱѡ (Systick clock select)
00SCLK
01LSCLK
10RC4M
11SYSCLK */
#define	CMU_SYSCLKCR_STCLKSEL_Msk	(0x3U << CMU_SYSCLKCR_STCLKSEL_Pos)
#define	CMU_SYSCLKCR_STCLKSEL_SCLK	(0x0U << CMU_SYSCLKCR_STCLKSEL_Pos)	/* SCLK */
#define	CMU_SYSCLKCR_STCLKSEL_LSCLK	(0x1U << CMU_SYSCLKCR_STCLKSEL_Pos)	/* LSCLK */
#define	CMU_SYSCLKCR_STCLKSEL_RC4M	(0x2U << CMU_SYSCLKCR_STCLKSEL_Pos)	/* RC4M */
#define	CMU_SYSCLKCR_STCLKSEL_SYSCLK	(0x3U << CMU_SYSCLKCR_STCLKSEL_Pos)	/* SYSCLK */

#define	CMU_SYSCLKCR_SYSCLKSEL_Pos	0	/* ϵͳʱԴѡ */
#define	CMU_SYSCLKCR_SYSCLKSEL_Msk	(0x7U << CMU_SYSCLKCR_SYSCLKSEL_Pos)
#define	CMU_SYSCLKCR_SYSCLKSEL_RCHF	(0x0U << CMU_SYSCLKCR_SYSCLKSEL_Pos)	/* RCHF */
#define	CMU_SYSCLKCR_SYSCLKSEL_XTHF	(0x1U << CMU_SYSCLKCR_SYSCLKSEL_Pos)	/* XTHF */
#define	CMU_SYSCLKCR_SYSCLKSEL_PLL_H	(0x2U << CMU_SYSCLKCR_SYSCLKSEL_Pos)	/* PLL_H */
#define	CMU_SYSCLKCR_SYSCLKSEL_LSCLK	(0x3U << CMU_SYSCLKCR_SYSCLKSEL_Pos)	/* LSCLK */

#define	CMU_RCHFCR_FSEL_Pos	16	/* RCHFƵѡĴ00008MHz
000116MHz
001024MHz
001132MHz
011140MHz
111148MHz
RFU */
#define	CMU_RCHFCR_FSEL_Msk	(0xfU << CMU_RCHFCR_FSEL_Pos)
#define	CMU_RCHFCR_FSEL_8MHZ	(0x0U << CMU_RCHFCR_FSEL_Pos)	/* 8MHz */
#define	CMU_RCHFCR_FSEL_16MHZ	(0x1U << CMU_RCHFCR_FSEL_Pos)	/* 16MHz */
#define	CMU_RCHFCR_FSEL_24MHZ	(0x2U << CMU_RCHFCR_FSEL_Pos)	/* 24MHz */
#define	CMU_RCHFCR_FSEL_32MHZ	(0x3U << CMU_RCHFCR_FSEL_Pos)	/* 32MHZ */
#define	CMU_RCHFCR_FSEL_40MHZ	(0x7U << CMU_RCHFCR_FSEL_Pos)	/* 40MHZ */
#define	CMU_RCHFCR_FSEL_48MHZ	(0xfU << CMU_RCHFCR_FSEL_Pos)	/* 48MHZ */

#define	CMU_RCHFCR_RCHFEN_Pos	0	/* RCHFʹܼĴ (RCHF Enable)
1ʹRCHF
0رRCHF */
#define	CMU_RCHFCR_RCHFEN_Msk	(0x1U << CMU_RCHFCR_RCHFEN_Pos)
	/* ʹRCHF */
	/* رRCHF */

#define	CMU_RCHFTR_RCHFTRIM_Pos	0	/* RCHFƵʵУĴ8h00ʾƵͣ8hFFʾƵߣУΧΪƵ+/-30%УΪƵ0.25%
ϵоƬԶLDT0ȡ8MHzУֵд˼Ĵ
ʹƵʱдLDT0ַָȡУϢд˼ĴӶȷƵ׼ȷ */
#define	CMU_RCHFTR_RCHFTRIM_Msk	(0xffU << CMU_RCHFTR_RCHFTRIM_Pos)

#define	CMU_PLLLCR_PLLDB_Pos	16
#define	CMU_PLLLCR_PLLDB_Msk	(0x3ffU << CMU_PLLLCR_PLLDB_Pos)

#define	CMU_PLLLCR_LOCKED_Pos	7
#define	CMU_PLLLCR_LOCKED_Msk	(0x1U << CMU_PLLLCR_LOCKED_Pos)

#define	CMU_PLLLCR_PLLEN_Pos	0	/* PLLʹܼĴ
1ʹPLL
0رPLL */
#define	CMU_PLLLCR_PLLEN_Msk	(0x1U << CMU_PLLLCR_PLLEN_Pos)

#define	CMU_PLLHCR_PLLHDB_Pos	16	/* PLLHƵȣοʱ1Mhz
000001111132Ƶ
010111148Ƶ */
#define	CMU_PLLHCR_PLLHDB_Msk	(0x3ffU << CMU_PLLHCR_PLLHDB_Pos)
#define	CMU_PLLHCR_PLLHDB_X32	(0x1fU << CMU_PLLHCR_PLLHDB_Pos)	/* 32Ƶ */
#define	CMU_PLLHCR_PLLHDB_X48	(0x2fU << CMU_PLLHCR_PLLHDB_Pos)	/* 48Ƶ */
#define	CMU_PLLHCR_PLLHDB_X64	(0x3fU << CMU_PLLHCR_PLLHDB_Pos)	/* 64Ƶ */

#define	CMU_PLLHCR_LOCKED_Pos	7	/* PLLH־ͨѯ˼ĴȷPLLѾ״̬
1PLL
0PLLδ */
#define	CMU_PLLHCR_LOCKED_Msk	(0x1U << CMU_PLLHCR_LOCKED_Pos)

#define	CMU_PLLHCR_REFPRSC_Pos	4	/* PLLHοʱԤƵĿǲ1MHzοʱӸPLL
000Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10012Ƶ
10116Ƶ
11024Ƶ
11132Ƶ */
#define	CMU_PLLHCR_REFPRSC_Msk	(0x7U << CMU_PLLHCR_REFPRSC_Pos)
#define	CMU_PLLHCR_REFPRSC_DIV1	(0x0U << CMU_PLLHCR_REFPRSC_Pos)	/* Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV2	(0x1U << CMU_PLLHCR_REFPRSC_Pos)	/* 2Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV4	(0x2U << CMU_PLLHCR_REFPRSC_Pos)	/* 4Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV8	(0x3U << CMU_PLLHCR_REFPRSC_Pos)	/* 8Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV12	(0x4U << CMU_PLLHCR_REFPRSC_Pos)	/* 12Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV16	(0x5U << CMU_PLLHCR_REFPRSC_Pos)	/* 16Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV24	(0x6U << CMU_PLLHCR_REFPRSC_Pos)	/* 24Ƶ */
#define	CMU_PLLHCR_REFPRSC_DIV32	(0x7U << CMU_PLLHCR_REFPRSC_Pos)	/* 32Ƶ */

#define	CMU_PLLHCR_OSEL_Pos	3	/* PLLHѡĴ
0ѡPLLHһΪֵ·ڵPLLʱ
1ѡPLLHΪֵ·ڵPLLʱ */
#define	CMU_PLLHCR_OSEL_Msk	(0x1U << CMU_PLLHCR_OSEL_Pos)
#define	CMU_PLLHCR_OSEL_X1	(0x0U << CMU_PLLHCR_OSEL_Pos)
#define	CMU_PLLHCR_OSEL_X2	(0x1U << CMU_PLLHCR_OSEL_Pos)

#define	CMU_PLLHCR_INSEL_Pos	1	/* PLLHѡĴ
0RCHF
1XTHF */
#define	CMU_PLLHCR_INSEL_Msk	(0x1U << CMU_PLLHCR_INSEL_Pos)
#define	CMU_PLLHCR_INSEL_RCHF	(0x0U << CMU_PLLHCR_INSEL_Pos)
#define	CMU_PLLHCR_INSEL_XTHF	(0x1U << CMU_PLLHCR_INSEL_Pos)

#define	CMU_PLLHCR_EN_Pos	0	/* PLLHʹܼĴ
1ʹPLLH
0رPLLH */
#define	CMU_PLLHCR_EN_Msk	(0x1U << CMU_PLLHCR_EN_Pos)

#define	CMU_XTHFCR_XTHF_CFG_Pos	8	/* XTHFǿ
000
111ǿ */
#define	CMU_XTHFCR_XTHF_CFG_Msk	(0x7U << CMU_XTHFCR_XTHF_CFG_Pos)
#define	CMU_XTHFCR_XTHF_CFG_MIN	(0x0U << CMU_XTHFCR_XTHF_CFG_Pos)	/*  */
#define	CMU_XTHFCR_XTHF_CFG_MAX	(0x7U << CMU_XTHFCR_XTHF_CFG_Pos)	/* ǿ */

#define	CMU_XTHFCR_XTHFEN_Pos	0	/* XTHFʹܼĴ
0رXTHF
1ʹXTHF */
#define	CMU_XTHFCR_XTHFEN_Msk	(0x1U << CMU_XTHFCR_XTHFEN_Pos)
	/* رXTHF */
	/* ʹXTHF */

#define	CMU_IER_SYSCSE_IE_Pos	1
#define	CMU_IER_SYSCSE_IE_Msk	(0x1U << CMU_IER_SYSCSE_IE_Pos)

#define	CMU_IER_HFDET_IE_Pos	0
#define	CMU_IER_HFDET_IE_Msk	(0x1U << CMU_IER_HFDET_IE_Pos)


#define	CMU_ISR_HFDETO_Pos	8	/* Ƶͣģ
1XTHFδͣ
0XTHFͣ */
#define	CMU_ISR_HFDETO_Msk	(0x1U << CMU_ISR_HFDETO_Pos)

#define	CMU_ISR_SYSCSE_IF_Pos	1/* SYSCLKʱѡжϱ־ĴѡеʱԴûʹʱжϱ־λд1㡣 */
#define	CMU_ISR_SYSCSE_IF_Msk	(0x1U << CMU_ISR_SYSCSE_IF_Pos)

#define	CMU_ISR_HFDET_IF_Pos	0	/* Ƶͣжϱ־ĴXTHFͣʱӲ첽λд1㣻ֻFFDETOΪ0²ܹ˼Ĵ */
#define	CMU_ISR_HFDET_IF_Msk	(0x1U << CMU_ISR_HFDET_IF_Pos)

#define	CMU_PCLKCR1_COMP_PCE_Pos	9	/* COMPʱʹܣʹ */
#define	CMU_PCLKCR1_COMP_PCE_Msk	(0x1U << CMU_PCLKCR1_COMP_PCE_Pos)

#define	CMU_PCLKCR1_SVD_PCE_Pos	8	/* SVDʱʹܣʹ */
#define	CMU_PCLKCR1_SVD_PCE_Msk	(0x1U << CMU_PCLKCR1_SVD_PCE_Pos)

#define	CMU_PCLKCR1_PAD_PCE_Pos	7	/* PADCFGʱʹܣʹ */
#define	CMU_PCLKCR1_PAD_PCE_Msk	(0x1U << CMU_PCLKCR1_PAD_PCE_Pos)

#define	CMU_PCLKCR1_ANAC_PCE_Pos	6	/* ANACʱʹܣʹ
˼ĴڿSVDȽʱ */
#define	CMU_PCLKCR1_ANAC_PCE_Msk	(0x1U << CMU_PCLKCR1_ANAC_PCE_Pos)

#define	CMU_PCLKCR1_IWDT_PCE_Pos	5	/* IWDTʱʹܣʹ */
#define	CMU_PCLKCR1_IWDT_PCE_Msk	(0x1U << CMU_PCLKCR1_IWDT_PCE_Pos)

#define	CMU_PCLKCR1_SCU_PCE_Pos	4	/* SCUʱʹܣʹ */
#define	CMU_PCLKCR1_SCU_PCE_Msk	(0x1U << CMU_PCLKCR1_SCU_PCE_Pos)

#define	CMU_PCLKCR1_PMU_PCE_Pos	3	/* PMUʱʹܣʹ */
#define	CMU_PCLKCR1_PMU_PCE_Msk	(0x1U << CMU_PCLKCR1_PMU_PCE_Pos)

#define	CMU_PCLKCR1_RTC_PCE_Pos	2	/* RTCʱʹܣʹ */
#define	CMU_PCLKCR1_RTC_PCE_Msk	(0x1U << CMU_PCLKCR1_RTC_PCE_Pos)

#define	CMU_PCLKCR1_LPT_PCE_Pos	0	/*  _32ʱʹܣʹ */
#define	CMU_PCLKCR1_LPT_PCE_Msk	(0x1U << CMU_PCLKCR1_LPT_PCE_Pos)

#define	CMU_PCLKCR2_PAE_PCE_Pos	17
#define	CMU_PCLKCR2_PAE_PCE_Msk	(0x1U << CMU_PCLKCR2_PAE_PCE_Pos)

#define	CMU_PCLKCR2_SHA_PCE_Pos	16
#define	CMU_PCLKCR2_SHA_PCE_Msk	(0x1U << CMU_PCLKCR2_SHA_PCE_Pos)

#define	CMU_PCLKCR2_CIC_PCE_Pos	8	/* ADCʱʹܣʹ */
#define	CMU_PCLKCR2_CIC_PCE_Msk	(0x1U << CMU_PCLKCR2_CIC_PCE_Pos)

#define	CMU_PCLKCR2_WWDT_PCE_Pos	7	/* WWDTʱʹܣʹ */
#define	CMU_PCLKCR2_WWDT_PCE_Msk	(0x1U << CMU_PCLKCR2_WWDT_PCE_Pos)

#define	CMU_PCLKCR2_RAMBIST_PCE_Pos	6	/* RAMBISTʱʹܣʹ */
#define	CMU_PCLKCR2_RAMBIST_PCE_Msk	(0x1U << CMU_PCLKCR2_RAMBIST_PCE_Pos)

#define	CMU_PCLKCR2_NVM_PCE_Pos	5	/* NVMIFFlashдʱʹܣʹ */
#define	CMU_PCLKCR2_NVM_PCE_Msk	(0x1U << CMU_PCLKCR2_NVM_PCE_Pos)

#define	CMU_PCLKCR2_DMA_PCE_Pos	4	/* DMAʱʹܣʹ */
#define	CMU_PCLKCR2_DMA_PCE_Msk	(0x1U << CMU_PCLKCR2_DMA_PCE_Pos)

#define	CMU_PCLKCR2_LCD_PCE_Pos	3	/* LCDʱʹܣʹ */
#define	CMU_PCLKCR2_LCD_PCE_Msk	(0x1U << CMU_PCLKCR2_LCD_PCE_Pos)

#define	CMU_PCLKCR2_AES_PCE_Pos	2	/* AESʱʹܣʹ */
#define	CMU_PCLKCR2_AES_PCE_Msk	(0x1U << CMU_PCLKCR2_AES_PCE_Pos)

#define	CMU_PCLKCR2_TRNG_PCE_Pos	1	/* RNGʱʹܣʹ */
#define	CMU_PCLKCR2_TRNG_PCE_Msk	(0x1U << CMU_PCLKCR2_TRNG_PCE_Pos)

#define	CMU_PCLKCR2_CRC_PCE_Pos	0	/* CRCʱʹܣʹ */
#define	CMU_PCLKCR2_CRC_PCE_Msk	(0x1U << CMU_PCLKCR2_CRC_PCE_Pos)

#define	CMU_PCLKCR3_I2C1_PCE_Pos	25	/* I2C1ʱʹܣЧ */
#define	CMU_PCLKCR3_I2C1_PCE_Msk	(0x1U << CMU_PCLKCR3_I2C1_PCE_Pos)

#define	CMU_PCLKCR3_I2C0_PCE_Pos	24	/* I2C0ʱʹܣЧ */
#define	CMU_PCLKCR3_I2C0_PCE_Msk	(0x1U << CMU_PCLKCR3_I2C0_PCE_Pos)

#define	CMU_PCLKCR3_LPUART1_PCE_Pos	18	/* LPUART1ʱʹܣЧ */
#define	CMU_PCLKCR3_LPUART1_PCE_Msk	(0x1U << CMU_PCLKCR3_LPUART1_PCE_Pos)

#define	CMU_PCLKCR3_U7816_PCE_Pos	16	/* 7816ʱʹܣЧ */
#define	CMU_PCLKCR3_U7816_PCE_Msk	(0x1U << CMU_PCLKCR3_U7816_PCE_Pos)

#define	CMU_PCLKCR3_LPUART0_PCE_Pos	15	/* LPUART0ʱʹܣЧ */
#define	CMU_PCLKCR3_LPUART0_PCE_Msk	(0x1U << CMU_PCLKCR3_LPUART0_PCE_Pos)

#define	CMU_PCLKCR3_UICR_PCE_Pos	14	/* UARTʹܣЧ */
#define	CMU_PCLKCR3_UICR_PCE_Msk	(0x1U << CMU_PCLKCR3_UICR_PCE_Pos)

#define	CMU_PCLKCR3_UART5_PCE_Pos	13	/* UART5ʱʹܣЧ */
#define	CMU_PCLKCR3_UART5_PCE_Msk	(0x1U << CMU_PCLKCR3_UART5_PCE_Pos)

#define	CMU_PCLKCR3_UART4_PCE_Pos	12	/* UART4ʱʹܣЧ */
#define	CMU_PCLKCR3_UART4_PCE_Msk	(0x1U << CMU_PCLKCR3_UART4_PCE_Pos)

#define	CMU_PCLKCR3_UART3_PCE_Pos	11	/* UART3ʱʹܣЧ */
#define	CMU_PCLKCR3_UART3_PCE_Msk	(0x1U << CMU_PCLKCR3_UART3_PCE_Pos)

#define	CMU_PCLKCR3_UART2_PCE_Pos	10	/* UART2ʱʹܣЧ */
#define	CMU_PCLKCR3_UART2_PCE_Msk	(0x1U << CMU_PCLKCR3_UART2_PCE_Pos)

#define	CMU_PCLKCR3_UART1_PCE_Pos	9	/* UART1ʱʹܣЧ */
#define	CMU_PCLKCR3_UART1_PCE_Msk	(0x1U << CMU_PCLKCR3_UART1_PCE_Pos)

#define	CMU_PCLKCR3_UART0_PCE_Pos	8	/* UART0ʱʹܣЧ */
#define	CMU_PCLKCR3_UART0_PCE_Msk	(0x1U << CMU_PCLKCR3_UART0_PCE_Pos)

#define	CMU_PCLKCR3_QSPI_PCE_Pos	7	/* QuadSPIʱʹܣЧ */
#define	CMU_PCLKCR3_QSPI_PCE_Msk	(0x1U << CMU_PCLKCR3_QSPI_PCE_Pos)

#define	CMU_PCLKCR3_SPI4_PCE_Pos	4	/* SPI4ʱʹܣЧ */
#define	CMU_PCLKCR3_SPI4_PCE_Msk	(0x1U << CMU_PCLKCR3_SPI4_PCE_Pos)

#define	CMU_PCLKCR3_SPI3_PCE_Pos	3	/* SPI3ʱʹܣЧ */
#define	CMU_PCLKCR3_SPI3_PCE_Msk	(0x1U << CMU_PCLKCR3_SPI3_PCE_Pos)

#define	CMU_PCLKCR3_SPI2_PCE_Pos	2	/* SPI2ʱʹܣЧ */
#define	CMU_PCLKCR3_SPI2_PCE_Msk	(0x1U << CMU_PCLKCR3_SPI2_PCE_Pos)

#define	CMU_PCLKCR3_SPI1_PCE_Pos	1	/* SPI1ʱʹܣЧ */
#define	CMU_PCLKCR3_SPI1_PCE_Msk	(0x1U << CMU_PCLKCR3_SPI1_PCE_Pos)

#define	CMU_PCLKCR3_SPI0_PCE_Pos	0	/* SPI0ʱʹܣЧ */
#define	CMU_PCLKCR3_SPI0_PCE_Msk	(0x1U << CMU_PCLKCR3_SPI0_PCE_Pos)

#define	CMU_PCLKCR4_ET4_PCE_Pos	6	/* Etimer4ʱʹܣʹ */
#define	CMU_PCLKCR4_ET4_PCE_Msk	(0x1U << CMU_PCLKCR4_ET4_PCE_Pos)

#define	CMU_PCLKCR4_ET3_PCE_Pos	5	/* Etimer3ʱʹܣʹ */
#define	CMU_PCLKCR4_ET3_PCE_Msk	(0x1U << CMU_PCLKCR4_ET3_PCE_Pos)

#define	CMU_PCLKCR4_ET2_PCE_Pos	4	/* Etimer2ʱʹܣʹ */
#define	CMU_PCLKCR4_ET2_PCE_Msk	(0x1U << CMU_PCLKCR4_ET2_PCE_Pos)

#define	CMU_PCLKCR4_ET1_PCE_Pos	3	/* Etimer1ʱʹܣʹ */
#define	CMU_PCLKCR4_ET1_PCE_Msk	(0x1U << CMU_PCLKCR4_ET1_PCE_Pos)

#define	CMU_PCLKCR4_BT2_PCE_Pos	2	/* BaseTimer2ʱʹܣʹ */
#define	CMU_PCLKCR4_BT2_PCE_Msk	(0x1U << CMU_PCLKCR4_BT2_PCE_Pos)

#define	CMU_PCLKCR4_BT1_PCE_Pos	1	/* BaseTimer1ʱʹܣʹ */
#define	CMU_PCLKCR4_BT1_PCE_Msk	(0x1U << CMU_PCLKCR4_BT1_PCE_Pos)

#define	CMU_PCLKCR4_BSTIM_PCE_Pos	0	/* ʱBSTIMʱʹܣʹ */
#define	CMU_PCLKCR4_BSTIM_PCE_Msk	(0x1U << CMU_PCLKCR4_BSTIM_PCE_Pos)

#define	CMU_OPCCR1_EXTICKE_Pos	31	/* EXTIʱʹܣЧ */
#define	CMU_OPCCR1_EXTICKE_Msk	(0x1U << CMU_OPCCR1_EXTICKE_Pos)

#define	CMU_OPCCR1_EXTICKSEL_Pos	30	/* EXTIжϲʱѡ */
#define	CMU_OPCCR1_EXTICKSEL_Msk	(0x1U << CMU_OPCCR1_EXTICKSEL_Pos)
#define	CMU_OPCCR1_EXTICKSEL_LSCLK	(0x1U << CMU_OPCCR1_EXTICKSEL_Pos) /* 1ⲿжʹLSCLK */
#define	CMU_OPCCR1_EXTICKSEL_HCLK	  (0x0U << CMU_OPCCR1_EXTICKSEL_Pos) /* 0ⲿжʹHCLK */

#define	CMU_OPCCR1_LPUART1CKE_Pos	29	/* LPUART1ʱʹܣЧ */
#define	CMU_OPCCR1_LPUART1CKE_Msk	(0x1U << CMU_OPCCR1_LPUART1CKE_Pos)

#define	CMU_OPCCR1_LPUART0CKE_Pos	28	/* LPUART0ʱʹܣЧ */
#define	CMU_OPCCR1_LPUART0CKE_Msk	(0x1U << CMU_OPCCR1_LPUART0CKE_Pos)

#define	CMU_OPCCR1_LPUART1CKS_Pos	26	/* LPUART1ʱѡ
00LSCLK
01RCHFƵRCHFλԶƵ32768Hz
10RFU
11RFU */
#define	CMU_OPCCR1_LPUART1CKS_Msk	(0x3U << CMU_OPCCR1_LPUART1CKS_Pos)
#define	CMU_OPCCR1_LPUART1CKS_LSCLK	(0x0U << CMU_OPCCR1_LPUART1CKS_Pos)	/* LSCLK */
#define	CMU_OPCCR1_LPUART1CKS_RCHF	(0x1U << CMU_OPCCR1_LPUART1CKS_Pos)	/* RCHFƵRCHFλԶƵ32768Hz */

#define	CMU_OPCCR1_LPUART0CKS_Pos	24	/* LPUART0ʱѡ
00LSCLK
01RCHFƵRCHFλԶƵ32768Hz
10RFU
11RFU */
#define	CMU_OPCCR1_LPUART0CKS_Msk	(0x3U << CMU_OPCCR1_LPUART0CKS_Pos)
#define	CMU_OPCCR1_LPUART0CKS_LSCLK	(0x0U << CMU_OPCCR1_LPUART0CKS_Pos)	/* LSCLK */
#define	CMU_OPCCR1_LPUART0CKS_RCHF	(0x1U << CMU_OPCCR1_LPUART0CKS_Pos)	/* RCHFƵRCHFλԶƵ32768Hz */

#define	CMU_OPCCR1_I2C1CKE_Pos	21	/* I2C1ʱʹ */
#define	CMU_OPCCR1_I2C1CKE_Msk	(0x1U << CMU_OPCCR1_I2C1CKE_Pos)

#define	CMU_OPCCR1_I2C0CKE_Pos	20	/* I2C0ʱʹ */
#define	CMU_OPCCR1_I2C0CKE_Msk	(0x1U << CMU_OPCCR1_I2C0CKE_Pos)

#define	CMU_OPCCR1_I2C1CKS_Pos	18	/* I2C1ʱѡ
00APBCLK
01RCHF
10SYSCLK
11APBCLK */
#define	CMU_OPCCR1_I2C1CKS_Msk	(0x3U << CMU_OPCCR1_I2C1CKS_Pos)
#define	CMU_OPCCR1_I2C1CKS_APBCLK	(0x0U << CMU_OPCCR1_I2C1CKS_Pos)	/* APBCLK */
#define	CMU_OPCCR1_I2C1CKS_RCHF	(0x1U << CMU_OPCCR1_I2C1CKS_Pos)	/* RCHF */
#define	CMU_OPCCR1_I2C1CKS_SYSCLK	(0x2U << CMU_OPCCR1_I2C1CKS_Pos)	/* SYSCLK */

#define	CMU_OPCCR1_I2C0CKS_Pos	16	/* I2C0ʱѡ
00APBCLK
01RCHF
10SYSCLK
11APBCLK */
#define	CMU_OPCCR1_I2C0CKS_Msk	(0x3U << CMU_OPCCR1_I2C0CKS_Pos)
#define	CMU_OPCCR1_I2C0CKS_APBCLK	(0x0U << CMU_OPCCR1_I2C0CKS_Pos)	/* APBCLK */
#define	CMU_OPCCR1_I2C0CKS_RCHF	(0x1U << CMU_OPCCR1_I2C0CKS_Pos)	/* RCHF */
#define	CMU_OPCCR1_I2C0CKS_SYSCLK	(0x2U << CMU_OPCCR1_I2C0CKS_Pos)	/* SYSCLK */

#define	CMU_OPCCR1_UART1CKE_Pos	9	/* UART1ʱʹܣЧ */
#define	CMU_OPCCR1_UART1CKE_Msk	(0x1U << CMU_OPCCR1_UART1CKE_Pos)

#define	CMU_OPCCR1_UART0CKE_Pos	8	/* UART0ʱʹܣЧ */
#define	CMU_OPCCR1_UART0CKE_Msk	(0x1U << CMU_OPCCR1_UART0CKE_Pos)

#define	CMU_OPCCR1_UART1CKS_Pos	2	/* UART1ʱѡ
00APBCLK
01RCHF
10SYSCLK
11RFU */
#define	CMU_OPCCR1_UART1CKS_Msk	(0x3U << CMU_OPCCR1_UART1CKS_Pos)
#define	CMU_OPCCR1_UART1CKS_APBCLK	(0x0U << CMU_OPCCR1_UART1CKS_Pos)	/* APBCLK */
#define	CMU_OPCCR1_UART1CKS_RCHF	(0x1U << CMU_OPCCR1_UART1CKS_Pos)	/* RCHF */
#define	CMU_OPCCR1_UART1CKS_SYSCLK	(0x2U << CMU_OPCCR1_UART1CKS_Pos)	/* SYSCLK */

#define	CMU_OPCCR1_UART0CKS_Pos	0	/* UART0ʱѡ
00APBCLK
01RCHF
10SYSCLK
11RFU */
#define	CMU_OPCCR1_UART0CKS_Msk	(0x3U << CMU_OPCCR1_UART0CKS_Pos)
#define	CMU_OPCCR1_UART0CKS_APBCLK	(0x0U << CMU_OPCCR1_UART0CKS_Pos)	/* APBCLK */
#define	CMU_OPCCR1_UART0CKS_RCHF	(0x1U << CMU_OPCCR1_UART0CKS_Pos)	/* RCHF */
#define	CMU_OPCCR1_UART0CKS_SYSCLK	(0x2U << CMU_OPCCR1_UART0CKS_Pos)	/* SYSCLK */

#define	CMU_OPCCR2_RNGPRSC_Pos	28	/* ʱӷƵ
000Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10016Ƶ
10132Ƶ
110, 111RFU */
#define	CMU_OPCCR2_RNGPRSC_Msk	(0x7U << CMU_OPCCR2_RNGPRSC_Pos)
#define	CMU_OPCCR2_RNGPRSC_DIV1	(0x0U << CMU_OPCCR2_RNGPRSC_Pos)	/* Ƶ */
#define	CMU_OPCCR2_RNGPRSC_DIV2	(0x1U << CMU_OPCCR2_RNGPRSC_Pos)	/* 2Ƶ */
#define	CMU_OPCCR2_RNGPRSC_DIV4	(0x2U << CMU_OPCCR2_RNGPRSC_Pos)	/* 4Ƶ */
#define	CMU_OPCCR2_RNGPRSC_DIV8	(0x3U << CMU_OPCCR2_RNGPRSC_Pos)	/* 8Ƶ */
#define	CMU_OPCCR2_RNGPRSC_DIV16	(0x4U << CMU_OPCCR2_RNGPRSC_Pos)	/* 16Ƶ */
#define	CMU_OPCCR2_RNGPRSC_DIV32	(0x5U << CMU_OPCCR2_RNGPRSC_Pos)	/* 32Ƶ */

#define	CMU_OPCCR2_NVMCKE_Pos	22	/* FlashдʱʹܣЧ */
#define	CMU_OPCCR2_NVMCKE_Msk	(0x1U << CMU_OPCCR2_NVMCKE_Pos)

#define	CMU_OPCCR2_RNGCKE_Pos	21	/* ʱʹܣЧ */
#define	CMU_OPCCR2_RNGCKE_Msk	(0x1U << CMU_OPCCR2_RNGCKE_Pos)

#define	CMU_OPCCR2_LPTCKE_Pos	12	/*  _ʱʹܣЧ */
#define	CMU_OPCCR2_LPTCKE_Msk	(0x1U << CMU_OPCCR2_LPTCKE_Pos)

#define	CMU_OPCCR2_LPTCKS_Pos	8	/*  _ʱѡ
00APBCLK
01LSCLK
10RCLP
11PLL_L */
#define	CMU_OPCCR2_LPTCKS_Msk	(0x3U << CMU_OPCCR2_LPTCKS_Pos)
#define	CMU_OPCCR2_LPTCKS_APBCLK	(0x0U << CMU_OPCCR2_LPTCKS_Pos)	/* APBCLK */
#define	CMU_OPCCR2_LPTCKS_LSCLK	(0x1U << CMU_OPCCR2_LPTCKS_Pos)	/* LSCLK */
#define	CMU_OPCCR2_LPTCKS_RCLP	(0x2U << CMU_OPCCR2_LPTCKS_Pos)	/* RCLP */
#define	CMU_OPCCR2_LPTCKS_PLL_L	(0x3U << CMU_OPCCR2_LPTCKS_Pos)	/* PLL_L */

#define	CMU_OPCCR2_BSTCKE_Pos	4	/* BSTIMʱʹܣЧ */
#define	CMU_OPCCR2_BSTCKE_Msk	(0x1U << CMU_OPCCR2_BSTCKE_Pos)

#define	CMU_OPCCR2_BSTCKS_Pos	0	/* BSTIMʱԴѡ
00APBCLK
01LSCLK
10RCLP
11SYSCLK */
#define	CMU_OPCCR2_BSTCKS_Msk	(0x3U << CMU_OPCCR2_BSTCKS_Pos)
#define	CMU_OPCCR2_BSTCKS_APBCLK	(0x0U << CMU_OPCCR2_BSTCKS_Pos)	/* APBCLK */
#define	CMU_OPCCR2_BSTCKS_LSCLK	(0x1U << CMU_OPCCR2_BSTCKS_Pos)	/* LSCLK */
#define	CMU_OPCCR2_BSTCKS_RCLP	(0x2U << CMU_OPCCR2_BSTCKS_Pos)	/* RCLP */
#define	CMU_OPCCR2_BSTCKS_SYSCLK	(0x3U << CMU_OPCCR2_BSTCKS_Pos)	/* SYSCLK */
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void CMU_Deinit(void);
extern void CMU_SYSCLKCR_SLP_ENEXTI_Setable(FunState NewState);
extern FunState CMU_SYSCLKCR_SLP_ENEXTI_Getable(void);

/* APBʱӷƵѡ (APBbus clock Prescaler)
000/001/010/011Ƶ
1002Ƶ
1014Ƶ
1108Ƶ
11116Ƶ غ */
extern void CMU_SYSCLKCR_APBPRES_Set(uint32_t SetValue);
extern uint32_t CMU_SYSCLKCR_APBPRES_Get(void);

/* AHBʱӷƵѡ (AHB bus clock Prescaler)
000/001/010/011Ƶ
1002Ƶ
1014Ƶ
1108Ƶ
11116Ƶ غ */
extern void CMU_SYSCLKCR_AHBPRES_Set(uint32_t SetValue);
extern uint32_t CMU_SYSCLKCR_AHBPRES_Get(void);

/* CPUںsystickʱѡ (Systick clock select)
00SCLK
01LSCLK
10RC4M
11SYSCLK غ */
extern void CMU_SYSCLKCR_STCLKSEL_Set(uint32_t SetValue);
extern uint32_t CMU_SYSCLKCR_STCLKSEL_Get(void);

/* ϵͳʱԴѡ غ */
extern void CMU_SYSCLKCR_SYSCLKSEL_Set(uint32_t SetValue);
extern uint32_t CMU_SYSCLKCR_SYSCLKSEL_Get(void);

/* RCHFƵѡĴ00008MHz
000116MHz
001024MHz
001132MHz
011140MHz
111148MHz
RFU غ */
extern void CMU_RCHFCR_FSEL_Set(uint32_t SetValue);
extern uint32_t CMU_RCHFCR_FSEL_Get(void);

/* RCHFʹܼĴ (RCHF Enable)
1ʹRCHF
0رRCHF غ */
extern void CMU_RCHFCR_RCHFEN_Setable(FunState NewState);
extern FunState CMU_RCHFCR_RCHFEN_Getable(void);

/* RCHFƵʵУĴ8h00ʾƵͣ8hFFʾƵߣУΧΪƵ+/-30%УΪƵ0.25%
ϵоƬԶLDT0ȡ8MHzУֵд˼Ĵ
ʹƵʱдLDT0ַָȡУϢд˼ĴӶȷƵ׼ȷ غ */
extern void CMU_RCHFTR_RCHFTRIM_Set(uint32_t SetValue);
extern uint32_t CMU_RCHFTR_RCHFTRIM_Get(void);
extern FlagStatus CMU_PLLLCR_LOCKED_Chk(void);
extern void CMU_PLLLCR_PLLDB_Set(uint32_t SetValue);
extern uint32_t CMU_PLLLCR_PLLDB_Get(void);

/* PLLʹܼĴ
1ʹPLL
0رPLL غ */
extern void CMU_PLLLCR_PLLEN_Setable(FunState NewState);
extern FunState CMU_PLLLCR_PLLEN_Getable(void);

/* PLLHƵȣοʱ1Mhz
000001111132Ƶ
010111148Ƶ غ */
extern void CMU_PLLHCR_PLLHDB_Set(uint32_t SetValue);
extern uint32_t CMU_PLLHCR_PLLHDB_Get(void);

/* PLLH־ͨѯ˼ĴȷPLLѾ״̬
1PLL
0PLLδ غ */
extern FlagStatus CMU_PLLHCR_LOCKED_Chk(void);

/* PLLHοʱԤƵĿǲ1MHzοʱӸPLL
000Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10012Ƶ
10116Ƶ
11024Ƶ
11132Ƶ غ */
extern void CMU_PLLHCR_REFPRSC_Set(uint32_t SetValue);
extern uint32_t CMU_PLLHCR_REFPRSC_Get(void);

/* PLLHѡĴ
0ѡPLLHһΪֵ·ڵPLLʱ
1ѡPLLHΪֵ·ڵPLLʱ غ */
extern void CMU_PLLHCR_OSEL_Set(uint32_t SetValue);
extern uint32_t CMU_PLLHCR_OSEL_Get(void);

/* PLLHѡĴ
00/11RCHF
01PLL_L
10XTHF غ */
extern void CMU_PLLHCR_INSEL_Set(uint32_t SetValue);
extern uint32_t CMU_PLLHCR_INSEL_Get(void);

/* PLLHʹܼĴ
1ʹPLLH
0رPLLH غ */
extern void CMU_PLLHCR_EN_Setable(FunState NewState);
extern FunState CMU_PLLHCR_EN_Getable(void);

/* XTHFǿ
000
111ǿ غ */
extern void CMU_XTHFCR_XTHF_CFG_Set(uint32_t SetValue);
extern uint32_t CMU_XTHFCR_XTHF_CFG_Get(void);

/* XTHFʹܼĴ
0رXTHF
1ʹXTHF غ */
extern void CMU_XTHFCR_XTHFEN_Setable(FunState NewState);
extern FunState CMU_XTHFCR_XTHFEN_Getable(void);
extern void CMU_IER_SYSCSE_IE_Setable(FunState NewState);
extern FunState CMU_IER_SYSCSE_IE_Getable(void);
extern void CMU_IER_HFDET_IE_Setable(FunState NewState);
extern FunState CMU_IER_HFDET_IE_Getable(void);


/* Ƶͣģ
1XTHFδͣ
0XTHFͣ غ */
extern FlagStatus CMU_ISR_HFDETO_Chk(void);

/* Ƶͣģ
1XTLFδͣ
0XTLFͣ غ */
extern FlagStatus CMU_ISR_LFDETO_Chk(void);

/* SYSCLKʱѡжϱ־ĴѡеʱԴûʹʱжϱ־λд1㡣 غ */
extern void CMU_ISR_SYSCSE_IF_Clr(void);
extern FlagStatus CMU_ISR_SYSCSE_IF_Chk(void);

/* Ƶͣжϱ־ĴXTHFͣʱӲ첽λд1㣻ֻFFDETOΪ0²ܹ˼Ĵ غ */
extern void CMU_ISR_HFDET_IF_Clr(void);
extern FlagStatus CMU_ISR_HFDET_IF_Chk(void);

/* EXTIʱʹܣЧ غ */
extern void CMU_OPCCR1_EXTICKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_EXTICKE_Getable(void);

/* EXTIжϲʱѡ */
extern void CMU_OPCCR1_EXTICKSEL_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_EXTICKSEL_Get(void);

/* LPUART1ʱʹܣЧ غ */
extern void CMU_OPCCR1_LPUART1CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_LPUART1CKE_Getable(void);

/* LPUART0ʱʹܣЧ غ */
extern void CMU_OPCCR1_LPUART0CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_LPUART0CKE_Getable(void);

/* LPUART1ʱѡ
00LSCLK
01RCHFƵRCHFλԶƵ32768Hz
10RFU
11RFU غ */
extern void CMU_OPCCR1_LPUART1CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_LPUART1CKS_Get(void);

/* LPUART0ʱѡ
00LSCLK
01RCHFƵRCHFλԶƵ32768Hz
10RFU
11RFU غ */
extern void CMU_OPCCR1_LPUART0CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_LPUART0CKS_Get(void);

/* I2C1ʱʹ غ */
extern void CMU_OPCCR1_I2C1CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_I2C1CKE_Getable(void);

/* I2C0ʱʹ غ */
extern void CMU_OPCCR1_I2C0CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_I2C0CKE_Getable(void);

/* I2C1ʱѡ
00APBCLK
01RCHF
10SYSCLK
11APBCLK غ */
extern void CMU_OPCCR1_I2C1CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_I2C1CKS_Get(void);


/* I2C0ʱѡ
00APBCLK
01RCHF
10SYSCLK
11APBCLK غ */
extern void CMU_OPCCR1_I2C0CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_I2C0CKS_Get(void);

/* UART1ʱʹܣЧ غ */
extern void CMU_OPCCR1_UART1CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_UART1CKE_Getable(void);

/* UART0ʱʹܣЧ غ */
extern void CMU_OPCCR1_UART0CKE_Setable(FunState NewState);
extern FunState CMU_OPCCR1_UART0CKE_Getable(void);

/* UART1ʱѡ
00APBCLK
01RCHF
10SYSCLK
11RFU غ */
extern void CMU_OPCCR1_UART1CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_UART1CKS_Get(void);

/* UART0ʱѡ
00APBCLK
01RCHF
10SYSCLK
11RFU غ */
extern void CMU_OPCCR1_UART0CKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR1_UART0CKS_Get(void);

/* ʱӷƵ
000Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10016Ƶ
10132Ƶ
110, 111RFU غ */
extern void CMU_OPCCR2_RNGPRSC_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR2_RNGPRSC_Get(void);

/* FlashдʱʹܣЧ غ */
extern void CMU_OPCCR2_NVMCKE_Setable(FunState NewState);
extern FunState CMU_OPCCR2_NVMCKE_Getable(void);

/* ʱʹܣЧ غ */
extern void CMU_OPCCR2_RNGCKE_Setable(FunState NewState);
extern FunState CMU_OPCCR2_RNGCKE_Getable(void);

/*  _ʱʹܣЧ غ */
extern void CMU_OPCCR2_LPTCKE_Setable(FunState NewState);
extern FunState CMU_OPCCR2_LPTCKE_Getable(void);

/*  _ʱѡ
00APBCLK
01LSCLK
10RCLP
11PLL_L غ */
extern void CMU_OPCCR2_LPTCKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR2_LPTCKS_Get(void);

/* BSTIMʱʹܣЧ غ */
extern void CMU_OPCCR2_BSTCKE_Setable(FunState NewState);
extern FunState CMU_OPCCR2_BSTCKE_Getable(void);

/* BSTIMʱԴѡ
00APBCLK
01LSCLK
10RCLP
11SYSCLK غ */
extern void CMU_OPCCR2_BSTCKS_Set(uint32_t SetValue);
extern uint32_t CMU_OPCCR2_BSTCKS_Get(void);
//Announce_End

/* ȡоƬõĲͬʱƵʺ */
extern void CMU_GetClocksFreq(CMU_ClocksType* para);

/* RCHFʼ */
extern void CMU_RCHF_Init(CMU_RCHF_InitTypeDef* para);

/* PLL Hʼ */
void CMU_PLL_H_Init(CMU_PLL_H_InitTypeDef* para);

/* rchfУ׼ֵ뺯 */
extern void CMU_Init_RCHF_Trim( uint8_t ClkMode );

/* ϵͳʱӳʼ */
extern void CMU_SysClk_Init(CMU_SYSCLK_InitTypeDef* para);

/* ʱӿƲ */
#define	COMPCLK		  (0x01000000 + CMU_PCLKCR1_COMP_PCE_Pos)	//9	/* COMPʱʹܣʹ */
#define	SVDCLK		  (0x01000000 + CMU_PCLKCR1_SVD_PCE_Pos)	//8	/* SVDʱʹܣʹ */
#define	PADCLK			(0x01000000 + CMU_PCLKCR1_PAD_PCE_Pos)	//7	/* PADCFGʱʹܣʹ */
#define	ANACCLK			(0x01000000 + CMU_PCLKCR1_ANAC_PCE_Pos)	//6	/* ANACʱʹܣʹܴ˼ĴڿSVDȽʱ */
#define	IWDTCLK			(0x01000000 + CMU_PCLKCR1_IWDT_PCE_Pos)	//5	/* IWDTʱʹܣʹ */
#define	SCUCLK			(0x01000000 + CMU_PCLKCR1_SCU_PCE_Pos)	//4	/* SCUʱʹܣʹ */
#define	PMUCLK			(0x01000000 + CMU_PCLKCR1_PMU_PCE_Pos)	//3	/* PMUʱʹܣʹ */
#define	RTCCLK			(0x01000000 + CMU_PCLKCR1_RTC_PCE_Pos)	//2	/* RTCʱʹܣʹ */
#define	LPTCLK			(0x01000000 + CMU_PCLKCR1_LPT_PCE_Pos)	//0	/* LPTIMʱʹܣʹ */
        

#define	PAECLK			(0x02000000 + CMU_PCLKCR2_PAE_PCE_Pos)	//17	/* PAEʱʹܣʹ */
#define	SHACLK			(0x02000000 + CMU_PCLKCR2_SHA_PCE_Pos)	//16	/* SHAʱʹܣʹ */
#define	CICCLK			(0x02000000 + CMU_PCLKCR2_CIC_PCE_Pos)	//8	  /* CICʱʹܣʹ */
#define	WWDTCLK			(0x02000000 + CMU_PCLKCR2_WWDT_PCE_Pos)	//7	/* WWDTʱʹܣʹ */
#define	RAMBISTCLK	(0x02000000 + CMU_PCLKCR2_RAMBIST_PCE_Pos)	//6	/* RAMBISTʱʹܣʹ */
#define	FLASHCLK		(0x02000000 + CMU_PCLKCR2_NVM_PCE_Pos)	//5	/* FLSCFlashдʱʹܣʹ */
#define	DMACLK			(0x02000000 + CMU_PCLKCR2_DMA_PCE_Pos)	//4	/* DMAʱʹܣʹ */
#define	LCDCLK			(0x02000000 + CMU_PCLKCR2_LCD_PCE_Pos)	//3	/* LCDʱʹܣʹ */
#define	AESCLK			(0x02000000 + CMU_PCLKCR2_AES_PCE_Pos)	//2	/* AESʱʹܣʹ */
#define	TRNGCLK			(0x02000000 + CMU_PCLKCR2_TRNG_PCE_Pos)	//1	/* RNGʱʹܣʹ */
#define	CRCCLK			(0x02000000 + CMU_PCLKCR2_CRC_PCE_Pos)	//0	/* CRCʱʹܣʹ */

#define	I2C1CLK			(0x03000000 + CMU_PCLKCR3_I2C1_PCE_Pos)	//25	/* I2C1ʱʹ */
#define	I2C0CLK			(0x03000000 + CMU_PCLKCR3_I2C0_PCE_Pos)	//24	/* I2C0ʱʹ */
#define	LPUART1CLK	(0x03000000 + CMU_PCLKCR3_LPUART1_PCE_Pos)	//18	/* LPUART1ʱʹ */
#define	U7816CLK		(0x03000000 + CMU_PCLKCR3_U7816_PCE_Pos)	//16	/* U7816-0ʱʹ */
#define	LPUART0CLK	(0x03000000 + CMU_PCLKCR3_LPUART0_PCE_Pos)	//15	/* LPUART0Ĵʱʹ */
#define	UARTIRCLK	   (0x03000000 + CMU_PCLKCR3_UICR_PCE_Pos)	//14	/* UARTʹܣʹ*/
#define	UART5CLK		(0x03000000 + CMU_PCLKCR3_UART5_PCE_Pos)	//13	/* UART5ʱʹ */
#define	UART4CLK		(0x03000000 + CMU_PCLKCR3_UART4_PCE_Pos)	//12	/* UART4ʱʹ */
#define	UART3CLK		(0x03000000 + CMU_PCLKCR3_UART3_PCE_Pos)	//11	/* UART3ʱʹ */
#define	UART2CLK		(0x03000000 + CMU_PCLKCR3_UART2_PCE_Pos)	//10	/* UART2ʱʹ */
#define	UART1CLK		(0x03000000 + CMU_PCLKCR3_UART1_PCE_Pos)	//9	/* UART1ʱʹ */
#define	UART0CLK		(0x03000000 + CMU_PCLKCR3_UART0_PCE_Pos)	//8	/* UART0ʱʹ */
#define	QSPICLK			(0x03000000 + CMU_PCLKCR3_QSPI_PCE_Pos)	//7	/* QSPIʱʹ */
#define	SPI4CLK			(0x03000000 + CMU_PCLKCR3_SPI4_PCE_Pos)	//4	/* SPI4ʱʹ */
#define	SPI3CLK			(0x03000000 + CMU_PCLKCR3_SPI3_PCE_Pos)	//3	/* SPI3ʱʹ */
#define	SPI2CLK			(0x03000000 + CMU_PCLKCR3_SPI2_PCE_Pos)	//2	/* SPI2ʱʹ */
#define	SPI1CLK			(0x03000000 + CMU_PCLKCR3_SPI1_PCE_Pos)	//1	/* SPI1ʱʹ */
#define	SPI0CLK			(0x03000000 + CMU_PCLKCR3_SPI0_PCE_Pos)	//0	/* SPI0ʱʹ */


#define	ET4CLK			(0x04000000 + CMU_PCLKCR4_ET4_PCE_Pos)	//6	/* Etimer4ʱʹܣʹ */
#define	ET3CLK			(0x04000000 + CMU_PCLKCR4_ET3_PCE_Pos)	//5	/* Etimer3ʱʹܣʹ */
#define	ET2CLK			(0x04000000 + CMU_PCLKCR4_ET2_PCE_Pos)	//4	/* Etimer2ʱʹܣʹ */
#define	ET1CLK			(0x04000000 + CMU_PCLKCR4_ET1_PCE_Pos)	//3	/* Etimer1ʱʹܣʹ */
#define	BT2CLK			(0x04000000 + CMU_PCLKCR4_BT2_PCE_Pos)	//2	/* BaseTimer2ʱʹܣʹ */
#define	BT1CLK			(0x04000000 + CMU_PCLKCR4_BT1_PCE_Pos)	//1	/* BaseTimer1ʱʹܣʹ */
#define	BSTIMCLK	  (0x04000000 + CMU_PCLKCR4_BSTIM_PCE_Pos)	//0	/* ʱBSTIMʱʹܣʹ */

/* ʱʹܺ */
extern void CMU_PERCLK_SetableEx(uint32_t periph_def, FunState NewState);
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_CMU_H */
