/**
  ******************************************************************************
  * @file    fm33a0xxev_crc.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the CRC firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_CRC_H
#define __FM33A0XXEV_CRC_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
	 
typedef struct
{       	
	uint32_t			CRCSEL;		/*!<CRCУʽλѡ*/
	uint32_t 			OPWD;			/*!<WORDʹ*/
	uint32_t      PARA;			/*!<CRCټʹ*/
	uint32_t			RFLTIN;		/*!<CRC뷴ת*/
	uint32_t			RFLTO;		/*!<CRCת*/
	FunState			XOR;			/*!<ʹ*/
	uint32_t			CRC_XOR;	/*!<Ĵ*/
	uint32_t 			CRCPOLY;	/*!<CRCʽĴ*/
	uint32_t			LFSR;			/*!<CRCʼֵ*/
}CRC_InitTypeDef;

#define	CRC_DR_DR_Pos	0	/* CRCDRΪĴ󱣴CRC(CRC Data Register)
ΪʱwordʹܣCRCDR[31:0]м㣬4byte㣨ɵ͵ߣCRCDR[7:0]м㣬1byte㡣
ʱΪ7λʽCRCDR[6:0]Ϊ8λʽCRCDR[7:0]Ϊ16λʽCRCDR[15:0]Ϊ32λʽCRCDR[31:0] */
#define	CRC_DR_DR_Msk	(0xffffffffU << CRC_DR_DR_Pos)

#define	CRC_CR_OPWD_Pos	9	/* WORDʹ (Operation by Word)
0ֽڲCRCCRCDRֽڽ
1ֲCRCCRCDRȫ4ֽڽ */
#define	CRC_CR_OPWD_Msk	(0x1U << CRC_CR_OPWD_Pos)
#define	CRC_CR_OPWD_BYTE	(0x0U << CRC_CR_OPWD_Pos)	/* ֽڲCRCCRCDRֽڽ */
#define	CRC_CR_OPWD_WORD	(0x1U << CRC_CR_OPWD_Pos)	/* ֲCRCCRCDRȫ4ֽڽ */

#define	CRC_CR_PARA_Pos	8	/* CRCټʹ (Parallel Calculation)
0㣬1ֽҪ8ʱ
1м㣬1ֽҪ1ʱ */
#define	CRC_CR_PARA_Msk	(0x1U << CRC_CR_PARA_Pos)
#define	CRC_CR_PARA_SERIAL	(0x0U << CRC_CR_PARA_Pos)	/* 㣬1ֽҪ8ʱ */
#define	CRC_CR_PARA_PARALLEL	(0x1U << CRC_CR_PARA_Pos)	/* м㣬1ֽҪ1ʱ */

#define	CRC_CR_RFLTIN_Pos	6	/* CRC뷴ת (Reflected Input)
00벻ת
01밴ֽڷת
10밴ַת
11밴ַת
磺Ϊ0x11223344
RFLTIN==01ݱΪ0x8844CC22ٽмRFLTIN==10ݱΪ0x448822CCٽм
RFLTIN==11ݱΪ0x22CC4488ٽм */
#define	CRC_CR_RFLTIN_Msk	(0x3U << CRC_CR_RFLTIN_Pos)
#define	CRC_CR_RFLTIN_NONE	(0x0U << CRC_CR_RFLTIN_Pos)	/* 벻ת */
#define	CRC_CR_RFLTIN_BYTE	(0x1U << CRC_CR_RFLTIN_Pos)	/* 밴ֽڷת */
#define	CRC_CR_RFLTIN_HALFWORD	(0x2U << CRC_CR_RFLTIN_Pos)	/* 밴ַת */
#define	CRC_CR_RFLTIN_WORD	(0x3U << CRC_CR_RFLTIN_Pos)	/* 밴ַת */

#define	CRC_CR_RFLTO_Pos	5	/* CRCת (Reflected Output)
0벻ת
1밴ֽڷת
磺
RFLTO==1ǰCRCΪ0x1234ĽΪ0x2C48 RFLTO==0ֱ0x1234ע⣺˽һΪҪXORǷΪ1Ĵbit2˵ */
#define	CRC_CR_RFLTO_Msk	(0x1U << CRC_CR_RFLTO_Pos)
#define	CRC_CR_RFLTO_NON	(0x0U << CRC_CR_RFLTO_Pos)	/* 벻ת */
#define	CRC_CR_RFLTO_BYTE	(0x1U << CRC_CR_RFLTO_Pos)	/* 밴ֽڷת */

#define	CRC_CR_RES_Pos	4	/* CRC־λֻ (Result Flag)
0CRCΪ0
1CRCȫ0 */
#define	CRC_CR_RES_Msk	(0x1U << CRC_CR_RES_Pos)

#define	CRC_CR_BUSY_Pos	3	/* CRC־λֻ (Busy)
0CRC
1CRC */
#define	CRC_CR_BUSY_Msk	(0x1U << CRC_CR_BUSY_Pos)

#define	CRC_CR_XOR_Pos	2	/* ʹ (Output XORed with CRC_XOR register enable)
0CRC_XORĴ
1CRC_XORĴ */
#define	CRC_CR_XOR_Msk	(0x1U << CRC_CR_XOR_Pos)
	/* CRC_XORĴ */
	/* CRC_XORĴ */

#define	CRC_CR_SEL_Pos	0	/* CRCʽλѡ (Polynomial width Selection)
0032λ
0116λ
108λ
117λ */
#define	CRC_CR_SEL_Msk	(0x3U << CRC_CR_SEL_Pos)
#define	CRC_CR_SEL_CRC32	(0x0U << CRC_CR_SEL_Pos)	/* 32λ */
#define	CRC_CR_SEL_CRC16	(0x1U << CRC_CR_SEL_Pos)	/* 16λ */
#define	CRC_CR_SEL_CRC8	(0x2U << CRC_CR_SEL_Pos)	/* 8λ */
#define	CRC_CR_SEL_CRC7	(0x3U << CRC_CR_SEL_Pos)	/* 7λ */

#define	CRC_LFSR_LFSR_Pos	0	/* CRCԷλĴ (Linear Feedback Shift Register)
㿪ʼǰдCRCʼֵ */
#define	CRC_LFSR_LFSR_Msk	(0xffffffffU << CRC_LFSR_LFSR_Pos)

#define	CRC_XOR_XOR_Pos	0	/* CRCĴ (eXclusive OR)
CRC_CR.XORΪ1ʱCRCǰ˼Ĵݡ */
#define	CRC_XOR_XOR_Msk	(0xffffffffU << CRC_XOR_XOR_Pos)

#define	CRC_POLY_POLY_Pos	0	/* CRCʽϵ (CRC Polynominals) */
#define	CRC_POLY_POLY_Msk	(0xffffffffU << CRC_POLY_POLY_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void CRC_Deinit(void);

/* CRCDRΪĴ󱣴CRC(CRC Data Register)
ΪʱwordʹܣCRCDR[31:0]м㣬4byte㣨ɵ͵ߣCRCDR[7:0]м㣬1byte㡣
ʱΪ7λʽCRCDR[6:0]Ϊ8λʽCRCDR[7:0]Ϊ16λʽCRCDR[15:0]Ϊ32λʽCRCDR[31:0] غ */
extern void CRC_DR_Write(uint32_t SetValue);
extern uint32_t CRC_DR_Read(void);

/* WORDʹ (Operation by Word)
0ֽڲCRCCRCDRֽڽ
1ֲCRCCRCDRȫ4ֽڽ غ */
extern void CRC_CR_OPWD_Set(uint32_t SetValue);
extern uint32_t CRC_CR_OPWD_Get(void);

/* CRCټʹ (Parallel Calculation)
0㣬1ֽҪ8ʱ
1м㣬1ֽҪ1ʱ غ */
extern void CRC_CR_PARA_Set(uint32_t SetValue);
extern uint32_t CRC_CR_PARA_Get(void);

/* CRC뷴ת (Reflected Input)
00벻ת
01밴ֽڷת
10밴ַת
11밴ַת
磺Ϊ0x11223344
RFLTIN==01ݱΪ0x8844CC22ٽмRFLTIN==10ݱΪ0x448822CCٽм
RFLTIN==11ݱΪ0x22CC4488ٽм غ */
extern void CRC_CR_RFLTIN_Set(uint32_t SetValue);
extern uint32_t CRC_CR_RFLTIN_Get(void);

/* CRCת (Reflected Output)
0벻ת
1밴ֽڷת
磺
RFLTO==1ǰCRCΪ0x1234ĽΪ0x2C48 RFLTO==0ֱ0x1234ע⣺˽һΪҪXORǷΪ1Ĵbit2˵ غ */
extern void CRC_CR_RFLTO_Set(uint32_t SetValue);
extern uint32_t CRC_CR_RFLTO_Get(void);

/* CRC־λֻ (Result Flag)
0CRCΪ0
1CRCȫ0 غ */
extern FlagStatus CRC_CR_RES_Chk(void);

/* CRC־λֻ (Busy)
0CRC
1CRC غ */
extern FlagStatus CRC_CR_BUSY_Chk(void);

/* ʹ (Output XORed with CRC_XOR register enable)
0CRC_XORĴ
1CRC_XORĴ غ */
extern void CRC_CR_XOR_Setable(FunState NewState);
extern FunState CRC_CR_XOR_Getable(void);

/* CRCʽλѡ (Polynomial width Selection)
0032λ
0116λ
108λ
117λ غ */
extern void CRC_CR_SEL_Set(uint32_t SetValue);
extern uint32_t CRC_CR_SEL_Get(void);

/* CRCԷλĴ (Linear Feedback Shift Register)
㿪ʼǰдCRCʼֵ غ */
extern void CRC_LFSR_Write(uint32_t SetValue);
extern uint32_t CRC_LFSR_Read(void);

/* CRCĴ (eXclusive OR)
CRC_CR.XORΪ1ʱCRCǰ˼Ĵݡ غ */
extern void CRC_XOR_Write(uint32_t SetValue);
extern uint32_t CRC_XOR_Read(void);

/* CRCʽϵ (CRC Polynominals) غ */
extern void CRC_POLY_Write(uint32_t SetValue);
extern uint32_t CRC_POLY_Read(void);
//Announce_End
extern void CRC_Init(CRC_InitTypeDef* para);
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_CRC_H */
