/**
  ******************************************************************************
  * @file    fm33a0xxev_dma.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the DMA firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_DMA_H
#define __FM33A0XXEV_DMA_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 

typedef enum {
    DMA_CH0 = 0, DMA_CH1 = 1, DMA_CH2 = 2, DMA_CH3 = 3, DMA_CH4 = 4, DMA_CH5 = 5, DMA_CH6 = 6, DMA_CH7 = 7,	
    DMA_CH8 = 8, DMA_CH9 = 9, DMA_CH10 = 10, DMA_CH11 = 11
}DMA_CH_Type;

typedef struct
{
	DMA_CH_Type CHx;			//DMAͨ
	uint32_t CHxTSIZE;          //ͨ䳤
	uint32_t CHxPRI;            //ͨȼ
	uint32_t CHxINC;            //CH0~CH10ַͨ,CH11
	uint32_t CHxSSEL;           //CH0~CH10ͨѡ,CH11䷽ʽѡ
    uint32_t CHxDIR;            //CH0~CH10ͨ䷽
    uint32_t CHxBDW;            //CH0~CH10ͨ
    FunState CHxCICR;           //CH0~CH10ѭģʽ
	FunState CHxFTIE;           //ͨжʹ
	FunState CHxHTIE;           //ͨжʹ
	FunState CHxEN;             //ͨʹ
	uint32_t CHxRAMAD;          //CH0~CH11ͨRAMַָ,עCH11ʹõwordַѴˣ

    uint32_t CH11RI;            // RAMַ(FlashRAMЧ)
    uint32_t CH11FI;            // FLSַ(FlashRAMЧ)
	uint32_t CH11FLSAD;			//ͨFLASHַָͨ11	
}DMA_InitTypeDef;

#define	DMA_GCR_DMA_ADDRERR_EN_Pos	1	/* DMAַжʹ
1ַж
0ַֹж */
#define	DMA_GCR_DMA_ADDRERR_EN_Msk	(0x1U << DMA_GCR_DMA_ADDRERR_EN_Pos)

#define	DMA_GCR_DMAEN_Pos	0	/* DMAȫʹ
1DMAʹ
0DMAر */
#define	DMA_GCR_DMAEN_Msk	(0x1U << DMA_GCR_DMAEN_Pos)

#define	DMA_CHxCR_CHxTSIZE_Pos	16	/* Channelx䳤ȣ1-8192δ */
#define	DMA_CHxCR_CHxTSIZE_Msk	(0x1fffU << DMA_CHxCR_CHxTSIZE_Pos)

#define	DMA_CHxCR_CHxPRI_Pos	12	/* Channelxȼ
00Low
01Medium
10High
11Very High */
#define	DMA_CHxCR_CHxPRI_Msk	(0x3U << DMA_CHxCR_CHxPRI_Pos)
#define	DMA_CHxCR_CHxPRI_LOW	(0x0U << DMA_CHxCR_CHxPRI_Pos)	/* Low */
#define	DMA_CHxCR_CHxPRI_MEDIUM	(0x1U << DMA_CHxCR_CHxPRI_Pos)	/* Medium */
#define	DMA_CHxCR_CHxPRI_HIGH	(0x2U << DMA_CHxCR_CHxPRI_Pos)	/* High */
#define	DMA_CHxCR_CHxPRI_VERY_HIGH	(0x3U << DMA_CHxCR_CHxPRI_Pos)	/* Very High */

#define	DMA_CHxCR_CHxINC_Pos	11	/* RAMַ
1RAMַ
0RAMַݼ */
#define	DMA_CHxCR_CHxINC_Msk	(0x1U << DMA_CHxCR_CHxINC_Pos)
#define	DMA_CHxCR_CHxINC_INCREASE	(0x1U << DMA_CHxCR_CHxINC_Pos)	/* RAMַ */
#define	DMA_CHxCR_CHxINC_DECREASE	(0x0U << DMA_CHxCR_CHxINC_Pos)	/* RAMַݼ */

#define	DMA_CHxCR_CHxSSEL_Pos	8	/* Channelxӳ
ÿͨԽ8ӳμ23.6.1DMAӳ */
#define	DMA_CHxCR_CHxSSEL_Msk	(0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH0SSEL_ADC               (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_SPI3_RX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_LPUART0_RX        (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_LPUART1_RX        (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_UART2_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_UART4_RX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_AES_IN            (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH0SSEL_QSPI              (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH1SSEL_ADC               (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_SPI0_RX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_SPI2_RX           (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_SPI3_TX           (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_UART0_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_UART2_TX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_UART3_RX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH1SSEL_U7816_RX          (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH2SSEL_SPI0_TX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_SPI2_TX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_SPI4_RX           (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_UART0_TX          (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_UART3_TX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_UART5_RX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_U7816_TX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH2SSEL_AES_OUT           (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH3SSEL_SPI1_RX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_SPI4_TX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_LPUART0_RX        (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_UART1_RX          (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_UART4_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_UART5_TX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_I2C0_TX           (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH3SSEL_AES_IN            (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH4SSEL_SPI1_TX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_SPI3_RX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_LPUART0_TX        (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_UART1_TX          (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_UART2_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_UART4_TX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_I2C0_RX           (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH4SSEL_AES_OUT           (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH5SSEL_SPI0_RX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_SPI2_RX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_LPUART1_RX        (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_UART1_RX          (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_UART2_TX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_UART3_RX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_UART5_RX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH5SSEL_I2C1_TX           (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH6SSEL_SPI0_TX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_SPI2_TX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_LPUART1_TX        (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_UART1_TX          (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_UART3_TX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_UART5_TX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_I2C1_RX           (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH6SSEL_CRC               (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH7SSEL_ADC               (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_SPI1_RX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_SPI3_RX           (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_LPUART0_RX        (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_UART0_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_UART2_RX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_UART4_RX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH7SSEL_QSPI              (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH8SSEL_ADC               (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_SPI1_TX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_SPI2_RX           (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_SPI3_TX           (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_LPUART0_TX        (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_UART0_TX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_UART2_TX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH8SSEL_UART4_TX          (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH9SSEL_SPI0_RX           (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_SPI2_TX           (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_SPI4_RX           (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_LPUART1_RX        (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_UART1_RX          (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_UART3_RX          (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_UART5_RX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH9SSEL_I2C0_TX           (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define DMA_CHxCR_CH10SSEL_SPI0_TX          (0x0U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_SPI4_TX          (0x1U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_LPUART1_TX       (0x2U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_UART1_TX         (0x3U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_UART3_TX         (0x4U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_UART5_TX         (0x5U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_I2C0_RX          (0x6U << DMA_CHxCR_CHxSSEL_Pos)
#define DMA_CHxCR_CH10SSEL_CRC              (0x7U << DMA_CHxCR_CHxSSEL_Pos)

#define	DMA_CHxCR_CIRC_UPD_Pos	7	/* ѭģʽԶ´ִɺshadowĴݸƵƼĴ
0ֹԶ
1ʹԶ
ע⣺ͨ0~6ִ֧˹ܣͨ7~10޴˼Ĵ */
#define	DMA_CHxCR_CIRC_UPD_Msk	(0x1U << DMA_CHxCR_CIRC_UPD_Pos)
	/* ֹԶ */
	/* ʹԶ */
	/* ͨ0~6ִ֧˹ܣͨ7~10޴˼Ĵ */

#define	DMA_CHxCR_DIR_Pos	6	/* ͨ䷽
0ȡдRAM
1RAMȡд */
#define	DMA_CHxCR_DIR_Msk	(0x1U << DMA_CHxCR_DIR_Pos)
#define	DMA_CHxCR_DIR_TO_RAM	(0x0U << DMA_CHxCR_DIR_Pos)	/* ȡдRAM */
#define	DMA_CHxCR_DIR_TO_PER	(0x1U << DMA_CHxCR_DIR_Pos)	/* RAMȡд */

#define	DMA_CHxCR_BDW_Pos	4	/* 
00ֽڣ8bit
01֣16bit
10֣32bit
11RFU */
#define	DMA_CHxCR_BDW_Msk	(0x3U << DMA_CHxCR_BDW_Pos)
#define	DMA_CHxCR_BDW_8BITS	(0x0U << DMA_CHxCR_BDW_Pos)	/* ֽڣ8bit */
#define	DMA_CHxCR_BDW_16BITS	(0x1U << DMA_CHxCR_BDW_Pos)	/* ֣16bit */
#define	DMA_CHxCR_BDW_32BITS	(0x2U << DMA_CHxCR_BDW_Pos)	/* ֣32bit */

#define	DMA_CHxCR_CIRC_Pos	3	/* ѭģʽ
0رѭģʽ
1ʹѭģʽ */
#define	DMA_CHxCR_CIRC_Msk	(0x1U << DMA_CHxCR_CIRC_Pos)
	/* رѭģʽ */
	/* ʹѭģʽ */

#define	DMA_CHxCR_CHxFTIE_Pos	2	/* Channelxжʹ
1ʹܴж
0رմж */
#define	DMA_CHxCR_CHxFTIE_Msk	(0x1U << DMA_CHxCR_CHxFTIE_Pos)
	/* ʹܴж */
	/* رմж */

#define	DMA_CHxCR_CHxHTIE_Pos	1	/* Channelx̴жʹ
1ʹܰж
0رհж */
#define	DMA_CHxCR_CHxHTIE_Msk	(0x1U << DMA_CHxCR_CHxHTIE_Pos)
	/* ʹܰж */
	/* رհж */

#define	DMA_CHxCR_ChxEN_Pos	0	/* Channelxʹ
1ͨ0
0رͨ0 */
#define	DMA_CHxCR_ChxEN_Msk	(0x1U << DMA_CHxCR_ChxEN_Pos)
	/* ͨ0 */
	/* رͨ0 */

#define	DMA_CHxMAR_CHxMEMAD_Pos	0	/* Channelx洢ַָDMAǰ˼Ĵд洢Ŀַ
ָָյַʱDMAʽhardfault
ָָFlashʱֹFlashдݡ
ԲѯǰDMAĿ洢ַ

ע⣺ָָֹ0x00080000~0x1FFFFFFFַεַΪflashϢָεַDMAе²ԤƵĽ */
#define	DMA_CHxMAR_CHxMEMAD_Msk	(0xffffffffU << DMA_CHxMAR_CHxMEMAD_Pos)

#define	DMA_CH11CR_CH11TSIZE_Pos	16	/* Channel11䳤ȣ1-8192δ䣬Flash->RAMʱЧRAM->FlashΪ̶64δ */
#define	DMA_CH11CR_CH11TSIZE_Msk	(0x1fffU << DMA_CH11CR_CH11TSIZE_Pos)

#define	DMA_CH11CR_CH11PRI_Pos	12	/* Channel11ȼ
00Low
01Medium
10High
11Very High */
#define	DMA_CH11CR_CH11PRI_Msk	(0x3U << DMA_CH11CR_CH11PRI_Pos)
#define	DMA_CH11CR_CH11PRI_LOW	(0x0U << DMA_CH11CR_CH11PRI_Pos)	/* Low */
#define	DMA_CH11CR_CH11PRI_MEDIUM	(0x1U << DMA_CH11CR_CH11PRI_Pos)	/* Medium */
#define	DMA_CH11CR_CH11PRI_HIGH	(0x2U << DMA_CH11CR_CH11PRI_Pos)	/* High */
#define	DMA_CH11CR_CH11PRI_VERY_HIGH	(0x3U << DMA_CH11CR_CH11PRI_Pos)	/* Very High */

#define	DMA_CH11CR_CH11DIR_Pos	10	/* Channel11䷽
1Flash->RAM
0RAM->Flash */
#define	DMA_CH11CR_CH11DIR_Msk	(0x1U << DMA_CH11CR_CH11DIR_Pos)
#define	DMA_CH11CR_CH11DIR_TO_RAM	(0x1U << DMA_CH11CR_CH11DIR_Pos)	/* Flash->RAM */
#define	DMA_CH11CR_CH11DIR_TO_FLASH	(0x0U << DMA_CH11CR_CH11DIR_Pos)	/* RAM->Flash */

#define	DMA_CH11CR_CH11RI_Pos	9	/* Channel11 RAMַãFlash->RAMЧ
1RAMַ
0RAMַݼ */
#define	DMA_CH11CR_CH11RI_Msk	(0x1U << DMA_CH11CR_CH11RI_Pos)
#define	DMA_CH11CR_CH11RI_INCREASE	(0x1U << DMA_CH11CR_CH11RI_Pos)	/* RAMַ */
#define	DMA_CH11CR_CH11RI_DECREASE	(0x0U << DMA_CH11CR_CH11RI_Pos)	/* RAMַݼ */

#define	DMA_CH11CR_CH11FI_Pos	8	/* Channel11 FlashַãFlash->RAMЧ
1Flashַ
0Flashַݼ */
#define	DMA_CH11CR_CH11FI_Msk	(0x1U << DMA_CH11CR_CH11FI_Pos)
#define	DMA_CH11CR_CH11FI_INCREASE	(0x1U << DMA_CH11CR_CH11FI_Pos)	/* Flashַ */
#define	DMA_CH11CR_CH11FI_DECREASE	(0x0U << DMA_CH11CR_CH11FI_Pos)	/* Flashַݼ */

#define	DMA_CH11CR_CH11FTIE_Pos	2	/* Channel11жʹ
1ʹܴж
0رմж */
#define	DMA_CH11CR_CH11FTIE_Msk	(0x1U << DMA_CH11CR_CH11FTIE_Pos)
	/* ʹܴж */
	/* رմж */

#define	DMA_CH11CR_CH11HTIE_Pos	1	/* Channel11̴жʹ
1ʹܰж
0رհж */
#define	DMA_CH11CR_CH11HTIE_Msk	(0x1U << DMA_CH11CR_CH11HTIE_Pos)
	/* ʹܰж */
	/* رհж */

#define	DMA_CH11CR_CH11EN_Pos	0	/* Channel11ʹ
1ͨ0
0رͨ0 */
#define	DMA_CH11CR_CH11EN_Msk	(0x1U << DMA_CH11CR_CH11EN_Pos)
	/* ͨ0 */
	/* رͨ0 */

#define	DMA_CH11FAR_CH11FLSAD_Pos	0	/* Channel11 FlashַָDMAǰ˼ĴдFlashĿַDMA˼ĴDMAԼ
ԲѯǰDMAĿFlashַ
˼Ĵλbit5-0Flash->RAMЧRAM->FlashĬ϶Flashhalf-sectorʼַ */
#define	DMA_CH11FAR_CH11FLSAD_Msk	(0x1ffffU << DMA_CH11FAR_CH11FLSAD_Pos)

#define	DMA_CH11RAR_CH7RAMAD_Pos	0	/* Channel7 RAMַָDMAǰ˼ĴдRAMĿַwordַDMA˼ĴDMAԼ
ԲѯǰDMAĿRAMַ */
#define	DMA_CH11RAR_CH7RAMAD_Msk	(0x3fffU << DMA_CH11RAR_CH7RAMAD_Pos)

#define	DMA_ISR_DMA_ADDRERR_Pos	28	/* DMAַ־洢ָ볬RAMFlashϷַΧʱλ */
#define	DMA_ISR_DMA_ADDRERR_Msk	(0x1U << DMA_ISR_DMA_ADDRERR_Pos)

#define	DMA_ISR_DMACHFT_Pos	16	/* DMAͨxɱ־Ӳλд1
1Ӧͨ
0Ӧͨδ */
#define	DMA_ISR_DMACHFT_Msk	(0xfffU << DMA_ISR_DMACHFT_Pos)

#define	DMA_ISR_DMACHHT_Pos	0	/* DMAͨx̱־Ӳλд1 */
#define	DMA_ISR_DMACHHT_Msk	(0xfffU << DMA_ISR_DMACHHT_Pos)

#define	DMA_CHxCSR_CHxTSIZE_SDW_Pos	16	/* Channelx䳤shadowĴѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxTSIZEС */
#define	DMA_CHxCSR_CHxTSIZE_SDW_Msk	(0x1fffU << DMA_CHxCSR_CHxTSIZE_SDW_Pos)

#define	DMA_CHxCSR_CHxINC_SDW_Pos	11	/* RAMַshadowĴѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxTSIZEС */
#define	DMA_CHxCSR_CHxINC_SDW_Msk	(0x1U << DMA_CHxCSR_CHxINC_SDW_Pos)

#define	DMA_CHxMASR_CHxMAD_SDW_Pos	0	/* Channelx洢ַָӰӼĴ
ѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxMEMADС */
#define	DMA_CHxMASR_CHxMAD_SDW_Msk	(0xffffffffU << DMA_CHxMASR_CHxMAD_SDW_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void DMA_Deinit(void);

/* DMAַжʹ
1ַж
0ַֹж غ */
extern void DMA_GCR_DMA_ADDRERR_EN_Setable(FunState NewState);
extern FunState DMA_GCR_DMA_ADDRERR_EN_Getable(void);

/* DMAȫʹ
1DMAʹ
0DMAر غ */
extern void DMA_GCR_DMAEN_Setable(FunState NewState);
extern FunState DMA_GCR_DMAEN_Getable(void);

/* Channelx䳤ȣ1-8192δ غ */
extern void DMA_CHxCR_CHxTSIZE_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_CHxTSIZE_Get(DMA_CH_Type CHx);

/* Channelxȼ
00Low
01Medium
10High
11Very High غ */
extern void DMA_CHxCR_CHxPRI_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_CHxPRI_Get(DMA_CH_Type CHx);

/* RAMַ
1RAMַ
0RAMַݼ غ */
extern void DMA_CHxCR_CHxINC_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_CHxINC_Get(DMA_CH_Type CHx);

/* Channelxӳ
ÿͨԽ8ӳμ23.6.1DMAӳ غ */
extern void DMA_CHxCR_CHxSSEL_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_CHxSSEL_Get(DMA_CH_Type CHx);

/* ѭģʽԶ´ִɺshadowĴݸƵƼĴ
0ֹԶ
1ʹԶ
ע⣺ͨ0~6ִ֧˹ܣͨ7~10޴˼Ĵ غ */
extern void DMA_CHxCR_CIRC_UPD_Setable(DMA_CH_Type CHx, FunState NewState);
extern FunState DMA_CHxCR_CIRC_UPD_Getable(DMA_CH_Type CHx);

/* ͨ䷽
0ȡдRAM
1RAMȡд غ */
extern void DMA_CHxCR_DIR_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_DIR_Get(DMA_CH_Type CHx);

/* 
00ֽڣ8bit
01֣16bit
10֣32bit
11RFU غ */
extern void DMA_CHxCR_BDW_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCR_BDW_Get(DMA_CH_Type CHx);

/* ѭģʽ
0رѭģʽ
1ʹѭģʽ غ */
extern void DMA_CHxCR_CIRC_Setable(DMA_CH_Type CHx, FunState NewState);
extern FunState DMA_CHxCR_CIRC_Getable(DMA_CH_Type CHx);

/* Channelxжʹ
1ʹܴж
0رմж غ */
extern void DMA_CHxCR_CHxFTIE_Setable(DMA_CH_Type CHx, FunState NewState);
extern FunState DMA_CHxCR_CHxFTIE_Getable(DMA_CH_Type CHx);

/* Channelx̴жʹ
1ʹܰж
0رհж غ */
extern void DMA_CHxCR_CHxHTIE_Setable(DMA_CH_Type CHx, FunState NewState);
extern FunState DMA_CHxCR_CHxHTIE_Getable(DMA_CH_Type CHx);

/* Channelxʹ
1ͨ0
0رͨ0 غ */
extern void DMA_CHxCR_ChxEN_Setable(DMA_CH_Type CHx, FunState NewState);
extern FunState DMA_CHxCR_ChxEN_Getable(DMA_CH_Type CHx);

/* Channelx洢ַָDMAǰ˼Ĵд洢Ŀַ
ָָյַʱDMAʽhardfault
ָָFlashʱֹFlashдݡ
ԲѯǰDMAĿ洢ַ

ע⣺ָָֹ0x00080000~0x1FFFFFFFַεַΪflashϢָεַDMAе²ԤƵĽ غ */
extern void DMA_CHxMAR_Write(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxMAR_Read(DMA_CH_Type CHx);

/* Channel11䳤ȣ1-8192δ䣬Flash->RAMʱЧRAM->FlashΪ̶64δ غ */
extern void DMA_CH11CR_CH11TSIZE_Set(uint32_t SetValue);
extern uint32_t DMA_CH11CR_CH11TSIZE_Get(void);

/* Channel11ȼ
00Low
01Medium
10High
11Very High غ */
extern void DMA_CH11CR_CH11PRI_Set(uint32_t SetValue);
extern uint32_t DMA_CH11CR_CH11PRI_Get(void);

/* Channel11䷽
1Flash->RAM
0RAM->Flash غ */
extern void DMA_CH11CR_CH11DIR_Set(uint32_t SetValue);
extern uint32_t DMA_CH11CR_CH11DIR_Get(void);

/* Channel11 RAMַãFlash->RAMЧ
1RAMַ
0RAMַݼ غ */
extern void DMA_CH11CR_CH11RI_Set(uint32_t SetValue);
extern uint32_t DMA_CH11CR_CH11RI_Get(void);

/* Channel11 FlashַãFlash->RAMЧ
1Flashַ
0Flashַݼ غ */
extern void DMA_CH11CR_CH11FI_Set(uint32_t SetValue);
extern uint32_t DMA_CH11CR_CH11FI_Get(void);

/* Channel11жʹ
1ʹܴж
0رմж غ */
extern void DMA_CH11CR_CH11FTIE_Setable(FunState NewState);
extern FunState DMA_CH11CR_CH11FTIE_Getable(void);

/* Channel11̴жʹ
1ʹܰж
0رհж غ */
extern void DMA_CH11CR_CH11HTIE_Setable(FunState NewState);
extern FunState DMA_CH11CR_CH11HTIE_Getable(void);

/* Channel11ʹ
1ͨ0
0رͨ0 غ */
extern void DMA_CH11CR_CH11EN_Setable(FunState NewState);
extern FunState DMA_CH11CR_CH11EN_Getable(void);

/* Channel11 FlashַָDMAǰ˼ĴдFlashĿַDMA˼ĴDMAԼ
ԲѯǰDMAĿFlashַ
˼Ĵλbit5-0Flash->RAMЧRAM->FlashĬ϶Flashhalf-sectorʼַ غ */
extern void DMA_CH11FAR_Write(uint32_t SetValue);
extern uint32_t DMA_CH11FAR_Read(void);

/* Channel7 RAMַָDMAǰ˼ĴдRAMĿַwordַDMA˼ĴDMAԼ
ԲѯǰDMAĿRAMַ غ */
extern void DMA_CH11RAR_Write(uint32_t SetValue);
extern uint32_t DMA_CH11RAR_Read(void);

/* DMAַ־洢ָ볬RAMFlashϷַΧʱλ غ */
extern void DMA_ISR_DMA_ADDRERR_Clr(void);
extern FlagStatus DMA_ISR_DMA_ADDRERR_Chk(void);

/* DMAͨxɱ־Ӳλд1
1Ӧͨ
0Ӧͨδ غ */
extern void DMA_ISR_DMACHFT_Clr(DMA_CH_Type CHx);
extern FlagStatus DMA_ISR_DMACHFT_Chk(DMA_CH_Type CHx);

/* DMAͨx̱־Ӳλд1 غ */
extern void DMA_ISR_DMACHHT_Clr(DMA_CH_Type CHx);
extern FlagStatus DMA_ISR_DMACHHT_Chk(DMA_CH_Type CHx);

/* Channelx䳤shadowĴѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxTSIZEС غ */
extern void DMA_CHxCSR_CHxTSIZE_SDW_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCSR_CHxTSIZE_SDW_Get(DMA_CH_Type CHx);

/* RAMַshadowĴѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxTSIZEС غ */
extern void DMA_CHxCSR_CHxINC_SDW_Set(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxCSR_CHxINC_SDW_Get(DMA_CH_Type CHx);

/* Channelx洢ַָӰӼĴ
ѭģʽʹCIRC_UPDĴڱִɺshadowĴֵƵCHxMEMADС غ */
extern void DMA_CHxMASR_Write(DMA_CH_Type CHx, uint32_t SetValue);
extern uint32_t DMA_CHxMASR_Read(DMA_CH_Type CHx);
//Announce_End

void DMA_DeInit(void);
void DMA_Init(DMA_InitTypeDef *para);
   
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_DMA_H */
