/**
  ******************************************************************************
  * @file    fm33a0xxev_flash.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the FLASH firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_FLASH_H
#define __FM33A0XXEV_FLASH_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
   
#define	FLS_RDCR_WAIT_Pos	0	/* Flashȴ (Wait Cycles Config)
00/110 wait cycle
011 wait cycle
102 wait cycles
CPUƵСڵ24MHzʱҪwaitƵ24MС48Mhzʱʹ1 waitƵ48Mhzʱʹ2 wait */
#define	FLS_RDCR_WAIT_Msk	(0x3U << FLS_RDCR_WAIT_Pos)
#define	FLS_RDCR_WAIT_0CYCLE	(0x0U << FLS_RDCR_WAIT_Pos)	/* 0 wait cycle */
#define	FLS_RDCR_WAIT_1CYCLE	(0x1U << FLS_RDCR_WAIT_Pos)	/* 1 wait cycle */
#define	FLS_RDCR_WAIT_2CYCLE	(0x2U << FLS_RDCR_WAIT_Pos)	/* 2 wait cycles */

#define	FLS_PFCR_PRFTEN_Pos	0	/* ָԤȡָʹܣWAIT==00д1Ч (Prefetch Enable)
1ʹPrefetch
0ֹPrefetch */
#define	FLS_PFCR_PRFTEN_Msk	(0x1U << FLS_PFCR_PRFTEN_Pos)
	/* ʹPrefetch */
	/* ֹPrefetch */

#define	FLS_OPTBR_DBGCFGEN_Pos	31	
#define	FLS_OPTBR_DBGCFGEN_Msk	(0x1U << FLS_OPTBR_DBGCFGEN_Pos)

#define	FLS_OPTBR_IF2LOCK_Pos	18	/* Information2־ (IF2 Lock Enable)
0δ
1ɸд */
#define	FLS_OPTBR_IF2LOCK_Msk	(0x1U << FLS_OPTBR_IF2LOCK_Pos)

#define	FLS_OPTBR_IF1LOCK_Pos	17	/* Information1־ (IF1 Lock Enable)
0δ
1ɸд */
#define	FLS_OPTBR_IF1LOCK_Msk	(0x1U << FLS_OPTBR_IF1LOCK_Pos)

#define	FLS_OPTBR_DFLSEN_Pos	10	/* DataFlashʹ (DataFlash Enable)
0data flash
1data flash */
#define	FLS_OPTBR_DFLSEN_Msk	(0x1U << FLS_OPTBR_DFLSEN_Pos)
	/* data flash */
	/* data flash */

#define	FLS_OPTBR_BTSEN_Pos	8	/* BootSwapʹ (BootSwap Enable)
00/01/11ֹBootSwap
10BootSwap */
#define	FLS_OPTBR_BTSEN_Msk	(0x3U << FLS_OPTBR_BTSEN_Pos)
	/* ֹBootSwap */
	/* ֹBootSwap */
	/* ֹBootSwap */
	/* BootSwap */

#define	FLS_OPTBR_ACLOCKEN_Pos	4	/* ӦôȨʹ (AppCode Lock Enable)
00/01/11ACLOCKʹ
10ACLOCKʹ */
#define	FLS_OPTBR_ACLOCKEN_Msk	(0x3U << FLS_OPTBR_ACLOCKEN_Pos)
	/* ACLOCKʹ */
	/* ACLOCKʹ */
	/* ACLOCKʹ */
	/* ACLOCKʹ */

#define	FLS_OPTBR_DBRDPEN_Pos	0	/* Debug Portȡʹ (Debug Read Protection Enable)
00/01/11DBRDPʹ
10DBRDPʹ */
#define	FLS_OPTBR_DBRDPEN_Msk	(0xfU << FLS_OPTBR_DBRDPEN_Pos)
	/* DBRDPʹ */
	/* DBRDPʹ */
	/* DBRDPʹ */
	/* DBRDPʹ */

#define	FLS_OPTBR_LOCK1_Pos	0	/* ACLOCKüĴ32bitֱڿBlock15~Block0ӦôдÿBlockСΪ8KBÿBlockʹ2bitȨ޿ơ */
#define	FLS_OPTBR_LOCK1_Msk	(0xffffffffU << FLS_OPTBR_LOCK1_Pos)

#define	FLS_OPTBR_LOCK2_Pos	0	/* ACLOCKüĴ32bitֱڿBlock31~Block16ӦôдÿBlockСΪ8KBÿBlockʹ2bitȨ޿ */
#define	FLS_OPTBR_LOCK2_Msk	(0xffffffffU << FLS_OPTBR_LOCK2_Pos)

#define	FLS_OPTBR_LOCK3_Pos	0	/* ACLOCKüĴ32bitֱڿBlock31~Block16ӦôдÿBlockСΪ8KBÿBlockʹ2bitȨ޿ */
#define	FLS_OPTBR_LOCK3_Msk	(0xffffffffU << FLS_OPTBR_LOCK3_Pos)

#define	FLS_OPTBR_LOCK4_Pos	0	/* ACLOCKüĴ32bitֱڿBlock31~Block16ӦôдÿBlockСΪ8KBÿBlockʹ2bitȨ޿ */
#define	FLS_OPTBR_LOCK4_Msk	(0xffffffffU << FLS_OPTBR_LOCK4_Pos)

#define	FLS_EPCR_ERTYPE_Pos	8	/* Flash (Erase Type)
00/11Page Erase
01Sector Erase
10Chip Erase (SWD only) */
#define	FLS_EPCR_ERTYPE_Msk	(0x3U << FLS_EPCR_ERTYPE_Pos)
#define	FLS_EPCR_ERTYPE_PAGE	(0x0U << FLS_EPCR_ERTYPE_Pos)	/* Page Erase */
#define	FLS_EPCR_ERTYPE_SECTOR	(0x1U << FLS_EPCR_ERTYPE_Pos)	/* Sector Erase */
#define	FLS_EPCR_ERTYPE_CHIP_ERASE	(0x2U << FLS_EPCR_ERTYPE_Pos)	/* Chip Erase (SWD only) */

#define	FLS_EPCR_PREQ_Pos	1	/* Program Request
λӲɱ̺Զ */
#define	FLS_EPCR_PREQ_Msk	(0x1U << FLS_EPCR_PREQ_Pos)

#define	FLS_EPCR_EREQ_Pos	0	/* Erase Request
λӲɲԶ */
#define	FLS_EPCR_EREQ_Msk	(0x1U << FLS_EPCR_EREQ_Pos)

#define	FLS_KEY_KEY_Pos	0	/* FlashдKeyĴSWDдǰȷ˵ַдϷKEYС (Flash Key) */
#define	FLS_KEY_KEY_Msk	(0xffffffffU << FLS_KEY_KEY_Pos)

#define	FLS_IER_OTPIE_Pos	11	/* OTP̴жʹܣ1Ч (OTP program error Interrupt Enable) */
#define	FLS_IER_OTPIE_Msk	(0x1U << FLS_IER_OTPIE_Pos)

#define	FLS_IER_AUTHIE_Pos	10	/* FlashдȨ޴жʹܣ1Ч (Flash Authentication Error Interrupt Enable) */
#define	FLS_IER_AUTHIE_Msk	(0x1U << FLS_IER_AUTHIE_Pos)

#define	FLS_IER_KEYIE_Pos	9	/* Flash KEYжʹܣ1Ч (Flash Key Error Interrupt Enable) */
#define	FLS_IER_KEYIE_Msk	(0x1U << FLS_IER_KEYIE_Pos)

#define	FLS_IER_CKIE_Pos	8	/* дʱʱӴжʹܣ1Ч (Erase/Program Clock Error Interrupt Enable) */
#define	FLS_IER_CKIE_Msk	(0x1U << FLS_IER_CKIE_Pos)

#define	FLS_IER_PRDIE_Pos	1	/* ɱ־жʹܣ1Ч (Program Done Interrupt Enable) */
#define	FLS_IER_PRDIE_Msk	(0x1U << FLS_IER_PRDIE_Pos)

#define	FLS_IER_ERDIE_Pos	0	/* дɱ־жʹܣ1Ч (Erase Done Interrupt Enable) */
#define	FLS_IER_ERDIE_Msk	(0x1U << FLS_IER_ERDIE_Pos)

#define	FLS_ISR_KEYSTA_Pos	17	/* FlashдKEY״̬ (Flash Key Status)
000Flashд״̬δKEY
001ȫ״̬
010״̬
011̽״̬
100KEY״̬Ҫλܽ
101/110/111RFU */
#define	FLS_ISR_KEYSTA_Msk	(0x7U << FLS_ISR_KEYSTA_Pos)
#define	FLS_ISR_KEYSTA_KEY_NONE_PROTECT	(0x0U << FLS_ISR_KEYSTA_Pos)	/* Flashд״̬δKEY */
#define	FLS_ISR_KEYSTA_CHIP_ERASE	(0x1U << FLS_ISR_KEYSTA_Pos)	/* ȫ״̬ */
#define	FLS_ISR_KEYSTA_SECTOR_ERASE	(0x2U << FLS_ISR_KEYSTA_Pos)	/* ״̬ */
#define	FLS_ISR_KEYSTA_PROGRAM	(0x3U << FLS_ISR_KEYSTA_Pos)	/* ̽״̬ */
#define	FLS_ISR_KEYSTA_KEY_ERR_PROTECT	(0x4U << FLS_ISR_KEYSTA_Pos)	/* KEY״̬Ҫλܽ */

#define	FLS_ISR_BTSF_Pos	16	/* BootSwap־Ĵ (BootSwap)
0ΪFlashַ0000H~1FFFH
1ΪFlashַ2000H~3FFFH */
#define	FLS_ISR_BTSF_Msk	(0x1U << FLS_ISR_BTSF_Pos)
#define	FLS_ISR_BTSF_START_0000H	(0x0U << FLS_ISR_BTSF_Pos)	/* ΪFlashַ0000H~1FFFH */
#define	FLS_ISR_BTSF_START_2000H	(0x1U << FLS_ISR_BTSF_Pos)	/* ΪFlashַ2000H~3FFFH */

#define	FLS_ISR_OTPERR_Pos	11	/* OTP pageȨ޴Ӳλд1 (OTP program Error Flag. Write 1 to clear)
1Զѱ̵OTPֽڽб
0OTP̴ */
#define	FLS_ISR_OTPERR_Msk	(0x1U << FLS_ISR_OTPERR_Pos)

#define	FLS_ISR_AUTHERR_Pos	10	/* FlashдȨ޴󣬶ȡLOCKݻLOCKдʱλд1㡣(Flash Authentication Error Flag, write 1 to clear)
1FlashȨ޴
0FlashûзȨ޴ */
#define	FLS_ISR_AUTHERR_Msk	(0x1U << FLS_ISR_AUTHERR_Pos)

#define	FLS_ISR_KEYERR_Pos	9	/* Flash KEYӲλд1 (Flash Key Error Flag, write 1 to clear) */
#define	FLS_ISR_KEYERR_Msk	(0x1U << FLS_ISR_KEYERR_Pos)

#define	FLS_ISR_CKERR_Pos	8	/* дʱʱӴNVMIFдFlashʱRCHFδʹܣ򴥷CKERRжϣд1㡣(Erase/Program Clock Error Flag, write 1 to clear) */
#define	FLS_ISR_CKERR_Msk	(0x1U << FLS_ISR_CKERR_Pos)

#define	FLS_ISR_PRD_Pos	1	/* Program Doneɱ־Ӳλд1(Program Done Flag,write 1 to clear) */
#define	FLS_ISR_PRD_Msk	(0x1U << FLS_ISR_PRD_Pos)

#define	FLS_ISR_ERD_Pos	0	/* Erase Doneдɱ־Ӳλд1(Erase Done Flag,write 1 to clear) */
#define	FLS_ISR_ERD_Msk	(0x1U << FLS_ISR_ERD_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void FLS_Deinit(void);

/* Flashȴ (Wait Cycles Config)
00/110 wait cycle
011 wait cycle
102 wait cycles
CPUƵСڵ24MHzʱҪwaitƵ24MС48Mhzʱʹ1 waitƵ48Mhzʱʹ2 wait غ */
extern void FLS_RDCR_WAIT_Set(uint32_t SetValue);
extern uint32_t FLS_RDCR_WAIT_Get(void);

/* ָԤȡָʹܣWAIT==00д1Ч (Prefetch Enable)
1ʹPrefetch
0ֹPrefetch غ */
extern void FLS_PFCR_PRFTEN_Setable(FunState NewState);
extern FunState FLS_PFCR_PRFTEN_Getable(void);

/* IWDTģʽǷӦͣ (IWDT Sleep Enable)
1ӦģʽͣIWDT
0ֹӦģʽͣIWDT غ */
extern FunState FLS_OPTBR_IWDTSLP_Getable(void);

extern FunState FLS_OPTBR_DBGCFGEN_Chk(void);

/* Information2־ (IF2 Lock Enable)
0δ
1ɸд غ */
extern FlagStatus FLS_OPTBR_IF2LOCK_Chk(void);

/* Information1־ (IF1 Lock Enable)
0δ
1ɸд غ */
extern FlagStatus FLS_OPTBR_IF1LOCK_Chk(void);

/* DataFlashʹ (DataFlash Enable)
0data flash
1data flash غ */
extern FunState FLS_OPTBR_DFLSEN_Getable(void);

/* BootSwapʹ (BootSwap Enable)
00/01/11ֹBootSwap
10BootSwap غ */
extern FunState FLS_OPTBR_BTSEN_Getable(void);

/* ӦôȨʹ (AppCode Lock Enable)
00/01/11ACLOCKʹ
10ACLOCKʹ غ */
extern FunState FLS_OPTBR_ACLOCKEN_Getable(void);

/* Debug Portȡʹ (Debug Read Protection Enable)
00/01/11DBRDPʹ
10DBRDPʹ غ */
extern uint32_t FLS_OPTBR_DBRDPEN_Getable(void);

/* Flash (Erase Type)
00/11Page Erase
01Sector Erase
10Chip Erase (SWD only) غ */
extern void FLS_EPCR_ERTYPE_Set(uint32_t SetValue);
extern uint32_t FLS_EPCR_ERTYPE_Get(void);

/* Program Request
λӲɱ̺Զ غ */
extern void FLS_EPCR_PREQ_Set(uint32_t SetValue);
extern uint32_t FLS_EPCR_PREQ_Get(void);

/* Erase Request
λӲɲԶ غ */
extern void FLS_EPCR_EREQ_Set(uint32_t SetValue);
extern uint32_t FLS_EPCR_EREQ_Get(void);

/* FlashдKeyĴSWDдǰȷ˵ַдϷKEYС (Flash Key) غ */
extern void FLS_KEY_Write(uint32_t SetValue);

/* OTP̴жʹܣ1Ч (OTP program error Interrupt Enable) غ */
extern void FLS_IER_OTPIE_Setable(FunState NewState);
extern FunState FLS_IER_OTPIE_Getable(void);

/* FlashдȨ޴жʹܣ1Ч (Flash Authentication Error Interrupt Enable) غ */
extern void FLS_IER_AUTHIE_Setable(FunState NewState);
extern FunState FLS_IER_AUTHIE_Getable(void);

/* Flash KEYжʹܣ1Ч (Flash Key Error Interrupt Enable) غ */
extern void FLS_IER_KEYIE_Setable(FunState NewState);
extern FunState FLS_IER_KEYIE_Getable(void);

/* дʱʱӴжʹܣ1Ч (Erase/Program Clock Error Interrupt Enable) غ */
extern void FLS_IER_CKIE_Setable(FunState NewState);
extern FunState FLS_IER_CKIE_Getable(void);

/* ɱ־жʹܣ1Ч (Program Done Interrupt Enable) غ */
extern void FLS_IER_PRDIE_Setable(FunState NewState);
extern FunState FLS_IER_PRDIE_Getable(void);

/* дɱ־жʹܣ1Ч (Erase Done Interrupt Enable) غ */
extern void FLS_IER_ERDIE_Setable(FunState NewState);
extern FunState FLS_IER_ERDIE_Getable(void);

/* FlashдKEY״̬ (Flash Key Status)
000Flashд״̬δKEY
001ȫ״̬
010״̬
011̽״̬
100KEY״̬Ҫλܽ
101/110/111RFU غ */
extern uint32_t FLS_ISR_KEYSTA_Get(void);

/* BootSwap־Ĵ (BootSwap)
0ΪFlashַ0000H~1FFFH
1ΪFlashַ2000H~3FFFH غ */
extern uint32_t FLS_ISR_BTSF_Get(void);

/* OTP pageȨ޴Ӳλд1 (OTP program Error Flag. Write 1 to clear)
1Զѱ̵OTPֽڽб
0OTP̴ غ */
extern void FLS_ISR_OTPERR_Clr(void);
extern FlagStatus FLS_ISR_OTPERR_Chk(void);

/* FlashдȨ޴󣬶ȡLOCKݻLOCKдʱλд1㡣(Flash Authentication Error Flag, write 1 to clear)
1FlashȨ޴
0FlashûзȨ޴ غ */
extern void FLS_ISR_AUTHERR_Clr(void);
extern FlagStatus FLS_ISR_AUTHERR_Chk(void);

/* Flash KEYӲλд1 (Flash Key Error Flag, write 1 to clear) غ */
extern void FLS_ISR_KEYERR_Clr(void);
extern FlagStatus FLS_ISR_KEYERR_Chk(void);

/* дʱʱӴNVMIFдFlashʱRCHFδʹܣ򴥷CKERRжϣд1㡣(Erase/Program Clock Error Flag, write 1 to clear) غ */
extern void FLS_ISR_CKERR_Clr(void);
extern FlagStatus FLS_ISR_CKERR_Chk(void);

/* Program Doneɱ־Ӳλд1(Program Done Flag,write 1 to clear) غ */
extern void FLS_ISR_PRD_Clr(void);
extern FlagStatus FLS_ISR_PRD_Chk(void);

/* Erase Doneдɱ־Ӳλд1(Erase Done Flag,write 1 to clear) غ */
extern void FLS_ISR_ERD_Clr(void);
extern FlagStatus FLS_ISR_ERD_Chk(void);
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_FLASH_H */
