/**
  ******************************************************************************
  * @file    fm33a0xxev_i2c.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the I2C firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_I2C_H
#define __FM33A0XXEV_I2C_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
   
#define I2C_SEND_STARTBIT(I2Cx)			I2Cx_CR_SEN_Setable(I2Cx,ENABLE)
#define I2C_SEND_RESTARTBIT(I2Cx)		I2Cx_CR_RSEN_Setable(I2Cx,ENABLE)
#define I2C_SEND_STOPBIT(I2Cx)			I2Cx_CR_PEN_Setable(I2Cx,ENABLE)
	 
#define I2C_SEND_ACK_0(I2Cx)				I2Cx_SR_ACKMO_Set(I2Cx,I2Cx_SR_ACKMO_RESET)
#define I2C_SEND_ACK_1(I2Cx)				I2Cx_SR_ACKMO_Set(I2Cx,I2Cx_SR_ACKMO_SET)	 
	 
	 
#define	I2Cx_CFGR_AUTOEND_Pos	17	/* DMAԶֹ (Automatic Ending)
1DMAָȴɺԶSTOPʱ
0DMAָȴɺ󣬵ȴӹ */
#define	I2Cx_CFGR_AUTOEND_Msk	(0x1U << I2Cx_CFGR_AUTOEND_Pos)
#define	I2Cx_CFGR_AUTOEND_AUTO	(0x1U << I2Cx_CFGR_AUTOEND_Pos)	/* DMAָȴɺԶSTOPʱ */
#define	I2Cx_CFGR_AUTOEND_MANUAL	(0x0U << I2Cx_CFGR_AUTOEND_Pos)	/* DMAָȴɺ󣬵ȴӹ */

#define	I2Cx_CFGR_MSP_DMAEN_Pos	16	/* DMAʹ (Master DMA Enable)
0رDMA
1ʹDMA */
#define	I2Cx_CFGR_MSP_DMAEN_Msk	(0x1U << I2Cx_CFGR_MSP_DMAEN_Pos)
	/* رDMA */
	/* ʹDMA */

#define	I2Cx_CFGR_TOEN_Pos	1	/* SCLͳʱʹܣTimeOut
1ʹܳʱܣʱMSPTOĴ
0رճʱ */
#define	I2Cx_CFGR_TOEN_Msk	(0x1U << I2Cx_CFGR_TOEN_Pos)
#define	I2Cx_CFGR_TOEN_ENABLE	(0x1U << I2Cx_CFGR_TOEN_Pos)	/* ʹܳʱܣʱMSPTOĴ */
#define	I2Cx_CFGR_TOEN_DISABLE	(0x0U << I2Cx_CFGR_TOEN_Pos)	/* رճʱ */

#define	I2Cx_CFGR_MSPEN_Pos	0	/* I2Cģʹܿλ (Master Enable) 1 = I2Cʹ 0 = I2Cֹ */
#define	I2Cx_CFGR_MSPEN_Msk	(0x1U << I2Cx_CFGR_MSPEN_Pos)

#define	I2Cx_CR_RCEN_Pos	3	/* ؽģʽ£ʹλ (Receive Enable)
1 = ʹ
0 = սֹ */
#define	I2Cx_CR_RCEN_Msk	(0x1U << I2Cx_CR_RCEN_Pos)

#define	I2Cx_CR_PEN_Pos	2	/* STOPʱʹܿλд1STOPʱ򣬷ɺӲԶ (Stop Enable) */
#define	I2Cx_CR_PEN_Msk	(0x1U << I2Cx_CR_PEN_Pos)

#define	I2Cx_CR_RSEN_Pos	1	/* Repeated STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (ReStart Enable) */
#define	I2Cx_CR_RSEN_Msk	(0x1U << I2Cx_CR_RSEN_Pos)

#define	I2Cx_CR_SEN_Pos	0	/* STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (Start Enable) */
#define	I2Cx_CR_SEN_Msk	(0x1U << I2Cx_CR_SEN_Pos)

#define	I2Cx_IER_WCOLE_Pos	6	/* WCOLжʹܼĴ (Write Collide Enable)
1дͻж
0ֹдͻж */
#define	I2Cx_IER_WCOLE_Msk	(0x1U << I2Cx_IER_WCOLE_Pos)
	/* дͻж */
	/* ֹдͻж */

#define	I2Cx_IER_TOE_Pos	5	/* SCLʱжʹܼĴ (Time-Out Enable)
1ʱж
0ֹʱж */
#define	I2Cx_IER_TOE_Msk	(0x1U << I2Cx_IER_TOE_Pos)

#define	I2Cx_IER_SE_Pos	4	/* STARTʱжʹܼĴ (START interrupt Enable)
1STARTʱж
0ֹSTARTʱж */
#define	I2Cx_IER_SE_Msk	(0x1U << I2Cx_IER_SE_Pos)
	/* STARTʱж */
	/* ֹSTARTʱж */

#define	I2Cx_IER_PE_Pos	3	/* STOPʱжʹܼĴ (STOP interrupt Enable)
1STOPʱж
0ֹSTOPʱж */
#define	I2Cx_IER_PE_Msk	(0x1U << I2Cx_IER_PE_Pos)
	/* STOPʱж */
	/* ֹSTOPʱж */

#define	I2Cx_IER_NACKE_Pos	2	/* ģʽNACKжʹܼĴ (Non-ACK interrupt Enable)
1յNACKж
0ֹNACKж */
#define	I2Cx_IER_NACKE_Msk	(0x1U << I2Cx_IER_NACKE_Pos)
	/* յNACKж */
	/* ֹNACKж */

#define	I2Cx_IER_TXIE_Pos	1	/* I2Cжʹ (Transmit done interrupt enable)
1ж
0ֹж */
#define	I2Cx_IER_TXIE_Msk	(0x1U << I2Cx_IER_TXIE_Pos)
	/* ж */
	/* ֹж */

#define	I2Cx_IER_RXIE_Pos	0	/* I2Cжʹ (Receive done interrupt enable)
1ж
0ֹж */
#define	I2Cx_IER_RXIE_Msk	(0x1U << I2Cx_IER_RXIE_Pos)
	/* ж */
	/* ֹж */

#define	I2Cx_ISR_WCOL_Pos	6	/* дͻλMCUֻSTARTʱһ֡д֮дSSPBUFдͻӲλд1
(Write Collide)
1 = дͻ
0 = δͻ */
#define	I2Cx_ISR_WCOL_Msk	(0x1U << I2Cx_ISR_WCOL_Pos)

#define	I2Cx_ISR_TO_Pos	5	/* SCLжϱ־*/
#define	I2Cx_ISR_TO_Msk	(0x1U << I2Cx_ISR_TO_Pos)


#define	I2Cx_ISR_S_Pos	4	/* STARTʱжϱ־Ӳλȡ
(START done) */
#define	I2Cx_ISR_S_Msk	(0x1U << I2Cx_ISR_S_Pos)

#define	I2Cx_ISR_P_Pos	3	/* STOPʱжϱ־Ӳλȡ
(STOP done) */
#define	I2Cx_ISR_P_Msk	(0x1U << I2Cx_ISR_P_Pos)

#define	I2Cx_ISR_ACKSTA_Pos	2	/* طģʽ£ԴӻĻӦźţͺյNACK˱־ԲжϣӲλд1㡣(Acknowledge Status)
1ӻӦNACK
0ӻӦACK */
#define	I2Cx_ISR_ACKSTA_Msk	(0x1U << I2Cx_ISR_ACKSTA_Pos)

#define	I2Cx_ISR_TXIF_Pos	1	/* I2Cжϱ־Ӳλд1 (Transmit done interrupt flag) */
#define	I2Cx_ISR_TXIF_Msk	(0x1U << I2Cx_ISR_TXIF_Pos)

#define	I2Cx_ISR_RXIF_Pos	0	/* I2Cжϱ־Ӳλд1 (Receive done interrupt flag) */
#define	I2Cx_ISR_RXIF_Msk	(0x1U << I2Cx_ISR_RXIF_Pos)

#define	I2Cx_SR_BUSY_Pos	5	/* I2Cͨ״̬λ (I2C is busy)
1ӿڴڶд״̬ڽݴ䣬
0ݴ */
#define	I2Cx_SR_BUSY_Msk	(0x1U << I2Cx_SR_BUSY_Pos)

#define	I2Cx_SR_RW_Pos	4	/* I2C䷽״̬λ (Read or Write Bar)
1Ӵӻȡ
0ӻд */
#define	I2Cx_SR_RW_Msk	(0x1U << I2Cx_SR_RW_Pos)

#define	I2Cx_SR_BF_Pos	2	/* ״̬λ (Buffer full)
գ
1 = ɣSSPBUF
0 = δɣSSPBUF
ͣ
1 = ڷͣSSPBUF
0 = ɣSSPBUF */
#define	I2Cx_SR_BF_Msk	(0x1U << I2Cx_SR_BF_Pos)

#define	I2Cx_SR_ACKMO_Pos	0	/* ؽģʽ£Ӧźŵ״̬ (Acknowledge mode)
1طNACK
0طACK

ע⣺P־Ĵ£λACKMO
 */
#define	I2Cx_SR_ACKMO_Msk	(0x1U << I2Cx_SR_ACKMO_Pos)
#define	I2Cx_SR_ACKMO_RESET	(0x0U << I2Cx_SR_ACKMO_Pos)	/* 0طACK */
#define	I2Cx_SR_ACKMO_SET	(0x1U << I2Cx_SR_ACKMO_Pos)	/* 1طNACK */

#define	I2Cx_BRG_MSPBRGH_Pos	16	/* ͵SCLʱӸߵƽȣI2CʱӼ */
#define	I2Cx_BRG_MSPBRGH_Msk	(0x1ffU << I2Cx_BRG_MSPBRGH_Pos)

#define	I2Cx_BRG_MSPBRGL_Pos	0	/* ͵SCLʱӵ͵ƽȣI2CʱӼ */
#define	I2Cx_BRG_MSPBRGL_Msk	(0x1ffU << I2Cx_BRG_MSPBRGL_Pos)

#define	I2Cx_BUF_WR_Pos	0	/* SSPBUF[7:0]ݵĶдͨSSPBUFĲɡʱSSPBUFִдͬʱҲշλĴ(SSPSR)ʱSSPBUFSSPSR˫ṹΪSSPBUFݡһֽڵݣSSPSRSSPBUFͬʱλI2CIFSSPSRֱӼĴûַ */
#define	I2Cx_BUF_WR_Msk	(0xffU << I2Cx_BUF_WR_Pos)

#define	I2Cx_TIMING_SDAHD_Pos	0	/* SDASCL½صıʱI2CʱӼ
(SDA hold time) */
#define	I2Cx_TIMING_SDAHD_Msk	(0x1ffU << I2Cx_TIMING_SDAHD_Pos)

#define	I2Cx_TO_TIMEOUT_Pos	0	/* ӻSCL͵ƽչʱڣMSPEN=0¸д
TSCL_STRETCHING_TIMEOUT=TIMEOUT[11:0] * TSCL */
#define	I2Cx_TO_TIMEOUT_Msk	(0xfffU << I2Cx_TO_TIMEOUT_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void I2Cx_Deinit(I2C_Type* I2Cx);

/* DMAԶֹ (Automatic Ending)
1DMAָȴɺԶSTOPʱ
0DMAָȴɺ󣬵ȴӹ غ */
extern void I2Cx_CFGR_AUTOEND_Set(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_CFGR_AUTOEND_Get(I2C_Type* I2Cx);

/* DMAʹ (Master DMA Enable)
0رDMA
1ʹDMA غ */
extern void I2Cx_CFGR_MSP_DMAEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CFGR_MSP_DMAEN_Getable(I2C_Type* I2Cx);

/* SCLͳʱʹܣTimeOut
1ʹܳʱܣʱMSPTOĴ
0رճʱ غ */
extern void I2Cx_CFGR_TOEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CFGR_TOEN_Getable(I2C_Type* I2Cx);

/* I2Cģʹܿλ (Master Enable)1 = I2Cʹ0 = I2Cֹ غ */
extern void I2Cx_CFGR_MSPEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CFGR_MSPEN_Getable(I2C_Type* I2Cx);

/* ؽģʽ£ʹλ (Receive Enable)
1 = ʹ
0 = սֹ غ */
extern void I2Cx_CR_RCEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CR_RCEN_Getable(I2C_Type* I2Cx);

/* STOPʱʹܿλд1STOPʱ򣬷ɺӲԶ (Stop Enable) غ */
extern void I2Cx_CR_PEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CR_PEN_Getable(I2C_Type* I2Cx);

/* Repeated STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (ReStart Enable) غ */
extern void I2Cx_CR_RSEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CR_RSEN_Getable(I2C_Type* I2Cx);

/* STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (Start Enable) غ */
extern void I2Cx_CR_SEN_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_CR_SEN_Getable(I2C_Type* I2Cx);

/* WCOLжʹܼĴ (Write Collide Enable)
1дͻж
0ֹдͻж غ */
extern void I2Cx_IER_WCOLE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_WCOLE_Getable(I2C_Type* I2Cx);

/* SCLʱжʹܼĴ (Time-Out Enable)
1ʱж
0ֹʱж غ */
extern void I2Cx_IER_TOE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_TOE_Getable(I2C_Type* I2Cx);

/* STARTʱжʹܼĴ (START interrupt Enable)
1STARTʱж
0ֹSTARTʱж غ */
extern void I2Cx_IER_SE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_SE_Getable(I2C_Type* I2Cx);

/* STOPʱжʹܼĴ (STOP interrupt Enable)
1STOPʱж
0ֹSTOPʱж غ */
extern void I2Cx_IER_PE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_PE_Getable(I2C_Type* I2Cx);

/* ģʽNACKжʹܼĴ (Non-ACK interrupt Enable)
1յNACKж
0ֹNACKж غ */
extern void I2Cx_IER_NACKE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_NACKE_Getable(I2C_Type* I2Cx);

/* I2Cжʹ (Transmit done interrupt enable)
1ж
0ֹж غ */
extern void I2Cx_IER_TXIE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_TXIE_Getable(I2C_Type* I2Cx);

/* I2Cжʹ (Receive done interrupt enable)
1ж
0ֹж غ */
extern void I2Cx_IER_RXIE_Setable(I2C_Type* I2Cx, FunState NewState);
extern FunState I2Cx_IER_RXIE_Getable(I2C_Type* I2Cx);

/* дͻλMCUֻSTARTʱһ֡д֮дSSPBUFдͻӲλд1
(Write Collide)
1 = дͻ
0 = δͻ غ */
extern void I2Cx_ISR_WCOL_Clr(I2C_Type* I2Cx);
extern FlagStatus I2Cx_ISR_WCOL_Chk(I2C_Type* I2Cx);

extern void I2Cx_ISR_OVT_Clr(I2C_Type* I2Cx);
extern FlagStatus I2Cx_ISR_OVT_Chk(I2C_Type* I2Cx);

/* STARTʱжϱ־Ӳλȡ
(START done) غ */
extern FlagStatus I2Cx_ISR_S_Chk(I2C_Type* I2Cx);

/* STOPʱжϱ־Ӳλȡ
(STOP done) غ */
extern FlagStatus I2Cx_ISR_P_Chk(I2C_Type* I2Cx);

/* طģʽ£ԴӻĻӦźţͺյNACK˱־ԲжϣӲλд1㡣(Acknowledge Status)
1ӻӦNACK
0ӻӦACK غ */
extern void I2Cx_ISR_ACKSTA_Clr(I2C_Type* I2Cx);
extern FlagStatus I2Cx_ISR_ACKSTA_Chk(I2C_Type* I2Cx);

/* I2Cжϱ־Ӳλд1 (Transmit done interrupt flag) غ */
extern void I2Cx_ISR_TXIF_Clr(I2C_Type* I2Cx);
extern FlagStatus I2Cx_ISR_TXIF_Chk(I2C_Type* I2Cx);

/* I2Cжϱ־Ӳλд1 (Receive done interrupt flag) غ */
extern void I2Cx_ISR_RXIF_Clr(I2C_Type* I2Cx);
extern FlagStatus I2Cx_ISR_RXIF_Chk(I2C_Type* I2Cx);

/* I2Cͨ״̬λ (I2C is busy)
1ӿڴڶд״̬ڽݴ䣬
0ݴ غ */
extern FlagStatus I2Cx_SR_BUSY_Chk(I2C_Type* I2Cx);

/* I2C䷽״̬λ (Read or Write Bar)
1Ӵӻȡ
0ӻд غ */
extern FlagStatus I2Cx_SR_RW_Chk(I2C_Type* I2Cx);

/* ״̬λ (Buffer full)
գ
1 = ɣSSPBUF
0 = δɣSSPBUF
ͣ
1 = ڷͣSSPBUF
0 = ɣSSPBUF غ */
extern FlagStatus I2Cx_SR_BF_Chk(I2C_Type* I2Cx);

/* ؽģʽ£Ӧźŵ״̬ (Acknowledge mode)
1طNACK
0طACK

ע⣺P־Ĵ£λACKMO
 غ */
extern void I2Cx_SR_ACKMO_Set(I2C_Type* I2Cx,uint32_t SetValue);
extern uint32_t I2Cx_SR_ACKMO_Get(I2C_Type* I2Cx);

/* ͵SCLʱӸߵƽȣI2CʱӼ غ */
extern void I2Cx_BRG_MSPBRGH_Set(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_BRG_MSPBRGH_Get(I2C_Type* I2Cx);

/* ͵SCLʱӵ͵ƽȣI2CʱӼ غ */
extern void I2Cx_BRG_MSPBRGL_Set(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_BRG_MSPBRGL_Get(I2C_Type* I2Cx);

/* SSPBUF[7:0]ݵĶдͨSSPBUFĲɡʱSSPBUFִдͬʱҲշλĴ(SSPSR)ʱSSPBUFSSPSR˫ṹΪSSPBUFݡһֽڵݣSSPSRSSPBUFͬʱλI2CIFSSPSRֱӼĴûַ غ */
extern void I2Cx_BUF_Write(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_BUF_Read(I2C_Type* I2Cx);

/* SDASCL½صıʱI2CʱӼ
(SDA hold time) غ */
extern void I2Cx_TIMING_Write(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_TIMING_Read(I2C_Type* I2Cx);

/* ӻSCL͵ƽչʱڣMSPEN=0¸д
TSCL_STRETCHING_TIMEOUT=TIMEOUT[11:0] * TSCL غ */
extern void I2Cx_TO_Write(I2C_Type* I2Cx, uint32_t SetValue);
extern uint32_t I2Cx_TO_Read(I2C_Type* I2Cx);

extern void I2Cx_Deinit(I2C_Type* I2Cx);
extern uint32_t I2C_BaudREG_Calc(uint32_t I2CClk, uint32_t APBClk);
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /*__FM33A0XXEV_I2C_H */
