/**
  ******************************************************************************
  * @file    fm33a0xxev_lptim.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the LPTIM firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_LPTIM_H
#define __FM33A0XXEV_LPTIM_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h"
   
#define	LPTIM_CFGR_ETR_AFEN_Pos	24	/* LPT_ETRģ˲ʹ(External Trigger input Analog Filter Enable)
0رģ˲
1ʹģ˲˲Լ100ns */
#define	LPTIM_CFGR_ETR_AFEN_Msk	(0x1U << LPTIM_CFGR_ETR_AFEN_Pos)
	/* رģ˲ */
	/* ʹģ˲˲Լ100ns */

#define	LPTIM_CFGR_PSCSEL_Pos	14	/* ʱԤƵѡ(Prescaler input Select)
0CLKSELѡʱ
1LPTETRF */
#define	LPTIM_CFGR_PSCSEL_Msk	(0x1U << LPTIM_CFGR_PSCSEL_Pos)
#define	LPTIM_CFGR_PSCSEL_CLKSEL	(0x0U << LPTIM_CFGR_PSCSEL_Pos)	/* CLKSELѡʱ */
#define	LPTIM_CFGR_PSCSEL_LPTETRF	(0x1U << LPTIM_CFGR_PSCSEL_Pos)	/* LPTETRF */

#define	LPTIM_CFGR_DIVSEL_Pos	10	/* ʱӷƵѡ(Counter Clock Divider Select)
0001Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10016Ƶ
10132Ƶ
11064Ƶ
111128Ƶ */
#define	LPTIM_CFGR_DIVSEL_Msk	(0x7U << LPTIM_CFGR_DIVSEL_Pos)
#define	LPTIM_CFGR_DIVSEL_DIV1	(0x0U << LPTIM_CFGR_DIVSEL_Pos)	/* 1Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV2	(0x1U << LPTIM_CFGR_DIVSEL_Pos)	/* 2Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV4	(0x2U << LPTIM_CFGR_DIVSEL_Pos)	/* 4Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV8	(0x3U << LPTIM_CFGR_DIVSEL_Pos)	/* 8Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV16	(0x4U << LPTIM_CFGR_DIVSEL_Pos)	/* 16Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV32	(0x5U << LPTIM_CFGR_DIVSEL_Pos)	/* 32Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV64	(0x6U << LPTIM_CFGR_DIVSEL_Pos)	/* 64Ƶ */
#define	LPTIM_CFGR_DIVSEL_DIV128	(0x7U << LPTIM_CFGR_DIVSEL_Pos)	/* 128Ƶ */

#define	LPTIM_CFGR_EDGESEL_Pos	7	/* ETRѡ(ETR Clock Edge Select)
0LPT_ETRؼ
1LPT_ETR½ؼ */
#define	LPTIM_CFGR_EDGESEL_Msk	(0x1U << LPTIM_CFGR_EDGESEL_Pos)
#define	LPTIM_CFGR_EDGESEL_RISING	(0x0U << LPTIM_CFGR_EDGESEL_Pos)	/* LPT_ETRؼ */
#define	LPTIM_CFGR_EDGESEL_FALLING	(0x1U << LPTIM_CFGR_EDGESEL_Pos)	/* LPT_ETR½ؼ */

#define	LPTIM_CFGR_TRIGCFG_Pos	5	/* ⲿѡʹڲʱͬLPT_ETR(ETR trigger Configuration)
00: LPT_ETRźش
01: LPT_ETRź½ش
10/11: ⲿź½ش */
#define	LPTIM_CFGR_TRIGCFG_Msk	(0x3U << LPTIM_CFGR_TRIGCFG_Pos)
#define	LPTIM_CFGR_TRIGCFG_RISING	(0x0U << LPTIM_CFGR_TRIGCFG_Pos)
#define	LPTIM_CFGR_TRIGCFG_FALLING	(0x1U << LPTIM_CFGR_TRIGCFG_Pos)
#define	LPTIM_CFGR_TRIGCFG_EXTERNAL	(0x2U << LPTIM_CFGR_TRIGCFG_Pos)

#define	LPTIM_CFGR_ONST_Pos	2	/* μģʽʹ(One State Timer)
0ģʽ󱣳УֱرΪֹﵽĿֵص0¿ʼжϡ
1μģʽĿֵص0Զֹͣжϡ */
#define	LPTIM_CFGR_ONST_Msk	(0x1U << LPTIM_CFGR_ONST_Pos)
#define	LPTIM_CFGR_ONST_CONTINUE	(0x0U << LPTIM_CFGR_ONST_Pos)	/* ģʽ */
#define	LPTIM_CFGR_ONST_SINGLE	(0x1U << LPTIM_CFGR_ONST_Pos)	/* μģʽ */

#define	LPTIM_CFGR_TMODE_Pos	0	/* ģʽѡ(Timer operation Mode)
00ͨʱģʽ
01Trigger崥ģʽ
10ⲿ첽ģʽ
11Timeoutģʽ */
#define	LPTIM_CFGR_TMODE_Msk	(0x3U << LPTIM_CFGR_TMODE_Pos)
#define	LPTIM_CFGR_TMODE_COUNTER	(0x0U << LPTIM_CFGR_TMODE_Pos)
#define	LPTIM_CFGR_TMODE_PULSE	(0x1U << LPTIM_CFGR_TMODE_Pos)
#define	LPTIM_CFGR_TMODE_ASY	(0x2U << LPTIM_CFGR_TMODE_Pos)
#define	LPTIM_CFGR_TMODE_TIMEOUT	(0x3U << LPTIM_CFGR_TMODE_Pos)

#define	LPTIM_CNTR_CNT32_Pos	0	/* 32bitǰֵ(Counter 32bits-wide) */
#define	LPTIM_CNTR_CNT32_Msk	(0xffffffffU << LPTIM_CNTR_CNT32_Pos)

#define	LPTIM_CCSR_CAP1SSEL_Pos	24	/* ͨ1׽Դѡ(Capture channel 1 source select)CH1ͨΪ벶׽ʱЧ
00LPT_CH1
01XTLF
10RCLP
11RCMF */
#define	LPTIM_CCSR_CAP1SSEL_Msk	(0x3U << LPTIM_CCSR_CAP1SSEL_Pos)
#define	LPTIM_CCSR_CAP1SSEL_LPT_CH1	(0x0U << LPTIM_CCSR_CAP1SSEL_Pos)
#define	LPTIM_CCSR_CAP1SSEL_XTLF	(0x1U << LPTIM_CCSR_CAP1SSEL_Pos)
#define	LPTIM_CCSR_CAP1SSEL_RCLP	(0x2U << LPTIM_CCSR_CAP1SSEL_Pos)
#define	LPTIM_CCSR_CAP1SSEL_RCMF	(0x3U << LPTIM_CCSR_CAP1SSEL_Pos)

#define	LPTIM_CCSR_CAP4EDGE_Pos	23	/* ͨ4ǰ׽ıأCC2IFλʱ(Channel 4 Captured Edge)
0
1½ */
#define	LPTIM_CCSR_CAP4EDGE_Msk	(0x1U << LPTIM_CCSR_CAP4EDGE_Pos)
#define	LPTIM_CCSR_CAP4EDGE_RISING	(0x0U << LPTIM_CCSR_CAP4EDGE_Pos)
#define	LPTIM_CCSR_CAP4EDGE_FALLING	(0x1U << LPTIM_CCSR_CAP4EDGE_Pos)

#define	LPTIM_CCSR_CAP3EDGE_Pos	22
#define	LPTIM_CCSR_CAP3EDGE_Msk	(0x1U << LPTIM_CCSR_CAP3EDGE_Pos)
#define	LPTIM_CCSR_CAP3EDGE_RISING	(0x0U << LPTIM_CCSR_CAP3EDGE_Pos)
#define	LPTIM_CCSR_CAP3EDGE_FALLING	(0x1U << LPTIM_CCSR_CAP3EDGE_Pos)

#define	LPTIM_CCSR_CAP2EDGE_Pos	21
#define	LPTIM_CCSR_CAP2EDGE_Msk	(0x1U << LPTIM_CCSR_CAP2EDGE_Pos)
#define	LPTIM_CCSR_CAP2EDGE_RISING	(0x0U << LPTIM_CCSR_CAP2EDGE_Pos)
#define	LPTIM_CCSR_CAP2EDGE_FALLING	(0x1U << LPTIM_CCSR_CAP2EDGE_Pos)

#define	LPTIM_CCSR_CAP1EDGE_Pos	20
#define	LPTIM_CCSR_CAP1EDGE_Msk	(0x1U << LPTIM_CCSR_CAP1EDGE_Pos)
#define	LPTIM_CCSR_CAP1EDGE_RISING	(0x0U << LPTIM_CCSR_CAP1EDGE_Pos)
#define	LPTIM_CCSR_CAP1EDGE_FALLING	(0x1U << LPTIM_CCSR_CAP1EDGE_Pos)

#define	LPTIM_CCSR_POLAR4_Pos	19	/* ͨ4Ƚμѡ (Channel 4 compare output Polarity)
0ԲΣʼΪֵͣ==Ƚֵʱøߣֵ==ARRʱָΪ
1ԲΣԲȡ */
#define	LPTIM_CCSR_POLAR4_Msk	(0x1U << LPTIM_CCSR_POLAR4_Pos)
#define	LPTIM_CCSR_POLAR4_POS	(0x0U << LPTIM_CCSR_POLAR4_Pos)
#define	LPTIM_CCSR_POLAR4_NEG	(0x1U << LPTIM_CCSR_POLAR4_Pos)

#define	LPTIM_CCSR_POLAR3_Pos	18
#define	LPTIM_CCSR_POLAR3_Msk	(0x1U << LPTIM_CCSR_POLAR3_Pos)
#define	LPTIM_CCSR_POLAR3_POS	(0x0U << LPTIM_CCSR_POLAR3_Pos)
#define	LPTIM_CCSR_POLAR3_NEG	(0x1U << LPTIM_CCSR_POLAR3_Pos)

#define	LPTIM_CCSR_POLAR2_Pos	17
#define	LPTIM_CCSR_POLAR2_Msk	(0x1U << LPTIM_CCSR_POLAR2_Pos)
#define	LPTIM_CCSR_POLAR2_POS	(0x0U << LPTIM_CCSR_POLAR2_Pos)
#define	LPTIM_CCSR_POLAR2_NEG	(0x1U << LPTIM_CCSR_POLAR2_Pos)

#define	LPTIM_CCSR_POLAR1_Pos	16
#define	LPTIM_CCSR_POLAR1_Msk	(0x1U << LPTIM_CCSR_POLAR1_Pos)
#define	LPTIM_CCSR_POLAR1_POS	(0x0U << LPTIM_CCSR_POLAR1_Pos)
#define	LPTIM_CCSR_POLAR1_NEG	(0x1U << LPTIM_CCSR_POLAR1_Pos)

#define	LPTIM_CCSR_CAPCFG4_Pos	14	/* ͨ4׽ѡ(Channel 4 Capture edge Config)
00ز׽
01½ز׽
10½ز׽
11RFU */
#define	LPTIM_CCSR_CAPCFG4_Msk	(0x3U << LPTIM_CCSR_CAPCFG4_Pos)
#define	LPTIM_CCSR_CAPCFG4_RISING	(0x0U << LPTIM_CCSR_CAPCFG4_Pos)
#define	LPTIM_CCSR_CAPCFG4_FALLING	(0x1U << LPTIM_CCSR_CAPCFG4_Pos)
#define	LPTIM_CCSR_CAPCFG4_BOTH	(0x2U << LPTIM_CCSR_CAPCFG4_Pos)

#define	LPTIM_CCSR_CAPCFG3_Pos	12
#define	LPTIM_CCSR_CAPCFG3_Msk	(0x3U << LPTIM_CCSR_CAPCFG3_Pos)
#define	LPTIM_CCSR_CAPCFG3_RISING	(0x0U << LPTIM_CCSR_CAPCFG3_Pos)
#define	LPTIM_CCSR_CAPCFG3_FALLING	(0x1U << LPTIM_CCSR_CAPCFG3_Pos)
#define	LPTIM_CCSR_CAPCFG3_BOTH	(0x2U << LPTIM_CCSR_CAPCFG3_Pos)

#define	LPTIM_CCSR_CAPCFG2_Pos	10
#define	LPTIM_CCSR_CAPCFG2_Msk	(0x3U << LPTIM_CCSR_CAPCFG2_Pos)
#define	LPTIM_CCSR_CAPCFG2_RISING	(0x0U << LPTIM_CCSR_CAPCFG2_Pos)
#define	LPTIM_CCSR_CAPCFG2_FALLING	(0x1U << LPTIM_CCSR_CAPCFG2_Pos)
#define	LPTIM_CCSR_CAPCFG2_BOTH	(0x2U << LPTIM_CCSR_CAPCFG2_Pos)

#define	LPTIM_CCSR_CAPCFG1_Pos	8
#define	LPTIM_CCSR_CAPCFG1_Msk	(0x3U << LPTIM_CCSR_CAPCFG1_Pos)
#define	LPTIM_CCSR_CAPCFG1_RISING	(0x0U << LPTIM_CCSR_CAPCFG1_Pos)
#define	LPTIM_CCSR_CAPCFG1_FALLING	(0x1U << LPTIM_CCSR_CAPCFG1_Pos)
#define	LPTIM_CCSR_CAPCFG1_BOTH	(0x2U << LPTIM_CCSR_CAPCFG1_Pos)

#define	LPTIM_CCSR_CC4S_Pos	6	/* ͨ4׽/ȽϹʹ(Channel 4 Capture/Compare Select)
00,11ֹͨ4׽/ȽϹ
01ʹͨ4׽ܣLPT_CH4Ϊ룩
10ʹͨ4ȽϹܣLPT_CH4Ϊ */
#define	LPTIM_CCSR_CC4S_Msk	(0x3U << LPTIM_CCSR_CC4S_Pos)
#define	LPTIM_CCSR_CC4S_NON	(0x0U << LPTIM_CCSR_CC4S_Pos)
#define	LPTIM_CCSR_CC4S_LPT_CH4_IN	(0x1U << LPTIM_CCSR_CC4S_Pos)
#define	LPTIM_CCSR_CC4S_LPT_CH4_OUT	(0x2U << LPTIM_CCSR_CC4S_Pos)

#define	LPTIM_CCSR_CC3S_Pos	4
#define	LPTIM_CCSR_CC3S_Msk	(0x3U << LPTIM_CCSR_CC3S_Pos)
#define	LPTIM_CCSR_CC3S_NON	(0x0U << LPTIM_CCSR_CC3S_Pos)
#define	LPTIM_CCSR_CC3S_LPT_CH3_IN	(0x1U << LPTIM_CCSR_CC3S_Pos)
#define	LPTIM_CCSR_CC3S_LPT_CH3_OUT	(0x2U << LPTIM_CCSR_CC3S_Pos)

#define	LPTIM_CCSR_CC2S_Pos	2
#define	LPTIM_CCSR_CC2S_Msk	(0x3U << LPTIM_CCSR_CC2S_Pos)
#define	LPTIM_CCSR_CC2S_NON	(0x0U << LPTIM_CCSR_CC2S_Pos)
#define	LPTIM_CCSR_CC2S_LPT_CH2_IN	(0x1U << LPTIM_CCSR_CC2S_Pos)
#define	LPTIM_CCSR_CC2S_LPT_CH2_OUT	(0x2U << LPTIM_CCSR_CC2S_Pos)

#define	LPTIM_CCSR_CC1S_Pos	0
#define	LPTIM_CCSR_CC1S_Msk	(0x3U << LPTIM_CCSR_CC1S_Pos)
#define	LPTIM_CCSR_CC1S_NON	(0x0U << LPTIM_CCSR_CC1S_Pos)
#define	LPTIM_CCSR_CC1S_LPT_CH1_IN	(0x1U << LPTIM_CCSR_CC1S_Pos)
#define	LPTIM_CCSR_CC1S_LPT_CH1_OUT	(0x2U << LPTIM_CCSR_CC1S_Pos)

#define	LPTIM_ARR_ARR_Pos	0	/* ԶĿĴ(Auto-Reload Register)
ֵARRʱصֵ¿ʼϼ */
#define	LPTIM_ARR_ARR_Msk	(0xffffffffU << LPTIM_ARR_ARR_Pos)

#define	LPTIM_IER_OVR4IE_Pos	11	/* ͨ4׽жʹ(Channel 4 Over-Capture Interrupt Enable)
1ж
0ֹж */
#define	LPTIM_IER_OVR4IE_Msk	(0x1U << LPTIM_IER_OVR4IE_Pos)
	/* ж */
	/* ֹж */

#define	LPTIM_IER_OVR3IE_Pos	10	/* ͨ3׽жʹ(Channel 3 Over-Capture Interrupt Enable)
1ж
0ֹж */
#define	LPTIM_IER_OVR3IE_Msk	(0x1U << LPTIM_IER_OVR3IE_Pos)
	/* ж */
	/* ֹж */

#define	LPTIM_IER_OVR2IE_Pos	9	/* ͨ2׽жʹ(Channel 2 Over-Capture Interrupt Enable)
1ж
0ֹж */
#define	LPTIM_IER_OVR2IE_Msk	(0x1U << LPTIM_IER_OVR2IE_Pos)
	/* ж */
	/* ֹж */

#define	LPTIM_IER_OVR1IE_Pos	8	/* ͨ1׽жʹ(Channel 1 Over-Capture Interrupt Enable)
1ж
0ֹж */
#define	LPTIM_IER_OVR1IE_Msk	(0x1U << LPTIM_IER_OVR1IE_Pos)
	/* ж */
	/* ֹж */

#define	LPTIM_IER_TRIGIE_Pos	7	/* ⲿжʹλ(External Trigger Interrupt Enable)
1ⲿжʹ
0ⲿжϽֹ */
#define	LPTIM_IER_TRIGIE_Msk	(0x1U << LPTIM_IER_TRIGIE_Pos)
	/* ⲿжʹ */
	/* ⲿжϽֹ */

#define	LPTIM_IER_OVIE_Pos	6	/* жʹλ(Counter Over-Flow Interrupt Enable)
1жʹ
0жϽֹ */
#define	LPTIM_IER_OVIE_Msk	(0x1U << LPTIM_IER_OVIE_Pos)
	/* жʹ */
	/* жϽֹ */

#define	LPTIM_IER_CC4IE_Pos	3	/* ׽/Ƚͨ4жʹλ(Capture/Compare channel 4 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ */
#define	LPTIM_IER_CC4IE_Msk	(0x1U << LPTIM_IER_CC4IE_Pos)
	/* ׽/Ƚͨ2жʹ */
	/* ׽/Ƚͨ2жϽֹ */

#define	LPTIM_IER_CC3IE_Pos	2	/* ׽/Ƚͨ3жʹλ(Capture/Compare channel 3 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ */
#define	LPTIM_IER_CC3IE_Msk	(0x1U << LPTIM_IER_CC3IE_Pos)
	/* ׽/Ƚͨ1жʹ */
	/* ׽/Ƚͨ1жϽֹ */

#define	LPTIM_IER_CC2IE_Pos	1	/* ׽/Ƚͨ2жʹλ(Capture/Compare channel 2 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ */
#define	LPTIM_IER_CC2IE_Msk	(0x1U << LPTIM_IER_CC2IE_Pos)
	/* ׽/Ƚͨ2жʹ */
	/* ׽/Ƚͨ2жϽֹ */

#define	LPTIM_IER_CC1IE_Pos	0	/* ׽/Ƚͨ1жʹλ(Capture/Compare channel 1 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ */
#define	LPTIM_IER_CC1IE_Msk	(0x1U << LPTIM_IER_CC1IE_Pos)
	/* ׽/Ƚͨ1жʹ */
	/* ׽/Ƚͨ1жϽֹ */

#define	LPTIM_ISR_CAP4OVR_Pos	11
#define	LPTIM_ISR_CAP4OVR_Msk	(0x1U << LPTIM_ISR_CAP4OVR_Pos)

#define	LPTIM_ISR_CAP3OVR_Pos	10
#define	LPTIM_ISR_CAP3OVR_Msk	(0x1U << LPTIM_ISR_CAP3OVR_Pos)

#define	LPTIM_ISR_CAP2OVR_Pos	9
#define	LPTIM_ISR_CAP2OVR_Msk	(0x1U << LPTIM_ISR_CAP2OVR_Pos)

#define	LPTIM_ISR_CAP1OVR_Pos	8
#define	LPTIM_ISR_CAP1OVR_Msk	(0x1U << LPTIM_ISR_CAP1OVR_Pos)

#define	LPTIM_ISR_TRIGIF_Pos	7
#define	LPTIM_ISR_TRIGIF_Msk	(0x1U << LPTIM_ISR_TRIGIF_Pos)

#define	LPTIM_ISR_OVIF_Pos	6
#define	LPTIM_ISR_OVIF_Msk	(0x1U << LPTIM_ISR_OVIF_Pos)

#define	LPTIM_ISR_CC4IF_Pos	3
#define	LPTIM_ISR_CC4IF_Msk	(0x1U << LPTIM_ISR_CC4IF_Pos)

#define	LPTIM_ISR_CC3IF_Pos	2
#define	LPTIM_ISR_CC3IF_Msk	(0x1U << LPTIM_ISR_CC3IF_Pos)

#define	LPTIM_ISR_CC2IF_Pos	1
#define	LPTIM_ISR_CC2IF_Msk	(0x1U << LPTIM_ISR_CC2IF_Pos)

#define	LPTIM_ISR_CC1IF_Pos	0
#define	LPTIM_ISR_CC1IF_Msk	(0x1U << LPTIM_ISR_CC1IF_Pos)

#define	LPTIM_CR_EN_Pos	0	/* LPTIMʹλ(LPTIM Enable)
1ʹܼ
0ֹ */
#define	LPTIM_CR_EN_Msk	(0x1U << LPTIM_CR_EN_Pos)
	/* ʹܼ */
	/* ֹ */

#define	LPTIM_CCR1_CCR1_Pos	0	/* ׽/ȽֵĴ1 (Channel1 Capture/Compare Register) */
#define	LPTIM_CCR1_CCR1_Msk	(0xffffffffU << LPTIM_CCR1_CCR1_Pos)

#define	LPTIM_CCR2_CCR2_Pos	0	/* ׽/ȽֵĴ2 (Channel2 Capture/Compare Register) */
#define	LPTIM_CCR2_CCR2_Msk	(0xffffffffU << LPTIM_CCR2_CCR2_Pos)

#define	LPTIM_CCR3_CCR3_Pos	0	/* ׽/ȽֵĴ3 (Channel3 Capture/Compare Register) */
#define	LPTIM_CCR3_CCR3_Msk	(0xffffffffU << LPTIM_CCR3_CCR3_Pos)

#define	LPTIM_CCR4_CCR4_Pos	0	/* ׽/ȽֵĴ4 (Channel4 Capture/Compare Register) */
#define	LPTIM_CCR4_CCR4_Msk	(0xffffffffU << LPTIM_CCR4_CCR4_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void LPTIM_Deinit(void);

/* LPT_ETRģ˲ʹ(External Trigger input Analog Filter Enable)
0رģ˲
1ʹģ˲˲Լ100ns غ */
extern void LPTIM_CFGR_ETR_AFEN_Setable(FunState NewState);
extern FunState LPTIM_CFGR_ETR_AFEN_Getable(void);

/* ʱԤƵѡ(Prescaler input Select)
0CLKSELѡʱ
1LPTETRF غ */
extern void LPTIM_CFGR_PSCSEL_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_PSCSEL_Get(void);

/* ʱӷƵѡ(Counter Clock Divider Select)
0001Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10016Ƶ
10132Ƶ
11064Ƶ
111128Ƶ غ */
extern void LPTIM_CFGR_DIVSEL_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_DIVSEL_Get(void);

/* ETRѡ(ETR Clock Edge Select)
0LPT_ETRؼ
1LPT_ETR½ؼ غ */
extern void LPTIM_CFGR_EDGESEL_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_EDGESEL_Get(void);

/* ⲿѡʹڲʱͬLPT_ETR(ETR trigger Configuration)
00: LPT_ETRźش
01: LPT_ETRź½ش
10/11: ⲿź½ش غ */
extern void LPTIM_CFGR_TRIGCFG_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_TRIGCFG_Get(void);

/* μģʽʹ(One State Timer)
0ģʽ󱣳УֱرΪֹﵽĿֵص0¿ʼжϡ
1μģʽĿֵص0Զֹͣжϡ غ */
extern void LPTIM_CFGR_ONST_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_ONST_Get(void);

/* ģʽѡ(Timer operation Mode)
00ͨʱģʽ
01Trigger崥ģʽ
10ⲿ첽ģʽ
11Timeoutģʽ غ */
extern void LPTIM_CFGR_TMODE_Set(uint32_t SetValue);
extern uint32_t LPTIM_CFGR_TMODE_Get(void);

/* 32bitǰֵ(Counter 32bits-wide) غ */
extern uint32_t LPTIM_CNTR_Read(void);

/* ͨ1׽Դѡ(Capture channel 1 source select)CH1ͨΪ벶׽ʱЧ
00LPT_CH1
01XTLF
10RCLP
11RCMF غ */
extern void LPTIM_CCSR_CAP1SSEL_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAP1SSEL_Get(void);

/* ͨ4ǰ׽ıأCC2IFλʱ(Channel 4 Captured Edge)
0
1½ غ */
extern void LPTIM_CCSR_CAP4EDGE_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAP4EDGE_Get(void);
extern void LPTIM_CCSR_CAP3EDGE_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAP3EDGE_Get(void);
extern void LPTIM_CCSR_CAP2EDGE_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAP2EDGE_Get(void);
extern void LPTIM_CCSR_CAP1EDGE_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAP1EDGE_Get(void);

/* ͨ4Ƚμѡ (Channel 4 compare output Polarity)
0ԲΣʼΪֵͣ==Ƚֵʱøߣֵ==ARRʱָΪ
1ԲΣԲȡ غ */
extern void LPTIM_CCSR_POLAR4_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_POLAR4_Get(void);
extern void LPTIM_CCSR_POLAR3_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_POLAR3_Get(void);
extern void LPTIM_CCSR_POLAR2_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_POLAR2_Get(void);
extern void LPTIM_CCSR_POLAR1_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_POLAR1_Get(void);

/* ͨ4׽ѡ(Channel 4 Capture edge Config)
00ز׽
01½ز׽
10½ز׽
11RFU غ */
extern void LPTIM_CCSR_CAPCFG4_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAPCFG4_Get(void);
extern void LPTIM_CCSR_CAPCFG3_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAPCFG3_Get(void);
extern void LPTIM_CCSR_CAPCFG2_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAPCFG2_Get(void);
extern void LPTIM_CCSR_CAPCFG1_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CAPCFG1_Get(void);

/* ͨ4׽/ȽϹʹ(Channel 4 Capture/Compare Select)
00,11ֹͨ4׽/ȽϹ
01ʹͨ4׽ܣLPT_CH4Ϊ룩
10ʹͨ4ȽϹܣLPT_CH4Ϊ غ */
extern void LPTIM_CCSR_CC4S_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CC4S_Get(void);
extern void LPTIM_CCSR_CC3S_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CC3S_Get(void);
extern void LPTIM_CCSR_CC2S_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CC2S_Get(void);
extern void LPTIM_CCSR_CC1S_Set(uint32_t SetValue);
extern uint32_t LPTIM_CCSR_CC1S_Get(void);

/* ԶĿĴ(Auto-Reload Register)
ֵARRʱصֵ¿ʼϼ غ */
extern void LPTIM_ARR_Write(uint32_t SetValue);
extern uint32_t LPTIM_ARR_Read(void);

/* ͨ4׽жʹ(Channel 4 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
extern void LPTIM_IER_OVR4IE_Setable(FunState NewState);
extern FunState LPTIM_IER_OVR4IE_Getable(void);

/* ͨ3׽жʹ(Channel 3 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
extern void LPTIM_IER_OVR3IE_Setable(FunState NewState);
extern FunState LPTIM_IER_OVR3IE_Getable(void);

/* ͨ2׽жʹ(Channel 2 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
extern void LPTIM_IER_OVR2IE_Setable(FunState NewState);
extern FunState LPTIM_IER_OVR2IE_Getable(void);

/* ͨ1׽жʹ(Channel 1 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
extern void LPTIM_IER_OVR1IE_Setable(FunState NewState);
extern FunState LPTIM_IER_OVR1IE_Getable(void);

/* ⲿжʹλ(External Trigger Interrupt Enable)
1ⲿжʹ
0ⲿжϽֹ غ */
extern void LPTIM_IER_TRIGIE_Setable(FunState NewState);
extern FunState LPTIM_IER_TRIGIE_Getable(void);

/* жʹλ(Counter Over-Flow Interrupt Enable)
1жʹ
0жϽֹ غ */
extern void LPTIM_IER_OVIE_Setable(FunState NewState);
extern FunState LPTIM_IER_OVIE_Getable(void);

/* ׽/Ƚͨ4жʹλ(Capture/Compare channel 4 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ غ */
extern void LPTIM_IER_CC4IE_Setable(FunState NewState);
extern FunState LPTIM_IER_CC4IE_Getable(void);

/* ׽/Ƚͨ3жʹλ(Capture/Compare channel 3 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ غ */
extern void LPTIM_IER_CC3IE_Setable(FunState NewState);
extern FunState LPTIM_IER_CC3IE_Getable(void);

/* ׽/Ƚͨ2жʹλ(Capture/Compare channel 2 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ غ */
extern void LPTIM_IER_CC2IE_Setable(FunState NewState);
extern FunState LPTIM_IER_CC2IE_Getable(void);

/* ׽/Ƚͨ1жʹλ(Capture/Compare channel 1 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ غ */
extern void LPTIM_IER_CC1IE_Setable(FunState NewState);
extern FunState LPTIM_IER_CC1IE_Getable(void);
extern void LPTIM_ISR_CAP4OVR_Clr(void);
extern FlagStatus LPTIM_ISR_CAP4OVR_Chk(void);
extern void LPTIM_ISR_CAP3OVR_Clr(void);
extern FlagStatus LPTIM_ISR_CAP3OVR_Chk(void);
extern void LPTIM_ISR_CAP2OVR_Clr(void);
extern FlagStatus LPTIM_ISR_CAP2OVR_Chk(void);
extern void LPTIM_ISR_CAP1OVR_Clr(void);
extern FlagStatus LPTIM_ISR_CAP1OVR_Chk(void);
extern void LPTIM_ISR_TRIGIF_Clr(void);
extern FlagStatus LPTIM_ISR_TRIGIF_Chk(void);
extern void LPTIM_ISR_OVIF_Clr(void);
extern FlagStatus LPTIM_ISR_OVIF_Chk(void);
extern void LPTIM_ISR_CC4IF_Clr(void);
extern FlagStatus LPTIM_ISR_CC4IF_Chk(void);
extern void LPTIM_ISR_CC3IF_Clr(void);
extern FlagStatus LPTIM_ISR_CC3IF_Chk(void);
extern void LPTIM_ISR_CC2IF_Clr(void);
extern FlagStatus LPTIM_ISR_CC2IF_Chk(void);
extern void LPTIM_ISR_CC1IF_Clr(void);
extern FlagStatus LPTIM_ISR_CC1IF_Chk(void);




/* LPTIMʹλ(LPTIM Enable)
1ʹܼ
0ֹ غ */
extern void LPTIM_CR_EN_Setable(FunState NewState);
extern FunState LPTIM_CR_EN_Getable(void);

/* ׽/ȽֵĴ1 (Channel1 Capture/Compare Register) غ */
extern void LPTIM_CCR1_Write(uint32_t SetValue);
extern uint32_t LPTIM_CCR1_Read(void);
extern void LPTIM_Deinit(void);

/* ׽/ȽֵĴ2 (Channel2 Capture/Compare Register) غ */
extern void LPTIM_CCR2_Write(uint32_t SetValue);
extern uint32_t LPTIM_CCR2_Read(void);
extern void LPTIM_Deinit(void);

/* ׽/ȽֵĴ3 (Channel3 Capture/Compare Register) غ */
extern void LPTIM_CCR3_Write(uint32_t SetValue);
extern uint32_t LPTIM_CCR3_Read(void);

/* ׽/ȽֵĴ4 (Channel4 Capture/Compare Register) غ */
extern void LPTIM_CCR4_Write(uint32_t SetValue);
extern uint32_t LPTIM_CCR4_Read(void);
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /*__FM33A0XXEV_LPTIM_H */
