/**
  ******************************************************************************
  * @file    fm33a0xxev_lpuart.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the LPUART firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_LPUART_H
#define __FM33A0XXEV_LPUART_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
	 
#define LPUARTx_MCTL_FOR9600 (0x00000552<<16)
#define LPUARTx_MCTL_FOR4800 (0x00001EFB<<16)
#define LPUARTx_MCTL_FOR2400 (0x000016DB<<16)
#define LPUARTx_MCTL_FOR1200 (0x00000492<<16)
#define LPUARTx_MCTL_FOR600  (0x000016D6<<16)
#define LPUARTx_MCTL_FOR300  (0x00000842<<16)	 
	 
#define	LPUARTx_CSR_BUSY_Pos	24	/* LPUARTͨű־ֻ (Busy)
1LPUARTͨ
0LPUART */
#define	LPUARTx_CSR_BUSY_Msk	(0x1U << LPUARTx_CSR_BUSY_Pos)

#define	LPUARTx_CSR_WKBYTECFG_Pos	19	/* ݽջ (Wakeup Byte Config)
11ֽڣżУSTOPλȷŴж
01ֽڣУλSTOPλֱӴж */
#define	LPUARTx_CSR_WKBYTECFG_Msk	(0x1U << LPUARTx_CSR_WKBYTECFG_Pos)

#define	LPUARTx_CSR_RXEV_Pos	16	/* ж¼ãڿƺ¼CPUṩж
(Receive Wakeup Event)
00STARTλ⻽
011byteݽ
10ƥɹ
11RXD½ؼ */
#define	LPUARTx_CSR_RXEV_Msk	(0x3U << LPUARTx_CSR_RXEV_Pos)
#define	LPUARTx_CSR_RXEV_START	(0x0U << LPUARTx_CSR_RXEV_Pos)	/* STARTλ⻽ */
#define	LPUARTx_CSR_RXEV_1BYTE	(0x1U << LPUARTx_CSR_RXEV_Pos)	/* 1byteݽ */
#define	LPUARTx_CSR_RXEV_MATCH	(0x2U << LPUARTx_CSR_RXEV_Pos)	/* ƥɹ */
#define	LPUARTx_CSR_RXEV_FALLING	(0x3U << LPUARTx_CSR_RXEV_Pos)	/* RXD½ؼ */

#define	LPUARTx_CSR_IOSWAP_Pos	11	/* RXTXŽ
0Ĭ
1 */
#define	LPUARTx_CSR_IOSWAP_Msk	(0x1U << LPUARTx_CSR_IOSWAP_Pos)
#define	LPUARTx_CSR_IOSWAP_DEFUALT	(0x0U << LPUARTx_CSR_IOSWAP_Pos)	/* Ĭ */
#define	LPUARTx_CSR_IOSWAP_SWAP		(0x1U << LPUARTx_CSR_IOSWAP_Pos)	/*  */


#define	LPUARTx_CSR_DMATXIFCFG_Pos	10	/* DMAжʹܣLPUARTͨDMAзʱЧ (DMA Transmit Interrupt Config)
1IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź
0ǷжźIE */
#define	LPUARTx_CSR_DMATXIFCFG_Msk	(0x1U << LPUARTx_CSR_DMATXIFCFG_Pos)
	/* IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź */
	/* ǷжźIE */

#define	LPUARTx_CSR_BITORD_Pos	9	/* ݷ/ʱλ˳ (Bit Order)
0LSB first
1MSB first */
#define	LPUARTx_CSR_BITORD_Msk	(0x1U << LPUARTx_CSR_BITORD_Pos)
#define	LPUARTx_CSR_BITORD_LSB	(0x0U << LPUARTx_CSR_BITORD_Pos)	/* LSB first */
#define	LPUARTx_CSR_BITORD_MSB	(0x1U << LPUARTx_CSR_BITORD_Pos)	/* MSB first */

#define	LPUARTx_CSR_STOPCFG_Pos	8	/* ֹͣλãԷ֡ʽЧʱжֹͣλ (Stop bit Config)
01λֹͣλ
12λֹͣλ */
#define	LPUARTx_CSR_STOPCFG_Msk	(0x1U << LPUARTx_CSR_STOPCFG_Pos)
#define	LPUARTx_CSR_STOPCFG_1STOP	(0x0U << LPUARTx_CSR_STOPCFG_Pos)	/* 1λֹͣλ */
#define	LPUARTx_CSR_STOPCFG_2STOP	(0x1U << LPUARTx_CSR_STOPCFG_Pos)	/* 2λֹͣλ */

#define	LPUARTx_CSR_PDSEL_Pos	6	/* ÿ֡ݳѡ񣻴˼ĴݷͺͽͬʱЧ
(Payload Data length Select)
007λ
018λ
109λ
116λ */
#define	LPUARTx_CSR_PDSEL_Msk	(0x3U << LPUARTx_CSR_PDSEL_Pos)
#define	LPUARTx_CSR_PDSEL_7BITS	(0x0U << LPUARTx_CSR_PDSEL_Pos)	/* 7λ */
#define	LPUARTx_CSR_PDSEL_8BITS	(0x1U << LPUARTx_CSR_PDSEL_Pos)	/* 8λ */
#define	LPUARTx_CSR_PDSEL_9BITS	(0x2U << LPUARTx_CSR_PDSEL_Pos)	/* 9λ */
#define	LPUARTx_CSR_PDSEL_6BITS	(0x3U << LPUARTx_CSR_PDSEL_Pos)	/* 6λ */

#define	LPUARTx_CSR_PARITY_Pos	4	/* Уλã˼ĴݷͺͽͬʱЧ (Parity)
00Уλ
01żУ
10У
11RFU */
#define	LPUARTx_CSR_PARITY_Msk	(0x3U << LPUARTx_CSR_PARITY_Pos)
#define	LPUARTx_CSR_PARITY_NON	(0x0U << LPUARTx_CSR_PARITY_Pos)	/* Уλ */
#define	LPUARTx_CSR_PARITY_EVEN	(0x1U << LPUARTx_CSR_PARITY_Pos)	/* żУ */
#define	LPUARTx_CSR_PARITY_ODD	(0x2U << LPUARTx_CSR_PARITY_Pos)	/* У */

#define	LPUARTx_CSR_RXPOL_Pos	3	/* ݼ (Receive Polarity)
0
1ȡ */
#define	LPUARTx_CSR_RXPOL_Msk	(0x1U << LPUARTx_CSR_RXPOL_Pos)
#define	LPUARTx_CSR_RXPOL_POS	(0x0U << LPUARTx_CSR_RXPOL_Pos)	/*  */
#define	LPUARTx_CSR_RXPOL_NEG	(0x1U << LPUARTx_CSR_RXPOL_Pos)	/* ȡ */

#define	LPUARTx_CSR_TXPOL_Pos	2	/* ݼ (Transmit Polarity)
0
1ȡ */
#define	LPUARTx_CSR_TXPOL_Msk	(0x1U << LPUARTx_CSR_TXPOL_Pos)
#define	LPUARTx_CSR_TXPOL_POS	(0x0U << LPUARTx_CSR_TXPOL_Pos)	/*  */
#define	LPUARTx_CSR_TXPOL_NEG	(0x1U << LPUARTx_CSR_TXPOL_Pos)	/* ȡ */

#define	LPUARTx_CSR_RXEN_Pos	1	/* ʹܣ1Ч (Receive Enable) */
#define	LPUARTx_CSR_RXEN_Msk	(0x1U << LPUARTx_CSR_RXEN_Pos)

#define	LPUARTx_CSR_TXEN_Pos	0	/* ʹܣ1Ч (Transmit Enable) */
#define	LPUARTx_CSR_TXEN_Msk	(0x1U << LPUARTx_CSR_TXEN_Pos)

#define	LPUARTx_IER_RXEV_IE_Pos	12	/* ջ¼жʹܣ1Ч (Receive Event Interrupt Enable) */
#define	LPUARTx_IER_RXEV_IE_Msk	(0x1U << LPUARTx_IER_RXEV_IE_Pos)

#define	LPUARTx_IER_RXERR_IE_Pos	10	/* մжʹܣ1Ч (Receive Error Interrupt Enable) */
#define	LPUARTx_IER_RXERR_IE_Msk	(0x1U << LPUARTx_IER_RXERR_IE_Pos)

#define	LPUARTx_IER_RXBF_IE_Pos	8	/* ջжʹܣ1Ч (Receive Buffer Full Interrupt Enable) */
#define	LPUARTx_IER_RXBF_IE_Msk	(0x1U << LPUARTx_IER_RXBF_IE_Pos)

#define	LPUARTx_IER_TXBE_IE_Pos	1	/* ͻжʹܣ1Ч (Transmit Buffer Empty Interrupt Enable) */
#define	LPUARTx_IER_TXBE_IE_Msk	(0x1U << LPUARTx_IER_TXBE_IE_Pos)

#define	LPUARTx_IER_TXSE_IE_Pos	0	/* ͻҷλĴжʹܣ1Ч (Transmit Shift register Interrupt Enable) */
#define	LPUARTx_IER_TXSE_IE_Msk	(0x1U << LPUARTx_IER_TXSE_IE_Pos)

#define	LPUARTx_ISR_RXEVF_Pos	24	/* ջ¼жϱ־Ӳλд1 (Receive Event Interrupt Flag)
жϱ־ԴLPUxCR.RXEVĴá */
#define	LPUARTx_ISR_RXEVF_Msk	(0x1U << LPUARTx_ISR_RXEVF_Pos)

#define	LPUARTx_ISR_TXOV_Pos	19	/* ͻӲλд1 (Transmit Overflow Error)
ͻеݻδλĴʱͻдݣTXOV־λ */
#define	LPUARTx_ISR_TXOV_Msk	(0x1U << LPUARTx_ISR_TXOV_Pos)

#define	LPUARTx_ISR_PERR_Pos	18	/* żУжϱ־Ӳλд1 (Parity Error) */
#define	LPUARTx_ISR_PERR_Msk	(0x1U << LPUARTx_ISR_PERR_Pos)

#define	LPUARTx_ISR_FERR_Pos	17	/* ֡ʽжϱ־Ӳλд1 (Frame Error) */
#define	LPUARTx_ISR_FERR_Msk	(0x1U << LPUARTx_ISR_FERR_Pos)

#define	LPUARTx_ISR_OERR_Pos	16	/* ջжϱ־ջ£յµʱλӲλд1 (Receive Buffer Overflow Error) */
#define	LPUARTx_ISR_OERR_Msk	(0x1U << LPUARTx_ISR_OERR_Pos)

#define	LPUARTx_ISR_RXBF_Pos	8	/* ջжϱ־Ӳλд1߶ȡRXBUFʱ (Receive Buffer Full) */
#define	LPUARTx_ISR_RXBF_Msk	(0x1U << LPUARTx_ISR_RXBF_Pos)

#define	LPUARTx_ISR_TXBE_Pos	1	/* ͻжϱ־ӲλдTXBUFʱ (Transmit Buffer Empty) */
#define	LPUARTx_ISR_TXBE_Msk	(0x1U << LPUARTx_ISR_TXBE_Pos)

#define	LPUARTx_ISR_TXSE_Pos	0	/* ͻҷλĴжϱ־Ӳλд1߷ݱλĴʱ (Transmit Shift register Empty) */
#define	LPUARTx_ISR_TXSE_Msk	(0x1U << LPUARTx_ISR_TXSE_Pos)

#define	LPUARTx_BMR_MCTL_Pos	16	/* LPUARTÿbitλƿźţμ!δҵԴ!δҵԴ (Bit Modulation Control) */
#define	LPUARTx_BMR_MCTL_Msk	(0x1fffU << LPUARTx_BMR_MCTL_Pos)

#define	LPUARTx_BMR_BAUD_Pos	0	/* ʿƣbps
0009600
0014800
0102400
0111200
100600
101/110/111300 */
#define	LPUARTx_BMR_BAUD_Msk	(0x7U << LPUARTx_BMR_BAUD_Pos)
#define	LPUARTx_BMR_BAUD_9600	(0x0U << LPUARTx_BMR_BAUD_Pos)	/* 9600 */
#define	LPUARTx_BMR_BAUD_4800	(0x1U << LPUARTx_BMR_BAUD_Pos)	/* 4800 */
#define	LPUARTx_BMR_BAUD_2400	(0x2U << LPUARTx_BMR_BAUD_Pos)	/* 2400 */
#define	LPUARTx_BMR_BAUD_1200	(0x3U << LPUARTx_BMR_BAUD_Pos)	/* 1200 */
#define	LPUARTx_BMR_BAUD_600	(0x4U << LPUARTx_BMR_BAUD_Pos)	/* 600 */
#define	LPUARTx_BMR_BAUD_300	(0x5U << LPUARTx_BMR_BAUD_Pos)	/* 300 */

#define	LPUARTx_RXBUF_RXBUF_Pos	0	/* ݻĴ (Receive Buffer) */
#define	LPUARTx_RXBUF_RXBUF_Msk	(0x1ffU << LPUARTx_RXBUF_RXBUF_Pos)

#define	LPUARTx_TXBUF_TXBUF_Pos	0	/* ݻĴ (Transmit Buffer) */
#define	LPUARTx_TXBUF_TXBUF_Msk	(0x1ffU << LPUARTx_TXBUF_TXBUF_Pos)

#define	LPUARTx_DMR_MATD_Pos	0	/* һ֡ձȽݣRXEV=10յĵһ֡MATDͬʱRXEVFжϣģʽµݽջѡ
(Matched Data) */
#define	LPUARTx_DMR_MATD_Msk	(0x1ffU << LPUARTx_DMR_MATD_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void LPUARTx_Deinit(LPUART_Type* LPUARTx);

/* LPUARTͨű־ֻ (Busy)
1LPUARTͨ
0LPUART غ */
extern FlagStatus LPUARTx_CSR_BUSY_Chk(LPUART_Type* LPUARTx);

/* ݽջ غ */
extern void LPUARTx_CSR_WKBYTECFG_Setable(LPUART_Type* LPUARTx, FunState NewState);
	
/* ݽջ (Wakeup Byte Config)
11ֽڣżУSTOPλȷŴж
01ֽڣУλSTOPλֱӴж غ */
extern FlagStatus LPUARTx_CSR_WKBYTECFG_Chk(LPUART_Type* LPUARTx);

/* ж¼ãڿƺ¼CPUṩж
(Receive Wakeup Event)
00STARTλ⻽
011byteݽ
10ƥɹ
11RXD½ؼ غ */
extern void LPUARTx_CSR_RXEV_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_RXEV_Get(LPUART_Type* LPUARTx);

/* RXTXŽ
0Ĭ
1 */
extern void LPUARTx_CSR_IOSWAP_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_IOSWAP_Get(LPUART_Type* LPUARTx);


/* DMAжʹܣLPUARTͨDMAзʱЧ (DMA Transmit Interrupt Config)
1IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź
0ǷжźIE غ */
extern void LPUARTx_CSR_DMATXIFCFG_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_CSR_DMATXIFCFG_Getable(LPUART_Type* LPUARTx);

/* ݷ/ʱλ˳ (Bit Order)
0LSB first
1MSB first غ */
extern void LPUARTx_CSR_BITORD_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_BITORD_Get(LPUART_Type* LPUARTx);

/* ֹͣλãԷ֡ʽЧʱжֹͣλ (Stop bit Config)
01λֹͣλ
12λֹͣλ غ */
extern void LPUARTx_CSR_STOPCFG_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_STOPCFG_Get(LPUART_Type* LPUARTx);

/* ÿ֡ݳѡ񣻴˼ĴݷͺͽͬʱЧ
(Payload Data length Select)
007λ
018λ
109λ
116λ غ */
extern void LPUARTx_CSR_PDSEL_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_PDSEL_Get(LPUART_Type* LPUARTx);

/* Уλã˼ĴݷͺͽͬʱЧ (Parity)
00Уλ
01żУ
10У
11RFU غ */
extern void LPUARTx_CSR_PARITY_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_PARITY_Get(LPUART_Type* LPUARTx);

/* ݼ (Receive Polarity)
0
1ȡ غ */
extern void LPUARTx_CSR_RXPOL_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_RXPOL_Get(LPUART_Type* LPUARTx);

/* ݼ (Transmit Polarity)
0
1ȡ غ */
extern void LPUARTx_CSR_TXPOL_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_CSR_TXPOL_Get(LPUART_Type* LPUARTx);

/* ʹܣ1Ч (Receive Enable) غ */
extern void LPUARTx_CSR_RXEN_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_CSR_RXEN_Getable(LPUART_Type* LPUARTx);

/* ʹܣ1Ч (Transmit Enable) غ */
extern void LPUARTx_CSR_TXEN_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_CSR_TXEN_Getable(LPUART_Type* LPUARTx);

/* ջ¼жʹܣ1Ч (Receive Event Interrupt Enable) غ */
extern void LPUARTx_IER_RXEV_IE_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_IER_RXEV_IE_Getable(LPUART_Type* LPUARTx);

/* մжʹܣ1Ч (Receive Error Interrupt Enable) غ */
extern void LPUARTx_IER_RXERR_IE_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_IER_RXERR_IE_Getable(LPUART_Type* LPUARTx);

/* ջжʹܣ1Ч (Receive Buffer Full Interrupt Enable) غ */
extern void LPUARTx_IER_RXBF_IE_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_IER_RXBF_IE_Getable(LPUART_Type* LPUARTx);

/* ͻжʹܣ1Ч (Transmit Buffer Empty Interrupt Enable) غ */
extern void LPUARTx_IER_TXBE_IE_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_IER_TXBE_IE_Getable(LPUART_Type* LPUARTx);

/* ͻҷλĴжʹܣ1Ч (Transmit Shift register Interrupt Enable) غ */
extern void LPUARTx_IER_TXSE_IE_Setable(LPUART_Type* LPUARTx,FunState NewState);
extern FunState LPUARTx_IER_TXSE_IE_Getable(LPUART_Type* LPUARTx);

/* ջ¼жϱ־Ӳλд1 (Receive Event Interrupt Flag)
жϱ־ԴLPUxCR.RXEVĴá غ */
extern void LPUARTx_ISR_RXEVF_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_RXEVF_Chk(LPUART_Type* LPUARTx);

/* ͻӲλд1 (Transmit Overflow Error)
ͻеݻδλĴʱͻдݣTXOV־λ غ */
extern void LPUARTx_ISR_TXOV_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_TXOV_Chk(LPUART_Type* LPUARTx);

/* żУжϱ־Ӳλд1 (Parity Error) غ */
extern void LPUARTx_ISR_PERR_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_PERR_Chk(LPUART_Type* LPUARTx);

/* ֡ʽжϱ־Ӳλд1 (Frame Error) غ */
extern void LPUARTx_ISR_FERR_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_FERR_Chk(LPUART_Type* LPUARTx);

/* ջжϱ־ջ£յµʱλӲλд1 (Receive Buffer Overflow Error) غ */
extern void LPUARTx_ISR_OERR_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_OERR_Chk(LPUART_Type* LPUARTx);

/* ջжϱ־Ӳλд1߶ȡRXBUFʱ (Receive Buffer Full) غ */
extern void LPUARTx_ISR_RXBF_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_RXBF_Chk(LPUART_Type* LPUARTx);

/* ͻжϱ־ӲλдTXBUFʱ (Transmit Buffer Empty) غ */
extern FlagStatus LPUARTx_ISR_TXBE_Chk(LPUART_Type* LPUARTx);

/* ͻҷλĴжϱ־Ӳλд1߷ݱλĴʱ (Transmit Shift register Empty) غ */
extern void LPUARTx_ISR_TXSE_Clr(LPUART_Type* LPUARTx);
extern FlagStatus LPUARTx_ISR_TXSE_Chk(LPUART_Type* LPUARTx);

/* LPUARTÿbitλƿźţμ!δҵԴ!δҵԴ (Bit Modulation Control) غ */
extern void LPUARTx_BMR_MCTL_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_BMR_MCTL_Get(LPUART_Type* LPUARTx);

/* ʿƣbps
0009600
0014800
0102400
0111200
100600
101/110/111300 غ */
extern void LPUARTx_BMR_BAUD_Set(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_BMR_BAUD_Get(LPUART_Type* LPUARTx);

/* ݻĴ (Receive Buffer) غ */
extern uint32_t LPUARTx_RXBUF_Read(LPUART_Type* LPUARTx);

/* ݻĴ (Transmit Buffer) غ */
extern void LPUARTx_TXBUF_Write(LPUART_Type* LPUARTx,uint32_t SetValue);

/* һ֡ձȽݣRXEV=10յĵһ֡MATDͬʱRXEVFжϣģʽµݽջѡ
(Matched Data) غ */
extern void LPUARTx_DMR_Write(LPUART_Type* LPUARTx,uint32_t SetValue);
extern uint32_t LPUARTx_DMR_Read(LPUART_Type* LPUARTx);
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /*__FM33A0XXEV_LPUART_H */
