/**
  ******************************************************************************
  * @file    fm33a0xxev_pmu.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the PMU firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_PMU_H
#define __FM33A0XXEV_PMU_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "FM33A0XXEV.h"
typedef struct
{
	uint32_t PMOD;				/*!<͹ģʽ  */	
	uint32_t SLPDP;				/*!<DeepSleepƼĴ  */
	FunState CVS;				/*!<ں˵ѹʹܿ  */
	uint32_t SCR;				/*!<M0ϵͳƼĴһΪ0  */	
	uint32_t TIA;				/*!ɱ̶⻽ӳ */	
}PMU_SleepCfg_InitTypeDef;
   
#define	PMU_CR_LDO15EN_Pos	17	/* LDO15ʹܱ־λ
1LDO15ڹ״̬
0LDO15ر */
#define	PMU_CR_LDO15EN_Msk	(0x1U << PMU_CR_LDO15EN_Pos)

#define	PMU_CR_LDO15EN_B_Pos	16	/* LDO15ʹܱ־Уλ */
#define	PMU_CR_LDO15EN_B_Msk	(0x1U << PMU_CR_LDO15EN_B_Pos)

#define	PMU_CR_WKFSEL_Pos	10	/* Sleep/DeepSleepѺϵͳƵ
00RCHF-8MHz
01RCHF-16MHz
10RCHF-24MHz
11RCHF-32MHz */
#define	PMU_CR_WKFSEL_Msk	(0x3U << PMU_CR_WKFSEL_Pos)
#define	PMU_CR_WKFSEL_RCHF_8M	(0x0U << PMU_CR_WKFSEL_Pos)	/* RCHF-8MHz */
#define	PMU_CR_WKFSEL_RCHF_16M	(0x1U << PMU_CR_WKFSEL_Pos)	/* RCHF-16MHz */
#define	PMU_CR_WKFSEL_RCHF_24M	(0x2U << PMU_CR_WKFSEL_Pos)	/* RCHF-24MHz */
#define	PMU_CR_WKFSEL_RCHF_32M	(0x3U << PMU_CR_WKFSEL_Pos)	/* RCHF-32MHz */

#define	PMU_CR_SLPDP_Pos	9	/* DeepSleepƼĴ
1DeepSleepģʽʹܣ¹رջ׼ѹԴ
0Sleepģʽ
Sleep£λSLPDPλΪDeepSleepģʽ
λSleepЧ */
#define	PMU_CR_SLPDP_Msk	(0x1U << PMU_CR_SLPDP_Pos)
#define	PMU_CR_SLPDP_DEEPSLEEP	(0x1U << PMU_CR_SLPDP_Pos)	/* DeepSleepģʽʹܣ¹رջ׼ѹԴ */
#define	PMU_CR_SLPDP_SLEEP	(0x0U << PMU_CR_SLPDP_Pos)	/* Sleepģʽ */

#define	PMU_CR_CVS_Pos	8	/* CoreVoltageScaling
0͹ģʽ²ʹں˵ѹ
1͹ģʽ½ں˵ѹ
λSleep/DeepSleep/RTCBKPģʽ */
#define	PMU_CR_CVS_Msk	(0x1U << PMU_CR_CVS_Pos)
#define	PMU_CR_CVS_DISABLE	(0x0U << PMU_CR_CVS_Pos)	/* ͹ģʽ²ʹں˵ѹ */
#define	PMU_CR_CVS_ENABLE	(0x1U << PMU_CR_CVS_Pos)	/* ͹ģʽ½ں˵ѹ */

#define	PMU_CR_PMOD_Pos	0	/* ͹ģʽüĴ
00Active mode / LP Active mode
01LPRUN mode
10Sleep mode / DeepSleep mode
11RTCBKP mode */
#define	PMU_CR_PMOD_Msk	(0x3U << PMU_CR_PMOD_Pos)
#define	PMU_CR_PMOD_ACTIVE	(0x0U << PMU_CR_PMOD_Pos)	/* Active mode / LP Active mode */
#define	PMU_CR_PMOD_LPRUN 	(0x1U << PMU_CR_PMOD_Pos)	/* LPRUN mode */
#define	PMU_CR_PMOD_SLEEP	(0x2U << PMU_CR_PMOD_Pos)	/* Sleep mode / DeepSleep mode */

#define	PMU_WKTR_STPCLR_Pos	2	/* Flash Stopѿ
0Stopźŵȴʱӽͬ
1Stopź첽 */
#define	PMU_WKTR_STPCLR_Msk	(0x1U << PMU_WKTR_STPCLR_Pos)
#define	PMU_WKTR_STPCLR_SYN	(0x0U << PMU_WKTR_STPCLR_Pos)	/* Stopźŵȴʱӽͬ */
#define	PMU_WKTR_STPCLR_ASY	(0x1U << PMU_WKTR_STPCLR_Pos)	/* Stopź첽 */

#define	PMU_WKTR_T1A_Pos	0	/* ɱ̶⻽ӳ
DeepSleepģʽ£RCHFʱӵ󣬸ݴ˼ĴõȴӳʱٶȡFlashУ
000us
012us
104us
118us */
#define	PMU_WKTR_T1A_Msk	(0x3U << PMU_WKTR_T1A_Pos)
#define	PMU_WKTR_T1A_0US	(0x0U << PMU_WKTR_T1A_Pos)	/* 0us */
#define	PMU_WKTR_T1A_2US	(0x1U << PMU_WKTR_T1A_Pos)	/* 2us */
#define	PMU_WKTR_T1A_4US	(0x2U << PMU_WKTR_T1A_Pos)	/* 4us */
#define	PMU_WKTR_T1A_8US	(0x3U << PMU_WKTR_T1A_Pos)	/* 8us */

#define	PMU_WKFR_ADCWKF_Pos	31	/* ADCжϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_ADCWKF_Msk	(0x1U << PMU_WKFR_ADCWKF_Pos)

#define	PMU_WKFR_RTCWKF_Pos	28	/* RTCжϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_RTCWKF_Msk	(0x1U << PMU_WKFR_RTCWKF_Pos)

#define	PMU_WKFR_SVDWKF_Pos	27	/* SVDжϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_SVDWKF_Msk	(0x1U << PMU_WKFR_SVDWKF_Pos)

#define	PMU_WKFR_LFDETWKF_Pos	26	/* 32768Hzͣжϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_LFDETWKF_Msk	(0x1U << PMU_WKFR_LFDETWKF_Pos)

#define	PMU_WKFR_IOWKF_Pos	24	/* IOжϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_IOWKF_Msk	(0x1U << PMU_WKFR_IOWKF_Pos)

#define	PMU_WKFR_LPU1WKF_Pos	21	/* LPUART1жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_LPU1WKF_Msk	(0x1U << PMU_WKFR_LPU1WKF_Pos)

#define	PMU_WKFR_LPU0WKF_Pos	20	/* LPUART0жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_LPU0WKF_Msk	(0x1U << PMU_WKFR_LPU0WKF_Pos)

#define	PMU_WKFR_COMP_OOWF_Pos	19	/* Ƚout-of-windowѱ־жϳʱӲԶ */
#define	PMU_WKFR_COMP_OOWF_Msk	(0x1U << PMU_WKFR_COMP_OOWF_Pos)

#define	PMU_WKFR_COMP_WINF_Pos	18	/* Ƚwindowѱ־жϳʱӲԶ */
#define	PMU_WKFR_COMP_WINF_Msk	(0x1U << PMU_WKFR_COMP_WINF_Pos)

#define	PMU_WKFR_COMP2WKF_Pos	17	/* Ƚ2жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_COMP2WKF_Msk	(0x1U << PMU_WKFR_COMP2WKF_Pos)

#define	PMU_WKFR_COMP1WKF_Pos	16	/* Ƚ1жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_COMP1WKF_Msk	(0x1U << PMU_WKFR_COMP1WKF_Pos)

#define	PMU_WKFR_LPTWKF_Pos	10	/*  _32жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_LPTWKF_Msk	(0x1U << PMU_WKFR_LPTWKF_Pos)

#define	PMU_WKFR_BSTWKF_Pos	9	/* BSTIM32жϻѱ־жϳʱӲԶ */
#define	PMU_WKFR_BSTWKF_Msk	(0x1U << PMU_WKFR_BSTWKF_Pos)

#define	PMU_WKFR_DBGWKF_Pos	8	/* CPU Debuggerѱ־д1 */
#define	PMU_WKFR_DBGWKF_Msk	(0x1U << PMU_WKFR_DBGWKF_Pos)


#define	PMU_IER_SLPEIE_Pos	1	/* SLEEPжʹ (Sleep mode Error Interrupt Enable)
1ʹSLEEPж
0ֹSLEEPж */
#define	PMU_IER_SLPEIE_Msk	(0x1U << PMU_IER_SLPEIE_Pos)
	/* ʹSLEEPж */
	/* ֹSLEEPж */

#define	PMU_IER_RTCEIE_Pos	0	/* RTCBKPжʹ(RTCBKP mode Error Interrupt Enable)
1ʹRTCBKPж
0ֹRTCBKPж */
#define	PMU_IER_RTCEIE_Msk	(0x1U << PMU_IER_RTCEIE_Pos)
	/* ʹRTCBKPж */
	/* ֹRTCBKPж */

#define	PMU_ISR_SLPEIF_Pos	1	/* SLEEPжϱ־Ӳλд1 (Sleep Error Interrupt Flag, write 1 to clear)
1PMOD=2h2CPUִWFI/WFEָǰλSLEEPDEEPĴʱλ
0PMOD=2h2CPUȷSLEEP */
#define	PMU_ISR_SLPEIF_Msk	(0x1U << PMU_ISR_SLPEIF_Pos)

#define	PMU_ISR_RTCEIF_Pos	0	/* RTCBKPжϱ־Ӳλд1(RTC Error Interrupt Flag, write 1 to clear)
1PMOD=2h3δдCPUڲĴSLEEPDEEP=1ȻִWFI/WFEָϵͳʱUSB PHYͼRTCBKPģʽ
0PMOD=2h3CPUȷDEEP SLEEP */
#define	PMU_ISR_RTCEIF_Msk	(0x1U << PMU_ISR_RTCEIF_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void PMU_Deinit(void);

/* LDO15ʹܱ־λ
1LDO15ڹ״̬
0LDO15ر غ */
extern FlagStatus PMU_CR_LDO15EN_Chk(void);

/* LDO15ʹܱ־Уλ غ */
extern FlagStatus PMU_CR_LDO15EN_B_Chk(void);

/* Sleep/DeepSleepѺϵͳƵ
00RCHF-8MHz
01RCHF-16MHz
10RCHF-24MHz
11RCHF-32MHz غ */
extern void PMU_CR_WKFSEL_Set(uint32_t SetValue);
extern uint32_t PMU_CR_WKFSEL_Get(void);

/* DeepSleepƼĴ
1DeepSleepģʽʹܣ¹رջ׼ѹԴ
0Sleepģʽ
Sleep£λSLPDPλΪDeepSleepģʽ
λSleepЧ غ */
extern void PMU_CR_SLPDP_Set(uint32_t SetValue);
extern uint32_t PMU_CR_SLPDP_Get(void);

/* CoreVoltageScaling
0͹ģʽ²ʹں˵ѹ
1͹ģʽ½ں˵ѹ
λSleep/DeepSleep/RTCBKPģʽ غ */
extern void PMU_CR_CVS_Set(uint32_t SetValue);
extern uint32_t PMU_CR_CVS_Get(void);

/* ͹ģʽüĴ
00Active mode / LP Active mode
01LPRUN mode
10Sleep mode / DeepSleep mode
11RTCBKP mode غ */
extern void PMU_CR_PMOD_Set(uint32_t SetValue);
extern uint32_t PMU_CR_PMOD_Get(void);

/* Flash Stopѿ
0Stopźŵȴʱӽͬ
1Stopź첽 غ */
extern void PMU_WKTR_STPCLR_Set(uint32_t SetValue);
extern uint32_t PMU_WKTR_STPCLR_Get(void);

/* ɱ̶⻽ӳ
DeepSleepģʽ£RCHFʱӵ󣬸ݴ˼ĴõȴӳʱٶȡFlashУ
000us
012us
104us
118us غ */
extern void PMU_WKTR_T1A_Set(uint32_t SetValue);
extern uint32_t PMU_WKTR_T1A_Get(void);

/* ADCжϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_ADCWKF_Chk(void);

/* RTCжϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_RTCWKF_Chk(void);

/* SVDжϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_SVDWKF_Chk(void);

/* 32768Hzͣжϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_LFDETWKF_Chk(void);

/* IOжϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_IOWKF_Chk(void);

/* LPUART1жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_LPU1WKF_Chk(void);

/* LPUART0жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_LPU0WKF_Chk(void);

/* Ƚout-of-windowѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_COMP_OOWF_Chk(void);

/* Ƚwindowѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_COMP_WINF_Chk(void);

/* Ƚ2жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_COMP2WKF_Chk(void);

/* Ƚ1жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_COMP1WKF_Chk(void);

/*  _32жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_LPTWKF_Chk(void);

/* BSTIM32жϻѱ־жϳʱӲԶ غ */
extern FlagStatus PMU_WKFR_BSTWKF_Chk(void);

/* CPU Debuggerѱ־д1 غ */
extern void PMU_WKFR_DBGWKF_Clr(void);
extern FlagStatus PMU_WKFR_DBGWKF_Chk(void);


/* SLEEPжʹ (Sleep mode Error Interrupt Enable)
1ʹSLEEPж
0ֹSLEEPж غ */
extern void PMU_IER_SLPEIE_Setable(FunState NewState);
extern FunState PMU_IER_SLPEIE_Getable(void);

/* RTCBKPжʹ(RTCBKP mode Error Interrupt Enable)
1ʹRTCBKPж
0ֹRTCBKPж غ */
extern void PMU_IER_RTCEIE_Setable(FunState NewState);
extern FunState PMU_IER_RTCEIE_Getable(void);

/* SLEEPжϱ־Ӳλд1 (Sleep Error Interrupt Flag, write 1 to clear)
1PMOD=2h2CPUִWFI/WFEָǰλSLEEPDEEPĴʱλ
0PMOD=2h2CPUȷSLEEP غ */
extern void PMU_ISR_SLPEIF_Clr(void);
extern FlagStatus PMU_ISR_SLPEIF_Chk(void);

/* RTCBKPжϱ־Ӳλд1(RTC Error Interrupt Flag, write 1 to clear)
1PMOD=2h3δдCPUڲĴSLEEPDEEP=1ȻִWFI/WFEָϵͳʱUSB PHYͼRTCBKPģʽ
0PMOD=2h3CPUȷDEEP SLEEP غ */
extern void PMU_ISR_RTCEIF_Clr(void);
extern FlagStatus PMU_ISR_RTCEIF_Chk(void);

extern void PMU_WKFR_WKPxF_Clr(uint32_t NWKPinDef);
extern FlagStatus PMU_WKFR_WKPxF_Chk(uint32_t NWKPinDef);
extern void PMU_SleepCfg_Init(PMU_SleepCfg_InitTypeDef* SleepCfg_InitStruct);

//Announce_End
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_PMU_H */
