/**
  ******************************************************************************
  * @file    fm33a0xxev_qspi.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the QSPI firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_QSPI_H
#define __FM33A0XXEV_QSPI_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 

typedef struct
{
    uint32_t Instruction;
    uint32_t Address;
    uint32_t AlternateByte;
    
    uint32_t AddressSize;
    uint32_t AlternateByteSize;
    uint32_t DummyCycles;
    
    uint32_t InstructionMode;
    uint32_t AddressMode;
    uint32_t AlternateByteMode;
    uint32_t DataMode;
}QSPI_CmdTypeDef;
     
#define	QSPI_CR_PRESCALER_Pos	24
#define	QSPI_CR_PRESCALER_Msk	(0xffU << QSPI_CR_PRESCALER_Pos)

#define	QSPI_CR_PMM_Pos	23	/* ѯƥģʽ
0ANDģʽbitƥλSMF
1ORģʽ1bitƥͻλSMF */
#define	QSPI_CR_PMM_Msk	(0x1U << QSPI_CR_PMM_Pos)
#define	QSPI_CR_PMM_AND	(0x0U << QSPI_CR_PMM_Pos)	/* ANDģʽbitƥλSMF */
#define	QSPI_CR_PMM_OR	(0x1U << QSPI_CR_PMM_Pos)	/* ORģʽ1bitƥͻλSMF */

#define	QSPI_CR_TOIE_Pos	20	/* ʱжʹ
0ֹʱж
1ʱж */
#define	QSPI_CR_TOIE_Msk	(0x1U << QSPI_CR_TOIE_Pos)
	/* ֹʱж */
	/* ʱж */

#define	QSPI_CR_SMIE_Pos	19	/* ״̬ƥжʹ
0ֹ״̬ƥж
1״̬ƥж */
#define	QSPI_CR_SMIE_Msk	(0x1U << QSPI_CR_SMIE_Pos)
	/* ֹ״̬ƥж */
	/* ״̬ƥж */

#define	QSPI_CR_FTIE_Pos	18	/* FIFOˮλжʹ
0ֹFIFOˮλж
1FIFOˮλж */
#define	QSPI_CR_FTIE_Msk	(0x1U << QSPI_CR_FTIE_Pos)
	/* ֹFIFOˮλж */
	/* FIFOˮλж */

#define	QSPI_CR_TCIE_Pos	17	/* жʹ
0ֹж
1ж */
#define	QSPI_CR_TCIE_Msk	(0x1U << QSPI_CR_TCIE_Pos)
	/* ֹж */
	/* ж */

#define	QSPI_CR_TEIE_Pos	16	/* жʹ
0ֹж
1ж */
#define	QSPI_CR_TEIE_Msk	(0x1U << QSPI_CR_TEIE_Pos)
	/* ֹж */
	/* ж */

#define	QSPI_CR_FIFOTHR_Pos	8	/* FIFOˮλĴģʽЧ
дģʽ
0FTFFIFOֽڴڵ1ʱλ
1FTFFIFOֽڴڵ2ʱλ

15FTFFIFOֽڵ16ʱλ
ģʽ
0FTFFIFOֽڵ1ʱλ
1FTFFIFOֽڵ2ʱλ

15FTFFIFOֽ16ʱλ */
#define	QSPI_CR_FIFOTHR_Msk	(0xfU << QSPI_CR_FIFOTHR_Pos)
#define	QSPI_CR_FIFOTHR_1	(0x0U << QSPI_CR_FIFOTHR_Pos)   /* FTFFIFOֽڴڵ1ʱλ */
#define	QSPI_CR_FIFOTHR_2	(0x1U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ2ʱλ */
#define	QSPI_CR_FIFOTHR_3	(0x2U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ3ʱλ */
#define	QSPI_CR_FIFOTHR_4	(0x3U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ4ʱλ */
#define	QSPI_CR_FIFOTHR_5	(0x4U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ5ʱλ */
#define	QSPI_CR_FIFOTHR_6	(0x5U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ6ʱλ */
#define	QSPI_CR_FIFOTHR_7	(0x6U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ7ʱλ */
#define	QSPI_CR_FIFOTHR_8	(0x7U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ8ʱλ */
#define	QSPI_CR_FIFOTHR_9	(0x8U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ9ʱλ */
#define	QSPI_CR_FIFOTHR_10	(0x9U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ10ʱλ */
#define	QSPI_CR_FIFOTHR_11	(0x10U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ11ʱλ */
#define	QSPI_CR_FIFOTHR_12	(0x11U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ12ʱλ */
#define	QSPI_CR_FIFOTHR_13	(0x12U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ13ʱλ */
#define	QSPI_CR_FIFOTHR_14	(0x13U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ14ʱλ */
#define	QSPI_CR_FIFOTHR_15	(0x14U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڴڵ15ʱλ */
#define	QSPI_CR_FIFOTHR_16	(0x15U << QSPI_CR_FIFOTHR_Pos)	/* FTFFIFOֽڵ16ʱλ */

#define	QSPI_CR_SSHFT_Pos	4	/* ӳٲʹ
0رӳٲ
1ʹӳٲ */
#define	QSPI_CR_SSHFT_Msk	(0x1U << QSPI_CR_SSHFT_Pos)
	/* رӳٲ */
	/* ʹӳٲ */

#define	QSPI_CR_TCEN_Pos	3	/* ߳ʱʹܣ˼Ĵڴ洢ӳģʽЧ
BUSYλQuadSPIQSPI洢ķʣʱĴʼTIMEOUTĴ塣QSPI߳ʱ޶nCSԶߣǿƽǰ̡
0رճʱ
1ʹܳʱ */
#define	QSPI_CR_TCEN_Msk	(0x1U << QSPI_CR_TCEN_Pos)
	/* رճʱ */
	/* ʹܳʱ */

#define	QSPI_CR_DMAEN_Pos	2	/* DMAʹ
0DMAܹرգQuadSPIᷢDMA
1DMAܿQuadSPIʱDMA */
#define	QSPI_CR_DMAEN_Msk	(0x1U << QSPI_CR_DMAEN_Pos)
	/* DMAܹرգQuadSPIᷢDMA */
	/* DMAܿQuadSPIʱDMA */

#define	QSPI_CR_ABORT_Pos	1	/* ǰֹĴд1ֹ䣬ӲnCSԶ */
#define	QSPI_CR_ABORT_Msk	(0x1U << QSPI_CR_ABORT_Pos)

#define	QSPI_CR_EN_Pos	0	/* QuadSPIģʹ
0رQuadSPI
1ʹQuadSPI */
#define	QSPI_CR_EN_Msk	(0x1U << QSPI_CR_EN_Pos)
	/* رQuadSPI */
	/* ʹQuadSPI */

#define	QSPI_CFG_CSHT_Pos	8	/* nCSСߵƽʱ䣬֮֡nCS豣ָߵƽʱ䣬QSPI_CLKڼ
01 cycle
12 cycles

78 cycles */
#define	QSPI_CFG_CSHT_Msk	(0x7U << QSPI_CFG_CSHT_Pos)
#define	QSPI_CFG_CSHT_1CYCLE	(0x0U << QSPI_CFG_CSHT_Pos)	/* 1 cycle */
#define	QSPI_CFG_CSHT_2CYCLES	(0x1U << QSPI_CFG_CSHT_Pos)	/* 2 cycles */
#define	QSPI_CFG_CSHT_3CYCLES	(0x2U << QSPI_CFG_CSHT_Pos)	/* 3 cycles */
#define	QSPI_CFG_CSHT_4CYCLES	(0x3U << QSPI_CFG_CSHT_Pos)	/* 4 cycles */
#define	QSPI_CFG_CSHT_5CYCLES	(0x4U << QSPI_CFG_CSHT_Pos)	/* 5 cycles */
#define	QSPI_CFG_CSHT_6CYCLES	(0x5U << QSPI_CFG_CSHT_Pos)	/* 6 cycles */
#define	QSPI_CFG_CSHT_7CYCLES	(0x6U << QSPI_CFG_CSHT_Pos)	/* 7 cycles */
#define	QSPI_CFG_CSHT_8CYCLES	(0x7U << QSPI_CFG_CSHT_Pos)	/* 8 cycles */

#define	QSPI_CFG_CKMODE_Pos	0	/* SPI Clock ModeĴ
0mode 0
1mode 3 */
#define	QSPI_CFG_CKMODE_Msk	(0x1U << QSPI_CFG_CKMODE_Pos)
#define	QSPI_CFG_CKMODE_MODE0	(0x0U << QSPI_CFG_CKMODE_Pos)	/* mode 0 */
#define	QSPI_CFG_CKMODE_MODE3	(0x1U << QSPI_CFG_CKMODE_Pos)	/* mode 3 */

#define	QSPI_SR_FIFOLVL_Pos	8	/* FIFOˮλ־
˼ĴʾǰFIFOбֽ0ʾFIFOգ16ʾFIFO
Զѯģʽ´˼Ĵ0 */
#define	QSPI_SR_FIFOLVL_Msk	(0x1fU << QSPI_SR_FIFOLVL_Pos)

#define	QSPI_SR_BUSY_Pos	5	/* 1ʾQuadSPIУͨŽԶ */
#define	QSPI_SR_BUSY_Msk	(0x1U << QSPI_SR_BUSY_Pos)

#define	QSPI_SR_TOF_Pos	4	/* ʱ־Ӳλд1 */
#define	QSPI_SR_TOF_Msk	(0x1U << QSPI_SR_TOF_Pos)

#define	QSPI_SR_SMF_Pos	3	/* Զѯģʽ±״̬ĴƥɹӲλд1 */
#define	QSPI_SR_SMF_Msk	(0x1U << QSPI_SR_SMF_Pos)

#define	QSPI_SR_FTF_Pos	2	/* FIFO threshold־FIFOˮλ趨ֵʱԶλֵʱԶ
Զѯģʽ£ÿζһ״ֵ̬󶼻ԶλFTFȡQSPI_DATAĴFTF */
#define	QSPI_SR_FTF_Msk	(0x1U << QSPI_SR_FTF_Pos)

#define	QSPI_SR_TCF_Pos	1	/* ɱ־Ӳλд1 */
#define	QSPI_SR_TCF_Msk	(0x1U << QSPI_SR_TCF_Pos)

#define	QSPI_DATALEN_QSPI_DATALEN_Pos	0	/* ݳΪDATALEN+1bytes */
#define	QSPI_DATALEN_QSPI_DATALEN_Msk	(0xffffffffU << QSPI_DATALEN_QSPI_DATALEN_Pos)

#define	QSPI_CCR_CRM_Pos	28	/* Continuous Read Mode
0ÿͨŷʱҪָ
1ֻڵһͨʱָ */
#define	QSPI_CCR_CRM_Msk	(0x1U << QSPI_CCR_CRM_Pos)
#define	QSPI_CCR_CRM_ALWAYS	(0x0U << QSPI_CCR_CRM_Pos)	/* ÿͨŷʱҪָ */
#define	QSPI_CCR_CRM_ONLY	(0x1U << QSPI_CCR_CRM_Pos)	/* ֻڵһͨʱָ */

#define	QSPI_CCR_OPMODE_Pos	26	/* ģʽ
00дģʽ
01ģʽ
10Զѯģʽ
11洢ӳģʽ */
#define	QSPI_CCR_OPMODE_Msk	    (0x3U << QSPI_CCR_OPMODE_Pos)
#define	QSPI_CCR_OPMODE_WRITE	(0x0U << QSPI_CCR_OPMODE_Pos)	/* дģʽ */
#define	QSPI_CCR_OPMODE_READ	(0x1U << QSPI_CCR_OPMODE_Pos)	/* ģʽ */
#define	QSPI_CCR_OPMODE_QUERY	(0x2U << QSPI_CCR_OPMODE_Pos)	/* Զѯģʽ */
#define	QSPI_CCR_OPMODE_MAP	    (0x3U << QSPI_CCR_OPMODE_Pos)	/* 洢ӳģʽ */

#define	QSPI_CCR_DMODE_Pos	24	/* ͨģʽdata phase
00
01
10˫
11 */
#define	QSPI_CCR_DMODE_Msk	    (0x3U << QSPI_CCR_DMODE_Pos)
#define	QSPI_CCR_DMODE_NONE	    (0x0U << QSPI_CCR_DMODE_Pos)	/*  */
#define	QSPI_CCR_DMODE_SINGLE	(0x1U << QSPI_CCR_DMODE_Pos)	/*  */
#define	QSPI_CCR_DMODE_DOUBLE	(0x2U << QSPI_CCR_DMODE_Pos)	/* ˫ */
#define	QSPI_CCR_DMODE_FOUR	    (0x3U << QSPI_CCR_DMODE_Pos)	/*  */

#define	QSPI_CCR_DUMCYC_Pos	18	/* Dummy cycleãQSPI_CLKڼ㣩0~31 */
#define	QSPI_CCR_DUMCYC_Msk	(0x1fU << QSPI_CCR_DUMCYC_Pos)

#define	QSPI_CCR_ABSIZE_Pos	16	/* Alternate bytes
008bits alternate bytes
0116bits alternate bytes
1024bits alternate bytes
1132bits alternate bytes */
#define	QSPI_CCR_ABSIZE_Msk	    (0x3U << QSPI_CCR_ABSIZE_Pos)
#define	QSPI_CCR_ABSIZE_8BITS	(0x0U << QSPI_CCR_ABSIZE_Pos)	/* 8bits alternate bytes */
#define	QSPI_CCR_ABSIZE_16BITS	(0x1U << QSPI_CCR_ABSIZE_Pos)	/* 16bits alternate bytes */
#define	QSPI_CCR_ABSIZE_24BITS	(0x2U << QSPI_CCR_ABSIZE_Pos)	/* 24bits alternate bytes */
#define	QSPI_CCR_ABSIZE_32BITS	(0x3U << QSPI_CCR_ABSIZE_Pos)	/* 32bits alternate bytes */

#define	QSPI_CCR_ABMODE_Pos	14	/* Alternate bytesģʽ
00alternate bytes
01
10˫
11 */
#define	QSPI_CCR_ABMODE_Msk	    (0x3U << QSPI_CCR_ABMODE_Pos)
#define	QSPI_CCR_ABMODE_NONE	(0x0U << QSPI_CCR_ABMODE_Pos)	/* alternate bytes */
#define	QSPI_CCR_ABMODE_SINGLE	(0x1U << QSPI_CCR_ABMODE_Pos)	/*  */
#define	QSPI_CCR_ABMODE_DOUBLE	(0x2U << QSPI_CCR_ABMODE_Pos)	/* ˫ */
#define	QSPI_CCR_ABMODE_FOUR	(0x3U << QSPI_CCR_ABMODE_Pos)	/*  */

#define	QSPI_CCR_ADSIZE_Pos	12	/* ַֽڳ
008bitsַ
0116bitsַ
1024bitsַ
1132bitsַ */
#define	QSPI_CCR_ADSIZE_Msk	    (0x3U << QSPI_CCR_ADSIZE_Pos)
#define	QSPI_CCR_ADSIZE_8BITS	(0x0U << QSPI_CCR_ADSIZE_Pos)	/* 8bitsַ */
#define	QSPI_CCR_ADSIZE_16BITS	(0x1U << QSPI_CCR_ADSIZE_Pos)	/* 16bitsַ */
#define	QSPI_CCR_ADSIZE_24BITS	(0x2U << QSPI_CCR_ADSIZE_Pos)	/* 24bitsַ */
#define	QSPI_CCR_ADSIZE_32BITS	(0x3U << QSPI_CCR_ADSIZE_Pos)	/* 32bitsַ */

#define	QSPI_CCR_ADMODE_Pos	10	/* ַֽڷģʽ
00޵ַֽ
01
10˫
11 */
#define	QSPI_CCR_ADMODE_Msk	    (0x3U << QSPI_CCR_ADMODE_Pos)
#define	QSPI_CCR_ADMODE_NONE	(0x0U << QSPI_CCR_ADMODE_Pos)	/* ޵ַֽ */
#define	QSPI_CCR_ADMODE_SINGLE	(0x1U << QSPI_CCR_ADMODE_Pos)	/*  */
#define	QSPI_CCR_ADMODE_DOUBLE	(0x2U << QSPI_CCR_ADMODE_Pos)	/* ˫ */
#define	QSPI_CCR_ADMODE_FOUR	(0x3U << QSPI_CCR_ADMODE_Pos)	/*  */

#define	QSPI_CCR_IMODE_Pos	8	/* ָģʽ
00ָֽ
01
10˫
11 */
#define	QSPI_CCR_IMODE_Msk	    (0x3U << QSPI_CCR_IMODE_Pos)
#define	QSPI_CCR_IMODE_NONE	    (0x0U << QSPI_CCR_IMODE_Pos)	/* ָֽ */
#define	QSPI_CCR_IMODE_SINGLE	(0x1U << QSPI_CCR_IMODE_Pos)	/*  */
#define	QSPI_CCR_IMODE_DOUBLE	(0x2U << QSPI_CCR_IMODE_Pos)	/* ˫ */
#define	QSPI_CCR_IMODE_FOUR	    (0x3U << QSPI_CCR_IMODE_Pos)	/*  */

#define	QSPI_CCR_INSTRUCTION_Pos	0	/* QuadSPI͵ָֽ */
#define	QSPI_CCR_INSTRUCTION_Msk	(0xffU << QSPI_CCR_INSTRUCTION_Pos)

#define	QSPI_ADDR_QSPI_ADDR_Pos	0	/* ͸QSPI洢ĵַڴ洢ӳģʽЧ */
#define	QSPI_ADDR_QSPI_ADDR_Msk	(0xffffffffU << QSPI_ADDR_QSPI_ADDR_Pos)

#define	QSPI_ABR_QSPI_ABR_Pos	0	/* ͸QSPI洢alternate bytes */
#define	QSPI_ABR_QSPI_ABR_Msk	(0xffffffffU << QSPI_ABR_QSPI_ABR_Pos)

#define	QSPI_DR_QSPI_DATA_Pos	0	/* QSPIݼĴ
ģʽдʱQSPI_DRĴдݽpush FIFOֽ֧ڡ֡д룬ֱFIFOѹ124ֽڣдֽFIFOпֽǰдֱFIFO㹻ռɵǰдݡ
ģʽʱȡQSPI_DRĴpop FIFOֽ֧ڡֶ֡ȡֱFIFO124ֽڣȡֽFIFOЧֽǰֱFIFO㹻ֽڿԱȡߴɣһֻ󼸸ʵЧֽڡ
QSPI_DRķʱ͵ַֽڷʱQSPI_DR[7:0]ַʱQSPI_DR[15:0] */
#define	QSPI_DR_QSPI_DATA_Msk	(0xffffffffU << QSPI_DR_QSPI_DATA_Pos)

#define	QSPI_SMSK_QSPI_SMSK_Pos	0	/* Զ״̬ѯģʽµ״̬maskĴӦbitд0Ӧ״̬λ */
#define	QSPI_SMSK_QSPI_SMSK_Msk	(0xffffffffU << QSPI_SMSK_QSPI_SMSK_Pos)

#define	QSPI_SMAT_QSPI_SMAT_Pos	0	/* Զ״̬ѯģʽµ״̬ƥĴ
Ƚ϶QSPI_DATA & QSPI_SMSK */
#define	QSPI_SMAT_QSPI_SMAT_Msk	(0xffffffffU << QSPI_SMAT_QSPI_SMAT_Pos)

#define	QSPI_PITV_QSPI_PITV_Pos	0	/* Զ״̬ѯģʽµѯpolling intervalΪQSPI_CLK */
#define	QSPI_PITV_QSPI_PITV_Msk	(0xffffU << QSPI_PITV_QSPI_PITV_Pos)

#define	QSPI_TO_QSPI_TO_Pos	0	/* ʱãΪQSPI_CLKڴ洢ӳģʽЧ
FIFO֮QSPIΪֹͣʱʼֵQSPI_TO趨ֵ֮nCS */
#define	QSPI_TO_QSPI_TO_Msk	(0xffffU << QSPI_TO_QSPI_TO_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void QSPI_Deinit(void);
extern void QSPI_CR_PRESCALER_Set(uint32_t SetValue);
extern uint32_t QSPI_CR_PRESCALER_Get(void);

/* ѯƥģʽ
0ANDģʽbitƥλSMF
1ORģʽ1bitƥͻλSMF غ */
extern void QSPI_CR_PMM_Set(uint32_t SetValue);
extern uint32_t QSPI_CR_PMM_Get(void);

/* ʱжʹ
0ֹʱж
1ʱж غ */
extern void QSPI_CR_TOIE_Setable(FunState NewState);
extern FunState QSPI_CR_TOIE_Getable(void);

/* ״̬ƥжʹ
0ֹ״̬ƥж
1״̬ƥж غ */
extern void QSPI_CR_SMIE_Setable(FunState NewState);
extern FunState QSPI_CR_SMIE_Getable(void);

/* FIFOˮλжʹ
0ֹFIFOˮλж
1FIFOˮλж غ */
extern void QSPI_CR_FTIE_Setable(FunState NewState);
extern FunState QSPI_CR_FTIE_Getable(void);

/* жʹ
0ֹж
1ж غ */
extern void QSPI_CR_TCIE_Setable(FunState NewState);
extern FunState QSPI_CR_TCIE_Getable(void);

/* жʹ
0ֹж
1ж غ */
extern void QSPI_CR_TEIE_Setable(FunState NewState);
extern FunState QSPI_CR_TEIE_Getable(void);

/* FIFOˮλĴģʽЧ
дģʽ
0FTFFIFOֽڴڵ1ʱλ
1FTFFIFOֽڴڵ2ʱλ

15FTFFIFOֽڵ16ʱλ
ģʽ
0FTFFIFOֽڵ1ʱλ
1FTFFIFOֽڵ2ʱλ

15FTFFIFOֽ16ʱλ غ */
extern void QSPI_CR_FIFOTHR_Set(uint32_t SetValue);
extern uint32_t QSPI_CR_FIFOTHR_Get(void);

/* ӳٲʹ
0رӳٲ
1ʹӳٲ غ */
extern void QSPI_CR_SSHFT_Setable(FunState NewState);
extern FunState QSPI_CR_SSHFT_Getable(void);

/* ߳ʱʹܣ˼Ĵڴ洢ӳģʽЧ
BUSYλQuadSPIQSPI洢ķʣʱĴʼTIMEOUTĴ塣QSPI߳ʱ޶nCSԶߣǿƽǰ̡
0رճʱ
1ʹܳʱ غ */
extern void QSPI_CR_TCEN_Setable(FunState NewState);
extern FunState QSPI_CR_TCEN_Getable(void);

/* DMAʹ
0DMAܹرգQuadSPIᷢDMA
1DMAܿQuadSPIʱDMA غ */
extern void QSPI_CR_DMAEN_Setable(FunState NewState);
extern FunState QSPI_CR_DMAEN_Getable(void);

/* ǰֹĴд1ֹ䣬ӲnCSԶ غ */
extern void QSPI_CR_ABORT_Setable(FunState NewState);
extern FunState QSPI_CR_ABORT_Getable(void);

/* QuadSPIģʹ
0رQuadSPI
1ʹQuadSPI غ */
extern void QSPI_CR_EN_Setable(FunState NewState);
extern FunState QSPI_CR_EN_Getable(void);

/* nCSСߵƽʱ䣬֮֡nCS豣ָߵƽʱ䣬QSPI_CLKڼ
01 cycle
12 cycles

78 cycles غ */
extern void QSPI_CFG_CSHT_Set(uint32_t SetValue);
extern uint32_t QSPI_CFG_CSHT_Get(void);

/* SPI Clock ModeĴ
0mode 0
1mode 3 غ */
extern void QSPI_CFG_CKMODE_Set(uint32_t SetValue);
extern uint32_t QSPI_CFG_CKMODE_Get(void);

/* FIFOˮλ־
˼ĴʾǰFIFOбֽ0ʾFIFOգ16ʾFIFO
Զѯģʽ´˼Ĵ0 غ */
extern uint32_t QSPI_SR_FIFOLVL_Get(void);

/* 1ʾQuadSPIУͨŽԶ غ */
extern FlagStatus QSPI_SR_BUSY_Chk(void);

/* ʱ־Ӳλд1 غ */
extern void QSPI_SR_TOF_Clr(void);
extern FlagStatus QSPI_SR_TOF_Chk(void);

/* Զѯģʽ±״̬ĴƥɹӲλд1 غ */
extern void QSPI_SR_SMF_Clr(void);
extern FlagStatus QSPI_SR_SMF_Chk(void);

/* FIFO threshold־FIFOˮλ趨ֵʱԶλֵʱԶ
Զѯģʽ£ÿζһ״ֵ̬󶼻ԶλFTFȡQSPI_DATAĴFTF غ */
extern FlagStatus QSPI_SR_FTF_Chk(void);

/* ɱ־Ӳλд1 غ */
extern void QSPI_SR_TCF_Clr(void);
extern FlagStatus QSPI_SR_TCF_Chk(void);

/* ݳΪDATALEN+1bytes غ */
extern void QSPI_DATALEN_Write(uint32_t SetValue);
extern uint32_t QSPI_DATALEN_Read(void);

/* QuadSPI ͨſƼĴ غ */
#define QSPI_CCR_DUMCYC_SET(value)          ((value << QSPI_CCR_DUMCYC_Pos) & QSPI_CCR_DUMCYC_Msk)
#define QSPI_CCR_DUMCYC_GET(value)          ((value >> QSPI_CCR_DUMCYC_Pos) & QSPI_CCR_DUMCYC_Msk)
#define QSPI_CCR_INSTRUCTION_SET(value)     ((value << QSPI_CCR_INSTRUCTION_Pos) & QSPI_CCR_INSTRUCTION_Msk)
#define QSPI_CCR_INSTRUCTION_GET(value)     ((value >> QSPI_CCR_INSTRUCTION_Pos) & QSPI_CCR_INSTRUCTION_Msk)

extern void QSPI_CCR_Write(uint32_t SetValue);
extern uint32_t QSPI_CCR_Read(void);

/* ͸QSPI洢ĵַڴ洢ӳģʽЧ غ */
extern void QSPI_ADDR_Write(uint32_t SetValue);
extern uint32_t QSPI_ADDR_Read(void);

/* ͸QSPI洢alternate bytes غ */
extern void QSPI_ABR_Write(uint32_t SetValue);
extern uint32_t QSPI_ABR_Read(void);

/* QSPIݼĴ
ģʽдʱQSPI_DRĴдݽpush FIFOֽ֧ڡ֡д룬ֱFIFOѹ124ֽڣдֽFIFOпֽǰдֱFIFO㹻ռɵǰдݡ
ģʽʱȡQSPI_DRĴpop FIFOֽ֧ڡֶ֡ȡֱFIFO124ֽڣȡֽFIFOЧֽǰֱFIFO㹻ֽڿԱȡߴɣһֻ󼸸ʵЧֽڡ
QSPI_DRķʱ͵ַֽڷʱQSPI_DR[7:0]ַʱQSPI_DR[15:0] غ */
extern void QSPI_DR_WriteByte(uint8_t SetValue);
extern uint8_t QSPI_DR_ReadByte(void);
extern void QSPI_DR_WriteHalfword(uint16_t SetValue);
extern uint16_t QSPI_DR_ReadHalfword(void);
extern void QSPI_DR_WriteWord(uint32_t SetValue);
extern uint32_t QSPI_DR_ReadWord(void);

/* Զ״̬ѯģʽµ״̬maskĴӦbitд0Ӧ״̬λ غ */
extern void QSPI_SMSK_Write(uint32_t SetValue);
extern uint32_t QSPI_SMSK_Read(void);

/* Զ״̬ѯģʽµ״̬ƥĴ
Ƚ϶QSPI_DATA & QSPI_SMSK غ */
extern void QSPI_SMAT_Write(uint32_t SetValue);
extern uint32_t QSPI_SMAT_Read(void);

/* Զ״̬ѯģʽµѯpolling intervalΪQSPI_CLK غ */
extern void QSPI_PITV_Write(uint32_t SetValue);
extern uint32_t QSPI_PITV_Read(void);

/* ʱãΪQSPI_CLKڴ洢ӳģʽЧ
FIFO֮QSPIΪֹͣʱʼֵQSPI_TO趨ֵ֮nCS غ */
extern void QSPI_TO_Write(uint32_t SetValue);
extern uint32_t QSPI_TO_Read(void);

//Announce_End
void QSPI_Deinit(void);
void QSPI_Init(void);


#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_QSPI_H */
