/**
  ******************************************************************************
  * @file    fm33a0xxev_svd.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the SVD firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_SVD_H
#define __FM33A0XXEV_SVD_H


#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
   
#include "FM33A0XXEV.h"

typedef struct
{
	uint32_t	SVDMOD;		/*!<SVDģʽѡ  */
	uint32_t	SVDITVL;	/*!<SVDЪʹܼ  */
	uint32_t	SVDLVL;		/*!<SVDֵ  */
	FunState	DFEN;		/*!<SVD˲SVDMODE=1ʱ1  */
	FunState	PFIE;		/*!<SVDԴж  */
	FunState	PRIE;		/*!<SVDԴָж  */
	FunState	SVDEN;		/*!<SVDʹ  */
	
}SVD_InitTypeDef;
 
#define	SVD_CFGR_LVL_Pos	4	/* SVDֵλãμ11.3.3ֵ
(SVD threshold level) */
#define	SVD_CFGR_LVL_Msk	(0xfU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_1P800V	(0x0U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_2P014V	(0x1U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_2P229V	(0x2U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_2P443V	(0x3U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_2P657V	(0x4U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_2P871V	(0x5U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_3P086V	(0x6U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_3P300V	(0x7U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_3P514V	(0x8U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_3P729V	(0x9U << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_3P943V	(0xaU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_4P157V	(0xbU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_4P371V	(0xcU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_4P586V	(0xdU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_4P800V	(0xeU << SVD_CFGR_LVL_Pos)
#define	SVD_CFGR_LVL_SVS	(0xfU << SVD_CFGR_LVL_Pos)

#define	SVD_CFGR_DFEN_Pos	3	/* ˲ʹܣMOD=1ʱ1
(Digital Filter Enable)
1SVD˲
0رSVD˲ */
#define	SVD_CFGR_DFEN_Msk	(0x1U << SVD_CFGR_DFEN_Pos)
	/* SVD˲ */
	/* رSVD˲ */

#define	SVD_CFGR_MOD_Pos	2	/* SVDģʽѡģʽҪλSVD_CR.ENŻSVD
(SVD Mode)
1Ъʹģʽ
0ʹģʽ
ע⣺Ъʹģʽ±뿪˲ */
#define	SVD_CFGR_MOD_Msk	(0x1U << SVD_CFGR_MOD_Pos)
#define	SVD_CFGR_MOD_INTERVAL	(0x1U << SVD_CFGR_MOD_Pos)	/* Ъʹģʽ */
#define	SVD_CFGR_MOD_ALWAYSON	(0x0U << SVD_CFGR_MOD_Pos)	/* ʹģʽ */

#define	SVD_CFGR_ITVL_Pos	0	/* SVDЪʹܼ (SVD interval enable period)
0062.5ms
01256ms
101s
114s */
#define	SVD_CFGR_ITVL_Msk	(0x3U << SVD_CFGR_ITVL_Pos)
#define	SVD_CFGR_ITVL_62P5MS	(0x0U << SVD_CFGR_ITVL_Pos)	/* 62.5ms */
#define	SVD_CFGR_ITVL_256MS	(0x1U << SVD_CFGR_ITVL_Pos)	/* 256ms */
#define	SVD_CFGR_ITVL_1S	(0x2U << SVD_CFGR_ITVL_Pos)	/* 1s */
#define	SVD_CFGR_ITVL_4S	(0x3U << SVD_CFGR_ITVL_Pos)	/* 4s */
	/* 62.5ms */
	/* 256ms */
	/* 1s */
	/* 4s */
#define	SVD_CR_TE_Pos	8	/* SVDʹܣд1 (SVD test enable) */
#define	SVD_CR_TE_Msk	(0x1U << SVD_CR_TE_Pos)

#define	SVD_CR_SVSEN_Pos	1	/* SVSⲿԴͨź 
(SVS external monitor channel enable)
0SVSͨر
1SVSͨʹ
EN=1ʱSVDLVLĴSVSǷ񾭹ڲѹLVL=1111SVS벻ѹLVL != 1111SVS뾭ڲѹ */
#define	SVD_CR_SVSEN_Msk	(0x1U << SVD_CR_SVSEN_Pos)
	/* SVSͨر */
	/* SVSͨʹ */

#define	SVD_CR_SVDEN_Pos	0
#define	SVD_CR_SVDEN_Msk	(0x1U << SVD_CR_SVDEN_Pos)

#define	SVD_IER_PFIE_Pos	1	/* ԴжʹܼĴ1Ч (Power Fall interrupt enable) */
#define	SVD_IER_PFIE_Msk	(0x1U << SVD_IER_PFIE_Pos)

#define	SVD_IER_PRIE_Pos	0	/* ԴָжʹܼĴ1Ч (Power Rise interrupt enable) */
#define	SVD_IER_PRIE_Msk	(0x1U << SVD_IER_PRIE_Pos)

#define	SVD_ISR_SVDO_Pos	8	/* SVDԴ (SVD output)
1ԴѹSVDǰֵ
0ԴѹSVDǰֵ */
#define	SVD_ISR_SVDO_Msk	(0x1U << SVD_ISR_SVDO_Pos)

#define	SVD_ISR_SVDR_Pos	7	/* SVDźţֵ·SVD״̬
(SVD registered output) */
#define	SVD_ISR_SVDR_Msk	(0x1U << SVD_ISR_SVDR_Pos)

#define	SVD_ISR_PFF_Pos	1	/* Դжϱ־ĴԴѹ䵽SVDֵ֮ʱλд1 (Power fall flag) */
#define	SVD_ISR_PFF_Msk	(0x1U << SVD_ISR_PFF_Pos)

#define	SVD_ISR_PRF_Pos	0	/* Դָжϱ־ĴԴѹSVDֵ֮ʱλд1 (Power rise flag) */
#define	SVD_ISR_PRF_Msk	(0x1U << SVD_ISR_PRF_Pos)

#define	SVD_VSR_V1P0EN_Pos	2	/* 1.0V׼ʹź (1.0V reference enable)
1ʹ1.0V׼
0ر1.0V׼ */
#define	SVD_VSR_V1P0EN_Msk	(0x1U << SVD_VSR_V1P0EN_Pos)

#define	SVD_VSR_V0P95EN_Pos	1	/* 0.95V׼ʹź (0.95V reference enable)
1ʹ0.95V׼
0ر0.95V׼ */
#define	SVD_VSR_V0P95EN_Msk	(0x1U << SVD_VSR_V0P95EN_Pos)

#define	SVD_VSR_V0P9EN_Pos	0	/* 0.9V׼ʹź (0.9V reference enable)
1ʹ0.9V׼
0ر0.9V׼ */
#define	SVD_VSR_V0P9EN_Msk	(0x1U << SVD_VSR_V0P9EN_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void SVD_Deinit(void);

/* SVDֵλãμ11.3.3ֵ
(SVD threshold level) غ */
extern void SVD_CFGR_LVL_Set(uint32_t SetValue);
extern uint32_t SVD_CFGR_LVL_Get(void);

/* ˲ʹܣMOD=1ʱ1
(Digital Filter Enable)
1SVD˲
0رSVD˲ غ */
extern void SVD_CFGR_DFEN_Setable(FunState NewState);
extern FunState SVD_CFGR_DFEN_Getable(void);

/* SVDģʽѡģʽҪλSVD_CR.ENŻSVD
(SVD Mode)
1Ъʹģʽ
0ʹģʽ
ע⣺Ъʹģʽ±뿪˲ غ */
extern void SVD_CFGR_MOD_Set(uint32_t SetValue);
extern uint32_t SVD_CFGR_MOD_Get(void);

/* SVDЪʹܼ (SVD interval enable period)
0062.5ms
01256ms
101s
114s غ */
extern void SVD_CFGR_ITVL_Set(uint32_t SetValue);
extern uint32_t SVD_CFGR_ITVL_Get(void);

/* SVSⲿԴͨź 
(SVS external monitor channel enable)
0SVSͨر
1SVSͨʹ
EN=1ʱSVDLVLĴSVSǷ񾭹ڲѹLVL=1111SVS벻ѹLVL != 1111SVS뾭ڲѹ غ */
extern void SVD_CR_SVSEN_Setable(FunState NewState);
extern FunState SVD_CR_SVSEN_Getable(void);
extern void SVD_CR_SVDEN_Setable(FunState NewState);
extern FunState SVD_CR_SVDEN_Getable(void);

/* ԴжʹܼĴ1Ч (Power Fall interrupt enable) غ */
extern void SVD_IER_PFIE_Setable(FunState NewState);
extern FunState SVD_IER_PFIE_Getable(void);

/* ԴָжʹܼĴ1Ч (Power Rise interrupt enable) غ */
extern void SVD_IER_PRIE_Setable(FunState NewState);
extern FunState SVD_IER_PRIE_Getable(void);

/* SVDԴ (SVD output)
1ԴѹSVDǰֵ
0ԴѹSVDǰֵ غ */
extern FlagStatus SVD_ISR_SVDO_Chk(void);

/* SVDźţֵ·SVD״̬
(SVD registered output) غ */
extern FlagStatus SVD_ISR_SVDR_Chk(void);

/* Դжϱ־ĴԴѹ䵽SVDֵ֮ʱλд1 (Power fall flag) غ */
extern void SVD_ISR_PFF_Clr(void);
extern FlagStatus SVD_ISR_PFF_Chk(void);

/* Դָжϱ־ĴԴѹSVDֵ֮ʱλд1 (Power rise flag) غ */
extern void SVD_ISR_PRF_Clr(void);
extern FlagStatus SVD_ISR_PRF_Chk(void);

/* 1.0V׼ʹź (1.0V reference enable)
1ʹ1.0V׼
0ر1.0V׼ غ */
extern void SVD_VSR_V1P0EN_Setable(FunState NewState);
extern FunState SVD_VSR_V1P0EN_Getable(void);

/* 0.95V׼ʹź (0.95V reference enable)
1ʹ0.95V׼
0ر0.95V׼ غ */
extern void SVD_VSR_V0P95EN_Setable(FunState NewState);
extern FunState SVD_VSR_V0P95EN_Getable(void);

/* 0.9V׼ʹź (0.9V reference enable)
1ʹ0.9V׼
0ر0.9V׼ غ */
extern void SVD_VSR_V0P9EN_Setable(FunState NewState);
extern FunState SVD_VSR_V0P9EN_Getable(void);

extern void SVD_Init(SVD_InitTypeDef* para);
//Announce_End
#ifdef __cplusplus
}
#endif

#endif /* __FM33A0XXEV_SVD_H */
