/**
  ******************************************************************************
  * @file    fm33a0xxev_adc.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_adc.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */ 

/** @defgroup ADC 
  * @brief ADC driver modules
  * @{
  */ 

/* ADCڲۼģʽжʹ غ */
void ADC_CR_ADC_IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		ADC->CR |= (ADC_CR_ADC_IE_Msk);
	}
	else
	{
		ADC->CR &= ~(ADC_CR_ADC_IE_Msk);
	}
}

FunState ADC_CR_ADC_IE_Getable(void)
{
	if (ADC->CR & (ADC_CR_ADC_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ADCⲿۼģʽжʹ غ */
void ADC_CR_ACC_IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		ADC->CR |= (ADC_CR_ACC_IE_Msk);
	}
	else
	{
		ADC->CR &= ~(ADC_CR_ACC_IE_Msk);
	}
}

FunState ADC_CR_ACC_IE_Getable(void)
{
	if (ADC->CR & (ADC_CR_ACC_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ADCģʽ
0͹ģʽʱ1MHz
1߹ģʽʱ2MHz غ */
void ADC_CR_HPEN_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = ADC->CR;
	tmpreg &= ~(ADC_CR_HPEN_Msk);
	tmpreg |= (SetValue & ADC_CR_HPEN_Msk);
	ADC->CR = tmpreg;
}

uint32_t ADC_CR_HPEN_Get(void)
{
	return (ADC->CR & ADC_CR_HPEN_Msk);
}

/* ADCģʽ
0ڲۼģʽ
1ⲿۼCICģʽ غ */
void ADC_CR_MODE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = ADC->CR;
	tmpreg &= ~(ADC_CR_MODE_Msk);
	tmpreg |= (SetValue & ADC_CR_MODE_Msk);
	ADC->CR = tmpreg;
}

uint32_t ADC_CR_MODE_Get(void)
{
	return (ADC->CR & ADC_CR_MODE_Msk);
}

/* λʹܣMODE=1ʹⲿۼʱλ±뱣Ϊ0
0ֹⲿλ
1ⲿλ غ */
void ADC_CR_RSTCTRL_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		ADC->CR |= (ADC_CR_RSTCTRL_EN_Msk);
	}
	else
	{
		ADC->CR &= ~(ADC_CR_RSTCTRL_EN_Msk);
	}
}

FunState ADC_CR_RSTCTRL_EN_Getable(void)
{
	if (ADC->CR & (ADC_CR_RSTCTRL_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ADCʹź
0ADCʹ
1ADCʹ

ע⣺RTCִԶ²ʱӲԶʹADC غ */
void ADC_CR_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		ADC->CR |= (ADC_CR_EN_Msk);
	}
	else
	{
		ADC->CR &= ~(ADC_CR_EN_Msk);
	}
}

FunState ADC_CR_EN_Getable(void)
{
	if (ADC->CR & (ADC_CR_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ADC TRIMֵڲۼģʽ
ڲۼۼڣPeriod = TRIM*2*TADC_CLK
ۼھλTRIM=0x7FFֵʱʵۼΪ4095 cycleADCЧλ12bit غ */
void ADC_TRIM_Write(uint32_t SetValue)
{
	ADC->TRIM  = (SetValue & ADC_TRIM_ADC_TRIM_Msk);
}

uint32_t ADC_TRIM_Read(void)
{
	return (ADC->TRIM  & ADC_TRIM_ADC_TRIM_Msk);
}

/* ADCݣΪۼӽδ˲
MODE=0ʱΪADCڲۼӽ12bitλADC_TRIM
MODE=1ʱλADCⲿۼӽ14bitλACC_PERIOD غ */
uint32_t ADC_DR_Read(void)
{
	return (ADC->DR & ADC_DR_ADC_DATA_Msk);
}

/* ADCʼɱ־MODE=1ʱЧжϣ
0ADCδɳʼ
1ADCɳʼ

עMODE=1ADC_ENλ󣬵ȴMODE_CTRL_DELAYʱ֮󣬴˱־λʾADCڲɡMODE=0ʱʱADCڲпƣ˱־Ч غ */
FlagStatus ADC_ISR_INIT_RDY_Chk(void)
{
	if (ADC->ISR  & ADC_ISR_INIT_RDY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ۼжϱ־Ӳλд1㣬д0Ч غ */
void ADC_ISR_ACC_IF_Clr(void)
{
	ADC->ISR  = ADC_ISR_ACC_IF_Msk;
}

FlagStatus ADC_ISR_ACC_IF_Chk(void)
{
	if (ADC->ISR  & ADC_ISR_ACC_IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ADCתֻжϣ
תɺźűΪߵƽֻйرADCŻ0 غ */
FlagStatus ADC_ISR_ADC_DONE_Chk(void)
{
	if (ADC->ISR  & ADC_ISR_ADC_DONE_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ADCתжϱ־Ӳλд1㣬д0Ч غ */
void ADC_ISR_ADC_IF_Clr(void)
{
	ADC->ISR  = ADC_ISR_ADC_IF_Msk;
}

FlagStatus ADC_ISR_ADC_IF_Chk(void)
{
	if (ADC->ISR  & ADC_ISR_ADC_IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ⲿۼۼãλADC_CLK
0001023Ӧ10bit
0012047Ӧ11bit
0104095Ӧ12bit
0118191Ӧ13bit
10016383Ӧ14bit
Others4095 غ */
void ADC_CFGR_ACC_PERIOD_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = ADC->CFGR;
	tmpreg &= ~(ADC_CFGR_ACC_PERIOD_Msk);
	tmpreg |= (SetValue & ADC_CFGR_ACC_PERIOD_Msk);
	ADC->CFGR = tmpreg;
}

uint32_t ADC_CFGR_ACC_PERIOD_Get(void)
{
	return (ADC->CFGR & ADC_CFGR_ACC_PERIOD_Msk);
}

/* SDADCʹܺmode_ctrlӳٳãλADC_CLK
0x00ӳ
0xFFӳ255ADC_CLK

ע⣺λֵΪ16ʱڣ޸ΪС16ֵ غ */
void ADC_CFGR_RST_CTRL_DELAY_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = ADC->CFGR;
	tmpreg &= ~(ADC_CFGR_RST_CTRL_DELAY_Msk);
	tmpreg |= (SetValue & ADC_CFGR_RST_CTRL_DELAY_Msk);
	ADC->CFGR = tmpreg;
}

uint32_t ADC_CFGR_RST_CTRL_DELAY_Get(void)
{
	return (ADC->CFGR & ADC_CFGR_RST_CTRL_DELAY_Msk);
}

/* ADCͨBufferʹ غ */
void ADC_CFGR_BUFEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		ADC->CFGR |= (ADC_CFGR_BUFEN_Msk);
	}
	else
	{
		ADC->CFGR &= ~(ADC_CFGR_BUFEN_Msk);
	}
}

FunState ADC_CFGR_BUFEN_Getable(void)
{
	if (ADC->CFGR & (ADC_CFGR_BUFEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ADCͨѡ
0001ADC_IN1
0010ADC_IN2
0011ADC_IN3
0100ADC_IN4
0101ADC_IN5
0110ADC_IN6
0111ADC_IN7
1000ADC_IN8
1001ADC_IN9
1010ADC_IN10
1011ADC_IN11
1100ADC_IN12
1110VBAT
1111TS غ */
void ADC_CFGR_BUFSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = ADC->CFGR;
	tmpreg &= ~(ADC_CFGR_BUFSEL_Msk);
	tmpreg |= (SetValue & ADC_CFGR_BUFSEL_Msk);
	ADC->CFGR = tmpreg;
}

uint32_t ADC_CFGR_BUFSEL_Get(void)
{
	return (ADC->CFGR & ADC_CFGR_BUFSEL_Msk);
}

/* CIC˲ݣƵADCʱӺOSRͬעзʽΪƲ롣غ */
uint32_t CIC_DR_Read(void)
{
	return (CIC->DR & CIC_DR_ADC_CIC_DATA_Msk);
}

/* CIC˲ offsetƼĴдoffsetֵӲCIC_DROSԵõ޷Ͻغ */
void CIC_OS_Write(uint32_t SetValue)
{
	CIC ->OS = (SetValue & CIC_OS_ADC_CIC_OS_Msk);
}

uint32_t CIC_OS_Read(void)
{
	return (CIC ->OS & CIC_OS_ADC_CIC_OS_Msk);
}

/* CIC_DR + CIC_OSõ޷ غ */
uint32_t CIC_USDR_Read(void)
{
	return (CIC ->USDR  & CIC_USDR_ADC_CIC_USDR_Msk);
}

/* CIC˲ʹ (CIC enable)
0:رCIC
1:ʹCIC غ */
void CIC_CR_CIC_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		CIC ->CR |= (CIC_CR_CIC_EN_Msk);
	}
	else
	{
		CIC ->CR &= ~(CIC_CR_CIC_EN_Msk);
	}
}

FunState CIC_CR_CIC_EN_Getable(void)
{
	if (CIC ->CR & (CIC_CR_CIC_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* CICʹܺ (Number of Samples to be Discarded)
ԶADCʹܺͷ0~255
غ */
void CIC_CR_NS_DISC_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CIC->CR;
	tmpreg &= ~(CIC_CR_NS_DISC_Msk);
	tmpreg |= (SetValue & CIC_CR_NS_DISC_Msk);
	CIC ->CR = tmpreg;
}

uint32_t CIC_CR_NS_DISC_Get(void)
{
	return (CIC ->CR & CIC_CR_NS_DISC_Msk);
}

/* CIC Overrun Interrupt enable
0:ֹCICж
1:CICж غ */
void CIC_CR_OVR_IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		CIC ->CR |= (CIC_CR_OVR_IE_Msk);
	}
	else
	{
		CIC ->CR &= ~(CIC_CR_OVR_IE_Msk);
	}
}

FunState CIC_CR_OVR_IE_Getable(void)
{
	if (CIC ->CR & (CIC_CR_OVR_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* CICжʹ(CIC interrupt enable)
0:ֹCICж
1:CICж,CIC_IFλʱж¼ غ */
void CIC_CR_CIC_IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		CIC ->CR |= (CIC_CR_CIC_IE_Msk);
	}
	else
	{
		CIC ->CR &= ~(CIC_CR_CIC_IE_Msk);
	}
}

FunState CIC_CR_CIC_IE_Getable(void)
{
	if (CIC ->CR & (CIC_CR_CIC_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ݽȡ (Truncate),սжλλ;
غ */
void CIC_CR_TRUNC_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CIC ->CR;
	tmpreg &= ~(CIC_CR_TRUNC_Msk);
	tmpreg |= (SetValue & CIC_CR_TRUNC_Msk);
	CIC ->CR = tmpreg;
}

uint32_t CIC_CR_TRUNC_Get(void)
{
	return (CIC ->CR & CIC_CR_TRUNC_Msk);
}

/*  (Over Sampling Rate)
000:x8
001:x16
010:x32
011:x64
100:x128
101:x256
110:x512
111:x1024 غ */
void CIC_CR_OSR_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CIC ->CR;
	tmpreg &= ~(CIC_CR_OSR_Msk);
	tmpreg |= (SetValue & CIC_CR_OSR_Msk);
	CIC ->CR = tmpreg;
}

uint32_t CIC_CR_OSR_Get(void)
{
	return (CIC ->CR & CIC_CR_OSR_Msk);
}

void CIC_ISR_CIC_OVR_Clr(void)
{
	CIC ->ISR = CIC_ISR_CIC_OVR_Msk;
}

FlagStatus CIC_ISR_CIC_OVR_Chk(void)
{
	if (CIC ->ISR & CIC_ISR_CIC_OVR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void CIC_ISR_CIC_IF_Clr(void)
{
	CIC ->ISR = CIC_ISR_CIC_IF_Msk;
}

FlagStatus CIC_ISR_CIC_IF_Chk(void)
{
	if (CIC ->ISR & CIC_ISR_CIC_IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void ADC_Deinit(void)
{
	//ADC->CR = 0x00000000;
	//ADC->TRIM  = 0x00000000;
	//ADC->DR = 0x00000000;
	//ADC->ISR  = 0x00000000;
	//ADC->CFGR = 0x00000000;
}

/******END OF FILE****/
