/**
  ******************************************************************************
  * @file    fm33a0xxev_bstim.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_bstim.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup BSTIM 
  * @brief BSTIM driver modules
  * @{
  */ 
  
/* Auto-reloadԤװʹ (Auto-Reload Preload Enable)
0ARRĴʹpreload
1ARRĴʹpreload غ */
void BSTIM_CR1_ARPE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		BSTIM->CR1 |= (BSTIM_CR1_ARPE_Msk);
	}
	else
	{
		BSTIM->CR1 &= ~(BSTIM_CR1_ARPE_Msk);
	}
}

FunState BSTIM_CR1_ARPE_Getable(void)
{
	if (BSTIM->CR1 & (BSTIM_CR1_ARPE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ģʽ (One Pulse Mode)
0Update Eventʱֹͣ
1Update EventʱֹͣԶCEN غ */
void BSTIM_CR1_OPM_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BSTIM->CR1;
	tmpreg &= ~(BSTIM_CR1_OPM_Msk);
	tmpreg |= (SetValue & BSTIM_CR1_OPM_Msk);
	BSTIM->CR1 = tmpreg;
}

uint32_t BSTIM_CR1_OPM_Get(void)
{
	return (BSTIM->CR1 & BSTIM_CR1_OPM_Msk);
}

/* ѡ (Update Request Select)
0¼ܹupdateж

λUGĴ
ӻupdate
1updateжϻDMA غ */
void BSTIM_CR1_URS_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BSTIM->CR1;
	tmpreg &= ~(BSTIM_CR1_URS_Msk);
	tmpreg |= (SetValue & BSTIM_CR1_URS_Msk);
	BSTIM->CR1 = tmpreg;
}

uint32_t BSTIM_CR1_URS_Get(void)
{
	return (BSTIM->CR1 & BSTIM_CR1_URS_Msk);
}

/* ֹupdate (Update Disable)
0ʹupdate¼¼ʱupdate¼

λUGĴ
ӻupdate
1ֹupdate¼shadowĴUGλӻյӲresetʱ³ʼԤƵ غ */
void BSTIM_CR1_UDIS_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		BSTIM->CR1 |= (BSTIM_CR1_UDIS_Msk);
	}
	else
	{
		BSTIM->CR1 &= ~(BSTIM_CR1_UDIS_Msk);
	}
}

FunState BSTIM_CR1_UDIS_Getable(void)
{
	if (BSTIM->CR1 & (BSTIM_CR1_UDIS_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʹ (Counter Enable)
0ر
1ʹ
ע⣺ⲿģʽԶλCEN غ */
void BSTIM_CR1_CEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		BSTIM->CR1 |= (BSTIM_CR1_CEN_Msk);
	}
	else
	{
		BSTIM->CR1 &= ~(BSTIM_CR1_CEN_Msk);
	}
}

FunState BSTIM_CR1_CEN_Getable(void)
{
	if (BSTIM->CR1 & (BSTIM_CR1_CEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ģʽѡģʽӻ͵ͬźţTRGOԴ (Master Mode Select)
000BSTIM_EGRUGĴTRGO
001ʹźCNT_ENTRGOͬʱʱ
010UEupdate eventźűTRGO
011/100/111RFU

ע⣺ӻʱADCʹܹʱӣܽʱ͵TRGO غ */
void BSTIM_CR2_Write(uint32_t SetValue)
{
	BSTIM->CR2 = (SetValue & BSTIM_CR2_MMS_Msk);
}

uint32_t BSTIM_CR2_Read(void)
{
	return (BSTIM->CR2 & BSTIM_CR2_MMS_Msk);
}

/* Update¼жʹ (Update event Interrupt Enable)
0ֹUpdate¼ж
1Update¼ж غ */
void BSTIM_IER_UIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		BSTIM->IER |= (BSTIM_IER_UIE_Msk);
	}
	else
	{
		BSTIM->IER &= ~(BSTIM_IER_UIE_Msk);
	}
}

FunState BSTIM_IER_UIE_Getable(void)
{
	if (BSTIM->IER & (BSTIM_IER_UIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Update¼жϱ־Ӳλд1㡣(Update event Interrupt Flag,write 1 to flag)
¼ʱUIFλshadowĴ
-ظ=0UDIS=0£
-URS=0UDIS=0£λUGĴʼ
-URS=0UDIS=0£¼ʼ غ */
void BSTIM_ISR_UIF_Clr(void)
{
	BSTIM->ISR = BSTIM_ISR_UIF_Msk;
}

FlagStatus BSTIM_ISR_UIF_Chk(void)
{
	if (BSTIM->ISR & BSTIM_ISR_UIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Update¼λ˼ĴUpdate¼ӲԶ (User Generate)
λUGʱ³ʼshadowĴԤƵ㡣 غ */
void BSTIM_EGR_UG_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		BSTIM->EGR |= (BSTIM_EGR_UG_Msk);
	}
	else
	{
		BSTIM->EGR &= ~(BSTIM_EGR_UG_Msk);
	}
}

/* ֵ (Counter) غ */
void BSTIM_CNTR_Write(uint32_t SetValue)
{
	BSTIM->CNTR = (SetValue);
}

uint32_t BSTIM_CNTR_Read(void)
{
	return (BSTIM->CNTR);
}

/* ʱӣCK_CNTԤƵֵ (Counter Clock Prescaler)
fCK_CNT=fCK_PSC/(PSC[15:0]+1)
һpreloadĴupdate¼ʱݱshadowĴ غ */
void BSTIM_PSCR_Write(uint32_t SetValue)
{
	BSTIM->PSCR = (SetValue);
}

uint32_t BSTIM_PSCR_Read(void)
{
	return (BSTIM->PSCR);
}

/* ʱԶֵ (Auto-Reload Register)
һpreloadĴupdate¼ʱݱshadowĴ غ */
void BSTIM_ARR_Write(uint32_t SetValue)
{
	BSTIM->ARR = (SetValue);
}

uint32_t BSTIM_ARR_Read(void)
{
	return (BSTIM->ARR);
}


void BSTIM_Deinit(void)
{
	//BSTIM->CR1 = 0x00000000;
	//BSTIM->CR2 = 0x00000000;
	//BSTIM->IER = 0x00000000;
	//BSTIM->ISR = 0x00000000;
	//BSTIM->EGR = 0x00000000;
	//BSTIM->CNTR = 0x00000000;
	//BSTIM->PSCR = 0x00000000;
	//BSTIM->ARR = 0x00000000;
}

/******END OF FILE****/
