/**
  ******************************************************************************
  * @file    fm33a0xxev_bt.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_bt.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup BT
  * @brief BT driver modules
  * @{
  */ 
 
/* 8λBT1HBT2H  غ */
void BTx_CR1_CHEN_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR1  |= (BTx_CR1_CHEN_Msk);
	}
	else
	{
		BTx->CR1  &= ~(BTx_CR1_CHEN_Msk);
	}
}

FunState BTx_CR1_CHEN_Getable(BT_Type* BTx)
{
	if (BTx->CR1  & (BTx_CR1_CHEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* 8λBT1LBT2L
(Counter-Lowend enable)
18bitڼģʽʱԤֵͼֱֵͱȽϼĴ׽ģʽʱ㿪ʼɼ0xFFFFźȻ㿪ʼ¼׽ֻ16λģʽ16λĶʱ/Ͳ׽ģʽCHENΪƣCLENԶʧЧ
0ֹͣ8bit غ */
void BTx_CR1_CLEN_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR1  |= (BTx_CR1_CLEN_Msk);
	}
	else
	{
		BTx->CR1  &= ~(BTx_CR1_CLEN_Msk);
	}
}

FunState BTx_CR1_CLEN_Getable(BT_Type* BTx)
{
	if (BTx->CR1  & (BTx_CR1_CLEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ģʽѡ (work mode)
116λ׽ģʽ
08λʱ/ģʽλԴѡΪλźţʵ16λʱ/ģʽ غ */
void BTx_CR1_MODE_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR1 ;
	tmpreg &= ~(BTx_CR1_MODE_Msk);
	tmpreg |= (SetValue & BTx_CR1_MODE_Msk);
	BTx->CR1  = tmpreg;
}

uint32_t BTx_CR1_MODE_Get(BT_Type* BTx)
{
	return (BTx->CR1  & BTx_CR1_MODE_Msk);
}

/* ģʽµļغڲ׽ʱĲ׽ѡλ (edge select)
1ģʽԴ½أڲ׽ģʽʱز׽
0ģʽԴأڲ׽ģʽʱز׽
ע֧ϵͳʱӵ½ؼ׽ԴͼԴΪϵͳʱʱѡ½ؽЧ غ */
void BTx_CR1_EDGESEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR1 ;
	tmpreg &= ~(BTx_CR1_EDGESEL_Msk);
	tmpreg |= (SetValue & BTx_CR1_EDGESEL_Msk);
	BTx->CR1  = tmpreg;
}

uint32_t BTx_CR1_EDGESEL_Get(BT_Type* BTx)
{
	return (BTx->CR1  & BTx_CR1_EDGESEL_Msk);
}

/* ׽ģʽƣֻڲ׽ģʽЧ(capture mode)
1Ȳ׽
0ڲ׽ غ */
void BTx_CR1_CAPMOD_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR1 ;
	tmpreg &= ~(BTx_CR1_CAPMOD_Msk);
	tmpreg |= (SetValue & BTx_CR1_CAPMOD_Msk);
	BTx->CR1  = tmpreg;
}

uint32_t BTx_CR1_CAPMOD_Get(BT_Type* BTx)
{
	return (BTx->CR1  & BTx_CR1_CAPMOD_Msk);
}

/* 㲶׽ģʽ (capture clear)
1Ȼڲ׽£׽һغ󽫼жϣ׽ڶغ棨浽ߵλԤĴֵͬʱ
0׽㣬һֱɼ غ */
void BTx_CR1_CAPCLR_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR1  |= (BTx_CR1_CAPCLR_Msk);
	}
	else
	{
		BTx->CR1  &= ~(BTx_CR1_CAPCLR_Msk);
	}
}

FunState BTx_CR1_CAPCLR_Getable(BT_Type* BTx)
{
	if (BTx->CR1  & (BTx_CR1_CAPCLR_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* β׽ (capture once)
1β׽Чڲ׽һȻںֹͣҪٴβ׽
0׽ غ */
void BTx_CR1_CAPONCE_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR1 ;
	tmpreg &= ~(BTx_CR1_CAPONCE_Msk);
	tmpreg |= (SetValue & BTx_CR1_CAPONCE_Msk);
	BTx->CR1  = tmpreg;
}

uint32_t BTx_CR1_CAPONCE_Get(BT_Type* BTx)
{
	return (BTx->CR1  & BTx_CR1_CAPONCE_Msk);
}

/* PWMģʽ (pulse width modulation)
1PWMʹ
0PWM ʹ غ */
void BTx_CR1_PWM_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR1  |= (BTx_CR1_PWM_Msk);
	}
	else
	{
		BTx->CR1  &= ~(BTx_CR1_PWM_Msk);
	}
}

FunState BTx_CR1_PWM_Getable(BT_Type* BTx)
{
	if (BTx->CR1  & (BTx_CR1_PWM_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ڲԴźѡ (signal group2 select)
1ڲԴźѡGroup2
0ڲԴźѡGroup1 غ */
void BTx_CR2_SIG2SEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR2 ;
	tmpreg &= ~(BTx_CR2_SIG2SEL_Msk);
	tmpreg |= (SetValue & BTx_CR2_SIG2SEL_Msk);
	BTx->CR2  = tmpreg;
}

uint32_t BTx_CR2_SIG2SEL_Get(BT_Type* BTx)
{
	return (BTx->CR2  & BTx_CR2_SIG2SEL_Msk);
}

/* ڲ׽Դźѡ (signal group1 select)
1ڲ׽ԴźѡGroup1
0ڲ׽ԴźѡGroup2 غ */
void BTx_CR2_SIG1SEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR2 ;
	tmpreg &= ~(BTx_CR2_SIG1SEL_Msk);
	tmpreg |= (SetValue & BTx_CR2_SIG1SEL_Msk);
	BTx->CR2  = tmpreg;
}

uint32_t BTx_CR2_SIG1SEL_Get(BT_Type* BTx)
{
	return (BTx->CR2  & BTx_CR2_SIG1SEL_Msk);
}

/* 8λԴѡ (Counter Highend source select)
00/11ѡET1ĵλźţλ16λ
01ѡڲ׽Դź
10ѡڲԴźŻⲿDIRź غ */
void BTx_CR2_CNTHSEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR2 ;
	tmpreg &= ~(BTx_CR2_CNTHSEL_Msk);
	tmpreg |= (SetValue & BTx_CR2_CNTHSEL_Msk);
	BTx->CR2  = tmpreg;
}

uint32_t BTx_CR2_CNTHSEL_Get(BT_Type* BTx)
{
	return (BTx->CR2  & BTx_CR2_CNTHSEL_Msk);
}

/* ⲿDIRʹܡͨʱͬʱһɸߵ͵ƽָʾķźDIR·ͨDIRźŵƽĸߵֱͣƸλ͵λʹܣʵ򡢷ĸּ (direction bit enable)
1ⲿDIRźЧʱߵλǷⲿDIRźſơ
0ⲿDIRźЧʱߵλǷڲźſơ غ */
void BTx_CR2_DIREN_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR2  |= (BTx_CR2_DIREN_Msk);
	}
	else
	{
		BTx->CR2  &= ~(BTx_CR2_DIREN_Msk);
	}
}

FunState BTx_CR2_DIREN_Getable(BT_Type* BTx)
{
	if (BTx->CR2  & (BTx_CR2_DIREN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ڲDIRźţDIRENΪ0ⲿDIRЧʱɸźŴDIR룬ֱӿڲļҪⲿDIR룬DIRENΪ1ʱλӦΪ0 (set direction)
1ڲDIRźΪߵƽ8λ غ */
void BTx_CR2_STDIR_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR2  |= (BTx_CR2_STDIR_Msk);
	}
	else
	{
		BTx->CR2  &= ~(BTx_CR2_STDIR_Msk);
	}
}

FunState BTx_CR2_STDIR_Getable(BT_Type* BTx)
{
	if (BTx->CR2  & (BTx_CR2_STDIR_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* λʹܿѡź (source select)
1λʹܶѡʹܡʱλDIRƣɽһ
0λʹܶѡɼĴSTDIRⲿEX_SIG2ơ غ */
void BTx_CR2_SRCSEL_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->CR2  |= (BTx_CR2_SRCSEL_Msk);
	}
	else
	{
		BTx->CR2  &= ~(BTx_CR2_SRCSEL_Msk);
	}
}

FunState BTx_CR2_SRCSEL_Getable(BT_Type* BTx)
{
	if (BTx->CR2  & (BTx_CR2_SRCSEL_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ź2ѡ (direction polarity)
1ⲿDIRźEX_SIG2
0ⲿDIRźEX_SIG2 غ */
void BTx_CR2_DIRPO_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CR2 ;
	tmpreg &= ~(BTx_CR2_DIRPO_Msk);
	tmpreg |= (SetValue & BTx_CR2_DIRPO_Msk);
	BTx->CR2  = tmpreg;
}

uint32_t BTx_CR2_DIRPO_Get(BT_Type* BTx)
{
	return (BTx->CR2  & BTx_CR2_DIRPO_Msk);
}

/* RTCOUT2źѡ2 (RTCOUT2 source select)
00 = 32768HzXTLFʱ
01 = RTCSecRTCģź
10 = RTCMinRTCģķź
11 = RFU غ */
void BTx_CFGR1_RTCSEL2_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR1 ;
	tmpreg &= ~(BTx_CFGR1_RTCSEL2_Msk);
	tmpreg |= (SetValue & BTx_CFGR1_RTCSEL2_Msk);
	BTx->CFGR1  = tmpreg;
}

uint32_t BTx_CFGR1_RTCSEL2_Get(BT_Type* BTx)
{
	return (BTx->CFGR1  & BTx_CFGR1_RTCSEL2_Msk);
}

/* RTCOUT1źѡ1 (RTCOUT1 source select)
00 = 32768HzXTLFʱ
01 = RTCSecRTCģź
10 = RTCMinRTCģķź
11 = RFU غ */
void BTx_CFGR1_RTCSEL1_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR1 ;
	tmpreg &= ~(BTx_CFGR1_RTCSEL1_Msk);
	tmpreg |= (SetValue & BTx_CFGR1_RTCSEL1_Msk);
	BTx->CFGR1  = tmpreg;
}

uint32_t BTx_CFGR1_RTCSEL1_Get(BT_Type* BTx)
{
	return (BTx->CFGR1  & BTx_CFGR1_RTCSEL1_Msk);
}

/* Group2źѡ (Group2 source select)
00 = APBCLK
01 = RTCOUT2
10 = IN_SIG2ڲź2
11 = EX_SIG2ⲿź2
ע֧APBCLK½ز׽ͼ׽ԴͼԴΪAPBCLKʱѡ½ؽЧ׽ͼ غ */
void BTx_CFGR1_GRP2SEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR1 ;
	tmpreg &= ~(BTx_CFGR1_GRP2SEL_Msk);
	tmpreg |= (SetValue & BTx_CFGR1_GRP2SEL_Msk);
	BTx->CFGR1  = tmpreg;
}

uint32_t BTx_CFGR1_GRP2SEL_Get(BT_Type* BTx)
{
	return (BTx->CFGR1  & BTx_CFGR1_GRP2SEL_Msk);
}

/* Group1źѡƣΪ׽ģʽ²ʱѡͬʱΪźŲ׽Դ(Group1 source select)
00 = APBCLK
01 = RTCOUT1
10 = IN_SIG1ڲź1
11 = EX_SIG1ⲿź1
ע֧APBCLK½ز׽ͼ׽ԴͼԴΪAPBCLKʱѡ½ؽЧ׽ͼ غ */
void BTx_CFGR1_GRP1SEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR1 ;
	tmpreg &= ~(BTx_CFGR1_GRP1SEL_Msk);
	tmpreg |= (SetValue & BTx_CFGR1_GRP1SEL_Msk);
	BTx->CFGR1  = tmpreg;
}

uint32_t BTx_CFGR1_GRP1SEL_Get(BT_Type* BTx)
{
	return (BTx->CFGR1  & BTx_CFGR1_GRP1SEL_Msk);
}

/* ⲿźѡ2 (external source select2)
00 = BT1_IN0/BT2_IN0
01 = BT1_IN1/BT2_IN1
10 = BT1_IN2/BT2_IN2
11 = BT1_IN3/BT2_IN3 غ */
void BTx_CFGR2_EXSEL2_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR2 ;
	tmpreg &= ~(BTx_CFGR2_EXSEL2_Msk);
	tmpreg |= (SetValue & BTx_CFGR2_EXSEL2_Msk);
	BTx->CFGR2  = tmpreg;
}

uint32_t BTx_CFGR2_EXSEL2_Get(BT_Type* BTx)
{
	return (BTx->CFGR2  & BTx_CFGR2_EXSEL2_Msk);
}

/* ⲿźѡ1 (external source select1)
00 = BT1_IN0/BT2_IN0
01 = BT1_IN1/BT2_IN1
10 = BT1_IN2/BT2_IN2
11 = BT1_IN3/BT2_IN3 غ */
void BTx_CFGR2_EXSEL1_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR2 ;
	tmpreg &= ~(BTx_CFGR2_EXSEL1_Msk);
	tmpreg |= (SetValue & BTx_CFGR2_EXSEL1_Msk);
	BTx->CFGR2  = tmpreg;
}

uint32_t BTx_CFGR2_EXSEL1_Get(BT_Type* BTx)
{
	return (BTx->CFGR2  & BTx_CFGR2_EXSEL1_Msk);
}

/* ڲźѡ2 (internal source select 2)
00 = UART_RX3/UART_RX3
01 = UART_RX4/UART_RX4
10 = UART_RX5/UART_RX5
11 = RCLP/BT1_OUT غ */
void BTx_CFGR2_INSEL2_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR2 ;
	tmpreg &= ~(BTx_CFGR2_INSEL2_Msk);
	tmpreg |= (SetValue & BTx_CFGR2_INSEL2_Msk);
	BTx->CFGR2  = tmpreg;
}

uint32_t BTx_CFGR2_INSEL2_Get(BT_Type* BTx)
{
	return (BTx->CFGR2  & BTx_CFGR2_INSEL2_Msk);
}

/* ڲźѡ1 (internal source select 1)
00 = UART_RX0/UART_RX0
01 = UART_RX1/UART_RX1
10 = UART_RX2/UART_RX2
11 = RCLP/RCLP غ */
void BTx_CFGR2_INSEL1_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->CFGR2 ;
	tmpreg &= ~(BTx_CFGR2_INSEL1_Msk);
	tmpreg |= (SetValue & BTx_CFGR2_INSEL1_Msk);
	BTx->CFGR2  = tmpreg;
}

uint32_t BTx_CFGR2_INSEL1_Get(BT_Type* BTx)
{
	return (BTx->CFGR2  & BTx_CFGR2_INSEL1_Msk);
}

/* Group1ԤƵĴ (Group1 input signal prescaler)
Ƶ=X+100ʾ1ƵFFʾ256ƵԤƵźŶΪʽռձ1:X غ */
void BTx_PRES_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->PRES  = (SetValue & BTx_PRES_PRESCALE_Msk);
}

uint32_t BTx_PRES_Read(BT_Type* BTx)
{
	return (BTx->PRES  & BTx_PRES_PRESCALE_Msk);
}

/* λؿ (Counter highend load enable)
д1ԤĴET1PRESETHͼؼĴET1LOADHֱصֵĴET1CNTHͱȽϼĴET1CMPHд0ЧλӲԶ016λĶʱ/LHENΪļؿƣLLENԶʧЧ غ */
void BTx_LOADCR_LHEN_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->LOADCR |= (BTx_LOADCR_LHEN_Msk);
	}
	else
	{
		BTx->LOADCR &= ~(BTx_LOADCR_LHEN_Msk);
	}
}

FunState BTx_LOADCR_LHEN_Getable(BT_Type* BTx)
{
	if (BTx->LOADCR & (BTx_LOADCR_LHEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* λؿ (Counter lowend load enable)
д1ԤĴPRESETLͼؼĴLOADLֱصֵĴET1CNTLͱȽϼĴET1CMPLд0ЧλӲԶ016λĶʱ/LHENΪļؿƣLLENԶʧЧ غ */
void BTx_LOADCR_LLEN_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->LOADCR |= (BTx_LOADCR_LLEN_Msk);
	}
	else
	{
		BTx->LOADCR &= ~(BTx_LOADCR_LLEN_Msk);
	}
}

FunState BTx_LOADCR_LLEN_Getable(BT_Type* BTx)
{
	if (BTx->LOADCR & (BTx_LOADCR_LLEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* λֵĴ (counter lowend)
LLENЧʱԤüĴ غ */
uint32_t BTx_CNTL_Read(BT_Type* BTx)
{
	return (BTx->CNTL  & BTx_CNTL_CNTL_Msk);
}

/* λֵĴ (counter highend)
LHENЧʱԤüĴ غ */
uint32_t BTx_CNTH_Read(BT_Type* BTx)
{
	return (BTx->CNTH  & BTx_CNTH_CNTH_Msk);
}

/* λԤĴ (preset highend)
ڱλֵ򱣴沶׽8bit غ */
void BTx_PRESET_PRESETH_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->PRESET ;
	tmpreg &= ~(BTx_PRESET_PRESETH_Msk);
	tmpreg |= (SetValue & BTx_PRESET_PRESETH_Msk);
	BTx->PRESET  = tmpreg;
}

uint32_t BTx_PRESET_PRESETH_Get(BT_Type* BTx)
{
	return (BTx->PRESET  & BTx_PRESET_PRESETH_Msk);
}

/* λԤĴ (preset lowend)
ڱλֵ򱣴沶׽8bit غ */
void BTx_PRESET_PRESETL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->PRESET ;
	tmpreg &= ~(BTx_PRESET_PRESETL_Msk);
	tmpreg |= (SetValue & BTx_PRESET_PRESETL_Msk);
	BTx->PRESET  = tmpreg;
}

uint32_t BTx_PRESET_PRESETL_Get(BT_Type* BTx)
{
	return (BTx->PRESET  & BTx_PRESET_PRESETL_Msk);
}

/* λؼĴ (load lowend counter)
ڼƥִмʱؼĴֵȽϹĴ غ */
void BTx_LOADL_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->LOADL  = (SetValue & BTx_LOADL_LOADL_Msk);
}

uint32_t BTx_LOADL_Read(BT_Type* BTx)
{
	return (BTx->LOADL  & BTx_LOADL_LOADL_Msk);
}

/* λؼĴ (load highend counter)
ڼƥִмʱؼĴֵȽϹĴ8λʱ/ģʽʱøλؼĴֵּ֧Ϊ0x00á غ */
void BTx_LOADH_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->LOADH  = (SetValue & BTx_LOADH_LOADH_Msk);
}

uint32_t BTx_LOADH_Read(BT_Type* BTx)
{
	return (BTx->LOADH  & BTx_LOADH_LOADH_Msk);
}

/* λȽϼĴ (compare lowend )
ؼĴֵغдüĴüĴȽϣֵڵڸüĴֵƥźģ飬Ӧжϡ غ */
void BTx_CMPL_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->CMPL  = (SetValue & BTx_CMPL_CMPL_Msk);
}

uint32_t BTx_CMPL_Read(BT_Type* BTx)
{
	return (BTx->CMPL  & BTx_CMPL_CMPL_Msk);
}

/* λȽϼĴ (compare highend)
ؼĴֵغдüĴüĴȽϣֵڵڸüĴֵƥźģ飬Ӧжϡ غ */
void BTx_CMPH_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->CMPH  = (SetValue & BTx_CMPH_CMPH_Msk);
}

uint32_t BTx_CMPH_Read(BT_Type* BTx)
{
	return (BTx->CMPH  & BTx_CMPH_CMPH_Msk);
}

/* ȼ (output pulse width counter)
üĴڵȡʱΪ32768HzӦȷΧΪ30.5uS~125mS=(OUTCNT+1)/32768 غ */
void BTx_OUTCNT_Write(BT_Type* BTx, uint32_t SetValue)
{
	BTx->OUTCNT  = (SetValue & BTx_OUTCNT_OUTCNT_Msk);
}

uint32_t BTx_OUTCNT_Read(BT_Type* BTx)
{
	return (BTx->OUTCNT  & BTx_OUTCNT_OUTCNT_Msk);
}

void BTx_OCR_OUTCLR_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->OCR;
	tmpreg &= ~(BTx_OCR_OUTCLR_Msk);
	tmpreg |= (SetValue & BTx_OCR_OUTCLR_Msk);
	BTx->OCR = tmpreg;
}

uint32_t BTx_OCR_OUTCLR_Get(BT_Type* BTx)
{
	return (BTx->OCR & BTx_OCR_OUTCLR_Msk);
}

void BTx_OCR_OUTINV_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->OCR;
	tmpreg &= ~(BTx_OCR_OUTINV_Msk);
	tmpreg |= (SetValue & BTx_OCR_OUTINV_Msk);
	BTx->OCR = tmpreg;
}

uint32_t BTx_OCR_OUTINV_Get(BT_Type* BTx)
{
	return (BTx->OCR & BTx_OCR_OUTINV_Msk);
}

void BTx_OCR_OUTMOD_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->OCR;
	tmpreg &= ~(BTx_OCR_OUTMOD_Msk);
	tmpreg |= (SetValue & BTx_OCR_OUTMOD_Msk);
	BTx->OCR = tmpreg;
}

uint32_t BTx_OCR_OUTMOD_Get(BT_Type* BTx)
{
	return (BTx->OCR & BTx_OCR_OUTMOD_Msk);
}

void BTx_OCR_OUTSEL_Set(BT_Type* BTx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = BTx->OCR;
	tmpreg &= ~(BTx_OCR_OUTSEL_Msk);
	tmpreg |= (SetValue & BTx_OCR_OUTSEL_Msk);
	BTx->OCR = tmpreg;
}

uint32_t BTx_OCR_OUTSEL_Get(BT_Type* BTx)
{
	return (BTx->OCR & BTx_OCR_OUTSEL_Msk);
}

/* չʱλȽϷź (compare highend interrupt enable)
1 = жʹ
0 = жϽֹ غ */
void BTx_IER_CMPHIE_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->IER |= (BTx_IER_CMPHIE_Msk);
	}
	else
	{
		BTx->IER &= ~(BTx_IER_CMPHIE_Msk);
	}
}

FunState BTx_IER_CMPHIE_Getable(BT_Type* BTx)
{
	if (BTx->IER & (BTx_IER_CMPHIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* չʱλȽϷź (compare lowend interrupt enable)
1 = жʹ
0 = жϽֹ غ */
void BTx_IER_CMPLIE_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->IER |= (BTx_IER_CMPLIE_Msk);
	}
	else
	{
		BTx->IER &= ~(BTx_IER_CMPLIE_Msk);
	}
}

FunState BTx_IER_CMPLIE_Getable(BT_Type* BTx)
{
	if (BTx->IER & (BTx_IER_CMPLIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʱλź (highend overflow interrupt enable)
1 = жʹ
0 = жϽֹ غ */
void BTx_IER_OVHIE_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->IER |= (BTx_IER_OVHIE_Msk);
	}
	else
	{
		BTx->IER &= ~(BTx_IER_OVHIE_Msk);
	}
}

FunState BTx_IER_OVHIE_Getable(BT_Type* BTx)
{
	if (BTx->IER & (BTx_IER_OVHIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʱλź (lowend overflow interrupt enable)
1 = жʹ
0 = жϽֹ غ */
void BTx_IER_OVLIE_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->IER |= (BTx_IER_OVLIE_Msk);
	}
	else
	{
		BTx->IER &= ~(BTx_IER_OVLIE_Msk);
	}
}

FunState BTx_IER_OVLIE_Getable(BT_Type* BTx)
{
	if (BTx->IER & (BTx_IER_OVLIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʱ׽ź (capture interrupt enable)
1 = жʹ
0 = жϽֹ غ */
void BTx_IER_CAPIE_Setable(BT_Type* BTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		BTx->IER |= (BTx_IER_CAPIE_Msk);
	}
	else
	{
		BTx->IER &= ~(BTx_IER_CAPIE_Msk);
	}
}

FunState BTx_IER_CAPIE_Getable(BT_Type* BTx)
{
	if (BTx->IER & (BTx_IER_CAPIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʱλȽϷź (compare highend interrupt flag)
1 = ǰֵڵڱȽϼĴֵźŽ¼µļؼĴֵĴ
0 = ǰֵСڱȽϼĴֵ غ */
void BTx_ISR_CMPHIF_Clr(BT_Type* BTx)
{
	BTx->ISR = BTx_ISR_CMPHIF_Msk;
}

FlagStatus BTx_ISR_CMPHIF_Chk(BT_Type* BTx)
{
	if (BTx->ISR & BTx_ISR_CMPHIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʱλȽϷź (compare lowend interrupt flag)
1 = ǰֵڵڱȽϼĴֵźŽ¼µļؼĴֵĴ
0 = ǰֵСڱȽϼĴֵ غ */
void BTx_ISR_CMPLIF_Clr(BT_Type* BTx)
{
	BTx->ISR = BTx_ISR_CMPLIF_Msk;
}

FlagStatus BTx_ISR_CMPLIF_Chk(BT_Type* BTx)
{
	if (BTx->ISR & BTx_ISR_CMPLIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʱλź (highend overflow interrupt flag)
1 = 
0 = δ غ */
void BTx_ISR_OVHIF_Clr(BT_Type* BTx)
{
	BTx->ISR = BTx_ISR_OVHIF_Msk;
}

FlagStatus BTx_ISR_OVHIF_Chk(BT_Type* BTx)
{
	if (BTx->ISR & BTx_ISR_OVHIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʱλź (lowend overflow interrupt flag)
1 = 
0 = δ غ */
void BTx_ISR_OVLIF_Clr(BT_Type* BTx)
{
	BTx->ISR = BTx_ISR_OVLIF_Msk;
}

FlagStatus BTx_ISR_OVLIF_Chk(BT_Type* BTx)
{
	if (BTx->ISR & BTx_ISR_OVLIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʱ׽ź (capture interrupt flag)
1 = ׽ָ
0 = δ׽ָ غ */
void BTx_ISR_CAPIF_Clr(BT_Type* BTx)
{
	BTx->ISR = BTx_ISR_CAPIF_Msk;
}

FlagStatus BTx_ISR_CAPIF_Chk(BT_Type* BTx)
{
	if (BTx->ISR & BTx_ISR_CAPIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}


void BTx_Deinit(BT_Type* BTx)
{
	//BTx->CR1  = 0x00000000;
	//BTx->CR2  = 0x00000000;
	//BTx->CFGR1  = 0x00000000;
	//BTx->CFGR2  = 0x00000000;
	//BTx->PRES  = 0x00000000;
	//BTx->LOADCR = 0x00000000;
	//BTx->CNTL  = 0x00000000;
	//BTx->CNTH  = 0x00000000;
	//BTx->PRESET  = 0x00000000;
	//BTx->LOADL  = 0x00000000;
	//BTx->LOADH  = 0x00000000;
	//BTx->CMPL  = 0x00000000;
	//BTx->CMPH  = 0x00000000;
	//BTx->OUTCNT  = 0x00000000;
	//BTx->OCR = ;
	//BTx->IER = 0x00000000;
	//BTx->ISR = 0x00000000;
}
/******END OF FILE****/
