/**
  ******************************************************************************
  * @file    fm33a0xxev_crc.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_crc.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup CRC 
  * @brief CRC driver modules
  * @{
  */ 

/* CRCDRΪĴ󱣴CRC(CRC Data Register)
ΪʱwordʹܣCRCDR[31:0]м㣬4byte㣨ɵ͵ߣCRCDR[7:0]м㣬1byte㡣
ʱΪ7λʽCRCDR[6:0]Ϊ8λʽCRCDR[7:0]Ϊ16λʽCRCDR[15:0]Ϊ32λʽCRCDR[31:0] غ */
void CRC_DR_Write(uint32_t SetValue)
{
	CRC->DR = (SetValue);
}

uint32_t CRC_DR_Read(void)
{
	return (CRC->DR);
}

/* WORDʹ (Operation by Word)
0ֽڲCRCCRCDRֽڽ
1ֲCRCCRCDRȫ4ֽڽ غ */
void CRC_CR_OPWD_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CRC->CR;
	tmpreg &= ~(CRC_CR_OPWD_Msk);
	tmpreg |= (SetValue & CRC_CR_OPWD_Msk);
	CRC->CR = tmpreg;
}

uint32_t CRC_CR_OPWD_Get(void)
{
	return (CRC->CR & CRC_CR_OPWD_Msk);
}

/* CRCټʹ (Parallel Calculation)
0㣬1ֽҪ8ʱ
1м㣬1ֽҪ1ʱ غ */
void CRC_CR_PARA_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CRC->CR;
	tmpreg &= ~(CRC_CR_PARA_Msk);
	tmpreg |= (SetValue & CRC_CR_PARA_Msk);
	CRC->CR = tmpreg;
}

uint32_t CRC_CR_PARA_Get(void)
{
	return (CRC->CR & CRC_CR_PARA_Msk);
}

/* CRC뷴ת (Reflected Input)
00벻ת
01밴ֽڷת
10밴ַת
11밴ַת
磺Ϊ0x11223344
RFLTIN==01ݱΪ0x8844CC22ٽмRFLTIN==10ݱΪ0x448822CCٽм
RFLTIN==11ݱΪ0x22CC4488ٽм غ */
void CRC_CR_RFLTIN_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CRC->CR;
	tmpreg &= ~(CRC_CR_RFLTIN_Msk);
	tmpreg |= (SetValue & CRC_CR_RFLTIN_Msk);
	CRC->CR = tmpreg;
}

uint32_t CRC_CR_RFLTIN_Get(void)
{
	return (CRC->CR & CRC_CR_RFLTIN_Msk);
}

/* CRCת (Reflected Output)
0벻ת
1밴ֽڷת
磺
RFLTO==1ǰCRCΪ0x1234ĽΪ0x2C48 RFLTO==0ֱ0x1234ע⣺˽һΪҪXORǷΪ1Ĵbit2˵ غ */
void CRC_CR_RFLTO_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CRC->CR;
	tmpreg &= ~(CRC_CR_RFLTO_Msk);
	tmpreg |= (SetValue & CRC_CR_RFLTO_Msk);
	CRC->CR = tmpreg;
}

uint32_t CRC_CR_RFLTO_Get(void)
{
	return (CRC->CR & CRC_CR_RFLTO_Msk);
}

/* CRC־λֻ (Result Flag)
0CRCΪ0
1CRCȫ0 غ */
FlagStatus CRC_CR_RES_Chk(void)
{
	if (CRC->CR & CRC_CR_RES_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* CRC־λֻ (Busy)
0CRC
1CRC غ */
FlagStatus CRC_CR_BUSY_Chk(void)
{
	if (CRC->CR & CRC_CR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʹ (Output XORed with CRC_XOR register enable)
0CRC_XORĴ
1CRC_XORĴ غ */
void CRC_CR_XOR_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		CRC->CR |= (CRC_CR_XOR_Msk);
	}
	else
	{
		CRC->CR &= ~(CRC_CR_XOR_Msk);
	}
}

FunState CRC_CR_XOR_Getable(void)
{
	if (CRC->CR & (CRC_CR_XOR_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* CRCʽλѡ (Polynomial width Selection)
0032λ
0116λ
108λ
117λ غ */
void CRC_CR_SEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = CRC->CR;
	tmpreg &= ~(CRC_CR_SEL_Msk);
	tmpreg |= (SetValue & CRC_CR_SEL_Msk);
	CRC->CR = tmpreg;
}

uint32_t CRC_CR_SEL_Get(void)
{
	return (CRC->CR & CRC_CR_SEL_Msk);
}

/* CRCԷλĴ (Linear Feedback Shift Register)
㿪ʼǰдCRCʼֵ غ */
void CRC_LFSR_Write(uint32_t SetValue)
{
	CRC->LFSR = (SetValue);
}

uint32_t CRC_LFSR_Read(void)
{
	return (CRC->LFSR);
}

/* CRCĴ (eXclusive OR)
CRC_CR.XORΪ1ʱCRCǰ˼Ĵݡ غ */
void CRC_XOR_Write(uint32_t SetValue)
{
	CRC->XOR = (SetValue);
}

uint32_t CRC_XOR_Read(void)
{
	return (CRC->XOR);
}

/* CRCʽϵ (CRC Polynominals) غ */
void CRC_POLY_Write(uint32_t SetValue)
{
	CRC->POLY = (SetValue);
}

uint32_t CRC_POLY_Read(void)
{
	return (CRC->POLY);
}


void CRC_Deinit(void)
{
	//CRC->DR = 0xFFFFFFFF;
	//CRC->CR = 0x00000002;
	//CRC->LFSR = 0xFFFFFFFF;
	//CRC->XOR = 0x00000000;
	//CRC->POLY = 0x00000000;
}
//Code_End

/* CRCʼú*/
void CRC_Init(CRC_InitTypeDef* para)
{
	CRC_LFSR_Write(para->LFSR);				/*!<ֵĴ*/
	CRC_CR_SEL_Set(para->CRCSEL);			/*!<CRCУʽλѡ*/
	CRC_CR_OPWD_Set(para->OPWD);			/*!<WORDʹ*/
	CRC_CR_PARA_Set(para->PARA);			/*!<CRCټʹ*/
	CRC_CR_RFLTIN_Set(para->RFLTIN);	/*!<CRC뷴ת*/
	CRC_CR_RFLTO_Set(para->RFLTO);		/*!<CRCת*/
	CRC_CR_XOR_Setable(para->XOR);		/*!<ʹ*/
	CRC_XOR_Write(para->CRC_XOR);			/*!<Ĵ*/
	CRC_POLY_Write(para->CRCPOLY);		/*!<CRCʽĴ*/
}

/******END OF FILE****/
