/**
  ******************************************************************************
  * @file    fm33a0xxev_flash.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_flash.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup FLASH 
  * @brief FLASH driver modules
  * @{
  */ 

/* Flashȴ (Wait Cycles Config)
00/110 wait cycle
011 wait cycle
102 wait cycles
CPUƵСڵ24MHzʱҪwaitƵ24MС48Mhzʱʹ1 waitƵ48Mhzʱʹ2 wait غ */
void FLS_RDCR_WAIT_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = FLS->RDCR;
	tmpreg &= ~(FLS_RDCR_WAIT_Msk);
	tmpreg |= (SetValue & FLS_RDCR_WAIT_Msk);
	FLS->RDCR = tmpreg;
}

uint32_t FLS_RDCR_WAIT_Get(void)
{
	return (FLS->RDCR & FLS_RDCR_WAIT_Msk);
}

/* ָԤȡָʹܣWAIT==00д1Ч (Prefetch Enable)
1ʹPrefetch
0ֹPrefetch غ */
void FLS_PFCR_PRFTEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->PFCR |= (FLS_PFCR_PRFTEN_Msk);
	}
	else
	{
		FLS->PFCR &= ~(FLS_PFCR_PRFTEN_Msk);
	}
}

FunState FLS_PFCR_PRFTEN_Getable(void)
{
	if (FLS->PFCR & (FLS_PFCR_PRFTEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* DBGƼĴʹ
0
1ʹ */
FunState FLS_OPTBR_DBGCFGEN_Chk(void)
{
	if (FLS->OPTBR & FLS_OPTBR_DBGCFGEN_Msk)
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Information2־ (IF2 Lock Enable)
0δ
1ɸд غ */
FlagStatus FLS_OPTBR_IF2LOCK_Chk(void)
{
	if (FLS->OPTBR & FLS_OPTBR_IF2LOCK_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Information1־ (IF1 Lock Enable)
0δ
1ɸд غ */
FlagStatus FLS_OPTBR_IF1LOCK_Chk(void)
{
	if (FLS->OPTBR & FLS_OPTBR_IF1LOCK_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* DataFlashʹ (DataFlash Enable)
0data flash
1data flash غ */
FunState FLS_OPTBR_DFLSEN_Getable(void)
{
	if (FLS->OPTBR & (FLS_OPTBR_DFLSEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* BootSwapʹ (BootSwap Enable)
00/01/11ֹBootSwap
10BootSwap غ */
FunState FLS_OPTBR_BTSEN_Getable(void)
{
	if ((FLS->OPTBR & (FLS_OPTBR_BTSEN_Msk))==(0x2<<8))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ӦôȨʹ (AppCode Lock Enable)
00/01/11ACLOCKʹ
10ACLOCKʹ غ */
FunState FLS_OPTBR_ACLOCKEN_Getable(void)
{
	if ((FLS->OPTBR & (FLS_OPTBR_ACLOCKEN_Msk))==(0x2<<4))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Debug Portȡʹ (Debug Read Protection Enable)
00/01/11DBRDPʹ
10DBRDPʹ غ */
uint32_t FLS_OPTBR_DBRDPEN_Getable(void)
{
	 return (FLS->OPTBR & (FLS_OPTBR_DBRDPEN_Msk));
	
}

uint32_t FLS_OPTBR_Read(void)
{
	return (FLS->OPTBR);
}



/* Flash (Erase Type)
00/11Page Erase
01Sector Erase
10Chip Erase (SWD only) غ */
void FLS_EPCR_ERTYPE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = FLS->EPCR;
	tmpreg &= ~(FLS_EPCR_ERTYPE_Msk);
	tmpreg |= (SetValue & FLS_EPCR_ERTYPE_Msk);
	FLS->EPCR = tmpreg;
}

uint32_t FLS_EPCR_ERTYPE_Get(void)
{
	return (FLS->EPCR & FLS_EPCR_ERTYPE_Msk);
}

/* Program Request
λӲɱ̺Զ غ */
void FLS_EPCR_PREQ_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = FLS->EPCR;
	tmpreg &= ~(FLS_EPCR_PREQ_Msk);
	tmpreg |= (SetValue & FLS_EPCR_PREQ_Msk);
	FLS->EPCR = tmpreg;
}

uint32_t FLS_EPCR_PREQ_Get(void)
{
	return (FLS->EPCR & FLS_EPCR_PREQ_Msk);
}

/* Erase Request
λӲɲԶ غ */
void FLS_EPCR_EREQ_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = FLS->EPCR;
	tmpreg &= ~(FLS_EPCR_EREQ_Msk);
	tmpreg |= (SetValue & FLS_EPCR_EREQ_Msk);
	FLS->EPCR = tmpreg;
}

uint32_t FLS_EPCR_EREQ_Get(void)
{
	return (FLS->EPCR & FLS_EPCR_EREQ_Msk);
}

/* FlashдKeyĴSWDдǰȷ˵ַдϷKEYС (Flash Key) غ */
void FLS_KEY_Write(uint32_t SetValue)
{
	FLS->KEY = (SetValue);
}

/* OTP̴жʹܣ1Ч (OTP program error Interrupt Enable) غ */
void FLS_IER_OTPIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_OTPIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_OTPIE_Msk);
	}
}

FunState FLS_IER_OTPIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_OTPIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* FlashдȨ޴жʹܣ1Ч (Flash Authentication Error Interrupt Enable) غ */
void FLS_IER_AUTHIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_AUTHIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_AUTHIE_Msk);
	}
}

FunState FLS_IER_AUTHIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_AUTHIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Flash KEYжʹܣ1Ч (Flash Key Error Interrupt Enable) غ */
void FLS_IER_KEYIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_KEYIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_KEYIE_Msk);
	}
}

FunState FLS_IER_KEYIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_KEYIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* дʱʱӴжʹܣ1Ч (Erase/Program Clock Error Interrupt Enable) غ */
void FLS_IER_CKIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_CKIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_CKIE_Msk);
	}
}

FunState FLS_IER_CKIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_CKIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ɱ־жʹܣ1Ч (Program Done Interrupt Enable) غ */
void FLS_IER_PRDIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_PRDIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_PRDIE_Msk);
	}
}

FunState FLS_IER_PRDIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_PRDIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* дɱ־жʹܣ1Ч (Erase Done Interrupt Enable) غ */
void FLS_IER_ERDIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		FLS->IER |= (FLS_IER_ERDIE_Msk);
	}
	else
	{
		FLS->IER &= ~(FLS_IER_ERDIE_Msk);
	}
}

FunState FLS_IER_ERDIE_Getable(void)
{
	if (FLS->IER & (FLS_IER_ERDIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* FlashдKEY״̬ (Flash Key Status)
000Flashд״̬δKEY
001ȫ״̬
010״̬
011̽״̬
100KEY״̬Ҫλܽ
101/110/111RFU غ */
uint32_t FLS_ISR_KEYSTA_Get(void)
{
	return (FLS->ISR & FLS_ISR_KEYSTA_Msk);
}

/* BootSwap־Ĵ (BootSwap)
0ΪFlashַ0000H~1FFFH
1ΪFlashַ2000H~3FFFH غ */
uint32_t FLS_ISR_BTSF_Get(void)
{
	return (FLS->ISR & FLS_ISR_BTSF_Msk);
}

/* OTP pageȨ޴Ӳλд1 (OTP program Error Flag. Write 1 to clear)
1Զѱ̵OTPֽڽб
0OTP̴ غ */
void FLS_ISR_OTPERR_Clr(void)
{
	FLS->ISR = FLS_ISR_OTPERR_Msk;
}

FlagStatus FLS_ISR_OTPERR_Chk(void)
{
	if (FLS->ISR & FLS_ISR_OTPERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* FlashдȨ޴󣬶ȡLOCKݻLOCKдʱλд1㡣(Flash Authentication Error Flag, write 1 to clear)
1FlashȨ޴
0FlashûзȨ޴ غ */
void FLS_ISR_AUTHERR_Clr(void)
{
	FLS->ISR = FLS_ISR_AUTHERR_Msk;
}

FlagStatus FLS_ISR_AUTHERR_Chk(void)
{
	if (FLS->ISR & FLS_ISR_AUTHERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Flash KEYӲλд1 (Flash Key Error Flag, write 1 to clear) غ */
void FLS_ISR_KEYERR_Clr(void)
{
	FLS->ISR = FLS_ISR_KEYERR_Msk;
}

FlagStatus FLS_ISR_KEYERR_Chk(void)
{
	if (FLS->ISR & FLS_ISR_KEYERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* дʱʱӴNVMIFдFlashʱRCHFδʹܣ򴥷CKERRжϣд1㡣(Erase/Program Clock Error Flag, write 1 to clear) غ */
void FLS_ISR_CKERR_Clr(void)
{
	FLS->ISR = FLS_ISR_CKERR_Msk;
}

FlagStatus FLS_ISR_CKERR_Chk(void)
{
	if (FLS->ISR & FLS_ISR_CKERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Program Doneɱ־Ӳλд1(Program Done Flag,write 1 to clear) غ */
void FLS_ISR_PRD_Clr(void)
{
	FLS->ISR = FLS_ISR_PRD_Msk;
}

FlagStatus FLS_ISR_PRD_Chk(void)
{
	if (FLS->ISR & FLS_ISR_PRD_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Erase Doneдɱ־Ӳλд1(Erase Done Flag,write 1 to clear) غ */
void FLS_ISR_ERD_Clr(void)
{
	FLS->ISR = FLS_ISR_ERD_Msk;
}

FlagStatus FLS_ISR_ERD_Chk(void)
{
	if (FLS->ISR & FLS_ISR_ERD_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}


void FLS_Deinit(void)
{
	//FLS->RDCR = 0x00000000;
	//FLS->PFCR = 0x00000000;
	//FLS->OPTBR = 0x00000105;
	//FLS->EPCR = 0x00000000;
	//FLS->KEY = 0x00000000;
	//FLS->IER = 0x00000000;
	//FLS->ISR = 0x00000000;
}

/******END OF FILE****/
