/**
  ******************************************************************************
  * @file    fm33a0xxev_i2c.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_i2c.h"
#include "fm33a0xxev_rmu.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup I2C 
  * @brief I2C driver modules
  * @{
  */

/* DMAԶֹ (Automatic Ending)
1DMAָȴɺԶSTOPʱ
0DMAָȴɺ󣬵ȴӹ غ */
void I2Cx_CFGR_AUTOEND_Set(I2C_Type* I2Cx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = I2Cx->CFGR ;
	tmpreg &= ~(I2Cx_CFGR_AUTOEND_Msk);
	tmpreg |= (SetValue & I2Cx_CFGR_AUTOEND_Msk);
	I2Cx->CFGR  = tmpreg;
}

uint32_t I2Cx_CFGR_AUTOEND_Get(I2C_Type* I2Cx)
{
	return (I2Cx->CFGR  & I2Cx_CFGR_AUTOEND_Msk);
}

/* DMAʹ (Master DMA Enable)
0رDMA
1ʹDMA غ */
void I2Cx_CFGR_MSP_DMAEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CFGR  |= (I2Cx_CFGR_MSP_DMAEN_Msk);
	}
	else
	{
		I2Cx->CFGR  &= ~(I2Cx_CFGR_MSP_DMAEN_Msk);
	}
}

FunState I2Cx_CFGR_MSP_DMAEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CFGR  & (I2Cx_CFGR_MSP_DMAEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SCLͳʱʹܣTimeOut
1ʹܳʱܣʱMSPTOĴ
0رճʱ غ */
void I2Cx_CFGR_TOEN_Setable(I2C_Type* I2Cx, FunState NewState)
{	
	if (NewState == ENABLE)
	{
		I2Cx->CFGR  |= (I2Cx_CFGR_TOEN_Msk);
	}
	else
	{
		I2Cx->CFGR  &= ~(I2Cx_CFGR_TOEN_Msk);
	}
}

FunState I2Cx_CFGR_TOEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CFGR  & (I2Cx_CFGR_TOEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* I2Cģʹܿλ (Master Enable)1 = I2Cʹ0 = I2Cֹ غ */
void I2Cx_CFGR_MSPEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CFGR  |= (I2Cx_CFGR_MSPEN_Msk);
	}
	else
	{
		I2Cx->CFGR  &= ~(I2Cx_CFGR_MSPEN_Msk);
	}
}

FunState I2Cx_CFGR_MSPEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CFGR  & (I2Cx_CFGR_MSPEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ؽģʽ£ʹλ (Receive Enable)
1 = ʹ
0 = սֹ غ */
void I2Cx_CR_RCEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CR  |= (I2Cx_CR_RCEN_Msk);
	}
	else
	{
		I2Cx->CR  &= ~(I2Cx_CR_RCEN_Msk);
	}
}

FunState I2Cx_CR_RCEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CR  & (I2Cx_CR_RCEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* STOPʱʹܿλд1STOPʱ򣬷ɺӲԶ (Stop Enable) غ */
void I2Cx_CR_PEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CR  |= (I2Cx_CR_PEN_Msk);
	}
	else
	{
		I2Cx->CR  &= ~(I2Cx_CR_PEN_Msk);
	}
}

FunState I2Cx_CR_PEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CR  & (I2Cx_CR_PEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Repeated STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (ReStart Enable) غ */
void I2Cx_CR_RSEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CR  |= (I2Cx_CR_RSEN_Msk);
	}
	else
	{
		I2Cx->CR  &= ~(I2Cx_CR_RSEN_Msk);
	}
}

FunState I2Cx_CR_RSEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CR  & (I2Cx_CR_RSEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* STARTʱʹܿλд1STOPʱ򣬷ɺӲԶ (Start Enable) غ */
void I2Cx_CR_SEN_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->CR  |= (I2Cx_CR_SEN_Msk);
	}
	else
	{
		I2Cx->CR  &= ~(I2Cx_CR_SEN_Msk);
	}
}

FunState I2Cx_CR_SEN_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->CR  & (I2Cx_CR_SEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* WCOLжʹܼĴ (Write Collide Enable)
1дͻж
0ֹдͻж غ */
void I2Cx_IER_WCOLE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_WCOLE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_WCOLE_Msk);
	}
}

FunState I2Cx_IER_WCOLE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_WCOLE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SCLʱжʹܼĴ (Time-Out Enable)
1ʱж
0ֹʱж غ */
void I2Cx_IER_TOE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_TOE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_TOE_Msk);
	}
}

FunState I2Cx_IER_TOE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_TOE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* STARTʱжʹܼĴ (START interrupt Enable)
1STARTʱж
0ֹSTARTʱж غ */
void I2Cx_IER_SE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_SE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_SE_Msk);
	}
}

FunState I2Cx_IER_SE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_SE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* STOPʱжʹܼĴ (STOP interrupt Enable)
1STOPʱж
0ֹSTOPʱж غ */
void I2Cx_IER_PE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_PE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_PE_Msk);
	}
}

FunState I2Cx_IER_PE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_PE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ģʽNACKжʹܼĴ (Non-ACK interrupt Enable)
1յNACKж
0ֹNACKж غ */
void I2Cx_IER_NACKE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_NACKE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_NACKE_Msk);
	}
}

FunState I2Cx_IER_NACKE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_NACKE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* I2Cжʹ (Transmit done interrupt enable)
1ж
0ֹж غ */
void I2Cx_IER_TXIE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_TXIE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_TXIE_Msk);
	}
}

FunState I2Cx_IER_TXIE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_TXIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* I2Cжʹ (Receive done interrupt enable)
1ж
0ֹж غ */
void I2Cx_IER_RXIE_Setable(I2C_Type* I2Cx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		I2Cx->IER  |= (I2Cx_IER_RXIE_Msk);
	}
	else
	{
		I2Cx->IER  &= ~(I2Cx_IER_RXIE_Msk);
	}
}

FunState I2Cx_IER_RXIE_Getable(I2C_Type* I2Cx)
{
	if (I2Cx->IER  & (I2Cx_IER_RXIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* дͻλMCUֻSTARTʱһ֡д֮дSSPBUFдͻӲλд1
(Write Collide)
1 = дͻ
0 = δͻ غ */
void I2Cx_ISR_WCOL_Clr(I2C_Type* I2Cx)
{
	I2Cx->ISR  = I2Cx_ISR_WCOL_Msk;
}

FlagStatus I2Cx_ISR_WCOL_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_WCOL_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* SCLʱжϱ־ غ */
void I2Cx_ISR_OVT_Clr(I2C_Type* I2Cx)
{
	I2Cx->ISR = I2Cx_ISR_TO_Msk;
}

FlagStatus I2Cx_ISR_OVT_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR & I2Cx_ISR_TO_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* STARTʱжϱ־Ӳλȡ
(START done) غ */
FlagStatus I2Cx_ISR_S_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_S_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* STOPʱжϱ־Ӳλȡ
(STOP done) غ */
FlagStatus I2Cx_ISR_P_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_P_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* طģʽ£ԴӻĻӦźţͺյNACK˱־ԲжϣӲλд1㡣(Acknowledge Status)
1ӻӦNACK
0ӻӦACK غ */
void I2Cx_ISR_ACKSTA_Clr(I2C_Type* I2Cx)
{
	I2Cx->ISR  = I2Cx_ISR_ACKSTA_Msk;
}

FlagStatus I2Cx_ISR_ACKSTA_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_ACKSTA_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* I2Cжϱ־Ӳλд1 (Transmit done interrupt flag) غ */
void I2Cx_ISR_TXIF_Clr(I2C_Type* I2Cx)
{
	I2Cx->ISR  = I2Cx_ISR_TXIF_Msk;
}

FlagStatus I2Cx_ISR_TXIF_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_TXIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* I2Cжϱ־Ӳλд1 (Receive done interrupt flag) غ */
void I2Cx_ISR_RXIF_Clr(I2C_Type* I2Cx)
{
	I2Cx->ISR  = I2Cx_ISR_RXIF_Msk;
}

FlagStatus I2Cx_ISR_RXIF_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->ISR  & I2Cx_ISR_RXIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* I2Cͨ״̬λ (I2C is busy)
1ӿڴڶд״̬ڽݴ䣬
0ݴ غ */
FlagStatus I2Cx_SR_BUSY_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->SR  & I2Cx_SR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* I2C䷽״̬λ (Read or Write Bar)
1Ӵӻȡ
0ӻд غ */
FlagStatus I2Cx_SR_RW_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->SR  & I2Cx_SR_RW_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ״̬λ (Buffer full)
գ
1 = ɣSSPBUF
0 = δɣSSPBUF
ͣ
1 = ڷͣSSPBUF
0 = ɣSSPBUF غ */
FlagStatus I2Cx_SR_BF_Chk(I2C_Type* I2Cx)
{
	if (I2Cx->SR  & I2Cx_SR_BF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ؽģʽ£Ӧźŵ״̬ (Acknowledge mode)
1طNACK
0طACK

ע⣺P־Ĵ£λACKMO
 غ */
void I2Cx_SR_ACKMO_Set(I2C_Type* I2Cx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = I2Cx->SR;
	tmpreg &= ~(I2Cx_SR_ACKMO_Msk);
	tmpreg |= (SetValue & I2Cx_SR_ACKMO_Msk);
	I2Cx->SR = tmpreg;
}

uint32_t I2Cx_SR_ACKMO_Get(I2C_Type* I2Cx)
{
	return (I2Cx->SR & I2Cx_SR_ACKMO_Msk);
}

/* ͵SCLʱӸߵƽȣI2CʱӼ غ */
void I2Cx_BRG_MSPBRGH_Set(I2C_Type* I2Cx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = I2Cx->BRG;
	tmpreg &= ~(I2Cx_BRG_MSPBRGH_Msk);
	tmpreg |= (SetValue & I2Cx_BRG_MSPBRGH_Msk);
	I2Cx->BRG = tmpreg;
}

uint32_t I2Cx_BRG_MSPBRGH_Get(I2C_Type* I2Cx)
{
	return (I2Cx->BRG & I2Cx_BRG_MSPBRGH_Msk);
}

/* ͵SCLʱӵ͵ƽȣI2CʱӼ غ */
void I2Cx_BRG_MSPBRGL_Set(I2C_Type* I2Cx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = I2Cx->BRG;
	tmpreg &= ~(I2Cx_BRG_MSPBRGL_Msk);
	tmpreg |= (SetValue & I2Cx_BRG_MSPBRGL_Msk);
	I2Cx->BRG = tmpreg;
}

uint32_t I2Cx_BRG_MSPBRGL_Get(I2C_Type* I2Cx)
{
	return (I2Cx->BRG & I2Cx_BRG_MSPBRGL_Msk);
}

/* SSPBUF[7:0]ݵĶдͨSSPBUFĲɡʱSSPBUFִдͬʱҲշλĴ(SSPSR)ʱSSPBUFSSPSR˫ṹΪSSPBUFݡһֽڵݣSSPSRSSPBUFͬʱλI2CIFSSPSRֱӼĴûַ غ */
void I2Cx_BUF_Write(I2C_Type* I2Cx, uint32_t SetValue)
{
	I2Cx->BUF = (SetValue & I2Cx_BUF_WR_Msk);
}

uint32_t I2Cx_BUF_Read(I2C_Type* I2Cx)
{
	return (I2Cx->BUF & I2Cx_BUF_WR_Msk);
}

/* SDASCL½صıʱI2CʱӼ
(SDA hold time) غ */
void I2Cx_TIMING_Write(I2C_Type* I2Cx, uint32_t SetValue)
{
	I2Cx->TIMING  = (SetValue & I2Cx_TIMING_SDAHD_Msk);
}

uint32_t I2Cx_TIMING_Read(I2C_Type* I2Cx)
{
	return (I2Cx->TIMING  & I2Cx_TIMING_SDAHD_Msk);
}

/* ӻSCL͵ƽչʱڣMSPEN=0¸д
TSCL_STRETCHING_TIMEOUT=TIMEOUT[11:0] * TSCL غ */
void I2Cx_TO_Write(I2C_Type* I2Cx, uint32_t SetValue)
{
	I2Cx->TO = (SetValue & I2Cx_TO_TIMEOUT_Msk);
}

uint32_t I2Cx_TO_Read(I2C_Type* I2Cx)
{
	return (I2Cx->TO & I2Cx_TO_TIMEOUT_Msk);
}


void I2Cx_Deinit(I2C_Type* I2Cx)
{
	RMU_PRSTEN_Write(0x13579BDF);
	if(I2Cx == I2C0)
	{
		RMU_APBRST2_I2C0RST_Setable(ENABLE);  
		RMU_APBRST2_I2C0RST_Setable(DISABLE);
	}
	if(I2Cx == I2C1)
	{
		RMU_APBRST1_I2C1RST_Setable(ENABLE);  
		RMU_APBRST1_I2C1RST_Setable(DISABLE);
	}
  RMU_PRSTEN_Write(0x00000000);
}

 /********************************
 I2CʼĴֵ㺯
ܣI2CʼĴֵ
룺@Hz APBClk @Hz
I2CBRGӦʵֵ
********************************/
uint32_t I2C_BaudREG_Calc(uint32_t I2CClk, uint32_t APBClk)
{
	uint32_t TempREG;
	
	TempREG = APBClk/(2*I2CClk);
	if( TempREG >= 1 ) TempREG = TempREG - 1;
	
	return TempREG;
}

/******END OF FILE****/
