/**
  ******************************************************************************
  * @file    fm33a0xxev_lcd.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_lcd.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup LCD 
  * @brief LCD driver modules
  * @{
  */ 

/* ģʽƵ
000ƵΪ֡Ƶ*COM
001ƵΪLSCLK/8
010ƵΪLSCLK/16
011ƵΪLSCLK/32
100ƵΪLSCLK/64
101ƵΪLSCLK/128
110ƵΪLSCLK/256
111ƵΪLSCLK/512غ */
void LCD_CR_SCSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_SCSEL_Msk);
	tmpreg |= (SetValue & LCD_CR_SCSEL_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_SCSEL_Get(void)
{
	return (LCD->CR & LCD_CR_SCSEL_Msk);
}

/* ģʽʽ 
00
012
104
11 غ */
void LCD_CR_SC_CTRL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_SC_CTRL_Msk);
	tmpreg |= (SetValue & LCD_CR_SC_CTRL_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_SC_CTRL_Get(void)
{
	return (LCD->CR & LCD_CR_SC_CTRL_Msk);
}

/* ƫõ·ԴС (Input bias Current Control)
00
01δ
10С
11С غ */
void LCD_CR_IC_CTRL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_IC_CTRL_Msk);
	tmpreg |= (SetValue & LCD_CR_IC_CTRL_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_IC_CTRL_Get(void)
{
	return (LCD->CR & LCD_CR_IC_CTRL_Msk);
}

/* ģʽѡ (LCD Enabling Mode)
0RFU
1Ƭڵ غ */
void LCD_CR_ENMODE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_ENMODE_Msk);
	tmpreg |= (SetValue & LCD_CR_ENMODE_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_ENMODE_Get(void)
{
	return (LCD->CR & LCD_CR_ENMODE_Msk);
}

/* ʾ˸ʹλ (LCD Flick Enable)
1ʾ˸˸ƵTONTOFFĴ
0ر˸ غ */
void LCD_CR_FLICK_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->CR |= (LCD_CR_FLICK_Msk);
	}
	else
	{
		LCD->CR &= ~(LCD_CR_FLICK_Msk);
	}
}

FunState LCD_CR_FLICK_Getable(void)
{
	if (LCD->CR & (LCD_CR_FLICK_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* LCDƫõƽѡλʾҶȿ (LCD Bias Voltage Select) غ */
void LCD_CR_BIAS_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_BIAS_Msk);
	tmpreg |= ((SetValue<<LCD_CR_BIAS_Pos) & LCD_CR_BIAS_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_BIAS_Get(void)
{
	return ((LCD->CR & LCD_CR_BIAS_Msk)>>LCD_CR_BIAS_Pos);
}

/* ƫѡ (Bias Mode)
11/3 Bias
01/4 Bias غ */
void LCD_CR_BIASMD_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_BIASMD_Msk);
	tmpreg |= (SetValue & LCD_CR_BIASMD_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_BIASMD_Get(void)
{
	return (LCD->CR & LCD_CR_BIASMD_Msk);
}

/* ʹ (Anti-Polarization)
1COMSEGLCDر½ӵ
0COMSEGLCDر¸ غ */
void LCD_CR_ANTIPOLAR_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_ANTIPOLAR_Msk);
	tmpreg |= (SetValue & LCD_CR_ANTIPOLAR_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_ANTIPOLAR_Get(void)
{
	return (LCD->CR & LCD_CR_ANTIPOLAR_Msk);
}

/* ѡ (Waveform Format)
1Bನ
0Aನ غ */
void LCD_CR_WFT_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_WFT_Msk);
	tmpreg |= (SetValue & LCD_CR_WFT_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_WFT_Get(void)
{
	return (LCD->CR & LCD_CR_WFT_Msk);
}

/* COMѡ (Segment Line Mux)
004COM
016COM
10/118COM غ */
void LCD_CR_LMUX_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->CR;
	tmpreg &= ~(LCD_CR_LMUX_Msk);
	tmpreg |= (SetValue & LCD_CR_LMUX_Msk);
	LCD->CR = tmpreg;
}

uint32_t LCD_CR_LMUX_Get(void)
{
	return (LCD->CR & LCD_CR_LMUX_Msk);
}

/* LCDʾʹλ (LCD Enable)
1LCDʾ
0رLCDʾ غ */
void LCD_CR_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->CR |= (LCD_CR_EN_Msk);
	}
	else
	{
		LCD->CR &= ~(LCD_CR_EN_Msk);
	}
}

FunState LCD_CR_EN_Getable(void)
{
	if (LCD->CR & (LCD_CR_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* LCDԿλڲģʽЧ (Line Constant Control)
COMSEGƽɲģʽµݼĴͬSEGCOMĽμı غ */
void LCD_TEST_LCCTRL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->TEST;
	tmpreg &= ~(LCD_TEST_LCCTRL_Msk);
	tmpreg |= (SetValue & LCD_TEST_LCCTRL_Msk);
	LCD->TEST = tmpreg;
}

uint32_t LCD_TEST_LCCTRL_Get(void)
{
	return (LCD->TEST & LCD_TEST_LCCTRL_Msk);
}

/* ģʽʹλ (Test mode Enable)
1LCDģʽʹܡLCDģʽ£LCDž̬ģֱƽ붯̬ɨʱԼɨ貨ؼĴЧ
0ģʽģʽЧزԼĴЧ غ */
void LCD_TEST_TESTEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->TEST |= (LCD_TEST_TESTEN_Msk);
	}
	else
	{
		LCD->TEST &= ~(LCD_TEST_TESTEN_Msk);
	}
}

FunState LCD_TEST_TESTEN_Getable(void)
{
	if (LCD->TEST & (LCD_TEST_TESTEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʾԤƵĴ (Display Frequency) غ */
void LCD_FCR_Write(uint32_t SetValue)
{
	LCD->FCR = (SetValue & LCD_FCR_DF_Msk);
}

uint32_t LCD_FCR_Read(void)
{
	return (LCD->FCR & LCD_FCR_DF_Msk);
}

/* ˸ʾϨʱĴ (Display-Off Time)
TOFFСΪTstep = COM*DF[7:0]*2*16/32768HzʵOFFʱΪTOFF * Tstep غ */
void LCD_FLKT_TOFF_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->FLKT;
	tmpreg &= ~(LCD_FLKT_TOFF_Msk);
	tmpreg |= (SetValue & LCD_FLKT_TOFF_Msk);
	LCD->FLKT = tmpreg;
}

uint32_t LCD_FLKT_TOFF_Get(void)
{
	return (LCD->FLKT & LCD_FLKT_TOFF_Msk);
}

/* ˸ʾʱĴ (Display-On Time)
TONСΪTstep = COM*DF[7:0]*2*16/32768HzʵONʱΪTON * Tstep غ */
void LCD_FLKT_TON_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->FLKT;
	tmpreg &= ~(LCD_FLKT_TON_Msk);
	tmpreg |= (SetValue & LCD_FLKT_TON_Msk);
	LCD->FLKT = tmpreg;
}

uint32_t LCD_FLKT_TON_Get(void)
{
	return (LCD->FLKT & LCD_FLKT_TON_Msk);
}

/* ʾжʹλ (Display-On Interrupt Enable)
1ʾжʹ
0ʾжϽֹ غ */
void LCD_IER_DONIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->IER |= (LCD_IER_DONIE_Msk);
	}
	else
	{
		LCD->IER &= ~(LCD_IER_DONIE_Msk);
	}
}

FunState LCD_IER_DONIE_Getable(void)
{
	if (LCD->IER & (LCD_IER_DONIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʾϨжʹλ (Display-OFF Interrupt Enable)
1ʾϨжʹ
0ʾϨжϽֹ غ */
void LCD_IER_DOFFIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->IER |= (LCD_IER_DOFFIE_Msk);
	}
	else
	{
		LCD->IER &= ~(LCD_IER_DOFFIE_Msk);
	}
}

FunState LCD_IER_DOFFIE_Getable(void)
{
	if (LCD->IER & (LCD_IER_DOFFIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʾжϱ־ (Display-On Interrupt Flag,write 1 to clear)
ʾʱӲжϱ־Ӳλ غ */
void LCD_ISR_DONIF_Clr(void)
{
	LCD->ISR = LCD_ISR_DONIF_Msk;
}

FlagStatus LCD_ISR_DONIF_Chk(void)
{
	if (LCD->ISR & LCD_ISR_DONIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʾϨжϱ־ (Display-OFF Interrupt Flag,write 1 to clear)
ʾʱӲжϱ־Ӳλ غ */
void LCD_ISR_DOFFIF_Clr(void)
{
	LCD->ISR = LCD_ISR_DOFFIF_Msk;
}

FlagStatus LCD_ISR_DOFFIF_Chk(void)
{
	if (LCD->ISR & LCD_ISR_DOFFIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LCD_BSTCR_OSC_CFG_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->BSTCR;
	tmpreg &= ~(LCD_BSTCR_OSC_CFG_Msk);
	tmpreg |= (SetValue & LCD_BSTCR_OSC_CFG_Msk);
	LCD->BSTCR = tmpreg;
}

uint32_t LCD_BSTCR_OSC_CFG_Get(void)
{
	return (LCD->BSTCR & LCD_BSTCR_OSC_CFG_Msk);
}

void LCD_BSTCR_BUF_OFF_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->BSTCR |= (LCD_BSTCR_BUF_OFF_Msk);
	}
	else
	{
		 LCD->BSTCR &= ~(LCD_BSTCR_BUF_OFF_Msk);		
	}
}

FunState LCD_BSTCR_BUF_OFF_Getable(void)
{
	if (LCD->BSTCR & (LCD_BSTCR_BUF_OFF_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void LCD_BSTCR_BUFBYP_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->BSTCR |= (LCD_BSTCR_BUFBYP_Msk);
	}
	else
	{
		LCD->BSTCR &= ~(LCD_BSTCR_BUFBYP_Msk);
	}
}

FunState LCD_BSTCR_BUFBYP_Getable(void)
{
	if (LCD->BSTCR & (LCD_BSTCR_BUFBYP_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void LCD_BSTCR_VLCDPD_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->BSTCR |= (LCD_BSTCR_VLCDPD_Msk);
	}
	else
	{
		LCD->BSTCR &= ~(LCD_BSTCR_VLCDPD_Msk);
	}
}

FunState LCD_BSTCR_VLCDPD_Getable(void)
{
	if (LCD->BSTCR & (LCD_BSTCR_VLCDPD_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void LCD_BSTCR_VLCDCFG_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LCD->BSTCR;
	tmpreg &= ~(LCD_BSTCR_VLCDCFG_Msk);
	tmpreg |= (SetValue & LCD_BSTCR_VLCDCFG_Msk);
	LCD->BSTCR = tmpreg;
}

uint32_t LCD_BSTCR_VLCDCFG_Get(void)
{
	return (LCD->BSTCR & LCD_BSTCR_VLCDCFG_Msk);
}

void LCD_BSTCR_BOOST_TEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->BSTCR |= (LCD_BSTCR_BOOST_TEN_Msk);
	}
	else
	{
		LCD->BSTCR &= ~(LCD_BSTCR_BOOST_TEN_Msk);
	}
}

FunState LCD_BSTCR_BOOST_TEN_Getable(void)
{
	if (LCD->BSTCR & (LCD_BSTCR_BOOST_TEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void LCD_BSTCR_BOOST_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LCD->BSTCR |= (LCD_BSTCR_BOOST_EN_Msk);
	}
	else
	{
		LCD->BSTCR &= ~(LCD_BSTCR_BOOST_EN_Msk);
	}
}

FunState LCD_BSTCR_BOOST_EN_Getable(void)
{
	if (LCD->BSTCR & (LCD_BSTCR_BOOST_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}


/*********************************
LCD ʾƺ 
ܣLCD COM0~3ʹܿ
: 00-0F COM0-COM3
: 
*********************************/
void LCD_COM_EN_Write(uint32_t SetValue)
{
	LCD->COMEN = (SetValue & LCD_COM_EN_COMEN_Msk);
}
/*********************************
LCD ʾƺ 
ܣȡLCD COM0~3ʹ״̬
: 
: 00-0F COM0-COM3
*********************************/
uint32_t LCD_COM_EN_Read(void)
{
	return (LCD->COMEN & LCD_COM_EN_COMEN_Msk);
}


/*********************************
LCD ʾƺ 
ܣLCD SEG0-31ʹܿ
: 00000000-ffffffff	SEG0-SEG31
: 
*********************************/
void LCD_SEG_EN0_Write(uint32_t SetValue)
{
	LCD->SEGEN0 = (SetValue);
}
/*********************************
LCD ʾƺ 
ܣȡLCD SEG0-31ʹ״̬
: 
: 00000000-ffffffff	SEG0-SEG31
*********************************/
uint32_t LCD_SEG_EN0_Read(void)
{
	return (LCD->SEGEN0);
}

/*********************************
LCD ʾƺ 
ܣLCD SEG32~43(COM4-7)ʹܿ
: 00000000-00000fff	SEG32~43(COM4-7)
: 
*********************************/
void LCD_SEG_EN1_Write(uint32_t SetValue)
{
	LCD->SEGEN1 = (SetValue & LCD_SEG_EN1_SEGENx_Msk);
}
/*********************************
LCD ʾƺ 
ܣȡLCD SEG32~43(COM4-7)ʹ״̬
: 
: 00000000-00000fff	SEG32~43(COM4-7)
*********************************/
uint32_t LCD_SEG_EN1_Read(void)
{
	return (LCD->SEGEN1 & LCD_SEG_EN1_SEGENx_Msk);
}

/*DISPDATAxʾݼĴˢ*/
void LCD_DISPDATAx_Refresh(uint32_t* DispBuf)
{
    LCD->DATA0 = DispBuf[0];
    LCD->DATA1 = DispBuf[1];
    LCD->DATA2 = DispBuf[2];
    LCD->DATA3 = DispBuf[3];
    LCD->DATA4 = DispBuf[4];
    LCD->DATA5 = DispBuf[5];
    LCD->DATA6 = DispBuf[6];
    LCD->DATA7 = DispBuf[7];
    LCD->DATA8 = DispBuf[8];
    LCD->DATA9 = DispBuf[9];
}

void LCD_Deinit(void)
{
	//LCD->CR = 0x00010e00;
	//LCD->TEST = 0x00000000;
	//LCD->FCR = 0x00000000;
	//LCD->FLKT = 0x00000000;
	//LCD->IER = 0x00000000;
	//LCD->ISR = 0x00000000;
	//LCD->BSTCR = 0x00000000;
}

/******END OF FILE****/
