/**
  ******************************************************************************
  * @file    fm33a0xxev_lptim.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_lptim.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup LPTIM 
  * @brief LPTIM driver modules
  * @{
  */ 

/* LPT_ETRģ˲ʹ(External Trigger input Analog Filter Enable)
0رģ˲
1ʹģ˲˲Լ100ns غ */
void LPTIM_CFGR_ETR_AFEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->CFGR |= (LPTIM_CFGR_ETR_AFEN_Msk);
	}
	else
	{
		LPTIM->CFGR &= ~(LPTIM_CFGR_ETR_AFEN_Msk);
	}
}

FunState LPTIM_CFGR_ETR_AFEN_Getable(void)
{
	if (LPTIM->CFGR & (LPTIM_CFGR_ETR_AFEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʱԤƵѡ(Prescaler input Select)
0CLKSELѡʱ
1LPTETRF غ */
void LPTIM_CFGR_PSCSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_PSCSEL_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_PSCSEL_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_PSCSEL_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_PSCSEL_Msk);
}

/* ʱӷƵѡ(Counter Clock Divider Select)
0001Ƶ
0012Ƶ
0104Ƶ
0118Ƶ
10016Ƶ
10132Ƶ
11064Ƶ
111128Ƶ غ */
void LPTIM_CFGR_DIVSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_DIVSEL_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_DIVSEL_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_DIVSEL_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_DIVSEL_Msk);
}

/* ETRѡ(ETR Clock Edge Select)
0LPT_ETRؼ
1LPT_ETR½ؼ غ */
void LPTIM_CFGR_EDGESEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_EDGESEL_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_EDGESEL_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_EDGESEL_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_EDGESEL_Msk);
}

/* ⲿѡʹڲʱͬLPT_ETR(ETR trigger Configuration)
00: LPT_ETRźش
01: LPT_ETRź½ش
10/11: ⲿź½ش غ */
void LPTIM_CFGR_TRIGCFG_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_TRIGCFG_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_TRIGCFG_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_TRIGCFG_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_TRIGCFG_Msk);
}

/* μģʽʹ(One State Timer)
0ģʽ󱣳УֱرΪֹﵽĿֵص0¿ʼжϡ
1μģʽĿֵص0Զֹͣжϡ غ */
void LPTIM_CFGR_ONST_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_ONST_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_ONST_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_ONST_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_ONST_Msk);
}

/* ģʽѡ(Timer operation Mode)
00ͨʱģʽ
01Trigger崥ģʽ
10ⲿ첽ģʽ
11Timeoutģʽ غ */
void LPTIM_CFGR_TMODE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CFGR;
	tmpreg &= ~(LPTIM_CFGR_TMODE_Msk);
	tmpreg |= (SetValue & LPTIM_CFGR_TMODE_Msk);
	LPTIM->CFGR = tmpreg;
}

uint32_t LPTIM_CFGR_TMODE_Get(void)
{
	return (LPTIM->CFGR & LPTIM_CFGR_TMODE_Msk);
}

/* 32bitǰֵ(Counter 32bits-wide) غ */
uint32_t LPTIM_CNTR_Read(void)
{
	return (LPTIM->CNTR);
}

/* ͨ1׽Դѡ(Capture channel 1 source select)CH1ͨΪ벶׽ʱЧ
00LPT_CH1
01XTLF
10RCLP
11RCMF غ */
void LPTIM_CCSR_CAP1SSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAP1SSEL_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAP1SSEL_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAP1SSEL_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAP1SSEL_Msk);
}

/* ͨ4ǰ׽ıأCC2IFλʱ(Channel 4 Captured Edge)
0
1½ غ */
void LPTIM_CCSR_CAP4EDGE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAP4EDGE_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAP4EDGE_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAP4EDGE_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAP4EDGE_Msk);
}

void LPTIM_CCSR_CAP3EDGE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAP3EDGE_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAP3EDGE_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAP3EDGE_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAP3EDGE_Msk);
}

void LPTIM_CCSR_CAP2EDGE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAP2EDGE_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAP2EDGE_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAP2EDGE_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAP2EDGE_Msk);
}

void LPTIM_CCSR_CAP1EDGE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAP1EDGE_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAP1EDGE_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAP1EDGE_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAP1EDGE_Msk);
}

/* ͨ4Ƚμѡ (Channel 4 compare output Polarity)
0ԲΣʼΪֵͣ==Ƚֵʱøߣֵ==ARRʱָΪ
1ԲΣԲȡ غ */
void LPTIM_CCSR_POLAR4_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_POLAR4_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_POLAR4_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_POLAR4_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_POLAR4_Msk);
}

void LPTIM_CCSR_POLAR3_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_POLAR3_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_POLAR3_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_POLAR3_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_POLAR3_Msk);
}

void LPTIM_CCSR_POLAR2_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_POLAR2_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_POLAR2_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_POLAR2_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_POLAR2_Msk);
}

void LPTIM_CCSR_POLAR1_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_POLAR1_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_POLAR1_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_POLAR1_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_POLAR1_Msk);
}

/* ͨ4׽ѡ(Channel 4 Capture edge Config)
00ز׽
01½ز׽
10½ز׽
11RFU غ */
void LPTIM_CCSR_CAPCFG4_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAPCFG4_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAPCFG4_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAPCFG4_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAPCFG4_Msk);
}

void LPTIM_CCSR_CAPCFG3_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAPCFG3_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAPCFG3_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAPCFG3_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAPCFG3_Msk);
}

void LPTIM_CCSR_CAPCFG2_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAPCFG2_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAPCFG2_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAPCFG2_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAPCFG2_Msk);
}

void LPTIM_CCSR_CAPCFG1_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CAPCFG1_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CAPCFG1_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CAPCFG1_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CAPCFG1_Msk);
}

/* ͨ4׽/ȽϹʹ(Channel 4 Capture/Compare Select)
00,11ֹͨ4׽/ȽϹ
01ʹͨ4׽ܣLPT_CH4Ϊ룩
10ʹͨ4ȽϹܣLPT_CH4Ϊ غ */
void LPTIM_CCSR_CC4S_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CC4S_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CC4S_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CC4S_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CC4S_Msk);
}

void LPTIM_CCSR_CC3S_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CC3S_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CC3S_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CC3S_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CC3S_Msk);
}

void LPTIM_CCSR_CC2S_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CC2S_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CC2S_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CC2S_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CC2S_Msk);
}

void LPTIM_CCSR_CC1S_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPTIM->CCSR;
	tmpreg &= ~(LPTIM_CCSR_CC1S_Msk);
	tmpreg |= (SetValue & LPTIM_CCSR_CC1S_Msk);
	LPTIM->CCSR = tmpreg;
}

uint32_t LPTIM_CCSR_CC1S_Get(void)
{
	return (LPTIM->CCSR & LPTIM_CCSR_CC1S_Msk);
}

/* ԶĿĴ(Auto-Reload Register)
ֵARRʱصֵ¿ʼϼ غ */
void LPTIM_ARR_Write(uint32_t SetValue)
{
	LPTIM->ARR = (SetValue);
}

uint32_t LPTIM_ARR_Read(void)
{
	return (LPTIM->ARR);
}

/* ͨ4׽жʹ(Channel 4 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
void LPTIM_IER_OVR4IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_OVR4IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_OVR4IE_Msk);
	}
}

FunState LPTIM_IER_OVR4IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_OVR4IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ͨ3׽жʹ(Channel 3 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
void LPTIM_IER_OVR3IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_OVR3IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_OVR3IE_Msk);
	}
}

FunState LPTIM_IER_OVR3IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_OVR3IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ͨ2׽жʹ(Channel 2 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
void LPTIM_IER_OVR2IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_OVR2IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_OVR2IE_Msk);
	}
}

FunState LPTIM_IER_OVR2IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_OVR2IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ͨ1׽жʹ(Channel 1 Over-Capture Interrupt Enable)
1ж
0ֹж غ */
void LPTIM_IER_OVR1IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_OVR1IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_OVR1IE_Msk);
	}
}

FunState LPTIM_IER_OVR1IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_OVR1IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ⲿжʹλ(External Trigger Interrupt Enable)
1ⲿжʹ
0ⲿжϽֹ غ */
void LPTIM_IER_TRIGIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_TRIGIE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_TRIGIE_Msk);
	}
}

FunState LPTIM_IER_TRIGIE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_TRIGIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* жʹλ(Counter Over-Flow Interrupt Enable)
1жʹ
0жϽֹ غ */
void LPTIM_IER_OVIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_OVIE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_OVIE_Msk);
	}
}

FunState LPTIM_IER_OVIE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_OVIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ׽/Ƚͨ4жʹλ(Capture/Compare channel 4 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ غ */
void LPTIM_IER_CC4IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_CC4IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_CC4IE_Msk);
	}
}

FunState LPTIM_IER_CC4IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_CC4IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ׽/Ƚͨ3жʹλ(Capture/Compare channel 3 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ غ */
void LPTIM_IER_CC3IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_CC3IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_CC3IE_Msk);
	}
}

FunState LPTIM_IER_CC3IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_CC3IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ׽/Ƚͨ2жʹλ(Capture/Compare channel 2 Interrupt Enable)
1׽/Ƚͨ2жʹ
0׽/Ƚͨ2жϽֹ غ */
void LPTIM_IER_CC2IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_CC2IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_CC2IE_Msk);
	}
}

FunState LPTIM_IER_CC2IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_CC2IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ׽/Ƚͨ1жʹλ(Capture/Compare channel 1 Interrupt Enable)
1׽/Ƚͨ1жʹ
0׽/Ƚͨ1жϽֹ غ */
void LPTIM_IER_CC1IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->IER |= (LPTIM_IER_CC1IE_Msk);
	}
	else
	{
		LPTIM->IER &= ~(LPTIM_IER_CC1IE_Msk);
	}
}

FunState LPTIM_IER_CC1IE_Getable(void)
{
	if (LPTIM->IER & (LPTIM_IER_CC1IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void LPTIM_ISR_CAP4OVR_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CAP4OVR_Msk;
}

FlagStatus LPTIM_ISR_CAP4OVR_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CAP4OVR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CAP3OVR_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CAP3OVR_Msk;
}

FlagStatus LPTIM_ISR_CAP3OVR_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CAP3OVR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CAP2OVR_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CAP2OVR_Msk;
}

FlagStatus LPTIM_ISR_CAP2OVR_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CAP2OVR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CAP1OVR_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CAP1OVR_Msk;
}

FlagStatus LPTIM_ISR_CAP1OVR_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CAP1OVR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_TRIGIF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_TRIGIF_Msk;
}

FlagStatus LPTIM_ISR_TRIGIF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_TRIGIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_OVIF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_OVIF_Msk;
}

FlagStatus LPTIM_ISR_OVIF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_OVIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CC4IF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CC4IF_Msk;
}

FlagStatus LPTIM_ISR_CC4IF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CC4IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CC3IF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CC3IF_Msk;
}

FlagStatus LPTIM_ISR_CC3IF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CC3IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CC2IF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CC2IF_Msk;
}

FlagStatus LPTIM_ISR_CC2IF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CC2IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

void LPTIM_ISR_CC1IF_Clr(void)
{
	LPTIM->ISR = LPTIM_ISR_CC1IF_Msk;
}

FlagStatus LPTIM_ISR_CC1IF_Chk(void)
{
	if (LPTIM->ISR & LPTIM_ISR_CC1IF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* LPTIMʹλ(LPTIM Enable)
1ʹܼ
0ֹ غ */
void LPTIM_CR_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPTIM->CR |= (LPTIM_CR_EN_Msk);
	}
	else
	{
		LPTIM->CR &= ~(LPTIM_CR_EN_Msk);
	}
}

FunState LPTIM_CR_EN_Getable(void)
{
	if (LPTIM->CR & (LPTIM_CR_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ׽/ȽֵĴ1 (Channel1 Capture/Compare Register) غ */
void LPTIM_CCR1_Write(uint32_t SetValue)
{
	LPTIM->CCR1 = (SetValue);
}

uint32_t LPTIM_CCR1_Read(void)
{
	return (LPTIM->CCR1);
}

/* ׽/ȽֵĴ2 (Channel2 Capture/Compare Register) غ */
void LPTIM_CCR2_Write(uint32_t SetValue)
{
	LPTIM->CCR2 = (SetValue);
}

uint32_t LPTIM_CCR2_Read(void)
{
	return (LPTIM->CCR2);
}

/* ׽/ȽֵĴ3 (Channel3 Capture/Compare Register) غ */
void LPTIM_CCR3_Write(uint32_t SetValue)
{
	LPTIM->CCR3 = (SetValue);
}

uint32_t LPTIM_CCR3_Read(void)
{
	return (LPTIM->CCR3);
}

/* ׽/ȽֵĴ4 (Channel4 Capture/Compare Register) غ */
void LPTIM_CCR4_Write(uint32_t SetValue)
{
	LPTIM->CCR4 = (SetValue);
}

uint32_t LPTIM_CCR4_Read(void)
{
	return (LPTIM->CCR4);
}

void LPTIM_Deinit(void)
{
	//LPTIM->CFGR = 0x00000000;
	//LPTIM->CNTR = 0x00000000;
	//LPTIM->CCSR = ;
	//LPTIM->ARR = 0x00000000;
	//LPTIM->IER = 0x00000000;
	//LPTIM->ISR = ;
	//LPTIM->CR = 0x00000000;
	//LPTIM->CCR1 = 0x00000000;
}

/******END OF FILE****/
