/**
  ******************************************************************************
  * @file    fm33a0xxev_lpuart.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_lpuart.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup LPUART 
  * @brief LPUART driver modules
  * @{
  */ 

/* LPUARTͨű־ֻ (Busy)
1LPUARTͨ
0LPUART غ */
FlagStatus LPUARTx_CSR_BUSY_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->CSR & LPUARTx_CSR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ݽջ غ */
void LPUARTx_CSR_WKBYTECFG_Setable(LPUART_Type* LPUARTx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx->CSR |= (LPUARTx_CSR_WKBYTECFG_Msk);
	}
	else
	{
		LPUARTx->CSR &= ~(LPUARTx_CSR_WKBYTECFG_Msk);
	}
}

/* ݽջ (Wakeup Byte Config)
11ֽڣżУSTOPλȷŴж
01ֽڣУλSTOPλֱӴж غ */
FlagStatus LPUARTx_CSR_WKBYTECFG_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->CSR & LPUARTx_CSR_WKBYTECFG_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ж¼ãڿƺ¼CPUṩж
(Receive Wakeup Event)
00STARTλ⻽
011byteݽ
10ƥɹ
11RXD½ؼ غ */
void LPUARTx_CSR_RXEV_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_RXEV_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_RXEV_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_RXEV_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_RXEV_Msk);
}

/* RXTXŽ
0Ĭ
1 */
void LPUARTx_CSR_IOSWAP_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_IOSWAP_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_IOSWAP_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_IOSWAP_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_IOSWAP_Msk);
}

/* DMAжʹܣLPUARTͨDMAзʱЧ (DMA Transmit Interrupt Config)
1IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź
0ǷжźIE غ */
void LPUARTx_CSR_DMATXIFCFG_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->CSR |= (LPUARTx_CSR_DMATXIFCFG_Msk);
	}
	else
	{
		LPUARTx ->CSR &= ~(LPUARTx_CSR_DMATXIFCFG_Msk);
	}
}

FunState LPUARTx_CSR_DMATXIFCFG_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->CSR & (LPUARTx_CSR_DMATXIFCFG_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ݷ/ʱλ˳ (Bit Order)
0LSB first
1MSB first غ */
void LPUARTx_CSR_BITORD_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_BITORD_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_BITORD_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_BITORD_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_BITORD_Msk);
}

/* ֹͣλãԷ֡ʽЧʱжֹͣλ (Stop bit Config)
01λֹͣλ
12λֹͣλ غ */
void LPUARTx_CSR_STOPCFG_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_STOPCFG_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_STOPCFG_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_STOPCFG_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_STOPCFG_Msk);
}

/* ÿ֡ݳѡ񣻴˼ĴݷͺͽͬʱЧ
(Payload Data length Select)
007λ
018λ
109λ
116λ غ */
void LPUARTx_CSR_PDSEL_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_PDSEL_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_PDSEL_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_PDSEL_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_PDSEL_Msk);
}

/* Уλã˼ĴݷͺͽͬʱЧ (Parity)
00Уλ
01żУ
10У
11RFU غ */
void LPUARTx_CSR_PARITY_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_PARITY_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_PARITY_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_PARITY_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_PARITY_Msk);
}

/* ݼ (Receive Polarity)
0
1ȡ غ */
void LPUARTx_CSR_RXPOL_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_RXPOL_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_RXPOL_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_RXPOL_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_RXPOL_Msk);
}

/* ݼ (Transmit Polarity)
0
1ȡ غ */
void LPUARTx_CSR_TXPOL_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->CSR;
	tmpreg &= ~(LPUARTx_CSR_TXPOL_Msk);
	tmpreg |= (SetValue & LPUARTx_CSR_TXPOL_Msk);
	LPUARTx ->CSR = tmpreg;
}

uint32_t LPUARTx_CSR_TXPOL_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->CSR & LPUARTx_CSR_TXPOL_Msk);
}

/* ʹܣ1Ч (Receive Enable) غ */
void LPUARTx_CSR_RXEN_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->CSR |= (LPUARTx_CSR_RXEN_Msk);
	}
	else
	{
		LPUARTx ->CSR &= ~(LPUARTx_CSR_RXEN_Msk);
	}
}

FunState LPUARTx_CSR_RXEN_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->CSR & (LPUARTx_CSR_RXEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ʹܣ1Ч (Transmit Enable) غ */
void LPUARTx_CSR_TXEN_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->CSR |= (LPUARTx_CSR_TXEN_Msk);
	}
	else
	{
		LPUARTx ->CSR &= ~(LPUARTx_CSR_TXEN_Msk);
	}
}

FunState LPUARTx_CSR_TXEN_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->CSR & (LPUARTx_CSR_TXEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ջ¼жʹܣ1Ч (Receive Event Interrupt Enable) غ */
void LPUARTx_IER_RXEV_IE_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->IER |= (LPUARTx_IER_RXEV_IE_Msk);
	}
	else
	{
		LPUARTx ->IER &= ~(LPUARTx_IER_RXEV_IE_Msk);
	}
}

FunState LPUARTx_IER_RXEV_IE_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->IER & (LPUARTx_IER_RXEV_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* մжʹܣ1Ч (Receive Error Interrupt Enable) غ */
void LPUARTx_IER_RXERR_IE_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->IER |= (LPUARTx_IER_RXERR_IE_Msk);
	}
	else
	{
		LPUARTx ->IER &= ~(LPUARTx_IER_RXERR_IE_Msk);
	}
}

FunState LPUARTx_IER_RXERR_IE_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->IER & (LPUARTx_IER_RXERR_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ջжʹܣ1Ч (Receive Buffer Full Interrupt Enable) غ */
void LPUARTx_IER_RXBF_IE_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->IER |= (LPUARTx_IER_RXBF_IE_Msk);
	}
	else
	{
		LPUARTx ->IER &= ~(LPUARTx_IER_RXBF_IE_Msk);
	}
}

FunState LPUARTx_IER_RXBF_IE_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->IER & (LPUARTx_IER_RXBF_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ͻжʹܣ1Ч (Transmit Buffer Empty Interrupt Enable) غ */
void LPUARTx_IER_TXBE_IE_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->IER |= (LPUARTx_IER_TXBE_IE_Msk);
	}
	else
	{
		LPUARTx ->IER &= ~(LPUARTx_IER_TXBE_IE_Msk);
	}
}

FunState LPUARTx_IER_TXBE_IE_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->IER & (LPUARTx_IER_TXBE_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ͻҷλĴжʹܣ1Ч (Transmit Shift register Interrupt Enable) غ */
void LPUARTx_IER_TXSE_IE_Setable(LPUART_Type* LPUARTx,FunState NewState)
{
	if (NewState == ENABLE)
	{
		LPUARTx ->IER |= (LPUARTx_IER_TXSE_IE_Msk);
	}
	else
	{
		LPUARTx ->IER &= ~(LPUARTx_IER_TXSE_IE_Msk);
	}
}

FunState LPUARTx_IER_TXSE_IE_Getable(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->IER & (LPUARTx_IER_TXSE_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ջ¼жϱ־Ӳλд1 (Receive Event Interrupt Flag)
жϱ־ԴLPUxCR.RXEVĴá غ */
void LPUARTx_ISR_RXEVF_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_RXEVF_Msk;
}

FlagStatus LPUARTx_ISR_RXEVF_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_RXEVF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ͻӲλд1 (Transmit Overflow Error)
ͻеݻδλĴʱͻдݣTXOV־λ غ */
void LPUARTx_ISR_TXOV_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_TXOV_Msk;
}

FlagStatus LPUARTx_ISR_TXOV_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_TXOV_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* żУжϱ־Ӳλд1 (Parity Error) غ */
void LPUARTx_ISR_PERR_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_PERR_Msk;
}

FlagStatus LPUARTx_ISR_PERR_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_PERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ֡ʽжϱ־Ӳλд1 (Frame Error) غ */
void LPUARTx_ISR_FERR_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_FERR_Msk;
}

FlagStatus LPUARTx_ISR_FERR_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_FERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ջжϱ־ջ£յµʱλӲλд1 (Receive Buffer Overflow Error) غ */
void LPUARTx_ISR_OERR_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_OERR_Msk;
}

FlagStatus LPUARTx_ISR_OERR_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_OERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ջжϱ־Ӳλд1߶ȡRXBUFʱ (Receive Buffer Full) غ */
void LPUARTx_ISR_RXBF_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_RXBF_Msk;
}

FlagStatus LPUARTx_ISR_RXBF_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_RXBF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ͻжϱ־ӲλдTXBUFʱ (Transmit Buffer Empty) غ */
FlagStatus LPUARTx_ISR_TXBE_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_TXBE_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ͻҷλĴжϱ־Ӳλд1߷ݱλĴʱ (Transmit Shift register Empty) غ */
void LPUARTx_ISR_TXSE_Clr(LPUART_Type* LPUARTx)
{
	LPUARTx ->ISR = LPUARTx_ISR_TXSE_Msk;
}

FlagStatus LPUARTx_ISR_TXSE_Chk(LPUART_Type* LPUARTx)
{
	if (LPUARTx ->ISR & LPUARTx_ISR_TXSE_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* LPUARTÿbitλƿźţμ!δҵԴ!δҵԴ (Bit Modulation Control) غ */
void LPUARTx_BMR_MCTL_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->BMR;
	tmpreg &= ~(LPUARTx_BMR_MCTL_Msk);
	tmpreg |= (SetValue & LPUARTx_BMR_MCTL_Msk);
	LPUARTx ->BMR = tmpreg;
}

uint32_t LPUARTx_BMR_MCTL_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->BMR & LPUARTx_BMR_MCTL_Msk);
}

/* ʿƣbps
0009600
0014800
0102400
0111200
100600
101/110/111300 غ */
void LPUARTx_BMR_BAUD_Set(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = LPUARTx ->BMR;
	tmpreg &= ~(LPUARTx_BMR_BAUD_Msk);
	tmpreg |= (SetValue & LPUARTx_BMR_BAUD_Msk);
	LPUARTx ->BMR = tmpreg;
}

uint32_t LPUARTx_BMR_BAUD_Get(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->BMR & LPUARTx_BMR_BAUD_Msk);
}

/* ݻĴ (Receive Buffer) غ */
uint32_t LPUARTx_RXBUF_Read(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->RXBUF & LPUARTx_RXBUF_RXBUF_Msk);
}

/* ݻĴ (Transmit Buffer) غ */
void LPUARTx_TXBUF_Write(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	LPUARTx ->TXBUF = (SetValue & LPUARTx_TXBUF_TXBUF_Msk);
}

/* һ֡ձȽݣRXEV=10յĵһ֡MATDͬʱRXEVFжϣģʽµݽջѡ
(Matched Data) غ */
void LPUARTx_DMR_Write(LPUART_Type* LPUARTx,uint32_t SetValue)
{
	LPUARTx ->DMR = (SetValue & LPUARTx_DMR_MATD_Msk);
}

uint32_t LPUARTx_DMR_Read(LPUART_Type* LPUARTx)
{
	return (LPUARTx ->DMR & LPUARTx_DMR_MATD_Msk);
}


void LPUARTx_Deinit(LPUART_Type* LPUARTx)
{
	//LPUARTx ->CSR = 0x00000000;
	//LPUARTx ->IER = 0x00000000;
	//LPUARTx ->ISR = 0x00000002;
	//LPUARTx ->BMR = 0x00000000;
	//LPUARTx ->RXBUF = ;
	//LPUARTx ->TXBUF = ;
	//LPUARTx ->DMR = ;
}

/******END OF FILE****/
