/**
  ******************************************************************************
  * @file    fm33a0xxev_pae.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_pae.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup PAE 
  * @brief PAE driver modules
  * @{
  */ 

/* ˳򽻻
00
01half-word
10byte
11bit
˼Ĵڶ32bitݵλеΪ0x1234_5678half-wordΪ0x5678_1234byteΪ0x7856_3412bitΪ0x1E6A_2C48 غ */
void PAE_CSR_DATA_TYPE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->CSR;
	tmpreg &= ~(PAE_CSR_DATA_TYPE_Msk);
	tmpreg |= (SetValue & PAE_CSR_DATA_TYPE_Msk);
	PAE->CSR = tmpreg;
}

uint32_t PAE_CSR_DATA_TYPE_Get(void)
{
	return (PAE->CSR & PAE_CSR_DATA_TYPE_Msk);
}

/* λд1㣬ӲԶ غ */
void PAE_CSR_START_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		PAE->CSR |= (PAE_CSR_START_Msk);
	}
	else
	{
		PAE->CSR &= ~(PAE_CSR_START_Msk);
	}
}

/* ģ鹤ģʽ
00ģʽ0ģ㣬4Block/16Block
01ģʽ1ε㣬̶16Block
10ģʽ232bitԿRSA㣬̶4Block
11ģʽ332bitԿECC㣬̶16Block غ */
void PAE_CSR_RUN_MODE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->CSR;
	tmpreg &= ~(PAE_CSR_RUN_MODE_Msk);
	tmpreg |= (SetValue & PAE_CSR_RUN_MODE_Msk);
	PAE->CSR = tmpreg;
}

uint32_t PAE_CSR_RUN_MODE_Get(void)
{
	return (PAE->CSR & PAE_CSR_RUN_MODE_Msk);
}

/* PAEλд1λPAEڲҪĴģĴPAE_MLRģμĴPAE_MPRӰ죩λӲԶ
ע⣺дSOFT_RST֮һNOPָдPAEĴSOFT_RSTдPAEĴдʧܡ غ */
void PAE_CSR_SOFT_RST_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		PAE->CSR |= (PAE_CSR_SOFT_RST_Msk);
	}
	else
	{
		PAE->CSR &= ~(PAE_CSR_SOFT_RST_Msk);
	}
}

/* PAEжʹ
1ʹж
0ֹж غ */
void PAE_CSR_PAEIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		PAE->CSR |= (PAE_CSR_PAEIE_Msk);
	}
	else
	{
		PAE->CSR &= ~(PAE_CSR_PAEIE_Msk);
	}
}

FunState PAE_CSR_PAEIE_Getable(void)
{
	if (PAE->CSR & (PAE_CSR_PAEIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}
/* PAEжϱ־Ĵ غ */
void PAE_CSR_PAEIF_Clr(void)
{
	PAE->CSR = PAE_CSR_PAEIF_Msk;
}

FlagStatus PAE_CSR_PAEIF_Chk(void)
{
	if (PAE->CSR & PAE_CSR_PAEIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* PAE־
1PAE
0PAE غ */
FlagStatus PAE_CSR_BUSY_Chk(void)
{
	if (PAE->CSR & PAE_CSR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* RAMΪ4 BlockʱģΪ2048bit
ģ=MLR[5:0]+1*32λ
RAMΪ16 BlockʱģΪ512bitMLRֻе4bitЧ
ģ=MLR[3:0]+1*32λ غ */
void PAE_MLR_Write(uint32_t SetValue)
{
	PAE->MLR = (SetValue & PAE_MLR_PAE_MLR_Msk);
}

uint32_t PAE_MLR_Read(void)
{
	return (PAE->MLR & PAE_MLR_PAE_MLR_Msk);
}

/* 32bitģμĴֻдɶ غ */
void PAE_MPR_Write(uint32_t SetValue)
{
	PAE->MPR = (SetValue);
}

/* RAM Blockλ
1㷨RAMΪ16 Block
0㷨RAMλ4 Block غ */
void PAE_M0CFG_BLKCFG_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M0CFG;
	tmpreg &= ~(PAE_M0CFG_BLKCFG_Msk);
	tmpreg |= (SetValue & PAE_M0CFG_BLKCFG_Msk);
	PAE->M0CFG = tmpreg;
}

uint32_t PAE_M0CFG_BLKCFG_Get(void)
{
	return (PAE->M0CFG & PAE_M0CFG_BLKCFG_Msk);
}

/* ģָ
00ɸģ
01ɸģ
10RFU
11ɸģ غ */
void PAE_M0CFG_INS_M0_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M0CFG;
	tmpreg &= ~(PAE_M0CFG_INS_M0_Msk);
	tmpreg |= (SetValue & PAE_M0CFG_INS_M0_Msk);
	PAE->M0CFG = tmpreg;
}

uint32_t PAE_M0CFG_INS_M0_Get(void)
{
	return (PAE->M0CFG & PAE_M0CFG_INS_M0_Msk);
}

/* Block
BLK_CFG==14bitЧʾBlock0~Block15
BLK_CFG==02bitЧʾBlock0~Block3 غ */
void PAE_M0CFG_RES_BLK_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M0CFG;
	tmpreg &= ~(PAE_M0CFG_RES_BLK_Msk);
	tmpreg |= (SetValue & PAE_M0CFG_RES_BLK_Msk);
	PAE->M0CFG = tmpreg;
}

uint32_t PAE_M0CFG_RES_BLK_Get(void)
{
	return (PAE->M0CFG & PAE_M0CFG_RES_BLK_Msk);
}

/* 1ַ
BLK_CFG==14bitЧʾBlock0~Block15
BLK_CFG==02bitЧʾBlock0~Block3 غ */
void PAE_M0CFG_OP1_BLK_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M0CFG;
	tmpreg &= ~(PAE_M0CFG_OP1_BLK_Msk);
	tmpreg |= (SetValue & PAE_M0CFG_OP1_BLK_Msk);
	PAE->M0CFG = tmpreg;
}

uint32_t PAE_M0CFG_OP1_BLK_Get(void)
{
	return (PAE->M0CFG & PAE_M0CFG_OP1_BLK_Msk);
}

/* 2ַ
BLK_CFG==14bitЧʾBlock0~Block15
BLK_CFG==02bitЧʾBlock0~Block3 غ */
void PAE_M0CFG_OP2_BLK_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M0CFG;
	tmpreg &= ~(PAE_M0CFG_OP2_BLK_Msk);
	tmpreg |= (SetValue & PAE_M0CFG_OP2_BLK_Msk);
	PAE->M0CFG = tmpreg;
}

uint32_t PAE_M0CFG_OP2_BLK_Get(void)
{
	return (PAE->M0CFG & PAE_M0CFG_OP2_BLK_Msk);
}

/* εָ
000Jacobin걶
001
010RFU
011ɸ˱㣨ECDBL
100ɸ˱㣨ECADDDBL
101ɸYָ㣨ECYRecover
110RFU
111RFU غ */
void PAE_M1CFG_INS_M1_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M1CFG;
	tmpreg &= ~(PAE_M1CFG_INS_M1_Msk);
	tmpreg |= (SetValue & PAE_M1CFG_INS_M1_Msk);
	PAE->M1CFG = tmpreg;
}

uint32_t PAE_M1CFG_INS_M1_Get(void)
{
	return (PAE->M1CFG & PAE_M1CFG_INS_M1_Msk);
}

/* Aָʾź
1A-3
0A-3 غ */
void PAE_M1CFG_AEN3_M1_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M1CFG;
	tmpreg &= ~(PAE_M1CFG_AEN3_M1_Msk);
	tmpreg |= (SetValue & PAE_M1CFG_AEN3_M1_Msk);
	PAE->M1CFG = tmpreg;
}

uint32_t PAE_M1CFG_AEN3_M1_Get(void)
{
	return (PAE->M1CFG & PAE_M1CFG_AEN3_M1_Msk);
}

/* ǰԿλָʾ־
1ǰλΪ1
0ǰλΪ0 غ */
void PAE_M1CFG_BIT_VALUE_M1_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M1CFG;
	tmpreg &= ~(PAE_M1CFG_BIT_VALUE_M1_Msk);
	tmpreg |= (SetValue & PAE_M1CFG_BIT_VALUE_M1_Msk);
	PAE->M1CFG = tmpreg;
}

uint32_t PAE_M1CFG_BIT_VALUE_M1_Get(uint32_t SetValue)
{
	return (PAE->M1CFG & PAE_M1CFG_BIT_VALUE_M1_Msk);
}

/* ģ˿λ
1ǰԿλΪ0ʱͬģ˲
0ǰԿλΪ0ʱģ˲ غ */
void PAE_M2CFG_ALWAYS_MULT_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M2CFG;
	tmpreg &= ~(PAE_M2CFG_ALWAYS_MULT_Msk);
	tmpreg |= (SetValue & PAE_M2CFG_ALWAYS_MULT_Msk);
	PAE->M2CFG = tmpreg;
}

uint32_t PAE_M2CFG_ALWAYS_MULT_Get(void)
{
	return (PAE->M2CFG & PAE_M2CFG_ALWAYS_MULT_Msk);
}

/* ԿλָʾźţӲԶλ
1ҵΪ1bit
0δҵΪ1bit غ */
void PAE_M2CFG_FBO_M2_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M2CFG;
	tmpreg &= ~(PAE_M2CFG_FBO_M2_Msk);
	tmpreg |= (SetValue & PAE_M2CFG_FBO_M2_Msk);
	PAE->M2CFG = tmpreg;
}

uint32_t PAE_M2CFG_FBO_M2_Get(void)
{
	return (PAE->M2CFG & PAE_M2CFG_FBO_M2_Msk);
}

/* Aָʾź
1A-3
0A-3 غ */
void PAE_M3CFG_AEN3_M3_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M3CFG;
	tmpreg &= ~(PAE_M3CFG_AEN3_M3_Msk);
	tmpreg |= (SetValue & PAE_M3CFG_AEN3_M3_Msk);
	PAE->M3CFG = tmpreg;
}

uint32_t PAE_M3CFG_AEN3_M3_Get(void)
{
	return (PAE->M3CFG & PAE_M3CFG_AEN3_M3_Msk);
}

/* ԿλָʾźţӲԶλ
1ҵΪ1bit
0δҵΪ1bit غ */
void PAE_M3CFG_FBO_M3_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PAE->M3CFG;
	tmpreg &= ~(PAE_M3CFG_FBO_M3_Msk);
	tmpreg |= (SetValue & PAE_M3CFG_FBO_M3_Msk);
	PAE->M3CFG = tmpreg;
}

uint32_t PAE_M3CFG_FBO_M3_Get(void)
{
	return (PAE->M3CFG & PAE_M3CFG_FBO_M3_Msk);
}

/* 32bitݼĴֻдɶRSAECCģʽЧ غ */
void PAE_WORD_Write(uint32_t SetValue)
{
	PAE->WORD = (SetValue);
}


void PAE_Deinit(void)
{
	//PAE->CSR = 0x00000000;
	//PAE->MLR = 0x00000000;
	//PAE->MPR = 0x00000000;
	//PAE->M0CFG = 0x00000000;
	//PAE->M1CFG = 0x00000000;
	//PAE->M2CFG = 0x00000000;
	//PAE->M3CFG = 0x00000000;
	//PAE->WORD = 0x00000000;
}

/******END OF FILE****/

