/**
  ******************************************************************************
  * @file    fm33a0xxev_pmu.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_pmu.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup PMU 
  * @brief PMU driver modules
  * @{
  */ 

/* LDO15ʹܱ־λ
1LDO15ڹ״̬
0LDO15ر غ */
FlagStatus PMU_CR_LDO15EN_Chk(void)
{
	if (PMU->CR & PMU_CR_LDO15EN_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* LDO15ʹܱ־Уλ غ */
FlagStatus PMU_CR_LDO15EN_B_Chk(void)
{
	if (PMU->CR & PMU_CR_LDO15EN_B_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Sleep/DeepSleepѺϵͳƵ
00RCHF-8MHz
01RCHF-16MHz
10RCHF-24MHz
11RCHF-32MHz غ */
void PMU_CR_WKFSEL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->CR;
	tmpreg &= ~(PMU_CR_WKFSEL_Msk);
	tmpreg |= (SetValue & PMU_CR_WKFSEL_Msk);
	PMU->CR = tmpreg;
}

uint32_t PMU_CR_WKFSEL_Get(void)
{
	return (PMU->CR & PMU_CR_WKFSEL_Msk);
}

/* DeepSleepƼĴ
1DeepSleepģʽʹܣ¹رջ׼ѹԴ
0Sleepģʽ
Sleep£λSLPDPλΪDeepSleepģʽ
λSleepЧ غ */
void PMU_CR_SLPDP_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->CR;
	tmpreg &= ~(PMU_CR_SLPDP_Msk);
	tmpreg |= (SetValue & PMU_CR_SLPDP_Msk);
	PMU->CR = tmpreg;
}

uint32_t PMU_CR_SLPDP_Get(void)
{
	return (PMU->CR & PMU_CR_SLPDP_Msk);
}

/* CoreVoltageScaling
0͹ģʽ²ʹں˵ѹ
1͹ģʽ½ں˵ѹ
λSleep/DeepSleep/RTCBKPģʽ غ */
void PMU_CR_CVS_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->CR;
	tmpreg &= ~(PMU_CR_CVS_Msk);
	tmpreg |= (SetValue & PMU_CR_CVS_Msk);
	PMU->CR = tmpreg;
}

uint32_t PMU_CR_CVS_Get(void)
{
	return (PMU->CR & PMU_CR_CVS_Msk);
}

/* ͹ģʽüĴ
00Active mode / LP Active mode
01LPRUN mode
10Sleep mode / DeepSleep mode
11RTCBKP mode غ */
void PMU_CR_PMOD_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->CR;
	tmpreg &= ~(PMU_CR_PMOD_Msk);
	tmpreg |= (SetValue & PMU_CR_PMOD_Msk);
	PMU->CR = tmpreg;
}

uint32_t PMU_CR_PMOD_Get(void)
{
	return (PMU->CR & PMU_CR_PMOD_Msk);
}

/* Flash Stopѿ
0Stopźŵȴʱӽͬ
1Stopź첽 غ */
void PMU_WKTR_STPCLR_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->WKTR;
	tmpreg &= ~(PMU_WKTR_STPCLR_Msk);
	tmpreg |= (SetValue & PMU_WKTR_STPCLR_Msk);
	PMU->WKTR = tmpreg;
}

uint32_t PMU_WKTR_STPCLR_Get(void)
{
	return (PMU->WKTR & PMU_WKTR_STPCLR_Msk);
}

/* ɱ̶⻽ӳ
DeepSleepģʽ£RCHFʱӵ󣬸ݴ˼ĴõȴӳʱٶȡFlashУ
000us
012us
104us
118us غ */
void PMU_WKTR_T1A_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = PMU->WKTR;
	tmpreg &= ~(PMU_WKTR_T1A_Msk);
	tmpreg |= (SetValue & PMU_WKTR_T1A_Msk);
	PMU->WKTR = tmpreg;
}

uint32_t PMU_WKTR_T1A_Get(void)
{
	return (PMU->WKTR & PMU_WKTR_T1A_Msk);
}

/* ADCжϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_ADCWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_ADCWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* RTCжϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_RTCWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_RTCWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* SVDжϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_SVDWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_SVDWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* 32768Hzͣжϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_LFDETWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_LFDETWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* IOжϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_IOWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_IOWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* LPUART1жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_LPU1WKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_LPU1WKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* LPUART0жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_LPU0WKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_LPU0WKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Ƚout-of-windowѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_COMP_OOWF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_COMP_OOWF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Ƚwindowѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_COMP_WINF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_COMP_WINF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Ƚ2жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_COMP2WKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_COMP2WKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Ƚ1жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_COMP1WKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_COMP1WKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/*  _32жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_LPTWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_LPTWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* BSTIM32жϻѱ־жϳʱӲԶ غ */
FlagStatus PMU_WKFR_BSTWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_BSTWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* CPU Debuggerѱ־д1 غ */
void PMU_WKFR_DBGWKF_Clr(void)
{
	PMU->WKFR = PMU_WKFR_DBGWKF_Msk;
}

FlagStatus PMU_WKFR_DBGWKF_Chk(void)
{
	if (PMU->WKFR & PMU_WKFR_DBGWKF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}


/* SLEEPжʹ (Sleep mode Error Interrupt Enable)
1ʹSLEEPж
0ֹSLEEPж غ */
void PMU_IER_SLPEIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		PMU->IER |= (PMU_IER_SLPEIE_Msk);
	}
	else
	{
		PMU->IER &= ~(PMU_IER_SLPEIE_Msk);
	}
}

FunState PMU_IER_SLPEIE_Getable(void)
{
	if (PMU->IER & (PMU_IER_SLPEIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* RTCBKPжʹ(RTCBKP mode Error Interrupt Enable)
1ʹRTCBKPж
0ֹRTCBKPж غ */
void PMU_IER_RTCEIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		PMU->IER |= (PMU_IER_RTCEIE_Msk);
	}
	else
	{
		PMU->IER &= ~(PMU_IER_RTCEIE_Msk);
	}
}

FunState PMU_IER_RTCEIE_Getable(void)
{
	if (PMU->IER & (PMU_IER_RTCEIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SLEEPжϱ־Ӳλд1 (Sleep Error Interrupt Flag, write 1 to clear)
1PMOD=2h2CPUִWFI/WFEָǰλSLEEPDEEPĴʱλ
0PMOD=2h2CPUȷSLEEP غ */
void PMU_ISR_SLPEIF_Clr(void)
{
	PMU->ISR = PMU_ISR_SLPEIF_Msk;
}

FlagStatus PMU_ISR_SLPEIF_Chk(void)
{
	if (PMU->ISR & PMU_ISR_SLPEIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* RTCBKPжϱ־Ӳλд1(RTC Error Interrupt Flag, write 1 to clear)
1PMOD=2h3δдCPUڲĴSLEEPDEEP=1ȻִWFI/WFEָϵͳʱUSB PHYͼRTCBKPģʽ
0PMOD=2h3CPUȷDEEP SLEEP غ */
void PMU_ISR_RTCEIF_Clr(void)
{
	PMU->ISR = PMU_ISR_RTCEIF_Msk;
}

FlagStatus PMU_ISR_RTCEIF_Chk(void)
{
	if (PMU->ISR & PMU_ISR_RTCEIF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}


void PMU_Deinit(void)
{
	//PMU->CR = 0x00020000;
	//PMU->WKTR = 0x00000001;
	//PMU->WKFR = 0x00000000;
	//PMU->IER = ;
	//PMU->ISR = ;
}

/********************************
NWKUPx Pinѱ־
:NWKUPx Pinѱ־Sleep/DeepSleepģʽЧ
룺NWKUPx Pinѱ־ֵд1.
 ********************************/
void PMU_WKFR_WKPxF_Clr(uint32_t NWKPinDef)
{
	PMU->WKFR = NWKPinDef;
}
/********************************
ȡNWKUPx Pinѱ־
:ȡNWKUPx Pinѱ־
NWKUPx Pinѱ־ֵ
 ********************************/
FlagStatus PMU_WKFR_WKPxF_Chk(uint32_t NWKPinDef)
{
	if (PMU->WKFR & NWKPinDef)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/********************************
͹ģʽĳʼ
:͹ģʽ¸ãSLEEPģʽRAMãں˵ѹ·ȡ
룺͹Ĺܿǰ.
 ********************************/
void PMU_SleepCfg_Init(PMU_SleepCfg_InitTypeDef* SleepCfg_InitStruct)
{
	PMU_CR_SLPDP_Set(SleepCfg_InitStruct->SLPDP);				/*!<DeepSleepƼĴ  */	
	PMU_CR_CVS_Set(SleepCfg_InitStruct->CVS);				/*!<ں˵ѹʹܿ  */	
	if(SleepCfg_InitStruct->SLPDP == PMU_CR_SLPDP_DEEPSLEEP)
	{
		PMU_WKTR_T1A_Set(SleepCfg_InitStruct->TIA);
	}		
	SCB->SCR = SleepCfg_InitStruct->SCR;							/*!<M0ϵͳƼĴһΪ0  */
	PMU_CR_PMOD_Set(SleepCfg_InitStruct->PMOD);					/*!<͹ģʽ  */

}

/******END OF FILE****/
