/**
  ******************************************************************************
  * @file    fm33a0xxev_qspi.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_qspi.h"
#include "fm33a0xxev_rmu.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup QSPI 
  * @brief QSPI driver modules
  * @{
  */ 

void QSPI_CR_PRESCALER_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
    
    SetValue <<= QSPI_CR_PRESCALER_Pos;
    
	tmpreg = QSPI->CR;
	tmpreg &= ~(QSPI_CR_PRESCALER_Msk);
	tmpreg |= (SetValue & QSPI_CR_PRESCALER_Msk);
	QSPI->CR = tmpreg;
}

uint32_t QSPI_CR_PRESCALER_Get(void)
{
	return (QSPI->CR & QSPI_CR_PRESCALER_Msk);
}

/* ѯƥģʽ
0ANDģʽbitƥλSMF
1ORģʽ1bitƥͻλSMF غ */
void QSPI_CR_PMM_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = QSPI->CR;
	tmpreg &= ~(QSPI_CR_PMM_Msk);
	tmpreg |= (SetValue & QSPI_CR_PMM_Msk);
	QSPI->CR = tmpreg;
}

uint32_t QSPI_CR_PMM_Get(void)
{
	return (QSPI->CR & QSPI_CR_PMM_Msk);
}

/* ʱжʹ
0ֹʱж
1ʱж غ */
void QSPI_CR_TOIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_TOIE_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_TOIE_Msk);
	}
}

FunState QSPI_CR_TOIE_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_TOIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ״̬ƥжʹ
0ֹ״̬ƥж
1״̬ƥж غ */
void QSPI_CR_SMIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_SMIE_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_SMIE_Msk);
	}
}

FunState QSPI_CR_SMIE_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_SMIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* FIFOˮλжʹ
0ֹFIFOˮλж
1FIFOˮλж غ */
void QSPI_CR_FTIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_FTIE_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_FTIE_Msk);
	}
}

FunState QSPI_CR_FTIE_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_FTIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* жʹ
0ֹж
1ж غ */
void QSPI_CR_TCIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_TCIE_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_TCIE_Msk);
	}
}

FunState QSPI_CR_TCIE_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_TCIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* жʹ
0ֹж
1ж غ */
void QSPI_CR_TEIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_TEIE_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_TEIE_Msk);
	}
}

FunState QSPI_CR_TEIE_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_TEIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* FIFOˮλĴģʽЧ
дģʽ
0FTFFIFOֽڴڵ1ʱλ
1FTFFIFOֽڴڵ2ʱλ

15FTFFIFOֽڵ16ʱλ
ģʽ
0FTFFIFOֽڵ1ʱλ
1FTFFIFOֽڵ2ʱλ

15FTFFIFOֽ16ʱλ غ */
void QSPI_CR_FIFOTHR_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = QSPI->CR;
	tmpreg &= ~(QSPI_CR_FIFOTHR_Msk);
	tmpreg |= (SetValue & QSPI_CR_FIFOTHR_Msk);
	QSPI->CR = tmpreg;
}

uint32_t QSPI_CR_FIFOTHR_Get(void)
{
	return (QSPI->CR & QSPI_CR_FIFOTHR_Msk);
}

/* ӳٲʹ
0رӳٲ
1ʹӳٲ غ */
void QSPI_CR_SSHFT_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_SSHFT_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_SSHFT_Msk);
	}
}

FunState QSPI_CR_SSHFT_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_SSHFT_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ߳ʱʹܣ˼Ĵڴ洢ӳģʽЧ
BUSYλQuadSPIQSPI洢ķʣʱĴʼTIMEOUTĴ塣QSPI߳ʱ޶nCSԶߣǿƽǰ̡
0رճʱ
1ʹܳʱ غ */
void QSPI_CR_TCEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_TCEN_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_TCEN_Msk);
	}
}

FunState QSPI_CR_TCEN_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_TCEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* DMAʹ
0DMAܹرգQuadSPIᷢDMA
1DMAܿQuadSPIʱDMA غ */
void QSPI_CR_DMAEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_DMAEN_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_DMAEN_Msk);
	}
}

FunState QSPI_CR_DMAEN_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_DMAEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ǰֹĴд1ֹ䣬ӲnCSԶ غ */
void QSPI_CR_ABORT_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_ABORT_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_ABORT_Msk);
	}
}

FunState QSPI_CR_ABORT_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_ABORT_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* QuadSPIģʹ
0رQuadSPI
1ʹQuadSPI غ */
void QSPI_CR_EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		QSPI->CR |= (QSPI_CR_EN_Msk);
	}
	else
	{
		QSPI->CR &= ~(QSPI_CR_EN_Msk);
	}
}

FunState QSPI_CR_EN_Getable(void)
{
	if (QSPI->CR & (QSPI_CR_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* nCSСߵƽʱ䣬֮֡nCS豣ָߵƽʱ䣬QSPI_CLKڼ
01 cycle
12 cycles

78 cycles غ */
void QSPI_CFG_CSHT_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = QSPI->CFG;
	tmpreg &= ~(QSPI_CFG_CSHT_Msk);
	tmpreg |= (SetValue & QSPI_CFG_CSHT_Msk);
	QSPI->CFG = tmpreg;
}

uint32_t QSPI_CFG_CSHT_Get(void)
{
	return (QSPI->CFG & QSPI_CFG_CSHT_Msk);
}

/* SPI Clock ModeĴ
0mode 0
1mode 3 غ */
void QSPI_CFG_CKMODE_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = QSPI->CFG;
	tmpreg &= ~(QSPI_CFG_CKMODE_Msk);
	tmpreg |= (SetValue & QSPI_CFG_CKMODE_Msk);
	QSPI->CFG = tmpreg;
}

uint32_t QSPI_CFG_CKMODE_Get(void)
{
	return (QSPI->CFG & QSPI_CFG_CKMODE_Msk);
}

/* FIFOˮλ־
˼ĴʾǰFIFOбֽ0ʾFIFOգ16ʾFIFO
Զѯģʽ´˼Ĵ0 غ */
uint32_t QSPI_SR_FIFOLVL_Get(void)
{
	return (QSPI->SR & QSPI_SR_FIFOLVL_Msk) >> QSPI_SR_FIFOLVL_Pos;
}

/* 1ʾQuadSPIУͨŽԶ غ */
FlagStatus QSPI_SR_BUSY_Chk(void)
{
	if (QSPI->SR & QSPI_SR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ʱ־Ӳλд1 غ */
void QSPI_SR_TOF_Clr(void)
{
	QSPI->SR = QSPI_SR_TOF_Msk;
}

FlagStatus QSPI_SR_TOF_Chk(void)
{
	if (QSPI->SR & QSPI_SR_TOF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Զѯģʽ±״̬ĴƥɹӲλд1 غ */
void QSPI_SR_SMF_Clr(void)
{
	QSPI->SR = QSPI_SR_SMF_Msk;
}

FlagStatus QSPI_SR_SMF_Chk(void)
{
	if (QSPI->SR & QSPI_SR_SMF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* FIFO threshold־FIFOˮλ趨ֵʱԶλֵʱԶ
Զѯģʽ£ÿζһ״ֵ̬󶼻ԶλFTFȡQSPI_DATAĴFTF غ */
FlagStatus QSPI_SR_FTF_Chk(void)
{
	if (QSPI->SR & QSPI_SR_FTF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ɱ־Ӳλд1 غ */
void QSPI_SR_TCF_Clr(void)
{
	QSPI->SR = QSPI_SR_TCF_Msk;
}

FlagStatus QSPI_SR_TCF_Chk(void)
{
	if (QSPI->SR & QSPI_SR_TCF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ݳΪDATALEN+1bytes غ */
void QSPI_DATALEN_Write(uint32_t SetValue)
{
	QSPI->DATALEN = (SetValue);
}

uint32_t QSPI_DATALEN_Read(void)
{
	return (QSPI->DATALEN);
}

/* QuadSPI ͨſƼĴ غ */
void QSPI_CCR_Write(uint32_t SetValue)
{
    QSPI->CCR = (SetValue);
}

uint32_t QSPI_CCR_Read(void)
{
    return (QSPI->CCR);
}

/* ͸QSPI洢ĵַڴ洢ӳģʽЧ غ */
void QSPI_ADDR_Write(uint32_t SetValue)
{
	QSPI->ADDR = (SetValue);
}

uint32_t QSPI_ADDR_Read(void)
{
	return (QSPI->ADDR);
}

/* ͸QSPI洢alternate bytes غ */
void QSPI_ABR_Write(uint32_t SetValue)
{
	QSPI->ABR = (SetValue);
}

uint32_t QSPI_ABR_Read(void)
{
	return (QSPI->ABR);
}

/* QSPIݼĴ
ģʽдʱQSPI_DRĴдݽpush FIFOֽ֧ڡ֡д룬ֱFIFOѹ124ֽڣдֽFIFOпֽǰдֱFIFO㹻ռɵǰдݡ
ģʽʱȡQSPI_DRĴpop FIFOֽ֧ڡֶ֡ȡֱFIFO124ֽڣȡֽFIFOЧֽǰֱFIFO㹻ֽڿԱȡߴɣһֻ󼸸ʵЧֽڡ
QSPI_DRķʱ͵ַֽڷʱQSPI_DR[7:0]ַʱQSPI_DR[15:0] غ */
void QSPI_DR_WriteByte(uint8_t SetValue)
{
	*((uint8_t *)&(QSPI->DR)) = (SetValue);
}

uint8_t QSPI_DR_ReadByte(void)
{
	return (*((uint8_t *)&(QSPI->DR)));
}

void QSPI_DR_WriteHalfword(uint16_t SetValue)
{
	*((uint16_t *)&(QSPI->DR)) = (SetValue);
}

uint16_t QSPI_DR_ReadHalfword(void)
{
	return (*((uint16_t *)&(QSPI->DR)));
}

void QSPI_DR_WriteWord(uint32_t SetValue)
{
	*((uint32_t *)&(QSPI->DR)) = (SetValue);
}

uint32_t QSPI_DR_ReadWord(void)
{
	return (*((uint32_t *)&(QSPI->DR)));
}

/* Զ״̬ѯģʽµ״̬maskĴӦbitд0Ӧ״̬λ غ */
void QSPI_SMSK_Write(uint32_t SetValue)
{
	QSPI->SMSK = (SetValue);
}

uint32_t QSPI_SMSK_Read(void)
{
	return (QSPI->SMSK);
}

/* Զ״̬ѯģʽµ״̬ƥĴ
Ƚ϶QSPI_DATA & QSPI_SMSK غ */
void QSPI_SMAT_Write(uint32_t SetValue)
{
	QSPI->SMAT = (SetValue);
}

uint32_t QSPI_SMAT_Read(void)
{
	return (QSPI->SMAT);
}

/* Զ״̬ѯģʽµѯpolling intervalΪQSPI_CLK غ */
void QSPI_PITV_Write(uint32_t SetValue)
{
	QSPI->PITV = (SetValue & QSPI_PITV_QSPI_PITV_Msk);
}

uint32_t QSPI_PITV_Read(void)
{
	return (QSPI->PITV & QSPI_PITV_QSPI_PITV_Msk);
}

/* ʱãΪQSPI_CLKڴ洢ӳģʽЧ
FIFO֮QSPIΪֹͣʱʼֵQSPI_TO趨ֵ֮nCS غ */
void QSPI_TO_Write(uint32_t SetValue)
{
	QSPI->TO = (SetValue & QSPI_TO_QSPI_TO_Msk);
}

uint32_t QSPI_TO_Read(void)
{
	return (QSPI->TO & QSPI_TO_QSPI_TO_Msk);
}


void QSPI_Deinit(void)
{
    
}

void QSPI_Init(void)
{
    
}

/******END OF FILE****/
