/**
  ******************************************************************************
  * @file    fm33a0xxev_spi.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_spi.h"
#include "fm33a0xxev_rmu.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup SPI 
  * @brief SPI driver modules
  * @{
  */ 

/* MOSIMISOŽ (IO swapping)
0Ĭ˳
1˳ غ */
void SPIx_CR1_IOSWAP_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_IOSWAP_Msk);
	tmpreg |= (SetValue & SPIx_CR1_IOSWAP_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_IOSWAP_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_IOSWAP_Msk);
}

/* Master Sampling Position AdjustmentMasterMISOźŵĲλõڸͨʱPCBӳ
1ӳٰSCK
0 غ */
void SPIx_CR1_MSPA_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_MSPA_Msk);
	tmpreg |= (SetValue & SPIx_CR1_MSPA_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_MSPA_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_MSPA_Msk);
}

/* Slave Sending Position AdjustmentSlave MISOλõ
1ǰSCKڷ
0 غ */
void SPIx_CR1_SSPA_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_SSPA_Msk);
	tmpreg |= (SetValue & SPIx_CR1_SSPA_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_SSPA_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_SSPA_Msk);
}

/* Master/Slaveģʽѡ(Master Mode)1Masterģʽ0Slaveģʽ غ */
void SPIx_CR1_MM_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_MM_Msk);
	tmpreg |= (SetValue & SPIx_CR1_MM_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_MM_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_MM_Msk);
}

/* Masterģʽ£ÿһ֡(1+WAIT)SCK cycleȴʱ䣬ٴһ֡ݡSSNӲƣSSNM=1ӲԶSSN غ */
void SPIx_CR1_WAIT_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_WAIT_Msk);
	tmpreg |= (SetValue & SPIx_CR1_WAIT_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_WAIT_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_WAIT_Msk);
}

/* Masterģʽλ: (Baud rate)000 fAPBCLK/2001 fAPBCLK/4010 fAPBCLK/8011 fAPBCLK/16100 fAPBCLK/32101 fAPBCLK/64110 fAPBCLK/128111 fAPBCLK/256ͨڽеʱ򣬲޸Щλ غ */
void SPIx_CR1_BAUD_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_BAUD_Msk);
	tmpreg |= (SetValue & SPIx_CR1_BAUD_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_BAUD_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_BAUD_Msk);
}

/* ֡ʽ (LSB First) 0ȷMSB1ȷLSBעͨڽʱܸıλֵ غ */
void SPIx_CR1_LSBF_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_LSBF_Msk);
	tmpreg |= (SetValue & SPIx_CR1_LSBF_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_LSBF_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_LSBF_Msk);
}

void SPIx_CR1_CPHOL_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_CPHOL_Msk);
	tmpreg |= (SetValue & SPIx_CR1_CPHOL_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_CPHOL_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_CPHOL_Msk);
}

/* ʱλѡ (Clock Phase)1ڶʱӱǵһ׽0һʱӱǵһ׽עͨڽʱܸıλֵ غ */
void SPIx_CR1_CPHA_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR1 ;
	tmpreg &= ~(SPIx_CR1_CPHA_Msk);
	tmpreg |= (SetValue & SPIx_CR1_CPHA_Msk);
	SPIx->CR1  = tmpreg;
}

uint32_t SPIx_CR1_CPHA_Get(SPI_Type* SPIx)
{
	return (SPIx->CR1  & SPIx_CR1_CPHA_Msk);
}

/* 4߰˫ЭǷڶвdummy cycle
(Dummy cycle Enable)
0dummy cycle
1ڶ֮һdummy cycle غ */
void SPIx_CR2_DUMMY_EN_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_DUMMY_EN_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_DUMMY_EN_Msk);
	}
}

FunState SPIx_CR2_DUMMY_EN_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_DUMMY_EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* RXONLYλ˼ĴλʱSPIգдTXBUF (Receive Only mode)
1Masterĵģʽ
0رյģʽշȫ˫ غ */
void SPIx_CR2_RXO_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_RXO_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_RXO_Msk);
	}
}

FunState SPIx_CR2_RXO_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_RXO_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ֳͨ (Data Length)
008bit
0116bit
1024bit
1132bit غ */
void SPIx_CR2_DLEN_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_DLEN_Msk);
	tmpreg |= (SetValue & SPIx_CR2_DLEN_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_DLEN_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_DLEN_Msk);
}

/* ͨģʽѡ (Half-Duplex mode)
0׼SPIģʽ4ȫ˫
1DCNģʽ4߰˫ غ */
void SPIx_CR2_HALFDUPLEX_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_HALFDUPLEX_Msk);
	tmpreg |= (SetValue & SPIx_CR2_HALFDUPLEX_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_HALFDUPLEX_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_HALFDUPLEX_Msk);
}

/* ˫ģʽд (Read/Write config for Half-Duplex mode)
04߰˫Эдӻ
14߰˫Эȡӻ غ */
void SPIx_CR2_HD_RW_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_HD_RW_Msk);
	tmpreg |= (SetValue & SPIx_CR2_HD_RW_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_HD_RW_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_HD_RW_Msk);
}

/* ˫ģʽ¶command֡ (Command 8 bits)
1command̶֡Ϊ8bit
0command֡DLEN غ */
void SPIx_CR2_CMD8b_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_CMD8b_Msk);
	tmpreg |= (SetValue & SPIx_CR2_CMD8b_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_CMD8b_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_CMD8b_Msk);
}

/* MasterģʽSSNģʽѡ (SSN mode)
1ÿһ֡MasterSSNάָߵƽʱWAITĴ
0ÿһ֡MasterSSNΪ غ */
void SPIx_CR2_SSNM_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_SSNM_Msk);
	tmpreg |= (SetValue & SPIx_CR2_SSNM_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_SSNM_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_SSNM_Msk);
}

/* TXONLYӲԶյʹ (TXONLY auto-clear enable)1TXONLYӲԶЧʹTXO󣬵ȴϺӲ0رTXONLYӲԶ غ */
void SPIx_CR2_TXO_AC_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_TXO_AC_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_TXO_AC_Msk);
	}
}

FunState SPIx_CR2_TXO_AC_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_TXO_AC_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* TXONLYλ (Transmit Only mode enable)
1Masterĵģʽ
0رյģʽշȫ˫ غ */
void SPIx_CR2_TXO_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_TXO_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_TXO_Msk);
	}
}

FunState SPIx_CR2_TXO_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_TXO_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Masterģʽ£SSNSENΪ1ͨλSSNƽ 1SSNߵƽ
0SSN͵ƽ غ */
void SPIx_CR2_SSN_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->CR2 ;
	tmpreg &= ~(SPIx_CR2_SSN_Msk);
	tmpreg |= (SetValue & SPIx_CR2_SSN_Msk);
	SPIx->CR2  = tmpreg;
}

uint32_t SPIx_CR2_SSN_Get(SPI_Type* SPIx)
{
	return (SPIx->CR2  & SPIx_CR2_SSN_Msk);
}

/* Masterģʽ£SSNʹ (SSN Software Enable)
1MasterģʽSSN
0MasterģʽSSNӲԶ غ */
void SPIx_CR2_SSNSEN_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_SSNSEN_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_SSNSEN_Msk);
	}
}

FunState SPIx_CR2_SSNSEN_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_SSNSEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SPIʹ (SPI enable)1ʹSPI0رSPIշͽջ غ */
void SPIx_CR2_SPIEN_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->CR2  |= (SPIx_CR2_SPIEN_Msk);
	}
	else
	{
		SPIx->CR2  &= ~(SPIx_CR2_SPIEN_Msk);
	}
}

FunState SPIx_CR2_SPIEN_Getable(SPI_Type* SPIx)
{
	if (SPIx->CR2  & (SPIx_CR2_SPIEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* Transmit Buffer Clearд1ͻ棬д0Ч غ */
void SPIx_CR3_TXBFC_Clr(SPI_Type* SPIx)
{
	SPIx->CR3  = SPIx_CR3_TXBFC_Msk;
}

/* Receive Buffer Clearд1ͻ棬д0Ч غ */
void SPIx_CR3_RXBFC_Clr(SPI_Type* SPIx)
{
	SPIx->CR3  = SPIx_CR3_RXBFC_Msk;
}

/* Master Error Clearд1HSPISTA.MERRĴ غ */
void SPIx_CR3_MERRC_Clr(SPI_Type* SPIx)
{
	SPIx->CR3  = SPIx_CR3_MERRC_Msk;
}

/* Slave Error Clearд1HSPISTA.SERRĴ غ */
void SPIx_CR3_SERRC_Clr(SPI_Type* SPIx)
{
	SPIx->CR3  = SPIx_CR3_SERRC_Msk;
}

/* SPIжʹ (Error Interrupt Enable) غ */
void SPIx_IER_ERRIE_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->IER  |= (SPIx_IER_ERRIE_Msk);
	}
	else
	{
		SPIx->IER  &= ~(SPIx_IER_ERRIE_Msk);
	}
}

FunState SPIx_IER_ERRIE_Getable(SPI_Type* SPIx)
{
	if (SPIx->IER  & (SPIx_IER_ERRIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* жʹ (Transmit Interrupt Enable) غ */
void SPIx_IER_TXIE_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->IER  |= (SPIx_IER_TXIE_Msk);
	}
	else
	{
		SPIx->IER  &= ~(SPIx_IER_TXIE_Msk);
	}
}

FunState SPIx_IER_TXIE_Getable(SPI_Type* SPIx)
{
	if (SPIx->IER  & (SPIx_IER_TXIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* жʹ (Receive Interrupt Enable) غ */
void SPIx_IER_RXIE_Setable(SPI_Type* SPIx, FunState NewState)
{
	if (NewState == ENABLE)
	{
		SPIx->IER  |= (SPIx_IER_RXIE_Msk);
	}
	else
	{
		SPIx->IER  &= ~(SPIx_IER_RXIE_Msk);
	}
}

FunState SPIx_IER_RXIE_Getable(SPI_Type* SPIx)
{
	if (SPIx->IER  & (SPIx_IER_RXIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ˫ģʽ£HALFDUPLEX=1ÿ֡bit͵DCNźŵƽ (Data/Command transmit config)
0DCN=0ʾ֡
1DCN=1ʾ֡
ӦڷǰDCN_TXĴDCN_TX=0Ӳһ֡ͺԶDCN_TX1Ĭֻᷢһ֡֡ غ */
void SPIx_ISR_DCN_TX_Set(SPI_Type* SPIx, uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SPIx->ISR ;
	tmpreg &= ~(SPIx_ISR_DCN_TX_Msk);
	tmpreg |= (SetValue & SPIx_ISR_DCN_TX_Msk);
	SPIx->ISR  = tmpreg;
}

uint32_t SPIx_ISR_DCN_TX_Get(SPI_Type* SPIx)
{
	return (SPIx->ISR  & SPIx_ISR_DCN_TX_Msk);
}

/* ջд1 (Receive Collision flag,write 1 to flag) غ */
void SPIx_ISR_RXCOL_Clr(SPI_Type* SPIx)
{
	SPIx->ISR  = SPIx_ISR_RXCOL_Msk;
}

FlagStatus SPIx_ISR_RXCOL_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_RXCOL_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* ͻд1 (Transmit Collision flag,write 1 to clear) غ */
void SPIx_ISR_TXCOL_Clr(SPI_Type* SPIx)
{
	SPIx->ISR  = SPIx_ISR_TXCOL_Msk;
}

FlagStatus SPIx_ISR_TXCOL_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_TXCOL_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* SPIб־ֻ (busy flag)
1SPI
0SPI غ */
FlagStatus SPIx_ISR_BUSY_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_BUSY_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Master Error־(Master Error flag)
Master´δ8λSSNͱʱMERRλ غ */
FlagStatus SPIx_ISR_MERR_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_MERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Slave Error־(Slave Error flag)
Slave´δ8λSSNͱʱSERRλ غ */
FlagStatus SPIx_ISR_SERR_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_SERR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* TX Buffer Empty־λ(TX Buffer Empty flag)
1ͻգдTXBUF
0ͻ غ */
FlagStatus SPIx_ISR_TXBE_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_TXBE_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* RX Buffer Full־λ(RX Buffer Full flag)
1ջRXBUF
0ջ غ */
FlagStatus SPIx_ISR_RXBF_Chk(SPI_Type* SPIx)
{
	if (SPIx->ISR  & SPIx_ISR_RXBF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* SPIͻ (Transmit Buffer) غ */
void SPIx_TXBUF_Write(SPI_Type* SPIx, uint32_t SetValue)
{
	SPIx->TXBUF  = (SetValue);
}

/* SPIջ (Receive Buffer) غ */
uint32_t SPIx_RXBUF_Read(SPI_Type* SPIx)
{
	return (SPIx->RXBUF );
}

void SPIx_Deinit(SPI_Type* SPIx)
{
    RMU_PRSTEN_Write(0x13579BDF);
    if (SPIx == SPI0)
    {
        RMU_APBRST2_SPI0RST_Setable(ENABLE);
        RMU_APBRST2_SPI0RST_Setable(DISABLE);
    }
    else if (SPIx == SPI1)
    {
        RMU_APBRST2_SPI1RST_Setable(ENABLE);
        RMU_APBRST2_SPI1RST_Setable(DISABLE);
    }
    else if (SPIx == SPI2)
    {
        RMU_APBRST1_SPI2RST_Setable(ENABLE);
        RMU_APBRST1_SPI2RST_Setable(DISABLE);
    }
    else if (SPIx == SPI3)
    {
        RMU_APBRST1_SPI3RST_Setable(ENABLE);
        RMU_APBRST1_SPI3RST_Setable(DISABLE);
    }
    else if (SPIx == SPI4)
    {
        RMU_APBRST1_SPI4RST_Setable(ENABLE);
        RMU_APBRST1_SPI4RST_Setable(DISABLE);
    }
    RMU_PRSTEN_Write(0x00000000);
}

/********************************
 SPISSNõƽ͵ĺ
********************************/
void SPI_SSN_Set_Low(SPI_Type* SPIx)//
{
	SPIx_CR2_SSN_Set(SPIx, SPIx_CR2_SSN_LOW);//SSNź
}

/********************************
 SPISSNõƽߵĺ
********************************/
void SPI_SSN_Set_High(SPI_Type* SPIx)//
{
	SPIx_CR2_SSN_Set(SPIx, SPIx_CR2_SSN_HIGH);//SSNź
}

/********************************
 SPIͲһֽڵĺ
ܣSPIͺͽһֽ
룺SPIҪ͵ֽڣSPIֽ
********************************/
uint8_t SPI_RW_Byte(SPI_Type* SPIx, uint8_t data)//SPIͲһֽ
{
  unsigned char rx_data;
	uint32_t Temp32;
	uint32_t i;
	
	Temp32 = SPIx_CR1_BAUD_Get(SPIx);
	Temp32 = Temp32>>SPIx_CR1_BAUD_Pos;
	Temp32 = 0x1U<<(Temp32+1);
	Temp32 = Temp32*8;
	
    for(i=0; i<Temp32; i++)
    {
    	if (SPIx_ISR_TXBE_Chk(SPIx) == SET)
		{
			break;
		}		
    }
    SPIx_TXBUF_Write(SPIx, data);

    for(i=0; i<Temp32; i++)
    {
    	if (SPIx_ISR_RXBF_Chk(SPIx) == SET)
        {
            break;
        }
    }
    rx_data = SPIx_RXBUF_Read(SPIx);
    return rx_data;
}

/******END OF FILE****/
