/**
  ******************************************************************************
  * @file    fm33a0xxev_svd.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_svd.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup SVD 
  * @brief SVD driver modules
  * @{
  */ 
 
/* SVDֵλãμ11.3.3ֵ
(SVD threshold level) غ */
void SVD_CFGR_LVL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SVD->CFGR;
	tmpreg &= ~(SVD_CFGR_LVL_Msk);
	tmpreg |= (SetValue & SVD_CFGR_LVL_Msk);
	SVD->CFGR = tmpreg;
}

uint32_t SVD_CFGR_LVL_Get(void)
{
	return (SVD->CFGR & SVD_CFGR_LVL_Msk);
}

/* ˲ʹܣMOD=1ʱ1
(Digital Filter Enable)
1SVD˲
0رSVD˲ غ */
void SVD_CFGR_DFEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->CFGR |= (SVD_CFGR_DFEN_Msk);
	}
	else
	{
		SVD->CFGR &= ~(SVD_CFGR_DFEN_Msk);
	}
}

FunState SVD_CFGR_DFEN_Getable(void)
{
	if (SVD->CFGR & (SVD_CFGR_DFEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SVDģʽѡģʽҪλSVD_CR.ENŻSVD
(SVD Mode)
1Ъʹģʽ
0ʹģʽ
ע⣺Ъʹģʽ±뿪˲ غ */
void SVD_CFGR_MOD_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SVD->CFGR;
	tmpreg &= ~(SVD_CFGR_MOD_Msk);
	tmpreg |= (SetValue & SVD_CFGR_MOD_Msk);
	SVD->CFGR = tmpreg;
}

uint32_t SVD_CFGR_MOD_Get(void)
{
	return (SVD->CFGR & SVD_CFGR_MOD_Msk);
}

/* SVD IntervalSVDЪʹܼ (SVD periodic interval)
0062.5ms
01256ms
101s
114s غ */
void SVD_CFGR_ITVL_Set(uint32_t SetValue)
{
	uint32_t tmpreg;
	tmpreg = SVD->CFGR;
	tmpreg &= ~(SVD_CFGR_ITVL_Msk);
	tmpreg |= (SetValue & SVD_CFGR_ITVL_Msk);
	SVD->CFGR = tmpreg;
}

uint32_t SVD_CFGR_ITVL_Get(void)
{
	return (SVD->CFGR & SVD_CFGR_ITVL_Msk);
}

/* SVDʹܣд1 (SVD test enable) غ */
void SVD_CR_TE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->CR |= (SVD_CR_TE_Msk);
	}
	else
	{
		SVD->CR &= ~(SVD_CR_TE_Msk);
	}
}

FunState SVD_CR_TE_Getable(void)
{
	if (SVD->CR & (SVD_CR_TE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}


/* SVSⲿԴͨź 
(SVS external monitor channel enable)
0SVSͨر
1SVSͨʹ
EN=1ʱSVDLVLĴSVSǷ񾭹ڲѹLVL=1111SVS벻ѹLVL != 1111SVS뾭ڲѹ غ */
void SVD_CR_SVSEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->CR |= (SVD_CR_SVSEN_Msk);
	}
	else
	{
		SVD->CR &= ~(SVD_CR_SVSEN_Msk);
	}
}

FunState SVD_CR_SVSEN_Getable(void)
{
	if (SVD->CR & (SVD_CR_SVSEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

void SVD_CR_SVDEN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->CR |= (SVD_CR_SVDEN_Msk);
	}
	else
	{
		SVD->CR &= ~(SVD_CR_SVDEN_Msk);
	}
}

FunState SVD_CR_SVDEN_Getable(void)
{
	if (SVD->CR & (SVD_CR_SVDEN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ԴжʹܼĴ1Ч (Power Fall interrupt enable) غ */
void SVD_IER_PFIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->IER |= (SVD_IER_PFIE_Msk);
	}
	else
	{
		SVD->IER &= ~(SVD_IER_PFIE_Msk);
	}
}

FunState SVD_IER_PFIE_Getable(void)
{
	if (SVD->IER & (SVD_IER_PFIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* ԴָжʹܼĴ1Ч (Power Rise interrupt enable) غ */
void SVD_IER_PRIE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->IER |= (SVD_IER_PRIE_Msk);
	}
	else
	{
		SVD->IER &= ~(SVD_IER_PRIE_Msk);
	}
}

FunState SVD_IER_PRIE_Getable(void)
{
	if (SVD->IER & (SVD_IER_PRIE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* SVDԴ (SVD output)
1ԴѹSVDǰֵ
0ԴѹSVDǰֵ غ */
FlagStatus SVD_ISR_SVDO_Chk(void)
{
	if (SVD->ISR & SVD_ISR_SVDO_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* SVDźţֵ·SVD״̬
(SVD registered output) غ */
FlagStatus SVD_ISR_SVDR_Chk(void)
{
	if (SVD->ISR & SVD_ISR_SVDR_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Դжϱ־ĴԴѹ䵽SVDֵ֮ʱλд1 (Power fall flag) غ */
void SVD_ISR_PFF_Clr(void)
{
	SVD->ISR = SVD_ISR_PFF_Msk;
}

FlagStatus SVD_ISR_PFF_Chk(void)
{
	if (SVD->ISR & SVD_ISR_PFF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* Դָжϱ־ĴԴѹSVDֵ֮ʱλд1 (Power rise flag) غ */
void SVD_ISR_PRF_Clr(void)
{
	SVD->ISR = SVD_ISR_PRF_Msk;
}

FlagStatus SVD_ISR_PRF_Chk(void)
{
	if (SVD->ISR & SVD_ISR_PRF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* 1.0V׼ʹź (1.0V reference enable)
1ʹ1.0V׼
0ر1.0V׼ غ */
void SVD_VSR_V1P0EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->VSR |= (SVD_VSR_V1P0EN_Msk);
	}
	else
	{
		SVD->VSR &= ~(SVD_VSR_V1P0EN_Msk);
	}
}

FunState SVD_VSR_V1P0EN_Getable(void)
{
	if (SVD->VSR & (SVD_VSR_V1P0EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* 0.95V׼ʹź (0.95V reference enable)
1ʹ0.95V׼
0ر0.95V׼ غ */
void SVD_VSR_V0P95EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->VSR |= (SVD_VSR_V0P95EN_Msk);
	}
	else
	{
		SVD->VSR &= ~(SVD_VSR_V0P95EN_Msk);
	}
}

FunState SVD_VSR_V0P95EN_Getable(void)
{
	if (SVD->VSR & (SVD_VSR_V0P95EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* 0.9V׼ʹź (0.9V reference enable)
1ʹ0.9V׼
0ر0.9V׼ غ */
void SVD_VSR_V0P9EN_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		SVD->VSR |= (SVD_VSR_V0P9EN_Msk);
	}
	else
	{
		SVD->VSR &= ~(SVD_VSR_V0P9EN_Msk);
	}
}

FunState SVD_VSR_V0P9EN_Getable(void)
{
	if (SVD->VSR & (SVD_VSR_V0P9EN_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}


void SVD_Deinit(void)
{
	//SVD->CFGR = 0x00000008;
	//SVD->CR = 0x00000000;
	//SVD->IER = 0x00000000;
	//SVD->ISR = 0x00000000;
	//SVD->VSR = 0x00000000;
}

/*********************************
SVDʼú 
ܣSVDʼ
: ANAC_SVD_InitTypeDef* paraṹ庯ùģʽֵ˲ȹ
: 
*********************************/
void SVD_Init(SVD_InitTypeDef* para)
{
	SVD_CFGR_MOD_Set(para->SVDMOD);	//SVDģʽѡ
	SVD_CFGR_ITVL_Set(para->SVDITVL);	//SVDЪʹܼ
	SVD_CFGR_LVL_Set(para->SVDLVL);	//SVDֵ
	SVD_CFGR_DFEN_Setable(para->DFEN);	//SVD˲ãSVDMODE=1ʱ1
	SVD_IER_PFIE_Setable(para->PFIE);	//SVDԴж
	SVD_IER_PRIE_Setable(para->PRIE);	//SVDԴָж
	SVD_CR_TE_Setable(DISABLE);	//رղԹ	
}	

/******END OF FILE****/
