/**
  ******************************************************************************
  * @file    fm33a0xxev_wwdt.c
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of....:
  *
*/

/* Includes ------------------------------------------------------------------*/

#include "fm33a0xxev_wwdt.h"

/** @addtogroup fm33a0xxev_StdPeriph_Driver
  * @{
  */

/** @defgroup WWDT 
  * @brief WWDT driver modules
  * @{
  */ 

/* CPU˵ַд0x5AʱWWDTʱ (WWDT Control,write only)
WWDT󣬵CPU˵ַд0xACʱ غ */
void WWDT_CR_Write(uint32_t SetValue)
{
	WWDT->CR = (SetValue & WWDT_CR_CON_Msk);
}

/* WWDTŹʱ䣬λֵ011ϵϵͳʱĬΪ8MhzĬڴԼ32ms (WWDT Config)
000TPCLK * 4096 * 1
001TPCLK * 4096 * 4
010TPCLK * 4096 * 16
011TPCLK * 4096 * 64
100TPCLK * 4096 * 128
101TPCLK * 4096 * 256
110TPCLK * 4096 * 512
111TPCLK * 4096 * 1024 غ */
void WWDT_CFGR_Write(uint32_t SetValue)
{
	WWDT->CFGR = (SetValue & WWDT_CFGR_CFG_Msk);
}

uint32_t WWDT_CFGR_Read(void)
{
	return (WWDT->CFGR & WWDT_CFGR_CFG_Msk);
}

/* WWDTĴֵͨѯ˼Ĵ˽WWDTʱ (WWDT Counter value,read only) غ */
uint32_t WWDT_CNTR_Read(void)
{
	return (WWDT->CNTR & WWDT_CNTR_CNT_Msk);
}

/* WWDTжʹ (WWDT Interrupt Enable)
0жʹֹܽ
1жʹܴ غ */
void WWDT_IER_IE_Setable(FunState NewState)
{
	if (NewState == ENABLE)
	{
		WWDT->IER |= (WWDT_IER_IE_Msk);
	}
	else
	{
		WWDT->IER &= ~(WWDT_IER_IE_Msk);
	}
}

FunState WWDT_IER_IE_Getable(void)
{
	if (WWDT->IER & (WWDT_IER_IE_Msk))
	{
		return ENABLE;
	}
	else
	{
		return DISABLE;
	}
}

/* WWDT 75%ʱжϱ־д1 (Near Overflow Flag,write 1 to clear)
0жϲ
1жϱ־λ
IE=1˼Ĵλж غ */
void WWDT_ISR_NOVF_Clr(void)
{
	WWDT->ISR = WWDT_ISR_NOVF_Msk;
}

FlagStatus WWDT_ISR_NOVF_Chk(void)
{
	if (WWDT->ISR & WWDT_ISR_NOVF_Msk)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/* WWDT4096ԤƵǰֵֻ (WWDT prescaler Divider Counte,read only) غ */
uint32_t WWDT_PSCR_Read(void)
{
	return (WWDT->PSCR & WWDT_PSCR_DIV_CNT_Msk);
}


void WWDT_Deinit(void)
{
	//WWDT->CR = 0x00000000;
	//WWDT->CFGR = 0x00000003;
	//WWDT->CNTR = 0x00000000;
	//WWDT->IER = 0x00000000;
	//WWDT->ISR = 0x00000000;
	//WWDT->PSCR = 0x00000000;
}

/******END OF FILE****/
