/*!
 * \file      smtc_hal_bstim.c
 *
 * \brief     Implements Low Power Timer utilities functions.
 *
 * Revised BSD License
 * Copyright Semtech Corporation 2020. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Semtech corporation nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL SEMTECH CORPORATION BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * -----------------------------------------------------------------------------
 * --- DEPENDENCIES ------------------------------------------------------------
 */
#include <stdbool.h>  // bool type
#include <stdint.h>   // C99 types

#include "smtc_hal.h"
#include "system_FM33A0XXEV.h"
#include "fm33a0xxev_include_all.h"

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE MACROS-----------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE CONSTANTS -------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE TYPES -----------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE VARIABLES -------------------------------------------------------
 */

static hal_bstim_irq_t bstim_tmr_irq = { .context = NULL, .callback = NULL };

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE FUNCTIONS DECLARATION -------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PUBLIC FUNCTIONS DEFINITION ---------------------------------------------
 */

void hal_bstim_init( void )
{
    CMU_PERCLK_SetableEx( BSTIMCLK, ENABLE );          //ʱʹ
    CMU_OPCCR2_BSTCKS_Set( CMU_OPCCR2_BSTCKS_LSCLK );  //ѡʱԴ--XTLFϵĬʹ
    CMU_OPCCR2_BSTCKE_Setable( ENABLE );               //ʱԴʹ

    BSTIM_CR1_ARPE_Setable( DISABLE );        //Ԥװزʹ
    BSTIM_CR1_OPM_Set( BSTIM_CR1_OPM_STOP );  // UEʱֹͣ
    BSTIM_PSCR_Write( 1 - 1 );                //ʱӲƵ

    NVIC_DisableIRQ( BSTIM_IRQn );
    NVIC_SetPriority( BSTIM_IRQn, 2 );  //жȼ
    NVIC_EnableIRQ( BSTIM_IRQn );

    bstim_tmr_irq = ( hal_bstim_irq_t ){ .context = NULL, .callback = NULL };
}

void hal_bstim_start( const uint32_t milliseconds, const hal_bstim_irq_t* tmr_irq )
{
    uint32_t delay_ms_2_tick = 0;

    // Remark LSE_VALUE / BSTIM_PRESCALER_DIV1
    delay_ms_2_tick = ( uint32_t )( ( ( uint64_t ) milliseconds * ( 32768 ) ) / 1000 );

    BSTIM_ARR_Write( delay_ms_2_tick - 1 );  //ʱmsλ

    BSTIM_ISR_UIF_Clr( );
    BSTIM_IER_UIE_Setable( ENABLE );  // Updateжʹ
    BSTIM_CR1_CEN_Setable( ENABLE );  /* enable the counter */

    bstim_tmr_irq.callback = tmr_irq->callback;
    bstim_tmr_irq.context  = tmr_irq->context;
}

void hal_bstim_stop( void )
{
    BSTIM_CR1_CEN_Setable( DISABLE );  //
    BSTIM_IER_UIE_Setable( DISABLE );
}

void hal_bstim_irq_enable( void )
{
    NVIC_EnableIRQ( BSTIM_IRQn );
}

void hal_bstim_irq_disable( void )
{
    NVIC_DisableIRQ( BSTIM_IRQn );
}

void BSTIM_IRQHandler( void )
{
    if( BSTIM_ISR_UIF_Chk( ) == SET )
    {
        BSTIM_ISR_UIF_Clr( );
        BSTIM_CR1_CEN_Setable( DISABLE );  //
        BSTIM_IER_UIE_Setable( DISABLE );

        if( bstim_tmr_irq.callback != NULL )
        {
            bstim_tmr_irq.callback( bstim_tmr_irq.context );
        }
    }
}

/*
 * -----------------------------------------------------------------------------
 * --- PRIVATE FUNCTIONS DEFINITION --------------------------------------------
 */

/* --- EOF ------------------------------------------------------------------ */
