/*!
 * \file      smtc_hal_rng.h
 *
 * \brief     Random Number Generator Hardware Abstraction Layer definition
 *
 * Revised BSD License
 * Copyright Semtech Corporation 2020. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Semtech corporation nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL SEMTECH CORPORATION BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __SMTC_HAL_RNG_H__
#define __SMTC_HAL_RNG_H__

#ifdef __cplusplus
extern "C" {
#endif
/*
 * -----------------------------------------------------------------------------
 * --- DEPENDENCIES ------------------------------------------------------------
 */

#include <stdint.h>   // C99 types
#include <stdbool.h>  // bool type

/*
 * -----------------------------------------------------------------------------
 * --- PUBLIC MACROS -----------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PUBLIC CONSTANTS --------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PUBLIC TYPES ------------------------------------------------------------
 */

/*
 * -----------------------------------------------------------------------------
 * --- PUBLIC FUNCTIONS PROTOTYPES ---------------------------------------------
 */

/*!
 *
 */
void hal_rng_init( void );

/*!
 * Returns an hardware generated random number.
 *
 * \retval random Generated radom number
 */
uint32_t hal_rng_get_random( void );

/*!
 * Returns an hardware generated unsigned random number between min and max
 *
 * \param [IN] val_1 first range unsigned value
 * \param [IN] val_2 second range unsigned value
 *
 * \retval random Generated random unsigned number between smallest value and biggest
 * value between val_1 and val_2
 */
uint32_t hal_rng_get_random_in_range( const uint32_t val_1, const uint32_t val_2 );

/*!
 * Returns an hardware generated signed random number between min and max
 *
 * \param [IN] val_1 first range signed value
 * \param [IN] val_2 second range signed value
 *
 * \retval random Generated random signed number between smallest value and biggest
 * value between val_1 and val_2
 */
int32_t hal_rng_get_signed_random_in_range( const int32_t val_1, const int32_t val_2 );

#ifdef __cplusplus
}
#endif

#endif  // __SMTC_HAL_RNG_H__

/* --- EOF ------------------------------------------------------------------ */
